@@ -23,12 +23,14 @@
 #include "servicing.h"
 
 #include "../vehicle.h"
-#include "../unitctrl.h"
+#include "../spfst.h"
 #include "../containercontrols.h"
 #include "../gameoptions.h"
-#include "../replay.h"
 
+#include "consumeresource.h"
+#include "consumeammo.h"
 
+#include "context.h"
 
 enum TransferLimitation { NONE, GET, ALL };
 
@@ -178,8 +180,7 @@ bool ResourceWatch::getResources( Resources res )
 
    bool Transferrable::setDestAmount( long amount )
    {
-      setAmount( getDstContainer(), amount );
-      return true;
+      return setAmount( getDstContainer(), amount ) == amount;
    }
 
    void Transferrable::showAll()
@@ -207,61 +208,121 @@ bool ResourceWatch::getResources( Resources res )
 
 
 
+   class AmmoTransferrable : public Transferrable {
+      private:
+         int ammoType;
+         int sourceAmmo;
+         int destAmmo;
+         map<const ContainerBase*,int> produced;
+         map<const ContainerBase*,int> orgAmmo;
+         bool& allowAmmoProduction;
+
+
+         int& getAmmo( const ContainerBase* unit );
+         int put( ContainerBase* c, int toPut, bool queryOnly );
+         int get( ContainerBase* c, int toGet, bool queryOnly );
+         void executeTransfer( ContainerBase* from, ContainerBase* to, int amount, const Context& context );
+      
+      public:
+         AmmoTransferrable( int ammo, ResourceWatch& src, ResourceWatch& dst, bool& allowProduction );      
+         ASCString getName();
+      
+         int getID();
+         int getMax( ContainerBase* c, bool avail );
+         int getMin( ContainerBase* c, bool avail );
+         int getAmount ( const ContainerBase* target );
+         int transfer( ContainerBase* target, int delta );
+         bool isExchangable() const;
+         void commit( const Context& context );
+   };
+
+   class ResourceTransferrable : public Transferrable {
+      private:
+         int resourceType;
+         bool exchangable;
+         void warn();
+         void srcChanged( int res );
+         void dstChanged( int res );
+         void executeTransfer( ContainerBase* from, ContainerBase* to, int amount, const Context& context );
+      public:
+         ResourceTransferrable( int resource, ResourceWatch& src, ResourceWatch& dst, bool isExchangable = true );
+         ASCString getName();
+         int getID();
+         int getMax( ContainerBase* c, bool avail );
+         int getMin( ContainerBase* c, bool avail );
+         int getAmount ( const ContainerBase* target );
+         int getAvail ( const ContainerBase* target );
+         int transfer( ContainerBase* target, int delta );
+         bool isExchangable() const;
+         void commit(const Context& context);
+   };
+   
+   
       
 
 
-class ResourceTransferrable : public Transferrable {
-   private:
-      int resourceType;
-      bool exchangable;
-      
-      void warn()
+   void ResourceTransferrable::warn()
       {
-         warning( "Inconsistency in ResourceTransfer");
+         warningMessage( "Inconsistency in ResourceTransfer");
       };
 
-      void srcChanged( int res )
+      void ResourceTransferrable::srcChanged( int res )
       {
          if ( res == resourceType ) {
             show( source.getContainer() );
          }
       }
       
-      void dstChanged( int res )
+      void ResourceTransferrable::dstChanged( int res )
       {
          if ( res == resourceType ) {
             show( dest.getContainer() );
          }
       }
             
-      void executeTransfer( ContainerBase* from, ContainerBase* to, int amount )
+         
+      void ResourceTransferrable::executeTransfer( ContainerBase* from, ContainerBase* to, int amount, const Context& context )
       {
          if ( amount == 0 )
             return;
          
          if ( amount < 0 )
-            executeTransfer( to, from, -amount );
-         else {
-            int got = from->getResource( amount, resourceType, false );
-            if ( got != amount )
-               warning( ASCString("did not succeed in transfering resource ") + Resources::name( resourceType ) );
-            to->putResource( got, resourceType, false );
-
-            logtoreplayinfo( rpl_refuel3, from->getIdentification(), 1000+resourceType, got );
-            logtoreplayinfo( rpl_refuel3, to->getIdentification(), 1000+resourceType, -got );
-         }
+            executeTransfer( to, from, -amount, context );
+         else 
+            if ( amount > 0 ) {
+               Resources r;
+               r.resource(resourceType) = amount;
+               
+               ActionResult res = (new ConsumeResource( from, r ))->execute( context );
+               if ( !res.successful() )
+                  throw res;
+               
+               res = (new ConsumeResource( to, -r))->execute( context );
+               if ( !res.successful() )
+                  throw res;
+               
+            }
       }
-         
       
-   public:
-      ResourceTransferrable( int resource, ResourceWatch& src, ResourceWatch& dst, bool isExchangable = true ) : Transferrable( src, dst ), resourceType ( resource ), exchangable( isExchangable )
+      
+ResourceTransferrable::ResourceTransferrable( int resource, ResourceWatch& src, ResourceWatch& dst, bool isExchangable  ) 
+   : Transferrable( src, dst ), resourceType ( resource ), exchangable( isExchangable )
       {
          source.sigChanged.connect( SigC::slot( *this, &ResourceTransferrable::srcChanged ));
          dest.sigChanged.connect( SigC::slot( *this, &ResourceTransferrable::dstChanged ));
       };
-      ASCString getName() { return Resources::name( resourceType ); };
       
-      int getMax( ContainerBase* c, bool avail )
+      ASCString ResourceTransferrable::getName() 
+      { 
+         return Resources::name( resourceType ); 
+      }
+      
+      int ResourceTransferrable::getID()
+      {
+         return resourceType + 1000; 
+      };
+      
+      int ResourceTransferrable::getMax( ContainerBase* c, bool avail )
       {
          if ( avail ) {
             int needed = getResourceWatch( c ).limit().resource(resourceType) - getAvail( c );
@@ -272,7 +333,7 @@ class ResourceTransferrable : public Transferrable {
             return getResourceWatch( c ).limit().resource(resourceType);
       }
       
-      int getMin( ContainerBase* c, bool avail )
+      int ResourceTransferrable::getMin( ContainerBase* c, bool avail )
       {
          if ( avail ) {
             int space = getOpposingResourceWatch( c ).limit().resource(resourceType) - getOpposingResourceWatch( c ).avail().resource(resourceType);
@@ -284,17 +345,17 @@ class ResourceTransferrable : public Transferrable {
             return 0;
       }
       
-      int getAmount ( const ContainerBase* target )
+      int ResourceTransferrable::getAmount ( const ContainerBase* target )
       {
          return getResourceWatch( target ).amount().resource(resourceType);
       }
       
-      int getAvail ( const ContainerBase* target )
+      int ResourceTransferrable::getAvail ( const ContainerBase* target )
       {
          return getResourceWatch( target ).avail().resource(resourceType);
       }
       
-      int transfer( ContainerBase* target, int delta )
+      int ResourceTransferrable::transfer( ContainerBase* target, int delta )
       {
          if ( delta < 0 )
             return transfer( opposingContainer( target ), -delta );
@@ -307,30 +368,26 @@ class ResourceTransferrable : public Transferrable {
          }
       }
 
-      bool isExchangable() const
+      bool ResourceTransferrable::isExchangable() const
       {
          return exchangable;
       }
       
-      void commit()
+      void ResourceTransferrable::commit(const Context& context)
       {
          ContainerBase* target = dest.getContainer();
-         executeTransfer( source.getContainer(), target, getAmount( target ) - target->getResource( maxint, resourceType, true ));
+         executeTransfer( source.getContainer(), target, getAmount( target ) - target->getResource( maxint, resourceType, true ), context );
       }
       
-};
-
-class AmmoTransferrable : public Transferrable {
-   private:
-      int ammoType;
-      int sourceAmmo;
-      int destAmmo;
-      map<const ContainerBase*,int> produced;
-      map<const ContainerBase*,int> orgAmmo;
-      bool& allowAmmoProduction;
+      
+      
+      
+      
+      
+      
 
 
-      int& getAmmo( const ContainerBase* unit )
+      int& AmmoTransferrable::getAmmo( const ContainerBase* unit )
       {
          assert( unit == source.getContainer() || unit == dest.getContainer() );
          if ( unit == source.getContainer() )
@@ -339,7 +396,7 @@ class AmmoTransferrable : public Transferrable {
             return destAmmo;
       }
       
-      int put( ContainerBase* c, int toPut, bool queryOnly )
+      int AmmoTransferrable::put( ContainerBase* c, int toPut, bool queryOnly )
       {
          TransferLimitation limit = getTransferLimitation( c );
 
@@ -350,7 +407,7 @@ class AmmoTransferrable : public Transferrable {
          if ( undoProduction > 0 ) {
             if ( !queryOnly ) {
                for ( int r = 0; r < resourceTypeNum; ++r )
-                  getResourceWatch( c ).putResource(  r, cwaffenproduktionskosten[ammoType][r] * undoProduction );
+                  getResourceWatch( c ).putResource(  r, ammoProductionCost[ammoType][r] * undoProduction );
                
                produced[c] -= undoProduction;
             }
@@ -367,7 +424,7 @@ class AmmoTransferrable : public Transferrable {
          return toPut + undoProduction;
       }
             
-      int get( ContainerBase* c, int toGet, bool queryOnly )
+      int AmmoTransferrable::get( ContainerBase* c, int toGet, bool queryOnly )
       {
          TransferLimitation limit = getTransferLimitation( c );
 
@@ -386,10 +443,10 @@ class AmmoTransferrable : public Transferrable {
          int got = min ( toGet, gettable);
          int toProduce = toGet - got;
 
-         if ( allowAmmoProduction && toProduce > 0 && weaponAmmo[ammoType] ) {
+         if ( allowAmmoProduction && toProduce > 0 && weaponAmmo[ammoType] && c->baseType->hasFunction(ContainerBaseType::AmmoProduction) ) {
             for ( int r = 0; r < resourceTypeNum; ++r ) {
-               if ( cwaffenproduktionskosten[ammoType][r] ) {
-                  int produceable = getResourceWatch(c).avail().resource(r) / cwaffenproduktionskosten[ammoType][r];
+               if ( ammoProductionCost[ammoType][r] ) {
+                  int produceable = getResourceWatch(c).avail().resource(r) / ammoProductionCost[ammoType][r];
                   if ( produceable < toProduce )
                      toProduce = produceable;
                }
@@ -397,7 +454,7 @@ class AmmoTransferrable : public Transferrable {
             if ( !queryOnly ) {
                Resources res;
                for ( int r = 0; r < resourceTypeNum; ++r )
-                  res.resource(r) = cwaffenproduktionskosten[ammoType][r] * toProduce;
+                  res.resource(r) = ammoProductionCost[ammoType][r] * toProduce;
 
                getResourceWatch( c ).getResources( res );
                produced[c] += toProduce;
@@ -415,25 +472,32 @@ class AmmoTransferrable : public Transferrable {
          return got;
       }
 
-      void executeTransfer( ContainerBase* from, ContainerBase* to, int amount )
+     
+      void AmmoTransferrable::executeTransfer( ContainerBase* from, ContainerBase* to, int amount, const Context& context )
       {
          if ( amount < 0 )
-            executeTransfer( to, from, -amount );
-         else {
-            ContainerControls cc( from );
-            int got = cc.getammunition( ammoType, amount, true, allowAmmoProduction );
-            if ( got != amount )
-               warning( "did not succeed in transfering ammo" );
-            to->putAmmo( ammoType, got, false );
-
-            logtoreplayinfo( rpl_refuel3, from->getIdentification(), ammoType, got );
-            logtoreplayinfo( rpl_refuel3, to->getIdentification(), ammoType, -got );
-            
-         }
+            executeTransfer( to, from, -amount, context );
+         else 
+            if ( amount > 0 ) {
+               auto_ptr<ConsumeAmmo> ca1 ( new ConsumeAmmo( from, ammoType, -1, amount ));
+               ca1->setAmmoProduction( allowAmmoProduction );
+               
+               ActionResult res = ca1->execute( context );
+               if( res.successful() )
+                  ca1.release();
+               else
+                  throw res;
+               
+               auto_ptr<ConsumeAmmo> ca2 ( new ConsumeAmmo( to, ammoType, -1, -amount ));
+               res = ca2->execute( context );
+               if( res.successful() )
+                  ca2.release();
+               else
+                  throw res;
+            }
       }
       
-   public:
-      AmmoTransferrable( int ammo, ResourceWatch& src, ResourceWatch& dst, bool& allowProduction ) : Transferrable( src, dst ), ammoType ( ammo ), allowAmmoProduction( allowProduction )
+      AmmoTransferrable::AmmoTransferrable( int ammo, ResourceWatch& src, ResourceWatch& dst, bool& allowProduction ) : Transferrable( src, dst ), ammoType ( ammo ), allowAmmoProduction( allowProduction )
       {
          sourceAmmo = src.getContainer()->getAmmo( ammoType, maxint, true );
          destAmmo   = dst.getContainer()->getAmmo( ammoType, maxint, true );
@@ -442,10 +506,18 @@ class AmmoTransferrable : public Transferrable {
          orgAmmo[dst.getContainer()] = destAmmo;
       };
       
-      ASCString getName() { return cwaffentypen[ ammoType ]; };
+      ASCString AmmoTransferrable::getName() 
+      { 
+         return cwaffentypen[ ammoType ]; 
+      }
       
+      int AmmoTransferrable::getID()
+      {
+         // the AI depends on this value and ID scheme!
+         return ammoType + 2000; 
+      };
       
-      int getMax( ContainerBase* c, bool avail )
+      int AmmoTransferrable::getMax( ContainerBase* c, bool avail )
       {
          if ( avail ) {
             int needed = c->maxAmmo( ammoType ) - getAmount( c );
@@ -455,7 +527,7 @@ class AmmoTransferrable : public Transferrable {
             return c->maxAmmo( ammoType );
       }
       
-      int getMin( ContainerBase* c, bool avail )
+      int AmmoTransferrable::getMin( ContainerBase* c, bool avail )
       {
          if ( avail ) {
             int storable = opposingContainer(c)->maxAmmo(ammoType) - getAmmo( opposingContainer(c) );
@@ -465,12 +537,12 @@ class AmmoTransferrable : public Transferrable {
             return 0;
       }
       
-      int getAmount ( const ContainerBase* target )
+      int AmmoTransferrable::getAmount ( const ContainerBase* target )
       {
          return getAmmo( target );
       }
       
-      int transfer( ContainerBase* target, int delta )
+      int AmmoTransferrable::transfer( ContainerBase* target, int delta )
       {
          if ( delta < 0 )
             return transfer( opposingContainer( target ), -delta );
@@ -482,17 +554,15 @@ class AmmoTransferrable : public Transferrable {
          }
       }
       
-      bool isExchangable() const
+      bool AmmoTransferrable::isExchangable() const
       {
          return true;
       };
       
-      void commit()
+      void AmmoTransferrable::commit( const Context& context )
       {
-         executeTransfer( source.getContainer(), dest.getContainer(), destAmmo - orgAmmo[dest.getContainer()] );
+         executeTransfer( source.getContainer(), dest.getContainer(), destAmmo - orgAmmo[dest.getContainer()], context );
       }
-      
-};
 
 
 
@@ -582,7 +652,7 @@ void ServiceChecker :: check( ContainerBase* dest )
    /* it is important that the ammo transfers are in front of the resource transfers, because ammo production affects resource amounts
       and their prelimarny commitment would cause inconsistencies */
 
-      for ( int a = 0; a < cwaffentypennum; ++a ) {
+      for ( int a = 0; a < weaponTypeNum; ++a ) {
          if ( source->maxAmmo( a ) && dest->maxAmmo( a )) {
             if ( weaponAmmo[a] ) {
                if ( externalTransfer ) {
@@ -606,6 +676,24 @@ void ServiceChecker :: check( ContainerBase* dest )
          }
       }
 
+      
+      
+      if (  externalTransfer ) {
+         Vehicle* srcVehicle = dynamic_cast<Vehicle*>(source);
+         if ( srcVehicle ) {// it's a unit
+            if ( srcVehicle->baseType->hasFunction( ContainerBaseType::ExternalRepair ))
+               if ( serviceWeaponFits( dest ) && dest->damage > 0 ) {
+                  repair( dest );
+               }
+         }
+
+      } else {
+         if ( source->baseType->hasFunction( ContainerBaseType::InternalUnitRepair ))
+            repair( dest );
+      }
+      
+      
+      
 }
 
 
@@ -625,22 +713,31 @@ void ServiceTargetSearcher::addTarget( ContainerBase* target )
 
 void ServiceTargetSearcher::ammo( ContainerBase* dest, int type )
 {
-   addTarget ( dest );
+   if ( checks & checkAmmo )
+      addTarget ( dest );
 }
 
 void ServiceTargetSearcher::resource( ContainerBase* dest, int type, bool active )
 {
-   if ( active )
+   if ( checks & checkResources )
+      if ( active )
+         addTarget ( dest );
+}
+      
+void ServiceTargetSearcher::repair( ContainerBase* dest)
+{
+   if ( checks & checkRepair )
       addTarget ( dest );
 }
       
-ServiceTargetSearcher::ServiceTargetSearcher( ContainerBase* src ) : ServiceChecker( src )
+      
+ServiceTargetSearcher::ServiceTargetSearcher( ContainerBase* src, int checkFlags ) : ServiceChecker( src ), checks ( checkFlags )
 {
    gamemap = src->getMap();
 }
 
 
-bool ServiceTargetSearcher::available()
+bool ServiceTargetSearcher::externallyAvailable()
 {
    Vehicle* srcVehicle = dynamic_cast<Vehicle*>(source);
    if ( srcVehicle ) {
@@ -706,7 +803,7 @@ void TransferHandler::resource( ContainerBase* dest, int type, bool active )
    transfers.push_back(  new ResourceTransferrable( type, sourceRes, destRes, active ));
 }
 
-TransferHandler::TransferHandler( ContainerBase* src, ContainerBase* dst ) : ServiceChecker( src), sourceRes( src ), destRes( dst ), source(src), dest(dst)
+TransferHandler::TransferHandler( ContainerBase* src, ContainerBase* dst, int flags ) : ServiceChecker( src, flags ), sourceRes( src ), destRes( dst ), source(src), dest(dst)
 {
    allowProduction = CGameOptions::Instance()->autoproduceammunition ;
 
@@ -726,12 +823,6 @@ bool TransferHandler::allowAmmoProduction( bool allow )
       return false;
 }
 
-bool TransferHandler::allowAmmoProductionAllowed()
-{
-   return allowProduction;
-}
-
-
 
 TransferHandler::~TransferHandler()
 {
@@ -758,6 +849,23 @@ void TransferHandler::fillDest()
          (*i)->fill( dest );
 }
 
+void TransferHandler::fillDestAmmo()
+{
+   for ( Transfers::iterator i = transfers.begin(); i != transfers.end(); ++i )
+      if ( (*i)->isExchangable())
+         if ( dynamic_cast<AmmoTransferrable*>(*i))
+            (*i)->fill( dest );
+}
+
+void TransferHandler::fillDestResource()
+{
+   for ( Transfers::iterator i = transfers.begin(); i != transfers.end(); ++i )
+      if ( (*i)->isExchangable())
+         if ( dynamic_cast<ResourceTransferrable*>(*i))
+            (*i)->fill( dest );
+}
+
+
 void TransferHandler::emptyDest()
 {
    for ( Transfers::iterator i = transfers.begin(); i != transfers.end(); ++i )
@@ -765,10 +873,11 @@ void TransferHandler::emptyDest()
 
 }
 
-bool TransferHandler::commit()
+
+bool TransferHandler::commit( const Context& context )
 {
    for ( Transfers::iterator i = transfers.begin(); i != transfers.end(); ++i )
-      (*i)->commit();
+      (*i)->commit( context );
 
    return true;
 }
