@@ -22,6 +22,8 @@
     Boston, MA  02111-1307  USA
 */
 
+#ifndef aiH_included
+#define aiH_included
 
 
 #include <utility>
@@ -33,9 +35,10 @@
 #include "../terraintype.h"
 #include "../objecttype.h"
 #include "../spfst.h"
-#include "../unitctrl.h"
 #include "../buildingtype.h"
 #include "../astar2.h"
+#include "../actions/context.h"
+#include "../attack.h"
 
 
     class AI : public BaseAI {
@@ -55,7 +58,6 @@
 
            GameMap* activemap;
            MapDisplayInterface* mapDisplay;
-           MapDisplayInterface* rmd;
 
            class ServiceOrder {
                   AI* ai;
@@ -69,11 +71,16 @@
                   int nextServiceBuildingDistance;
                   bool active;
                public:
-                  VehicleService::Service requiredService;
+                  
+                  enum Service { srv_repair, srv_resource, srv_ammo };
+                  
+                  Service requiredService;
                   int position;
+                  
+                  int getServiceID() const;
 
                   ServiceOrder ( ) : ai ( NULL ), targetUnitID ( 0 ), serviceUnitID ( 0 ), failure ( 0 ), nextServiceBuilding ( 0 ), active ( false ) {};
-                  ServiceOrder ( AI* _ai, VehicleService::Service _requiredService, int UnitID, int _pos = -1 );
+                  ServiceOrder ( AI* _ai, Service _requiredService, int UnitID, int _pos = -1 );
                   ServiceOrder ( AI* _ai, tnstream& stream );
                   AStar3D::Path::iterator lastmatchServiceOrder ( AI* _ai, tnstream& stream );
                   Vehicle* getTargetUnit ( ) const { return ai->getMap()->getUnit ( targetUnitID );};
@@ -120,6 +127,8 @@
            };
 
 
+           //! this stores units that have been exhaustively used in tactics and should be skipped from strategy section
+           set<Vehicle*> unitsWorkedInTactics;
 
            static bool vehicleValueComp ( const Vehicle* v1, const Vehicle* v2 );
            static bool buildingValueComp ( const Building* v1, const Building* v2 );
@@ -130,7 +139,7 @@
            void issueServices ( );
 
            //! issues a single service. If the same service-order already exists, it will not be issued a second time
-           ServiceOrder& issueService ( VehicleService::Service requiredService, int UnitID, int pos = -1 );
+           ServiceOrder& issueService ( ServiceOrder::Service requiredService, int UnitID, int pos = -1 );
 
            ServiceOrder& issueRefuelOrder ( Vehicle* veh, bool returnImmediately );
            void runServiceUnit ( Vehicle* supplyUnit );
@@ -283,6 +292,8 @@
                   int moveDist;
                   bool neighbouringFieldsReachable[ sidenum ]; // used for the hemming tactic
                   float positionThreat;
+				  bool operator< ( const MoveVariant& a ) const;
+				  bool operator> ( const MoveVariant& a ) const;
             };
             static bool moveVariantComp ( const AI::MoveVariant* mv1, const AI::MoveVariant* mv2 );
          private:
@@ -329,7 +340,7 @@
             AiResult executeMoveAttack ( Vehicle* veh, TargetVector& tv );
             int getDirForBestTacticsMove ( const Vehicle* veh, TargetVector& tv );
             MapCoordinate getDestination ( Vehicle* veh );
-            AiResult moveToSavePlace ( Vehicle* veh, VehicleMovement& vm, int preferredHeight = -1 );
+            AiResult moveToSavePlace ( Vehicle* veh, int preferredHeight = -1 );
             int  getBestHeight (  Vehicle* veh );
             float getAttackValue ( const tfight& battle, const Vehicle* attackingUnit, const Vehicle* attackedUnit, float factor = 1 );
 
@@ -339,7 +350,7 @@
                          -1 = no space to change height
                          -2 = cannot change height here principially
             */
-            int changeVehicleHeight ( Vehicle* veh, VehicleMovement* vm, int preferredDirection = -1 );
+            int changeVehicleHeight ( Vehicle* veh, int preferredDirection = -1 );
 
             void  calculateThreat ( const Vehicletype* vt);
             void  calculateThreat ( Vehicle* eht );
@@ -447,6 +458,7 @@
             class Sections {
                   AI* ai;
                   Section* section;
+                  MapCoordinate getAlternativeField( const MapCoordinate& pos, map<MapCoordinate,int>* destinationCounter, int height );
                public:
                   int sizeX ;
                   int sizeY ;
@@ -457,9 +469,10 @@
                   Section& getForPos ( int xn, int yn );                //!< returns the xth and yth section
 
                   // secondRun should only be used when this function calls itself recursively
-                  Section* getBest ( int pass, Vehicle* veh, MapCoordinate3D* dest = NULL, bool allowRefuellOrder = false, bool secondRun = false );
+                  Section* getBest ( int pass, Vehicle* veh, MapCoordinate3D* dest = NULL, bool allowRefuellOrder = false, bool secondRun = false, map<MapCoordinate,int>* destinationCounter = NULL);
                   Sections ( AI* _ai );
-                  void reset( void );
+                  void reset();
+                  ~Sections();
             } sections;
             friend class Sections;
 
@@ -468,13 +481,14 @@
 
             void diplomacy ();
 
+            void checkGameEvents();
             
         public:
            AI ( GameMap* _map, int _player ) ;
 
-           void run (  ) { run ( false );};
+           void run (  MapDisplayInterface* mapDisplay ) { run ( false, mapDisplay );};
            //! starts the Ai. If benchmark is true, the AI might take longer since it is not time limited, it won't display any graphics and will output the time it needed to run completely
-           void run ( bool benchmark );
+           void run ( bool benchmark, MapDisplayInterface* mapDisplay );
 
            //! returns the map this AI runson
            GameMap* getMap ( void ) { return activemap; };
@@ -497,5 +511,13 @@
 
            void read ( tnstream& stream );
            void write ( tnstream& stream ) const ;
+           
+           Context getContext();
            ~AI ( );
     };
+
+//extern bool operator> ( const AI::MoveVariant& mv1, const AI::MoveVariant& mv2 );
+//extern bool operator< ( const AI::MoveVariant& mv1, const AI::MoveVariant& mv2 );
+
+
+#endif
