@@ -16,9 +16,12 @@
  ***************************************************************************/
 
 
+#include <iostream>
 #include "ai_common.h"
-
-
+#include "../actions/attackcommand.h"
+#include "../actions/constructunitcommand.h"
+#include "../actions/moveunitcommand.h"
+#include "../actions/servicecommand.h"
 
 bool AI :: runUnitTask ( Vehicle* veh )
 {
@@ -95,12 +98,13 @@ int  AI :: getBestHeight (  Vehicle* veh )
             if ( threat && threat != 1 )
                value *= ( log ( minAvgFieldThreat ) / log ( threat ) );
          } else {
-            if ( secminAvgFieldThreat < maxfloat)
+            if ( secminAvgFieldThreat < maxfloat) {
                if ( threat && threat != 1 )
                   value *= ( log ( secminAvgFieldThreat ) / log ( threat ) );
                else
                   value *= 1.5; // no threat
             }
+         }
 
          if ( value > bestHeightValue ) {
             bestHeightValue = value;
@@ -148,13 +152,13 @@ void AI::RefuelConstraint::findPath()
          else
             dist = maxint;
 
-         dist = min( dist, veh->maxMovement() * 5);
+         dist = min( dist, 1000 );
       } else
          dist = maxMove;
 
       ast = new AStar3D ( ai.getMap(), veh, true, dist );
       ast->findAllAccessibleFields ( );
-      // tanker planes may have a very large range; that's why we top the distance at 5 times the turn-range
+      // tanker planes may have a very large range; that's why we top the distance at 100 fields
    }
 }
 
@@ -166,13 +170,13 @@ MapCoordinate3D AI::RefuelConstraint::getNearestRefuellingPosition ( bool buildi
    int x1,y1,x2,y2;
    x1 = max(veh->xpos - fuel / veh->typ->fuelConsumption, 0 );
    y1 = max(veh->ypos - fuel / veh->typ->fuelConsumption, 0 );
-   x2 = min(veh->xpos + fuel / veh->typ->fuelConsumption, actmap->xsize );
-   y2 = min(veh->ypos + fuel / veh->typ->fuelConsumption, actmap->ysize );
+   x2 = min(veh->xpos + fuel / veh->typ->fuelConsumption, ai.getMap()->xsize );
+   y2 = min(veh->ypos + fuel / veh->typ->fuelConsumption, ai.getMap()->ysize );
 
    for ( AStar3D::Container::iterator i = ast->visited.begin(); i != ast->visited.end(); i++ ) {
       int dist = int(i->gval );
       if ( i->h.getNumericalHeight() == -1 ) {
-          tfield* fld = getfield( i->h.x, i->h.y );
+          tfield* fld = ai.getMap()->getField( i->h.x, i->h.y );
           if ( fld->building && fld->building->color == veh->color )
              reachableBuildings[ dist ] = fld->building;
       }
@@ -185,7 +189,7 @@ MapCoordinate3D AI::RefuelConstraint::getNearestRefuellingPosition ( bool buildi
        if ((veh->height > chfahrend) && (i->h.getNumericalHeight() == chfahrend) ) {
           // we don't want to land in hostile territory
           FieldInformation& fi = ai.getFieldInformation ( i->h.x, i->h.y );
-          if ( fi.control == -1 || !actmap->player[fi.control].diplomacy.isHostile( ai.getPlayerNum() ) )
+          if ( fi.control == -1 || !ai.getMap()->player[fi.control].diplomacy.isHostile( ai.getPlayerNum() ) )
               landingPositions[dist] = i->h;
        }
    }
@@ -302,7 +306,7 @@ bool AI::RefuelConstraint::necessary (const Vehicle* veh, AI& ai )
    if ( !veh->typ->fuelConsumption )
       return false;
    if ( !(veh->height & (chtieffliegend | chfliegend | chhochfliegend))) {
-      ServiceOrder so  ( &ai, VehicleService::srv_resource, veh->networkid, 2 );
+      ServiceOrder so  ( &ai, ServiceOrder::srv_resource, veh->networkid, 2 );
       if ( so.serviceUnitExists() )
          return false;
       else
@@ -354,7 +358,7 @@ AI::AiResult  AI :: container ( ContainerBase* cb )
    for ( vector<Vehicle*>::const_iterator j = cb->getCargo().begin(); j != cb->getCargo().end(); ++j ) {
       Vehicle* veh = *j;
       if ( veh )
-         if ( veh->canMove() )
+         if ( veh->canMove() && veh->getOwner() == getPlayerNum() )
             if ( veh->aiparam[ getPlayerNum() ]->getTask() == AiParameter::tsk_nothing
                || veh->aiparam[ getPlayerNum() ]->getJob() == AiParameter::job_script  )
             idleUnits.push_back ( veh );
@@ -371,18 +375,15 @@ AI::AiResult  AI :: container ( ContainerBase* cb )
 
       int preferredHeight = getBestHeight ( *i );
 
-      VehicleMovement* vm = ContainerControls::movement ( *i );
-      // auto_ptr<VehicleMovement> vm ( cc.movement ( *i ) );
-      if ( vm ) {
-         vm->registerMapDisplay ( mapDisplay );
-         auto_ptr<VehicleMovement> avm ( vm );
-
-         VehicleAttack va ( NULL, NULL );
+      auto_ptr<MoveUnitCommand> muc ( new MoveUnitCommand( *i ));
+      muc->searchFields();
+      
+      if ( muc->getReachableFields().size() ) {
+         
          int moved = 0;
-         if ( va.available ( *i )) {
+         if ( AttackCommand::avail ( *i )) {
             TargetVector tv;
 
-            // AStar3D ast ( getMap(), *i, false, (*i)->maxMovement() );  ?? Why was maxMovement used here ??
             AStar3D ast ( getMap(), *i, false, (*i)->getMovement() );
             ast.findAllAccessibleFields ();
 
@@ -410,7 +411,7 @@ AI::AiResult  AI :: container ( ContainerBase* cb )
 
          }
          if ( !moved ) {
-            AiResult res =  moveToSavePlace ( *i, *vm, preferredHeight );
+            AiResult res =  moveToSavePlace ( *i, preferredHeight );
             result += res;
             if ( !res.unitsDestroyed )
                (*i)->aiparam[ getPlayerNum() ]->resetTask();
@@ -440,10 +441,19 @@ AI::AiResult AI::buildings( int process )
          if ( veh ) {
             ++unitCounter;
             displaymessage2("processing building %d (unit %d)", buildingCounter, unitCounter );
+            
+            auto_ptr<ServiceCommand> sc ( new ServiceCommand( *bi ));
+            sc->setDestination( veh );
             if ( veh->aiparam[ getPlayerNum() ]->getJob() != AiParameter::job_supply )
-               bc.fillResource( veh, Resources::Fuel, maxint );
+               sc->getTransferHandler().fillDestAmmo();
             else
-               bc.refilleverything( veh );
+               sc->getTransferHandler().fillDest();
+            
+            sc->saveTransfers();
+            ActionResult res = sc->execute( getContext() );
+            if ( res.successful() )
+               sc.release();
+            
          }
       }
 
@@ -487,7 +497,7 @@ bool AI :: moveUnit ( Vehicle* veh, const MapCoordinate3D& destination, bool int
    #if 0
    if ( destination.getNumericalHeight() == -1 ) {
 
-      VehicleMovement vm ( mapDisplay, NULL );
+      Vehicle!!Movement vm ( mapDisplay, NULL );
       vm.execute ( veh, -1, -1, 0, -1, -1 );
 
       std::vector<MapCoordinate> path;
@@ -558,7 +568,7 @@ bool AI :: moveUnit ( Vehicle* veh, const MapCoordinate3D& destination, bool int
          if ( beeline ( veh->getPosition(), destination) > veh->maxMovement() )
             ast = new StratAStar3D ( this, veh );
          else
-            ast = new AStar3D ( getMap(), veh );
+            ast = new AntiMineAStar3D ( this, veh );
 
       auto_ptr<AStar3D> ap ( ast );
 
@@ -571,6 +581,11 @@ bool AI :: moveUnit ( Vehicle* veh, const MapCoordinate3D& destination, bool int
 
 int AI::moveUnit ( Vehicle* veh, const AStar3D::Path& path, bool intoBuildings, bool intoTransports )
 {
+   auto_ptr<MoveUnitCommand> mum ( new MoveUnitCommand( veh ));
+   mum->searchFields();
+   
+   
+   int oldHeight = veh->getPosition3D().getNumericalHeight();
    AStar3D::Path::const_iterator pi = path.begin();
    if ( pi == path.end() )
       return 0;
@@ -579,7 +594,7 @@ int AI::moveUnit ( Vehicle* veh, const AStar3D::Path& path, bool intoBuildings,
 
    AStar3D::Path::const_iterator lastmatch = pi;
    while ( pi != path.end() ) {
-      tfield* fld = getfield ( pi->x, pi->y );
+      tfield* fld = getMap()->getField ( pi->x, pi->y );
       bool ok = true;
       if ( fld->getContainer() ) {
          if ( pi+1 !=path.end() )
@@ -593,35 +608,33 @@ int AI::moveUnit ( Vehicle* veh, const AStar3D::Path& path, bool intoBuildings,
       }
 
       if ( ok )
-      // if ( !fld->building || getdiplomaticstatus2 ( fld->building->color, getPlayerNum()*8) == cawar || fld->building->getOwner() == veh->getOwner() )
-         if ( pi->dist <= veh->getMovement() )
+         if ( mum->isFieldReachable3D(*pi, true) )
             lastmatch = pi;
 
       ++pi;
    }
 
-   if ( getfield ( lastmatch->x, lastmatch->y )->building )
-      if ( checkReConquer ( getfield ( lastmatch->x, lastmatch->y )->building, veh ))
+   if ( getMap()->getField ( lastmatch->x, lastmatch->y )->building )
+      if ( checkReConquer ( getMap()->getField ( lastmatch->x, lastmatch->y )->building, veh ))
          return false;
 
    if ( lastmatch == path.begin() )
       return 0;
-
-   BaseVehicleMovement vm ( vat_move, NULL, mapDisplay );
-   vm.execute ( veh, lastmatch->x, lastmatch->y, 0, lastmatch->getNumericalHeight(), 1 );
-
-   if ( vm.getStatus() != 3 ) {
+  
+   mum->setDestination( *lastmatch );
+   mum->setFlags( MoveUnitCommand::NoInterrupt );
+   
+   
+   ActionResult res = mum->execute( getContext() );
+   if ( res.successful() ) {
+      mum.release();
+      if ( getMap()->getUnit ( nwid ) && (oldHeight != veh->getPosition3D().getNumericalHeight()) )
+         veh->aiparam[ getPlayerNum()]->setNewHeight();
+   } else {
       displaymessage ( "AI :: moveUnit (path) \n error in movement step 3 with unit %d", 1, veh->networkid );
       return -1;
    }
 
-   vm.execute ( NULL, lastmatch->x, lastmatch->y, 3, -1,  1 );
-   if ( vm.getStatus() != 1000 ) {
-      displaymessage ( "AI :: moveUnit (path) \n error in movement step 3 with unit %d", 1, veh->networkid );
-      return -1;
-   }
-
-
    //! the unit may have been shot down
    if ( ! getMap()->getUnit ( nwid ))
       return 1;
@@ -663,11 +676,12 @@ int AI:: CheckFieldRecon :: run ( int x, int y)
 void AI::CheckFieldRecon :: testfield ( const MapCoordinate& mc )
 {
    FieldInformation& fi = ai->getFieldInformation ( mc.x, mc.y );
-   if( fi.control != -1 )
-      if ( !actmap->player[player].diplomacy.isHostile( fi.control )  )
+   if( fi.control != -1 ) {
+      if ( !ai->getMap()->player[player].diplomacy.isHostile( fi.control )  )
          ownFields[dist]++;
       else
          enemyFields[dist]++;
+   }
 }
 
 void AI :: calcReconPositions()
@@ -689,43 +703,49 @@ void AI :: calcReconPositions()
 
 void AI ::  runReconUnits ( )
 {
+   
+   vector<int> reconUnits;
+   
    Player::VehicleList::iterator nvi;
-   for ( Player::VehicleList::iterator vi = getPlayer().vehicleList.begin(); vi != getPlayer().vehicleList.end(); ) {
-      nvi = vi;
-      ++nvi;
-
+   for ( Player::VehicleList::iterator vi = getPlayer().vehicleList.begin(); vi != getPlayer().vehicleList.end(); ++vi) {
       Vehicle* veh = *vi;
-
-      int maxUnitMovement = veh->typ->maxSpeed();
-
-      // the threat posed should be enemy units should be considered for position choosing too...
-      if ( veh->aiparam[getPlayerNum()]->getJob() == AiParameter::job_recon ) {
-         if ( reconPositions.find ( veh->getPosition()) == reconPositions.end()) {
-            // the unit is not standing on a reconposition
-            int mindist = maxint;
-            MapCoordinate mc;
-            for ( ReconPositions::iterator i = reconPositions.begin(); i != reconPositions.end(); i++ ) {
-               tfield* fld = getMap()->getField( i->first );
-               if ( !fld->vehicle && !fld->building ) {
-                  AStar ast ( getMap(), veh );
-                  ast.findAllAccessibleFields( maxUnitMovement );
-                  if ( fld->a.temp ) {
-                     int vdist = beeline ( veh->getPosition(), i->first )*(1+i->second/2);
-                     if( vdist < mindist ) {
-                        mindist = vdist;
-                        mc = i->first;
+      if ( veh->aiparam[getPlayerNum()] && veh->aiparam[getPlayerNum()]->getJob() == AiParameter::job_recon ) 
+         reconUnits.push_back( veh->networkid );
+   }
+      
+   for ( vector<int>::iterator ru = reconUnits.begin(); ru != reconUnits.end(); ++ru ) {
+      Vehicle* veh = getMap()->getUnit(*ru);
+      if ( veh ) {
+         int maxUnitMovement = veh->typ->maxSpeed();
+   
+         // the threat posed should be enemy units should be considered for position choosing too...
+         if ( veh->aiparam[getPlayerNum()] && veh->aiparam[getPlayerNum()]->getJob() == AiParameter::job_recon ) {
+            if ( reconPositions.find ( veh->getPosition()) == reconPositions.end()) {
+               // the unit is not standing on a reconposition
+               int mindist = maxint;
+               MapCoordinate mc;
+               for ( ReconPositions::iterator i = reconPositions.begin(); i != reconPositions.end(); i++ ) {
+                  tfield* fld = getMap()->getField( i->first );
+                  if ( !fld->vehicle && !fld->building ) {
+                     AStar ast ( getMap(), veh );
+                     ast.findAllAccessibleFields( maxUnitMovement );
+                     if ( fld->a.temp ) {
+                        int vdist = beeline ( veh->getPosition(), i->first )*(1+i->second/2);
+                        if( vdist < mindist ) {
+                           mindist = vdist;
+                           mc = i->first;
+                        }
                      }
                   }
                }
-            }
-            if( mindist < maxint ) {
-               veh->aiparam[getPlayerNum()]->dest = MapCoordinate3D(mc, veh->height);
-               veh->aiparam[getPlayerNum()]->setTask( AiParameter::tsk_move );
-               runUnitTask ( veh );
+               if( mindist < maxint ) {
+                  veh->aiparam[getPlayerNum()]->dest = MapCoordinate3D(mc, veh->height);
+                  veh->aiparam[getPlayerNum()]->setTask( AiParameter::tsk_move );
+                  runUnitTask ( veh );
+               }
             }
          }
       }
-      vi = nvi;
    }
 }
 
@@ -755,24 +775,25 @@ AI::UnitDistribution::Group AI::getUnitDistributionGroup ( Vehicletype* vt )
 
 AI::UnitDistribution::Group AI::getUnitDistributionGroup ( Vehicle* veh )
 {
-   switch ( veh->aiparam[getPlayerNum()]->getJob() ) {
-      case AiParameter::job_supply : return UnitDistribution::service;
-      case AiParameter::job_recon  : return UnitDistribution::recon;
-      case AiParameter::job_conquer: return UnitDistribution::conquer;
-      case AiParameter::job_fight:
-      case AiParameter::job_guard: {
-                                      bool range = false;
-                                      for ( int w = 0; w < veh->typ->weapons.count; w++ )
-                                         if ( veh->typ->weapons.weapon[w].offensive() )
-                                            if ( veh->typ->weapons.weapon[w].maxdistance >= 2 * minmalq )
-                                               range = true;
-                                      if ( range )
-                                         return UnitDistribution::rangeattack;
-                                      else
-                                         return UnitDistribution::attack;
-                                    }
-      default:;
-    } //switch job
+   if ( veh->aiparam[getPlayerNum()] )
+      switch ( veh->aiparam[getPlayerNum()]->getJob() ) {
+         case AiParameter::job_supply : return UnitDistribution::service;
+         case AiParameter::job_recon  : return UnitDistribution::recon;
+         case AiParameter::job_conquer: return UnitDistribution::conquer;
+         case AiParameter::job_fight:
+         case AiParameter::job_guard: {
+                                         bool range = false;
+                                         for ( int w = 0; w < veh->typ->weapons.count; w++ )
+                                            if ( veh->typ->weapons.weapon[w].offensive() )
+                                               if ( veh->typ->weapons.weapon[w].maxdistance >= 2 * minmalq )
+                                                  range = true;
+                                         if ( range )
+                                            return UnitDistribution::rangeattack;
+                                         else
+                                            return UnitDistribution::attack;
+                                       }
+         default:;
+       } //switch job
     return UnitDistribution::other;
 }
 
@@ -837,10 +858,12 @@ void AI::production()
       enemyValue[i] = 0;
 
    for ( int p = 0; p < 8; p++ )
-      if ( actmap->player[p].diplomacy.isHostile( getPlayerNum() ) )
+      if ( getMap()->player[p].diplomacy.isHostile( getPlayerNum() ) )
           for ( Player::VehicleList::iterator vli = getMap()->player[p].vehicleList.begin(); vli != getMap()->player[p].vehicleList.end(); vli++ ) {
-              enemyThreat += (*vli)->aiparam[getPlayerNum()]->threat;
-              enemyValue[(*vli)->aiparam[getPlayerNum()]->valueType] += (*vli)->aiparam[getPlayerNum()]->getValue();
+             if ( (*vli)->aiparam[getPlayerNum()] ) {
+                enemyThreat += (*vli)->aiparam[getPlayerNum()]->threat;
+                enemyValue[(*vli)->aiparam[getPlayerNum()]->valueType] += (*vli)->aiparam[getPlayerNum()]->getValue();
+             }
           }
 
 
@@ -894,12 +917,6 @@ void AI::production()
          }
    }
 
-   for ( Produceable::iterator i = produceable.begin(); i != produceable.end(); ++i ) {
-      float f = i->first;
-      const char* d = i->second.vt->description.c_str();
-      printf(" typ: %s  ; rating: %e \n", d, f );
-   }
-
    if ( !produceable.empty() ) {
 
       GetConnectedBuildings::BuildingContainer lockedBuildings;
@@ -915,16 +932,33 @@ void AI::production()
 
                       if ( find ( lockedBuildings.begin(), lockedBuildings.end(), pr.bld ) == lockedBuildings.end()) {
                          ContainerControls bc( pr.bld );
-                         int lack = bc.unitProductionPrerequisites( pr.vt );
+                         int lack = bc.unitProductionPrerequisites( pr.vt, true );
                          if  ( !lack && pr.bld->vehicleUnloadSystem ( pr.vt, 255 ) ) {
-                             Vehicle* veh = bc.produceUnit( pr.vt, true, true );
-                             if ( veh ) {
-                                 calculateThreat ( veh );
-                                 container ( pr.bld );
-                                 // currentUnitDistribution.group[i] += inc;
-                                 produced = true;
-                                 break;  // exit produceable loop
-                             }
+                            try {
+                               ConstructUnitCommand* cuc = new ConstructUnitCommand( pr.bld );
+                               cuc->setMode(ConstructUnitCommand::internal );
+                               cuc->setVehicleType( pr.vt );
+                               cuc->execute( getContext() );
+                               Vehicle* veh = cuc->getProducedUnit();
+                               if ( veh ) {
+                                  
+                                  auto_ptr<ServiceCommand> sc ( new ServiceCommand( pr.bld ));
+                                  sc->setDestination( veh );
+                                  sc->getTransferHandler().fillDest();
+                                  sc->saveTransfers();
+                                  ActionResult res = sc->execute( getContext() );
+                                  if ( res.successful() )
+                                     sc.release();
+                                  
+                                  calculateThreat ( veh );
+                                  container ( pr.bld );
+                                  // currentUnitDistribution.group[i] += inc;
+                                  produced = true;
+                                  break;  // exit produceable loop
+                              }
+                            } catch ( ActionResult res ) {
+                               // just ignore any errors 
+                            }
                          } else {
 
                             // the ai will save for move expensive units
