@@ -16,7 +16,8 @@
  ***************************************************************************/
 
 #include "ai_common.h"
-
+#include "../actions/attackcommand.h"
+#include "../actions/moveunitcommand.h"
 
 const int attack_unitdestroyed_bonus = 90;
 
@@ -36,22 +37,21 @@ void AI :: searchTargets ( Vehicle* veh, const MapCoordinate3D& pos, TargetVecto
    veh->ypos = pos.y;
    veh->height = pos.getBitmappedHeight();
    veh->addview();
-   int fieldsWithChangedVisibility = evaluateviewcalculation ( getMap(), veh->getPosition(), veh->typ->view, 0xff );
+   int fieldsWithChangedVisibility = evaluateviewcalculation ( getMap(), veh->getPosition(), veh->view, 0xff );
 
 
-   VehicleAttack va ( NULL, NULL );
-   if ( va.available ( veh )) {
-      va.execute ( veh, -1, -1, 0 , 0, -1 );
-      for ( int g = 0; g < va.attackableVehicles.getFieldNum(); g++ ) {
-         int xp, yp;
-         va.attackableVehicles.getFieldCoordinates ( g, &xp, &yp );
-         pattackweap aw = &va.attackableVehicles.getData ( g );
+   if ( AttackCommand::avail ( veh )) {
+      AttackCommand va ( veh );
+      va.searchTargets();
+      for ( AttackCommand::FieldList::const_iterator i = va.getAttackableUnits().begin(); i != va.getAttackableUnits().end(); ++i ) {
+         MapCoordinate targ = i->first;
+         const AttackWeap* aw = &(i->second);
 
          int bestweap = -1;
          int targdamage = -1;
          int weapstrength = -1;
          for ( int w = 0; w < aw->count; w++ ) {
-            tunitattacksunit uau ( veh, getfield ( xp, yp)->vehicle, 1, aw->num[w] );
+            tunitattacksunit uau ( veh, getMap()->getField (targ)->vehicle, 1, aw->num[w] );
             uau.calc();
             if ( uau.dv.damage > targdamage ) {
                bestweap = aw->num[w];
@@ -72,7 +72,7 @@ void AI :: searchTargets ( Vehicle* veh, const MapCoordinate3D& pos, TargetVecto
 
          MoveVariant* mv = new MoveVariant;
 
-         tunitattacksunit uau ( veh, getfield ( xp, yp)->vehicle, 1, bestweap );
+         tunitattacksunit uau ( veh, getMap()->getField(targ)->vehicle, 1, bestweap );
          mv->orgDamage = uau.av.damage;
          mv->damageAfterMove = uau.av.damage;
          mv->enemyOrgDamage = uau.dv.damage;
@@ -81,10 +81,10 @@ void AI :: searchTargets ( Vehicle* veh, const MapCoordinate3D& pos, TargetVecto
 
          mv->damageAfterAttack = uau.av.damage;
          mv->enemyDamage = uau.dv.damage;
-         mv->enemy = getfield ( xp, yp )->vehicle;
+         mv->enemy = getMap()->getField(targ)->vehicle;
          mv->movePos = pos;
-         mv->attackx = xp;
-         mv->attacky = yp;
+         mv->attackx = targ.x;
+         mv->attacky = targ.y;
          mv->weapNum = bestweap;
          mv->moveDist = moveDist;
          mv->attacker = veh;
@@ -102,7 +102,7 @@ void AI :: searchTargets ( Vehicle* veh, const MapCoordinate3D& pos, TargetVecto
 
          }
 
-         int attackerDirection = getdirection ( xp, yp, pos.x, pos.y );
+         int attackerDirection = getdirection ( targ.x, targ.y, pos.x, pos.y );
          float hemmingFactor = 1;
          for ( int nf = 0; nf < sidenum-1 && nf < hemmingBonus; nf++ ) {
             // we are starting opposite the attacker, since this is the highest hemming bonus
@@ -141,24 +141,40 @@ void AI :: searchTargets ( Vehicle* veh, const MapCoordinate3D& pos, TargetVecto
       veh->addview();
 
    if ( fieldsWithChangedVisibility || 1 )  // viewbug.sav !!!!!
-      evaluateviewcalculation ( getMap(), veh->getPosition(), veh->typ->view, 0xff );
+      evaluateviewcalculation ( getMap(), veh->getPosition(), veh->view, 0xff );
 }
 
-bool operator > ( const AI::MoveVariant& mv1, const AI::MoveVariant& mv2 )
+bool AI::MoveVariant::operator> ( const AI::MoveVariant& mv2 ) const
+{
+   return (     result > mv2.result
+            || (result == mv2.result && positionThreat < mv2.positionThreat  )
+            || (result == mv2.result && positionThreat == mv2.positionThreat && moveDist < mv2.moveDist) );  // mv1.moveDist < mv2.moveDist
+}
+
+
+bool AI::MoveVariant::operator< ( const AI::MoveVariant& mv2 ) const
+{
+   return (     result < mv2.result
+            || (result == mv2.result && positionThreat > mv2.positionThreat )
+            || (result == mv2.result && positionThreat == mv2.positionThreat && moveDist > mv2.moveDist) );  // mv1.moveDist < mv2.moveDist
+}
+
+/*
+bool operator> ( const AI::MoveVariant& mv1, const AI::MoveVariant& mv2 ) 
 {
    return (     mv1.result > mv2.result
             || (mv1.result == mv2.result && mv1.positionThreat < mv2.positionThreat  )
             || (mv1.result == mv2.result && mv1.positionThreat == mv2.positionThreat && mv1.moveDist < mv2.moveDist) );  // mv1.moveDist < mv2.moveDist
 }
 
-bool operator < ( const AI::MoveVariant& mv1, const AI::MoveVariant& mv2 )
+bool operator< ( const AI::MoveVariant& mv1, const AI::MoveVariant& mv2 )
 {
    return (     mv1.result < mv2.result
             || (mv1.result == mv2.result && mv1.positionThreat > mv2.positionThreat )
             || (mv1.result == mv2.result && mv1.positionThreat == mv2.positionThreat && mv1.moveDist > mv2.moveDist) );  // mv1.moveDist < mv2.moveDist
 }
 
-
+*/
 bool AI::moveVariantComp ( const AI::MoveVariant* mv1, const AI::MoveVariant* mv2 )
 {
    return *mv1 < *mv2;
@@ -210,7 +226,7 @@ void AI::getAttacks ( AStar3D& vm, Vehicle* veh, TargetVector& tv, int hemmingBo
    int orgxpos = veh->xpos ;
    int orgypos = veh->ypos ;
 
-   if ( getfield ( veh->xpos, veh->ypos )->unitHere ( veh ) )  // unit not inside a building or transport
+   if ( getMap()->getField ( veh->xpos, veh->ypos )->unitHere ( veh ) )  // unit not inside a building or transport
       searchTargets ( veh, veh->getPosition3D(), tv, 0, vm, hemmingBonus );
 
    if ( tv.size() && justOne )
@@ -255,18 +271,18 @@ AI::AiResult AI::executeMoveAttack ( Vehicle* veh, TargetVector& tv )
    MoveVariant* mv = *max_element( tv.begin(), tv.end(), moveVariantComp );
 
    if ( mv->movePos != veh->getPosition3D() ) {
-      BaseVehicleMovement vm2 ( mapDisplay );
-
       VisibilityStates org_vision =  _vision ;
       _vision = visible_now;
-
-      vm2.execute ( veh, mv->movePos.x, mv->movePos.y, 0, mv->movePos.getNumericalHeight(), 1 );
-      if ( vm2.getStatus() != 3 )
-         displaymessage ( "AI :: executeMoveAttack \n error in movement step 0 with unit %d", 1, veh->networkid );
-
-      vm2.execute ( NULL, mv->movePos.x, mv->movePos.y, 3, mv->movePos.getNumericalHeight(), 1 );
-      if ( vm2.getStatus() != 1000 )
-         displaymessage ( "AI :: executeMoveAttack \n error in movement step 2 with unit %d", 1, veh->networkid );
+      
+      auto_ptr<MoveUnitCommand> muc ( new MoveUnitCommand( veh ));
+      muc->setFlags ( MoveUnitCommand::NoInterrupt );
+      muc->setDestination( mv->movePos );
+      ActionResult res = muc->execute( getContext () );
+      if ( res.successful() )
+         muc.release();
+      else {
+         displaymessage ( "AI :: executeMoveAttack \n error in movement with unit %d", 1, veh->networkid );
+      }
 
       result.unitsMoved ++;
       _vision = org_vision;
@@ -281,16 +297,19 @@ AI::AiResult AI::executeMoveAttack ( Vehicle* veh, TargetVector& tv )
    if ( veh->attacked )
       return result;
 
-   VehicleAttack va ( mapDisplay, NULL );
-   va.execute ( veh, -1, -1, 0 , 0, -1 );
-   if ( va.getStatus() != 2 )
+   auto_ptr<AttackCommand> va (new AttackCommand( veh ));
+   ActionResult res = va->searchTargets();
+   if ( !res.successful() )
       displaymessage ( "AI :: executeMoveAttack \n error in attack step 2 with unit %d", 1, veh->networkid );
 
    VehicleTypeEfficiencyCalculator vtec( *this, veh, mv->enemy );
 
-   va.execute ( NULL, mv->attackx, mv->attacky, 2 , -1, mv->weapNum );
-   if ( va.getStatus() != 1000 )
+   va->setTarget( MapCoordinate( mv->attackx, mv->attacky), mv->weapNum );
+   res = va->execute( getContext() );
+   if ( !res.successful() )
       displaymessage ( "AI :: executeMoveAttack \n error in attack step 3 with unit %d", 1, veh->networkid );
+   else
+      va.release();
 
    vtec.calc();
 
@@ -303,9 +322,8 @@ AI::AiResult AI::executeMoveAttack ( Vehicle* veh, TargetVector& tv )
    }
 
 
-   VehicleMovement vm3 ( mapDisplay, NULL );
-   if ( vm3.available ( veh ))
-      result += moveToSavePlace ( veh, vm3 );
+   if ( MoveUnitCommand::avail( veh ))
+      result += moveToSavePlace ( veh );
 
    return result;
 }
@@ -359,57 +377,60 @@ MapCoordinate AI::getDestination ( Vehicle* veh )
 }
 
 
-AI::AiResult AI::moveToSavePlace ( Vehicle* veh, VehicleMovement& vm3, int preferredHeight )
+AI::AiResult AI::moveToSavePlace ( Vehicle* veh, int preferredHeight )
 {
    int unitNetworkID = veh->networkid;
 
    AiResult result;
 
-   if ( vm3.getStatus() == 0 )
-      vm3.execute ( veh, -1, -1, 0, -1, -1 );
+   auto_ptr<MoveUnitCommand> muc( new MoveUnitCommand( veh ));
 
-   // if there are no fields reachable
-   if ( vm3.getStatus() == -107 )
+   ActionResult res = muc->searchFields( preferredHeight );
+   if ( !res.successful() )
+      return result;
+   
+   const set<MapCoordinate3D>& fields = muc->getReachableFields();
+            
+   if ( fields.empty() )
       return result;
 
-   if ( vm3.getStatus() != 2 )
-      displaymessage ( "AI :: moveToSavePlace \n error in movement3 step 0 with unit %d", 1, veh->networkid );
 
    int xtogo = veh->xpos;
    int ytogo = veh->ypos;
    int threat = maxint;
    int dist = maxint;
 
-   if ( getfield ( veh->xpos, veh->ypos)->unitHere ( veh ) ) {  // vehicle not in building / transport
+   if ( getMap()->getField ( veh->xpos, veh->ypos)->unitHere ( veh ) ) {  // vehicle not in building / transport
       threat = int( getFieldThreat ( veh->xpos, veh->ypos).threat[ veh->aiparam[ getPlayerNum()]->valueType] * 1.5 + 1);
        // multiplying with 1.5 to make this field a bit unattractive, to allow other units (for example buggies) to attack from this field to, since it is probably quite a good position (else it would not have been chosen)
    }
 
-   for ( int f = 0; f < vm3.reachableFields.getFieldNum(); f++ )
-      if ( !vm3.reachableFields.getField( f )->vehicle && !vm3.reachableFields.getField( f )->building ) {
-         int x,y;
-         vm3.reachableFields.getFieldCoordinates ( f, &x, &y );
-         AiThreat& ait = getFieldThreat ( x, y );
-         int _dist = beeline ( x, y, veh->xpos, veh->ypos);
+   for ( set<MapCoordinate3D>::const_iterator i = fields.begin(); i != fields.end(); ++i ) {
+      tfield* fld = getMap()->getField( *i );
+      if ( !fld->vehicle && !fld->building ) {
+         AiThreat& ait = getFieldThreat ( i->x, i->y );
+         int _dist = beeline ( i->x, i->y, veh->xpos, veh->ypos);
 
             // make fields far away a bit unattractive; we don't want to move the whole distance back again next turn
          int t = int( ait.threat[ veh->aiparam[ getPlayerNum()]->valueType ] * log ( double(_dist) )/log(double(10)) );
 
-         if ( t < threat || ( t == threat && _dist < dist )) {
+         if ( t < threat || ( t == threat && (_dist < dist ))) {
             threat = t;
-            xtogo = x;
-            ytogo = y;
+            xtogo = i->x;
+            ytogo = i->y;
             dist = _dist;
          }
       }
+   }
 
    if ( veh->xpos != xtogo || veh->ypos != ytogo ) {
-      vm3.execute ( NULL, xtogo, ytogo, 2, -1, -1 );
-      if ( vm3.getStatus() != 3 )
-         displaymessage ( "AI :: moveToSavePlace \n error in movement3 step 2 with unit %d", 1, veh->networkid );
-
-      vm3.execute ( NULL, xtogo, ytogo, 3, -1,  1 );
-      if ( vm3.getStatus() != 1000 )
+      
+      muc->setDestination( MapCoordinate(xtogo, ytogo ));
+      
+      res = muc->execute( getContext() );
+      if ( res.successful() )
+         muc.release();
+      else
          displaymessage ( "AI :: moveToSavePlace \n error in movement3 step 3 with unit %d", 1, veh->networkid );
 
       result.unitsMoved++;
@@ -422,7 +443,7 @@ AI::AiResult AI::moveToSavePlace ( Vehicle* veh, VehicleMovement& vm3, int prefe
 }
 
 
-int AI::changeVehicleHeight ( Vehicle* veh, VehicleMovement* vm, int preferredDirection )
+int AI::changeVehicleHeight ( Vehicle* veh, int preferredDirection )
 {
 #if 0  // ####
    int bh = getBestHeight ( veh );
@@ -611,7 +632,11 @@ AI::AiResult AI::tactics( void )
                displaymessage2("tact: processing operation %d", unitCounter );
                checkKeys();
    
-               int stat = changeVehicleHeight ( veh, NULL );
+               int stat = changeVehicleHeight ( veh, -1 );
+               
+               if ( !getMap()->getUnit(*i) )
+                  continue;
+               
                if ( stat == -1 ) { // couldn't change height due to blocked way or something similar
                   veh->aiparam[ getPlayerNum() ]->setTask( AiParameter::tsk_wait );
                   result.unitsWaiting++;
@@ -623,6 +648,9 @@ AI::AiResult AI::tactics( void )
                   TargetVector tv;
                   getAttacks ( ast, veh, tv, hemmingBonus );
    
+                  if ( !getMap()->getUnit(*i) )
+                     continue;
+                  
                   if ( tv.size() ) {
                      MoveVariant* mv = *max_element( tv.begin(), tv.end(), moveVariantComp );
    
@@ -658,10 +686,14 @@ AI::AiResult AI::tactics( void )
                         AiResult res = executeMoveAttack ( veh, tv );
                         i = tactVehicles.erase ( i );
    
+                        if ( res.unitsMoved )
+                           unitsWorkedInTactics.insert(veh);
+                        
                         if ( !res.unitsDestroyed )
                            veh->aiparam[ getPlayerNum() ]->setTask( AiParameter::tsk_tactics );
    
                         result += res;
+                        
                         directAttackNum++;
    
                         _vision = org_vision;
@@ -748,6 +780,7 @@ AI::AiResult AI::tactics( void )
                         MapCoordinate affected =  MapCoordinate(finalPositions[i]->xpos, finalPositions[i]->ypos);
                         MapCoordinate3D dst = getNeighbouringFieldCoordinate( MapCoordinate3D( enemy->xpos, enemy->ypos, finalPositions[i]->height ), i);
                         dst.setnum ( dst.x, dst.y, -2 );
+                        
                         moveUnit ( finalPositions[i], dst );
                         _vision = visible_all;
 
@@ -755,7 +788,7 @@ AI::AiResult AI::tactics( void )
                         // the unit may have been shot down due to reaction fire
 
                         if ( !getMap()->getUnit ( nwid ) ) {
-                           TactVehicles::iterator att = find ( tactVehicles.begin(), tactVehicles.end(), finalPositions[i]->networkid ) ;
+                           TactVehicles::iterator att = find ( tactVehicles.begin(), tactVehicles.end(), nwid ) ;
                            tactVehicles.erase ( att );
                            finalPositions[i] = NULL;
                         }
@@ -778,35 +811,52 @@ AI::AiResult AI::tactics( void )
                      int finalAttackNum = maxint;
                      tactics_findBestAttackOrder ( finalPositions, attackOrder, enemy, 0, enemy->damage, finalDamage, finalOrder, finalAttackNum );
 
+                     vector<Vehicle*> unitsToMoveAwayAfterAttacking;
 
                      tfield* enemyField = getMap()->getField(enemy->xpos, enemy->ypos);
                      for ( int i = 0; i < finalAttackNum && enemyField->vehicle == enemy && finalAttackNum < maxint; i++ ) {
                         checkKeys();
                         if ( finalOrder[i] < 0 )
-                           warning("!!!");
+                           warningMessage("!!!");
                         // if ( i+1 < finalAttackNum ) {
                         if ( i < finalAttackNum && finalPositions[finalOrder[i]] ) {
                            Vehicle* a = finalPositions[finalOrder[i]];
-                           VehicleAttack va ( mapDisplay, NULL );
                            if ( finalOrder[i] < 0 )
-                              warning("!!!");
-
-                           va.execute ( finalPositions[finalOrder[i]], -1, -1, 0, 0, -1 );
-                           if ( va.getStatus() != 2 && strictChecks )
-                              displaymessage("inconsistency #1 in AI::tactics attack", 1 );
+                              warningMessage("!!!");
 
+                           unitsWorkedInTactics.insert(a);
+                           
+                           
+                           int nwid = a->networkid;
+                           auto_ptr<AttackCommand> va ( new AttackCommand ( a ));
+                           va->searchTargets();
+                           
                            VehicleTypeEfficiencyCalculator vtec (*this, finalPositions[finalOrder[i]], enemy );
-                           va.execute ( NULL, enemy->xpos, enemy->ypos, 2, 0, -1 );
-                           if ( va.getStatus() != 1000 && strictChecks )
+                           va->setTarget( enemy->getPosition() );
+                           ActionResult res = va->execute( getContext() );                           
+                           if ( !res.successful() && strictChecks )
                              displaymessage("inconsistency #1 in AI::tactics attack", 1 );
+                           
+                           if ( res.successful() )
+                              va.release();
+                             
 
                            vtec.calc();
 
-                           TactVehicles::iterator att = find ( tactVehicles.begin(), tactVehicles.end(), a->networkid ) ;
+                           TactVehicles::iterator att = find ( tactVehicles.begin(), tactVehicles.end(), nwid ) ;
                            tactVehicles.erase ( att );
+                           
+                           if ( getMap()->getUnit( nwid ) && a->typ->hasFunction( ContainerBaseType::MoveAfterAttack ))
+                              unitsToMoveAwayAfterAttacking.push_back( a );
                         }
                      }
+                     
+                     for ( vector<Vehicle*>::iterator i = unitsToMoveAwayAfterAttacking.begin(); i != unitsToMoveAwayAfterAttacking.end(); ++i )
+                        moveToSavePlace( *i );
+                     
                   } // unitcount > 0
+                  
+                  
                } // else { // if finalValue > 0
 
             } // if enemy
@@ -814,7 +864,7 @@ AI::AiResult AI::tactics( void )
             bool processable = true;
             do {
                currentTarget++;
-               if ( currentTarget != targets.end() )
+               if ( currentTarget != targets.end() ) {
                   if ( hemmingBonus == currentTarget->second.size()-1 ) {
                      for ( AffectedFields::iterator i = affectedFields.begin(); i != affectedFields.end(); i++ )
                         for ( MoveVariantContainer::iterator j = currentTarget->second.begin(); j != currentTarget->second.end(); j++ ) {
@@ -826,6 +876,7 @@ AI::AiResult AI::tactics( void )
                         }
                   } else
                      processable = false;
+               }
             } while ( !processable && currentTarget != targets.end() );
 
          } while ( currentTarget != targets.end() );
