@@ -15,8 +15,10 @@
  *                                                                         *
  ***************************************************************************/
 
-
+#include <iostream>
 #include "ai_common.h"
+#include "../actions/attackcommand.h"
+#include "../actions/moveunitcommand.h"
 
 const int value_armorfactor = 100;
 const int value_weaponfactor = 3000;
@@ -75,7 +77,7 @@ void         CalculateThreat_VehicleType :: calc_threat_vehicletype ( const Vehi
                if ( fzt->weapons.weapon[i].targ & (1 << j) ) {
                   int d = 0;
                   int m = 0;
-                  AttackFormula af;
+                  AttackFormula af( ai->getMap() );
                   for ( int e = (fzt->weapons.weapon[i].mindistance + maxmalq - 1)/ maxmalq; e <= fzt->weapons.weapon[i].maxdistance / maxmalq; e++ ) {    // the distance between two fields is maxmalq
                      d++;
                      int n = int( WeapDist::getWeaponStrength( &fzt->weapons.weapon[i], 0, e*maxmalq ) * fzt->weapons.weapon[i].maxstrength * af.strength_damage(getdamage()) * ( 1 + af.strength_experience(getexpirience())) );
@@ -263,7 +265,7 @@ void  AI :: calculateThreat ( Building* bld )
 void  AI :: calculateThreat ( Building* bld, int player )
 {
    if ( !bld->aiparam[ player ] )
-      bld->aiparam[ player ] = new AiValue ( log2 ( bld->typ->buildingheight ) );
+      bld->aiparam[ player ] = new AiValue ( log2 ( bld->typ->height ) );
 
    int b;
 
@@ -316,8 +318,8 @@ void AI :: WeaponThreatRange :: testfield ( const MapCoordinate& mc )
 {
    if ( dist*maxmalq <= veh->typ->weapons.weapon[weap].maxdistance )
       if ( dist*maxmalq >= veh->typ->weapons.weapon[weap].mindistance ) {
-         AttackFormula af;
-         int strength = int ( WeapDist::getWeaponStrength( &veh->typ->weapons.weapon[weap], ai->getMap()->getField(mc)->getweather(), dist*maxmalq, veh->height, 1 << height )
+         AttackFormula af ( ai->getMap() );
+         int strength = int ( WeapDist::getWeaponStrength( &veh->typ->weapons.weapon[weap], ai->getMap()->getField(mc)->getWeather(), dist*maxmalq, veh->height, 1 << height )
                               * veh->typ->weapons.weapon[weap].maxstrength
                               * (1 + af.strength_experience ( veh->experience ) + af.strength_attackbonus ( gamemap->getField(startPos)->getattackbonus() ))
                               * af.strength_damage ( veh->damage )
@@ -364,23 +366,18 @@ void AI :: calculateFieldInformation ( void )
 
                   fld->vehicle->setMovement ( fld->vehicle->maxMovement(), 0);
 
-                  VehicleMovement vm ( NULL, NULL );
-                  if ( vm.available ( fld->vehicle )) {
-                     vm.execute ( fld->vehicle, -1, -1, 0, -1, -1 );
+                  if ( MoveUnitCommand::avail ( fld->vehicle )) {
+                     MoveUnitCommand muc ( fld->vehicle );
+                     muc.searchFields();
 
-                     // Now we cycle through all fields that are reachable...
-                     for ( int f = 0; f < vm.reachableFields.getFieldNum(); f++ ) {
-                        int xp, yp;
-                        vm.reachableFields.getFieldCoordinates ( f, &xp, &yp );
-                        // ... and check for each which fields are threatened if the unit was standing there
-                        wr.run ( fld->vehicle, xp, yp, singleUnitThreat );
-                     }
-
-                     for ( int g = 0; g < vm.reachableFieldsIndirect.getFieldNum(); g++ ) {
-                        int xp, yp;
-                        vm.reachableFieldsIndirect.getFieldCoordinates ( g, &xp, &yp );
-                        wr.run ( fld->vehicle, xp, yp, singleUnitThreat );
-                     }
+                     const set<MapCoordinate3D>& fields =  muc.getReachableFields();
+                     for ( set<MapCoordinate3D>::const_iterator i = fields.begin(); i != fields.end(); ++i )
+                        wr.run ( fld->vehicle, i->x, i->y, singleUnitThreat );
+                  
+                     const set<MapCoordinate3D>& ifields  =  muc.getReachableFieldsIndirect();
+                     for ( set<MapCoordinate3D>::const_iterator i = ifields.begin(); i != ifields.end(); ++i )
+                        wr.run ( fld->vehicle, i->x, i->y, singleUnitThreat );
+                     
                   }
                   tus.restore();
                } else
@@ -412,6 +409,7 @@ void AI :: calculateFieldInformation ( void )
          fi.control = c;
       }
    }
+   delete[] singleUnitThreat;
 }
 
 
@@ -579,7 +577,7 @@ AI :: Sections :: Sections ( AI* _ai ) : ai ( _ai ) , section ( NULL )
    numY = ai->activemap->ysize * 2 / sizeY + 1;
 }
 
-void AI :: Sections :: reset ( void )
+void AI :: Sections :: reset ()
 {
    if ( section ) {
       delete[] section;
@@ -587,6 +585,11 @@ void AI :: Sections :: reset ( void )
    }
 }
 
+AI :: Sections :: ~Sections()
+{
+   reset();  
+}
+
 
 void AI :: Sections :: calculate ( void )
 {
@@ -627,7 +630,35 @@ AI::Section& AI :: Sections :: getForPos ( int xn, int yn )
    return section[xn+yn*numX];
 }
 
-AI::Section* AI :: Sections :: getBest ( int pass, Vehicle* veh, MapCoordinate3D* dest, bool allowRefuellOrder, bool secondRun )
+/*
+  This is like some ball rolling down from some high potential spike
+*/
+MapCoordinate AI :: Sections :: getAlternativeField( const MapCoordinate& pos, map<MapCoordinate,int>* destinationCounter, int height )
+{
+   MapCoordinate result  = pos;
+   int potential = destinationCounter->operator[](pos);
+   for ( int d = 0; d< 6; ++d ) {
+      MapCoordinate m = getNeighbouringFieldCoordinate( pos, d );
+      tfield* fld = ai->getMap()->getField(m );
+      if ( fld && (fld->a.temp & height) ) {
+         if ( destinationCounter->find( m ) == destinationCounter->end() )
+            return m;
+         else {
+            if ( (destinationCounter->find(m)->second * 11 / 10) < potential ) {
+               result = m;
+               potential = destinationCounter->find(m)->second;
+            }
+         }
+      }
+   }
+   if ( result == pos )
+      return result;
+   else
+      return getAlternativeField( result , destinationCounter, height );
+}
+
+
+AI::Section* AI :: Sections :: getBest ( int pass, Vehicle* veh, MapCoordinate3D* dest, bool allowRefuellOrder, bool secondRun, map<MapCoordinate,int>* destinationCounter )
 {
    /*
       In the first pass wwe check were all the units would go if there wouldn't be
@@ -717,8 +748,19 @@ AI::Section* AI :: Sections :: getBest ( int pass, Vehicle* veh, MapCoordinate3D
                             int mandist = abs( sec.centerx - xp ) + 2*abs ( sec.centery - yp );
                             if ( mandist < mindist ) {
                                mindist = mandist;
-                               xtogoSec = xp;
-                               ytogoSec = yp;
+                               
+                               if ( destinationCounter && destinationCounter->find( MapCoordinate(xp,yp)) != destinationCounter->end() ) {
+                                  /* we are checking if there are too many units heading for this field already 
+                                     check how many are for neighbouringfields */
+                                  
+                                  
+                                  MapCoordinate alt = getAlternativeField( MapCoordinate(xp,yp), destinationCounter, h );
+                                  xtogoSec = alt.x;
+                                  ytogoSec = alt.y;
+                               } else {
+                                 xtogoSec = xp;
+                                 ytogoSec = yp;
+                               }
                             }
 
                             ai->_vision = visible_all;
@@ -729,17 +771,17 @@ AI::Section* AI :: Sections :: getBest ( int pass, Vehicle* veh, MapCoordinate3D
                             veh->ypos = yp;
                             veh->height = h;
 
-                            VehicleAttack va ( NULL, NULL );
-                            if ( va.available ( veh )) {
-                               va.execute ( veh, -1, -1, 0, 0, -1 );
-                               targets += va.attackableVehicles.getFieldNum();
+                            if ( AttackCommand::avail( veh )) {
+                               AttackCommand attack ( veh ); 
+                               attack.searchTargets();
+                               targets += attack.getAttackableUnits().size();
                             }
                             ai->_vision = visible_ago;
                          } else
                             nac++;
                       }
 
-                   if ( xtogoSec >= 0 && ytogoSec >= 0 )
+                   if ( xtogoSec >= 0 && ytogoSec >= 0 ) {
                       if ( !rfc || rfc->returnFromPositionPossible ( MapCoordinate3D( xtogoSec, ytogoSec, h ))) {
                          int notAccessible = 100 * nac / (nac+ac);
                          if ( notAccessible < 85  && targets ) {   // less than 85% of fields not accessible
@@ -757,7 +799,7 @@ AI::Section* AI :: Sections :: getBest ( int pass, Vehicle* veh, MapCoordinate3D
                             if ( rfc && rfc->returnFromPositionPossible ( MapCoordinate3D( xtogoSec, ytogoSec, h ), veh->getStorageCapacity().fuel ))
                                sectionsPossibleWithMaxFuell++;
                          }
-
+                   }
                 }
             }
 
