@@ -43,6 +43,11 @@
 #include "gameoptions.h"
 #include "dialog.h"
 #include "widgets/textrenderer-addons.h"
+#include "spfst-legacy.h"
+
+#include "dialogs/fileselector.h"
+
+#include "actions/moveunitcommand.h"
 
 ASC_MainScreenWidget*  mainScreenWidget = NULL ;
 
@@ -125,6 +130,7 @@ void Menu::setup()
    addfield ( "Glo~b~al" );
    addbutton ( "~O~ptions", ua_gamepreferences );
    addbutton ( "~S~ound options", ua_soundDialog );
+   addbutton ( "~E~mail options", ua_emailOptions );
    currentMenu->addSeparator();
    addbutton ( "~M~ain menu\tESC", ua_mainmenu );
    currentMenu->addSeparator();
@@ -147,11 +153,20 @@ void Menu::setup()
    addbutton ( "~D~iplomacy", ua_setupalliances);
    addbutton ( "transfer ~U~nit control", ua_giveunitaway );
    addbutton ( "~T~ransfer resources", ua_settribute);
+   addbutton ( "Choose Technology", ua_chooseTechnology );
    addbutton ( "~C~ancel Research", ua_cancelResearch );
    currentMenu->addSeparator();
    addbutton ( "~E~dit Player Data", ua_editPlayerData);
 
 
+   addfield ( "~A~ction" );
+   addbutton ( "~U~ndo\tctrl-z", ua_undo );
+   addbutton ( "~R~edo\tctrl-shift-z", ua_redo );
+   addbutton ( "~M~anage", ua_actionManager );
+   currentMenu->addSeparator();
+   addbutton ( "Save Script", ua_writeLuaCommands );
+   addbutton ( "Run Script", ua_runLuaCommands );
+
 
    addfield ( "~I~nfo" );
    addbutton ( "~V~ehicle types", ua_vehicleinfo );
@@ -159,6 +174,7 @@ void Menu::setup()
    addbutton ( "Unit ~m~ovement range\t4", ua_viewunitmovementrange );
    addbutton ( "~G~ame Time", ua_GameStatus );
    addbutton ( "unit ~S~et information", ua_UnitSetInfo );
+   addbutton ( "unit guide dialog", ua_unitGuideDialog );
    addbutton ( "~T~errain\t7", ua_viewterraininfo );
    addbutton ( "~U~nit weight\t8", ua_unitweightinfo );
    currentMenu->addSeparator();
@@ -183,6 +199,7 @@ void Menu::setup()
 
    addfield ( "~M~essage");
    addbutton ( "~n~ew message", ua_newmessage );
+   addbutton ( "new reminder\tctrl-r", ua_createReminder );
    addbutton ( "view ~q~ueued messages", ua_viewqueuedmessages );
    addbutton ( "view ~s~ent messages", ua_viewsentmessages );
    addbutton ( "view ~r~eceived messages", ua_viewreceivedmessages);
@@ -195,19 +212,22 @@ void Menu::setup()
    addbutton ( "save ~S~creen as PCX", ua_writescreentopcx );
    addbutton ( "select graphic set", ua_selectgraphicset );
    currentMenu->addSeparator();
-   addbutton ( "~G~oto coorindate\tctrl-g", ua_gotoPosition );
+   addbutton ( "~G~oto coordinate\tctrl-g", ua_gotoPosition );
 
    addfield ( "~V~iew" );
    addbutton ( "toggle ~R~esourceview\t1", ua_changeresourceview );
    addbutton ( "toggle unit shading\t2", ua_toggleunitshading );
    addbutton ( "show ~P~ipeline net\t9", ua_viewPipeNet );
    addbutton ( "show ~V~isibility Range\t0", ua_visibilityInfo );
+   addbutton ( "show reaction~f~ire", ua_viewReactionfireOverlay );
    currentMenu->addSeparator();
    addbutton ( "Button Panel", ua_viewButtonPanel );
    addbutton ( "Wind Panel", ua_viewWindPanel );
    addbutton ( "Unit Info Panel", ua_viewUnitInfoPanel );
    addbutton ( "Overview Map Panel", ua_viewOverviewMapPanel );
    addbutton ( "Map Control Panel", ua_viewMapControlPanel );
+//   addbutton ( "Action Panel",    ua_viewActionPanel );
+
    // addbutton("Weathercast", ua_weathercast);
    currentMenu->addSeparator();
    addbutton ( "Increase Map Zoom\tKP+", ua_increase_zoom );
@@ -229,10 +249,15 @@ void Menu::setup()
    addbutton ( "show techadapter", ua_showTechAdapter );
    addbutton ( "calc airplane endurance", ua_showUnitEndurance );
    addbutton ( "show game ~P~arameters", ua_GameParameterInfo );
+   addbutton ( "Recompute View", ua_recompteview );
    currentMenu->addSeparator();
    addbutton ( "benchmark without view calc", ua_benchgamewov );
    addbutton ( "benchmark with view calc", ua_benchgamewv);
    addbutton ( "compiler benchmark (AI)", ua_aibench );
+   currentMenu->addSeparator();
+   addbutton ( "unit AI properties", ua_unitAiOptions );
+   currentMenu->addSeparator();
+   addbutton ( "show packages used by map", ua_showUsedPackages );
    // addbutton ( "test memory integrity", ua_heapcheck );
 
    addfield ( "~H~elp" );
@@ -366,30 +391,16 @@ class UnitMovementRangeLayer : public MapLayer, public SigC::Object {
 
       if ( fieldvisiblenow ( getfield ( veh->xpos, veh->ypos ))) {
          int counter = 0;
-         VehicleMovement vm ( NULL, NULL );
-
-
-         TemporaryContainerStorage tcs( veh );
-         veh->setMovement( veh->maxMovement(), 0 );
-         if ( vm.available ( veh )) {
-
-            vm.execute ( veh, -1, -1, 0, -1, -1 );
-            if ( vm.reachableFields.getFieldNum()) {
-               for  ( int i = 0; i < vm.reachableFields.getFieldNum(); i++ )
-                  if ( fieldvisiblenow ( vm.reachableFields.getField ( i ) )) {
-                     ++counter;
-                     markField( vm.reachableFields.getFieldCoordinates(i));
-                  }
-               for  ( int j = 0; j < vm.reachableFieldsIndirect.getFieldNum(); j++ )
-                  if ( fieldvisiblenow ( vm.reachableFieldsIndirect.getField ( j ))) {
-                     markField( vm.reachableFieldsIndirect.getFieldCoordinates(j));
-                     ++counter;
-                  }
-   
+         
+         AStar3D pathfinder( veh->getMap(), veh, false, veh->maxMovement() );
+         vector<MapCoordinate3D> fieldList;
+         pathfinder.findAllAccessibleFields( &fieldList );
+         
+         for ( vector<MapCoordinate3D>::iterator i = fieldList.begin(); i != fieldList.end(); ++i )
+            if ( fieldvisiblenow ( veh->getMap()->getField(*i) )) {
+               ++counter;
+               markField( *i );
             }
-
-         }
-         tcs.restore();
       
          if ( counter )
             fields[veh->getPosition()] |= 2;
@@ -470,7 +481,7 @@ class VisibilityLayer : public MapLayer {
 
 void VisibilityLayer::paintSingleField( const MapRenderer::FieldRenderInfo& fieldInfo, int layer, const SPoint& pos )
 {
-   if ( fieldvisiblenow( fieldInfo.fld, fieldInfo.gamemap->getPlayerView(), fieldInfo.gamemap )) {
+   if ( fieldVisibility( fieldInfo.fld, fieldInfo.gamemap->getPlayerView() ) >= visible_now ) {
 
       int view = 0;
       int jamming = 0;
@@ -516,7 +527,7 @@ void VisibilityLayer::renderText( const MapRenderer::FieldRenderInfo& fieldInfo,
 
 
 ASC_MainScreenWidget::ASC_MainScreenWidget( PG_Application& application )
-   : MainScreenWidget( application ), standardActionsLocked(0), guiHost(NULL), menu(NULL), unitInfoPanel(NULL), windInfoPanel(NULL), mapInfoPanel(NULL)
+   : MainScreenWidget( application ), standardActionsLocked(0), guiHost(NULL), menu(NULL), unitInfoPanel(NULL), windInfoPanel(NULL), mapInfoPanel(NULL), actionInfoPanel(NULL)
 {
 
    int mapWidth = Width() - 30;
@@ -600,11 +611,12 @@ void ASC_MainScreenWidget :: lockStandardActions( int dir, int options )
       lockOptionsChanged( lockOptions );
    }
    
-   if ( menu )
+   if ( menu ) {
       if ( standardActionsLocked <= 0 )
          menu->Show();
       else
          menu->Hide();
+   }
 }
 
 
@@ -646,6 +658,8 @@ void ASC_MainScreenWidget::spawnPanel ( const ASCString& panelName )
    if ( panelName == "MapInfo" )
       spawnPanel ( MapControl );
 
+   if ( panelName == "ActionInfo" )
+      spawnPanel ( ActionInfo );
 }
 
 void ASC_MainScreenWidget::spawnPanel ( Panels panel )
@@ -683,6 +697,15 @@ void ASC_MainScreenWidget::spawnPanel ( Panels panel )
       }
       mapInfoPanel->Show();
    }
+   
+   if ( panel == ActionInfo ) {
+      if ( !actionInfoPanel || !CGameOptions::Instance()->cacheASCGUI ) {
+         delete actionInfoPanel;
+         actionInfoPanel = new ActionInfoPanel( this, PG_Rect(Width()-170, 0, 170, 160) );
+      }
+      actionInfoPanel->Show();
+      actionInfoPanel->update( actmap );
+   }
 }
 
 
@@ -738,12 +761,6 @@ bool ASC_MainScreenWidget::eventKeyDown(const SDL_KeyboardEvent* key)
                }
                return true;
 
-               case SDLK_F11: {
-                  int sz = actmap->player[1].unreadmessage.size();
-                  printf(" %d \n", sz );
-               }
-               return true;
-               
             case SDLK_1:
                execUserAction_ev ( ua_changeresourceview );
                return true;
@@ -790,6 +807,10 @@ bool ASC_MainScreenWidget::eventKeyDown(const SDL_KeyboardEvent* key)
             case SDLK_0: execUserAction_ev( ua_visibilityInfo );
                return true;
 
+            case SDLK_BACKSPACE:               
+               execUserAction_ev( ua_undo );
+               return true;
+
 
             case SDLK_PLUS:   
             case SDLK_KP_PLUS: execUserAction_ev( ua_increase_zoom );
@@ -817,15 +838,30 @@ bool ASC_MainScreenWidget::eventKeyDown(const SDL_KeyboardEvent* key)
    if ( (mod & KMOD_SHIFT) && (mod & KMOD_CTRL)) {
       switch ( key->keysym.sym ) {
             case SDLK_F11: {
-               tfield* fld = actmap->getField(actmap->getCursor() );
-               if ( actmap && actmap->getCurrentPlayer().stat == Player::supervisor && fld && fld->vehicle )
-                  fld->vehicle->fillMagically( true, false );
                return true;
             }
-            case SDLK_l:
+            case SDLK_F10: {
+               return true;
+            }
+         case SDLK_l:
                execUserAction_ev ( ua_loadrecentgame );
                return true;
 
+            case SDLK_LEFT:
+               execUserAction_ev( ua_turnUnitLeft );
+               return true;
+
+            case SDLK_RIGHT:
+               execUserAction_ev( ua_turnUnitRight );
+               return true;
+
+            default:;
+      }
+      switch ( key->keysym.unicode ) {
+            case 26:
+               execUserAction_ev( ua_redo );
+               return true;
+               
             default:;
       }
    }
@@ -833,38 +869,9 @@ bool ASC_MainScreenWidget::eventKeyDown(const SDL_KeyboardEvent* key)
    if ( mod & KMOD_CTRL ) {
       switch ( key->keysym.sym ) {
 
-            case SDLK_f:
-               execUserAction_ev ( ua_unitsummary );
-               return true;
-
-            case SDLK_c:
-               execUserAction_ev ( ua_cargosummary );
-               return true;
-
-            case SDLK_g:
-               execUserAction_ev( ua_gotoPosition );
-               return true;
-
-            case SDLK_l:
-               execUserAction_ev ( ua_loadgame );
-               return true;
-
-            case SDLK_s:
-               execUserAction_ev ( ua_savegame );
-               return true;
-
-
-            case SDLK_n:
-               execUserAction_ev ( ua_newGame );
-               return true;
-
             case SDLK_0: execUserAction_ev( ua_writescreentopcx );
                return true;
 
-            case SDLK_F12:
-                  execUserAction_ev ( ua_exportUnitToFile );
-               return true;
-
             case SDLK_F10: {
                SDL_Surface* s = PG_Application::GetScreen();
                SDL_LockSurface( s );
@@ -873,15 +880,49 @@ bool ASC_MainScreenWidget::eventKeyDown(const SDL_KeyboardEvent* key)
                   i[x * s->pitch/4 + x] = 0xffffff;
                SDL_UnlockSurface( s );
                SDL_UpdateRect(s,0,0,0,0);
-                           }
+               return true;
+            }
 
-            return true;
             
+            default:;
+      }
+      switch ( key->keysym.unicode ) {
+            case 3: // C
+               execUserAction_ev ( ua_cargosummary );
+               return true;
 
-            case SDLK_q:
+            case 6: // F
+               execUserAction_ev ( ua_unitsummary );
+               return true;
+
+            case 7: // G
+               execUserAction_ev( ua_gotoPosition );
+               return true;
+
+            case 12: // L
+               execUserAction_ev ( ua_loadgame );
+               return true;
+
+            case 14: // N
+               execUserAction_ev ( ua_newGame );
+               return true;
+
+            case 17: // Q
                execUserAction_ev ( ua_exitgame );
                return true;
-            default:;
+
+            case 18: // R
+               execUserAction_ev( ua_createReminder );
+               return true;
+               
+            case 19: // S
+               execUserAction_ev ( ua_savegame );
+               return true;
+
+            case 26: // Z
+               execUserAction_ev( ua_undo );
+               return true;
+
       }
    }
 
