@@ -8,6 +8,7 @@
 #include <vector>
 #include <functional>
 #include <algorithm>
+#include <cmath>
 
 #include "vehicletype.h"
 #include "spfst.h"
@@ -23,17 +24,16 @@
 
 // mark -> temp3
 
-
-bool operator < ( const AStar::Node& a, const AStar::Node& b )
+bool AStar::Node::operator< ( const AStar::Node& a ) const
 {
     // To compare two nodes, we compare the `f' value, which is the
     // sum of the g and h values.
-    return (a.gval+a.hval) < (b.gval+b.hval);
+    return (gval+hval) < (a.gval+a.hval);
 }
 
-bool operator > ( const AStar::Node& a, const AStar::Node& b )
+bool AStar::Node::operator> ( const AStar::Node& a ) const
 {
-    return (a.gval+a.hval) > (b.gval+b.hval);
+    return (gval+hval) > (a.gval+a.hval);
 }
 
 
@@ -65,7 +65,7 @@ AStar :: AStar ( GameMap* actmap, Vehicle* veh )
    tempsMarked = NULL;
    _path = NULL;
    _veh = veh;
-   _actmap = actmap;
+   gamemap = actmap;
    MAXIMUM_PATH_LENGTH = maxint;
 }
 
@@ -104,7 +104,7 @@ int AStar::dist( HexCoord a, HexCoord b )
 
 int AStar::getMoveCost ( int x1, int y1, int x2, int y2, const Vehicle* vehicle )
 {
-    if ( !fieldAccessible ( getfield ( x2, y2 ), vehicle ))
+    if ( !fieldAccessible ( gamemap->getField ( x2, y2 ), vehicle ))
        return MAXIMUM_PATH_LENGTH;
 
     return calcMoveMalus ( MapCoordinate3D(x1, y1, vehicle->height), MapCoordinate3D(x2, y2, vehicle->height), vehicle ).first;
@@ -140,14 +140,13 @@ void AStar::findPath( HexCoord A, HexCoord B, Path& path )
 {
     _path = &path;
     Vehicle* veh = _veh;
-    GameMap* actmap = _actmap;
 
-    if ( actmap->getField(A.m, A.n)->unitHere(veh) )
+    if ( gamemap->getField(A.m, A.n)->unitHere(veh) )
        if ( !veh->canMove() )
           return;
 
-    for ( int y = actmap->xsize * actmap->ysize -1; y >= 0; y-- )
-       actmap->field[y].temp3 = DirNone;
+    for ( int y = gamemap->xsize * gamemap->ysize -1; y >= 0; y-- )
+       gamemap->field[y].temp3 = DirNone;
 
     Node N;
     Container open;
@@ -200,7 +199,7 @@ void AStar::findPath( HexCoord A, HexCoord B, Path& path )
             HexCoord hn = N.h;
             getnextfield ( hn.m, hn.n, d );
             // If it's off the end of the map, then don't keep scanning
-            if( hn.m < 0 || hn.n < 0 || hn.m >= actmap->xsize || hn.n >= actmap->ysize || !fieldAccessible ( getfield ( hn.m, hn.n ), veh ))
+            if( hn.m < 0 || hn.n < 0 || hn.m >= gamemap->xsize || hn.n >= gamemap->ysize || !fieldAccessible ( gamemap->getField ( hn.m, hn.n ), veh ))
                 continue;
 
             // cursor.gotoxy ( hn.m, hn.n );
@@ -218,12 +217,12 @@ void AStar::findPath( HexCoord A, HexCoord B, Path& path )
 
             N2.hval = dist( hn, B );
             // If this spot (hn) hasn't been visited, its mark is DirNone
-            if( getfield (hn.m,hn.n)->temp3 == DirNone ) {
+            if( gamemap->getField (hn.m,hn.n)->temp3 == DirNone ) {
 
                 // The space is not marked
 
                 if ( N.gval < MAXIMUM_PATH_LENGTH ) {
-                   getfield (hn.m,hn.n)->temp3 = ReverseDirection(d);
+                   gamemap->getField (hn.m,hn.n)->temp3 = ReverseDirection(d);
                    open.push_back( N2 );
                    push_heap( open.begin(), open.end(), comp );
                 }
@@ -244,7 +243,7 @@ void AStar::findPath( HexCoord A, HexCoord B, Path& path )
                     Node N3 = *find1;
                     if( N3.gval > N2.gval )
                     {
-                        getfield (hn.m,hn.n)->temp3 = ReverseDirection(d);
+                        gamemap->getField (hn.m,hn.n)->temp3 = ReverseDirection(d);
                         // Replace N3 with N2 in the open list
                         Container::iterator last = open.end() - 1;
                         *find1 = *last;
@@ -264,7 +263,7 @@ void AStar::findPath( HexCoord A, HexCoord B, Path& path )
         HexCoord h = B;
         while( !(h == A) )
         {
-            HexDirection dir = HexDirection ( getfield (h.m, h.n)->temp3 );
+            HexDirection dir = HexDirection ( gamemap->getField (h.m, h.n)->temp3 );
             tempPath.push_back( int( ReverseDirection( dir ) ) );
             getnextfield ( h.m, h.n, dir );
         }
@@ -285,12 +284,12 @@ void AStar::findPath( HexCoord A, HexCoord B, Path& path )
         // No path
     }
 
-    actmap->cleartemps ( 4 ); 
+    gamemap->cleartemps ( 4 ); 
 }
 
 int AStar::getDistance ( )
 {
-   if ( !_path || !_veh || ! _actmap )
+   if ( !_path || !_veh || ! gamemap )
       return -1;
 
    if ( _path->size() <= 0 )
@@ -298,7 +297,7 @@ int AStar::getDistance ( )
 
    int dist = 0;
    for ( Path::iterator i = _path->begin(); i != _path->end(); i++ )
-      dist += _actmap->getField ( *i )->getmovemalus ( _veh->typ->movemalustyp );
+      dist += gamemap->getField ( *i )->getmovemalus ( _veh->typ->movemalustyp );
 
    return dist;
 }
@@ -324,16 +323,16 @@ bool AStar::fieldVisited ( int x, int y)
 
 void AStar::findAllAccessibleFields ( int maxDist )
 {
-   _actmap->cleartemps ( 1 );
+   gamemap->cleartemps ( 1 );
 
    MAXIMUM_PATH_LENGTH = maxDist;
 
    Path dummy;
-   findPath ( dummy, _actmap->xsize, _actmap->ysize );  //this field does not exist...
+   findPath ( dummy, gamemap->xsize, gamemap->ysize );  //this field does not exist...
    for ( Container::iterator i = visited.begin(); i != visited.end(); i++ )
-      _actmap->getField ( (*i).h.m, (*i).h.n )->a.temp = 1;
+      gamemap->getField ( (*i).h.m, (*i).h.n )->a.temp = 1;
 
-   tempsMarked = _actmap;
+   tempsMarked = gamemap;
 }
 
 
@@ -458,6 +457,10 @@ AStar3D :: AStar3D ( GameMap* actmap_, Vehicle* veh_, bool markTemps_, int maxDi
       for ( int i = 0; i < 8; i++ )
           vehicleSpeedFactor[i] = float(veh->typ->movement[i]);
 
+   maxVehicleSpeedFactor = 0;
+   for ( int i = 0; i < 8; ++i )
+      maxVehicleSpeedFactor = max( maxVehicleSpeedFactor, vehicleSpeedFactor[i] );
+   
    int cnt = actmap->xsize*actmap->ysize*9;
    posDirs = new HexDirection[cnt];
    posHHops = new int[cnt];
@@ -514,6 +517,8 @@ AStar3D::DistanceType AStar3D::dist ( const MapCoordinate3D& a, const vector<Map
    DistanceType d = longestPath;
    for ( vector<MapCoordinate3D>::const_iterator i = b.begin(); i != b.end(); i++ ) {
       DistanceType e = dist(a,*i);
+      if ( maxVehicleSpeedFactor )
+         e /= maxVehicleSpeedFactor;
       if ( d > e )
          d = e;
    }
@@ -562,8 +567,6 @@ AStar3D::DistanceType AStar3D::getMoveCost ( const MapCoordinate3D& start, const
 // abstraction layer on priority_queue wouldn't let me do that.
 
 
-
-
 void AStar3D :: nodeVisited ( const Node& N2, HexDirection direc, Container& open, int prevHeight, int heightChangeDist )
 {
    if ( N2.gval <= MAXIMUM_PATH_LENGTH && N2.gval < longestPath ) {
@@ -620,8 +623,10 @@ void AStar3D::findPath( const MapCoordinate3D& A, const vector<MapCoordinate3D>&
     Node N;
     Container open;
 
+    /* this should be checked outside by giving the pathfinder the right movement
     if ( !veh->canMove() )
        return;
+    */
 
     if ( actmap->getField(A)->unitHere(veh) ) {
        // insert the original node
@@ -643,7 +648,7 @@ void AStar3D::findPath( const MapCoordinate3D& A, const vector<MapCoordinate3D>&
     // While there are still nodes to visit, visit them!
     while( !open.empty() ) {
         N = open.getFirst();
-
+       
         visited.add( N );
         // If we're at the goal, then exit
         for ( vector<MapCoordinate3D>::const_iterator i = B.begin(); i != B.end(); i++ )
@@ -861,7 +866,7 @@ void AStar3D::findPath( const MapCoordinate3D& A, const vector<MapCoordinate3D>&
 
         MapCoordinate3D h = endpos;
 
-        path.insert ( path.begin(), PathPoint ( h, int(N.gval), N.enterHeight, N.hasAttacked ) );
+        path.insert ( path.begin(), PathPoint ( h, int(ceil(N.gval)), N.enterHeight, N.hasAttacked ) );
         while( !(h == A) )
         {
             // tfield* fld = actmap->getField ( h );
@@ -904,7 +909,7 @@ void AStar3D::findPath( Path& path, const vector<MapCoordinate3D>& dest )
 }
 
 
-void AStar3D::findAllAccessibleFields ( )
+void AStar3D::findAllAccessibleFields ( vector<MapCoordinate3D>* path )
 {
    if ( markTemps )
       actmap->cleartemps ( 3 );
@@ -916,7 +921,11 @@ void AStar3D::findAllAccessibleFields ( )
       fa |= i->h.getBitmappedHeight();
       if ( markTemps )
          actmap->getField ( i->h )->a.temp  |= i->h.getBitmappedHeight();
+      
+      if ( path )
+         path->push_back( i->h );
    }
+   
    if ( markTemps )
       tempsMarked = actmap;
 }
@@ -942,3 +951,31 @@ int& AStar3D::getFieldAccess ( const MapCoordinate& mc )
 {
    return fieldAccess[mc.x + mc.y * actmap->xsize];
 }
+
+             
+void AStar3D::PathPoint::write( tnstream& stream ) const
+{
+   stream.writeInt(1);
+   MapCoordinate::write( stream );
+   stream.writeInt( dist );
+   stream.writeInt( enterHeight );
+   stream.writeInt( hasAttacked );
+}
+
+void AStar3D::PathPoint::read( tnstream& stream )
+{
+   stream.readInt(); // version  
+   MapCoordinate::read ( stream );
+   dist = stream.readInt();
+   enterHeight = stream.readInt(),
+   hasAttacked = stream.readInt();
+}
+
+
+AStar3D::PathPoint AStar3D::PathPoint::newFromStream( tnstream& stream )
+{
+   PathPoint pp;
+   pp.read(stream);
+   return pp;  
+}
+
