@@ -34,8 +34,20 @@
 #include "attack.h"
 #include "spfst.h"
 
+#include "actions/changeunitproperty.h"
+#include "actions/consumeammo.h"
+#include "actions/registerunitrftarget.h"
+#include "actions/inflictdamage.h"
+#include "actions/removemine.h"
+#include "actions/removeobject.h"
+#include "actions/changeobjectproperty.h"
 
 
+AttackFormula::AttackFormula( GameMap* gamemap ) 
+{
+   this->gamemap = gamemap;
+}
+
 bool  AttackFormula :: checkHemming ( Vehicle*     d_eht,  int     direc )
 { 
    Vehicle*     s_eht;
@@ -43,7 +55,7 @@ bool  AttackFormula :: checkHemming ( Vehicle*     d_eht,  int     direc )
    int x = d_eht->xpos;
    int y = d_eht->ypos; 
    getnextfield(x, y, direc);
-   tfield* fld = getfield(x,y);
+   tfield* fld = d_eht->getMap()->getField(x,y);
 
    if ( fld )
       s_eht = fld->vehicle;
@@ -70,7 +82,7 @@ float AttackFormula :: getHemmingFactor ( int relDir )
       relDir += sidenum;
 
    if ( relDir == 5 )
-      warning("float AttackFormula :: getHemmingFactor - invalid direction" );
+      warningMessage("float AttackFormula :: getHemmingFactor - invalid direction" );
       
    return hemming[relDir]*maxHemmingFactor/maxHemmingSum;
 }
@@ -107,7 +119,7 @@ float AttackFormula :: strength_experience ( int experience )
 	float e =		(experience < 0)
 				?	0	
 				:	experience ;
-   return e/maxunitexperience * 2.875 / actmap->getgameparameter( cgp_experienceDivisorAttack );
+   return e/maxunitexperience * 2.875 / gamemap->getgameparameter( cgp_experienceDivisorAttack );
 }
 
 float AttackFormula :: defense_experience ( int experience )
@@ -116,7 +128,7 @@ float AttackFormula :: defense_experience ( int experience )
 				?	0
 				:	experience ;
 
-   return e/maxunitexperience * 1.15 / actmap->getgameparameter( cgp_experienceDivisorDefense );
+   return e/maxunitexperience * 1.15 / gamemap->getgameparameter( cgp_experienceDivisorDefense );
 }
 
 float AttackFormula :: strength_attackbonus ( int abonus )
@@ -134,14 +146,9 @@ float AttackFormula :: defense_defensebonus ( int defensebonus )
 
 
 
-tfight :: tfight ( void )
-{
-}
-
-
 void tfight :: calc ( void )
 {
-   int damagefactor = actmap->getgameparameter ( cgp_attackPower );
+   int damagefactor = gamemap->getgameparameter ( cgp_attackPower );
    const float armordivisor = 5;
 
 
@@ -157,7 +164,7 @@ void tfight :: calc ( void )
       int w = int( ceil(dv.damage + absstrength / absdefense * 1000 / damagefactor ));
 
       if (dv.damage > w ) 
-         warning("fatal error at attack: \ndecrease of damage d!");
+         warningMessage("fatal error at attack: \ndecrease of damage d!");
 
       if (dv.damage == w )
          w = dv.damage+1;
@@ -199,7 +206,7 @@ void tfight :: calc ( void )
       int w = int( ceil(av.damage + absstrength / absdefense * 1000 / damagefactor ));
 
       if (av.damage > w ) 
-         warning("fatal error at attack: \ndecrease of damage a!");
+         warningMessage("fatal error at attack: \ndecrease of damage a!");
 
       if (w > 100) 
          av.damage = 100; 
@@ -230,6 +237,7 @@ void tfight :: calc ( void )
 
 
 tunitattacksunit :: tunitattacksunit ( Vehicle* &attackingunit, Vehicle* &attackedunit, bool respond, int weapon, bool reactionfire )
+   : UnitAttacksSomething( attackingunit->getMap() )
 {
    this->reactionfire = reactionfire;
    setup ( attackingunit, attackedunit, respond, weapon );
@@ -247,7 +255,7 @@ void tunitattacksunit :: setup ( Vehicle* &attackingunit, Vehicle* &attackedunit
    int _weapon;
 
    if ( weapon == -1 ) {
-      pattackweap atw = attackpossible( attackingunit, attackedunit->xpos, attackedunit->ypos );
+      AttackWeap* atw = attackpossible( attackingunit, attackedunit->xpos, attackedunit->ypos );
       int n = -1;
       int s = 0;
       for ( int i = 0; i < atw->count; i++ )
@@ -266,7 +274,7 @@ void tunitattacksunit :: setup ( Vehicle* &attackingunit, Vehicle* &attackedunit
    const SingleWeapon* weap = attackingunit->getWeapon(_weapon);
 
 
-   int targetWeather = attackedunit->getMap()->getField( attackedunit->getPosition() )->getweather();
+   int targetWeather = attackedunit->getMap()->getField( attackedunit->getPosition() )->getWeather();
 
    av.strength = int ( ceil( attackingunit->weapstrength[_weapon]
                         * WeapDist::getWeaponStrength(weap, targetWeather, dist, attackingunit->height, attackedunit->height )
@@ -283,7 +291,7 @@ void tunitattacksunit :: setup ( Vehicle* &attackingunit, Vehicle* &attackedunit
    av.height = attackingunit->height;
    av.weapontype = attackingunit->typ->weapons.weapon[ _weapon ].getScalarWeaponType();
 
-   tfield* field = getfield ( attackingunit->xpos, attackingunit->ypos );
+   tfield* field = attackingunit->getMap()->getField ( attackingunit->xpos, attackingunit->ypos );
 
    if ( attackingunit->height <= chfahrend ) {
       // if ( dist <= maxmalq )
@@ -324,12 +332,12 @@ void tunitattacksunit :: setup ( Vehicle* &attackingunit, Vehicle* &attackedunit
       weap = attackedunit->getWeapon( dv.weapnum );
 
 
-      int attackerWeather = attackingunit->getMap()->getField( attackingunit->getPosition() )->getweather();
+      int attackerWeather = attackingunit->getMap()->getField( attackingunit->getPosition() )->getWeather();
 
       dv.strength  = int ( ceil( attackedunit->weapstrength[ dv.weapnum ]
                            * WeapDist::getWeaponStrength(weap, attackerWeather, dist, attackedunit->height, attackingunit->height )
                            * attackedunit->typ->weapons.weapon[ dv.weapnum ].targetingAccuracy[attackingunit->typ->movemalustyp] / 100 ));
-      field = getfield ( attackedunit->xpos, attackedunit->ypos );
+      field = attackingunit->getMap()->getField ( attackedunit->xpos, attackedunit->ypos );
       dv.attackbonus  = field->getattackbonus();
       _respond = 1;
 
@@ -353,7 +361,7 @@ void tunitattacksunit :: setup ( Vehicle* &attackingunit, Vehicle* &attackedunit
 
 
    if ( attackedunit->height <= chfahrend )
-      dv.defensebonus = getfield ( attackedunit->xpos, attackedunit->ypos ) -> getdefensebonus();
+      dv.defensebonus = attackedunit->getMap()->getField ( attackedunit->xpos, attackedunit->ypos ) -> getdefensebonus();
    else
       dv.defensebonus = 0;
 
@@ -394,7 +402,49 @@ void tunitattacksunit :: setresult ( void )
      delete *_pattackedunit;
      *_pattackedunit = NULL;
    }
-   actmap->time.set ( actmap->time.turn(), actmap->time.move()+1);
+   gamemap->time.set ( gamemap->time.turn(), gamemap->time.move()+1);
+}
+
+void tunitattacksunit :: setresult( const Context& context )
+{
+   int nwid = _attackingunit->networkid;
+   GameMap* map = _attackingunit->getMap();
+   
+   GameAction* a = new ChangeUnitProperty( _attackingunit, ChangeUnitProperty::Experience, av.experience );
+   a->execute ( context );
+   
+   GameAction* b = new ConsumeAmmo( _attackingunit, _attackingunit->typ->weapons.weapon[av.weapnum].getScalarWeaponType(), av.weapnum, _attackingunit->ammo[ av.weapnum ] - av.weapcount );
+   b->execute ( context );
+   
+   _attackingunit->postAttack( reactionfire, context );
+   
+   GameAction* c = new RegisterUnitRFTarget( map, nwid, av.weapnum, _attackedunit->networkid  );
+   c->execute ( context );
+   
+   if ( _respond ) {
+      GameAction* d = new ChangeUnitProperty( _attackedunit, ChangeUnitProperty::Experience, dv.experience );
+      d->execute ( context );
+      
+      GameAction* e = new ConsumeAmmo( _attackedunit, _attackedunit->typ->weapons.weapon[dv.weapnum].getScalarWeaponType(), dv.weapnum, _attackedunit->ammo[ dv.weapnum ] - dv.weapcount );
+      e->execute ( context );
+   }
+   
+   GameAction* f = new InflictDamage( _attackingunit, av.damage - _attackingunit->damage );
+   f->execute ( context );
+
+   GameAction* g = new InflictDamage( _attackedunit, dv.damage - _attackedunit->damage  );
+   g->execute ( context );
+   
+   
+   /* If the attacking vehicle was destroyed, remove it */
+   if ( av.damage >= 100 ) {
+     *_pattackingunit = NULL;
+   }
+
+   /* If the attacked vehicle was destroyed, remove it */
+   if ( dv.damage >= 100 ) {
+     *_pattackedunit = NULL;
+   }
 }
 
 
@@ -403,8 +453,8 @@ void tunitattacksunit :: setresult ( void )
 
 
 
-
 tunitattacksbuilding :: tunitattacksbuilding ( Vehicle* attackingunit, int x, int y, int weapon )
+   : UnitAttacksSomething( attackingunit->getMap() )
 {
    setup ( attackingunit, x, y, weapon );
 }
@@ -415,13 +465,13 @@ void tunitattacksbuilding :: setup ( Vehicle* attackingunit, int x, int y, int w
    _attackingunit = attackingunit;
    _x = x;
    _y = y;
-   _attackedbuilding  = getfield ( x, y ) -> building;
+   _attackedbuilding  = attackingunit->getMap()->getField ( x, y ) -> building;
 
    dist = beeline ( attackingunit->xpos, attackingunit->ypos, x, y );
    int _weapon;
 
    if ( weapon == -1 ) {
-      pattackweap atw = attackpossible( attackingunit, x, y );
+      AttackWeap* atw = attackpossible( attackingunit, x, y );
       int n = -1;
       int s = 0;
       for ( int i = 0; i < atw->count; i++ )
@@ -438,10 +488,10 @@ void tunitattacksbuilding :: setup ( Vehicle* attackingunit, int x, int y, int w
 
    const SingleWeapon *weap = &attackingunit->typ->weapons.weapon[_weapon];
 
-   int targetWeather = getfield(x,y)->getweather();
+   int targetWeather = attackingunit->getMap()->getField(x,y)->getWeather();
 
    av.strength  = int (ceil( attackingunit->weapstrength[_weapon]
-                        * WeapDist::getWeaponStrength(weap, targetWeather, dist, attackingunit->height, _attackedbuilding->typ->buildingheight )
+                        * WeapDist::getWeaponStrength(weap, targetWeather, dist, attackingunit->height, _attackedbuilding->typ->height )
                         * attackingunit->typ->weapons.weapon[_weapon].targetingAccuracy[cmm_building] / 100 ));
 
    av.armor = attackingunit->getArmor();
@@ -456,7 +506,7 @@ void tunitattacksbuilding :: setup ( Vehicle* attackingunit, int x, int y, int w
    av.kamikaze   = attackingunit->typ->hasFunction( ContainerBaseType::KamikazeOnly  );
    av.height = attackingunit->height;
 
-   tfield* field = getfield ( attackingunit->xpos, attackingunit->ypos );
+   tfield* field = attackingunit->getMap()->getField ( attackingunit->xpos, attackingunit->ypos );
 
    if ( attackingunit->height <= chfahrend ) {
       av.defensebonus = field->getdefensebonus();
@@ -484,7 +534,7 @@ void tunitattacksbuilding :: setup ( Vehicle* attackingunit, int x, int y, int w
    dv.color      = _attackedbuilding->getOwner();
    dv.initiative = 0;
    dv.kamikaze = 0;
-   dv.height = _attackedbuilding->typ->buildingheight;
+   dv.height = _attackedbuilding->typ->height;
 }
 
 
@@ -511,18 +561,39 @@ void tunitattacksbuilding :: setresult ( void )
      _attackedbuilding = NULL;
    }
 
-   actmap->time.set ( actmap->time.turn(), actmap->time.move()+1);
+   gamemap->time.set ( gamemap->time.turn(), gamemap->time.move()+1);
+}
+
+void tunitattacksbuilding :: setresult( const Context& context )
+{
+   MapCoordinate target = _attackedbuilding->getPosition();
+   
+   GameAction* b = new ConsumeAmmo( _attackingunit, _attackingunit->typ->weapons.weapon[av.weapnum].getScalarWeaponType(), av.weapnum, _attackingunit->ammo[ av.weapnum ] - av.weapcount );
+   b->execute ( context );
+   
+   _attackingunit->postAttack( false, context );
+   
+   GameAction* f = new InflictDamage( _attackingunit, av.damage - _attackingunit->damage );
+   f->execute ( context );
+   
+   GameAction* g = new InflictDamage( _attackedbuilding, dv.damage - _attackedbuilding->damage  );
+   g->execute ( context );
 }
 
 
 
-tmineattacksunit :: tmineattacksunit ( tfield* mineposition, int minenum, Vehicle* &attackedunit )
+tmineattacksunit :: tmineattacksunit ( const MapCoordinate& mineposition, int minenum, Vehicle* &attackedunit )
+   : tfight( attackedunit->getMap() )
 {
    setup ( mineposition, minenum, attackedunit );
 }
 
-void tmineattacksunit :: setup ( tfield* mineposition, int minenum, Vehicle* &attackedunit )
+void tmineattacksunit :: setup ( const MapCoordinate& position, int minenum, Vehicle* &attackedunit )
 {
+   this->position = position;
+   
+   tfield* mineposition = attackedunit->getMap()->getField( position );
+   
    if ( mineposition->mines.empty() )
       errorMessage(" tmineattacksunit :: setup \n no mine to attack !\n" );
 
@@ -604,11 +675,31 @@ void tmineattacksunit :: setresult ( void )
    _attackedunit->damage = dv.damage;
 
    /* Remove the mined vehicle if it was destroyed */
-   if ( _attackedunit->damage >= 100 ) {
-     delete *_pattackedunit;
+   if ( dv.damage >= 100 ) 
      *_pattackedunit = NULL;
+
+}
+
+void tmineattacksunit :: setresult( const Context& context )
+{
+   vector<GameAction*> actions;
+   if ( _minenum == -1 ) {
+      for ( tfield::MineContainer::iterator m = _mineposition->mines.begin(); m != _mineposition->mines.end(); ++m)
+         if ( m->attacksunit ( _attackedunit ))
+            actions.push_back ( new RemoveMine(_attackedunit->getMap(), position, m->identifier));
+   } else {
+      int counter = 0;
+      for ( tfield::MineContainer::iterator m = _mineposition->mines.begin(); m != _mineposition->mines.end(); ++m, ++counter)
+         if ( counter == _minenum )
+            actions.push_back ( new RemoveMine(_attackedunit->getMap(), position, m->identifier));
    }
 
+   for ( vector<GameAction*>::iterator i = actions.begin(); i != actions.end(); ++i )
+      (*i)->execute( context );
+      
+   
+   (new InflictDamage( _attackedunit, dv.damage - _attackedunit->damage  ))->execute ( context );
+ 
 }
 
 
@@ -623,6 +714,7 @@ Mine* tmineattacksunit :: getFirstMine()
 
 
 tunitattacksobject :: tunitattacksobject ( Vehicle* attackingunit, int obj_x, int obj_y, int weapon )
+   : UnitAttacksSomething( attackingunit->getMap() )
 {
    setup ( attackingunit, obj_x, obj_y, weapon );
 }
@@ -633,7 +725,7 @@ void tunitattacksobject :: setup ( Vehicle* attackingunit, int obj_x, int obj_y,
    _x = obj_x;
    _y = obj_y;
 
-   targetField = getfield ( obj_x, obj_y );
+   targetField = attackingunit->getMap()->getField ( obj_x, obj_y );
 
    _attackingunit = attackingunit;
 
@@ -649,7 +741,7 @@ void tunitattacksobject :: setup ( Vehicle* attackingunit, int obj_x, int obj_y,
 
    if ( weapon == -1 ) {
 
-      pattackweap atw = attackpossible( attackingunit, obj_x, obj_y );
+      AttackWeap* atw = attackpossible( attackingunit, obj_x, obj_y );
       int n = -1;
       int s = 0;
       for ( int i = 0; i < atw->count; i++ )
@@ -667,7 +759,7 @@ void tunitattacksobject :: setup ( Vehicle* attackingunit, int obj_x, int obj_y,
 
    const SingleWeapon *weap = &attackingunit->typ->weapons.weapon[weapon];
    av.strength  = int ( ceil( attackingunit->weapstrength[weapon]
-                        * WeapDist::getWeaponStrength(weap, targetField->getweather(), dist, attackingunit->height, _obji->typ->getEffectiveHeight() )
+                        * WeapDist::getWeaponStrength(weap, targetField->getWeather(), dist, attackingunit->height, _obji->typ->getEffectiveHeight() )
                         * attackingunit->typ->weapons.weapon[_weapon].targetingAccuracy[cmm_building] / 100 ));
 
    av.armor = attackingunit->getArmor();
@@ -679,7 +771,7 @@ void tunitattacksobject :: setup ( Vehicle* attackingunit, int obj_x, int obj_y,
    av.height = attackingunit->height;
    av.weapontype = attackingunit->typ->weapons.weapon[ _weapon ].getScalarWeaponType();
 
-   tfield* field2 = getfield ( attackingunit->xpos, attackingunit->ypos );
+   tfield* field2 = attackingunit->getMap()->getField ( attackingunit->xpos, attackingunit->ypos );
 
    if ( attackingunit->height <= chfahrend ) {
       av.defensebonus = field2->getdefensebonus();
@@ -720,7 +812,7 @@ void tunitattacksobject :: setresult ( void )
 
    /* Remove the object if it was destroyed */
    if ( _obji->damage >= 100 ) {
-     getfield ( _x, _y )-> removeobject ( _obji->typ );
+      _attackingunit->getMap()->getField ( _x, _y )-> removeObject ( _obji->typ );
    }
 
    /* Remove the attacking unit if it was destroyed */
@@ -730,53 +822,74 @@ void tunitattacksobject :: setresult ( void )
    }
 
 
-   actmap->time.set ( actmap->time.turn(), actmap->time.move()+1);
+   gamemap->time.set ( gamemap->time.turn(), gamemap->time.move()+1);
 
 }
 
+void tunitattacksobject :: setresult( const Context& context )
+{
+   GameMap* map = _attackingunit->getMap();
+   
+   GameAction* b = new ConsumeAmmo( _attackingunit, _attackingunit->typ->weapons.weapon[av.weapnum].getScalarWeaponType(), av.weapnum, _attackingunit->ammo[ av.weapnum ] - av.weapcount );
+   b->execute ( context );
+   
+   _attackingunit->postAttack( false, context );
+   
+   GameAction* f = new InflictDamage( _attackingunit, av.damage - _attackingunit->damage );
+   f->execute ( context );
+   
+   
+   MapCoordinate position( _x, _y );
+   
+   if ( dv.damage >= 100 ) {
+      (new RemoveObject(map, position, _obji->typ->id))->execute(context);
+   } else {
+      GameAction* g = new ChangeObjectProperty( map, position, _obji, ChangeObjectProperty::Damage, dv.damage );
+      g->execute ( context );
+   }
+}
+
 
 
 
 
-
-pattackweap  attackpossible( const Vehicle*     angreifer, int x, int y)
+AttackWeap*  attackpossible( const Vehicle*     attacker, int x, int y)
 {
-  pattackweap atw = new AttackWeap;
+  AttackWeap* atw = new AttackWeap;
 
   memset(atw, 0, sizeof(*atw));
 
 
-   if ((x < 0) || (y < 0) || (x >= actmap->xsize) || (y >= actmap->ysize))
+   if ((x < 0) || (y < 0) || (x >= attacker->getMap()->xsize) || (y >= attacker->getMap()->ysize))
       return atw;
-   if (angreifer == NULL)
+   if (attacker == NULL)
       return atw;
-   if (angreifer->typ->weapons.count == 0)
+   if (attacker->typ->weapons.count == 0)
       return atw;
 
-   tfield* efield = getfield(x,y);
+   tfield* efield = attacker->getMap()->getField(x,y);
 
-   if ( efield->vehicle ) {
-      if (fieldvisiblenow(efield, angreifer->color/8))
-         attackpossible2n ( angreifer, efield->vehicle, atw );
+   if ( efield->getVehicle() ) {
+      if (fieldvisiblenow(efield, attacker->color/8))
+         attackpossible2n ( attacker, efield->getVehicle(), atw );
    }
-   else
-      if (efield->building != NULL) {
-         if ( actmap->getPlayer(angreifer).diplomacy.isHostile( efield->building->getOwner() ) || efield->building->color == 8*8 )
-            for (int i = 0; i < angreifer->typ->weapons.count ; i++)
-               if (angreifer->typ->weapons.weapon[i].shootable() )
-                  if (angreifer->typ->weapons.weapon[i].offensive() )
-                     if ( angreifer->typ->weapons.weapon[i].targetingAccuracy[cmm_building] > 0 ) {
-                        int tm = efield->building->typ->buildingheight;
-                        if (tm & angreifer->typ->weapons.weapon[i].targ) {
-                           if (fieldvisiblenow(efield, angreifer->color/8)) {
-                              int d = beeline(angreifer->xpos,angreifer->ypos,x,y);
-                              if (d <= angreifer->typ->weapons.weapon[i].maxdistance)
-                                 if (d >= angreifer->typ->weapons.weapon[i].mindistance) {
-                                    if (angreifer->height & angreifer->typ->weapons.weapon[i].sourceheight)
-                                       if ( angreifer->typ->weapons.weapon[i].efficiency[6 + getheightdelta ( log2( angreifer->height), log2(tm))] )
-                                          if (angreifer->ammo[i] > 0) {
-                                             atw->strength[atw->count ] = angreifer->weapstrength[i];
-                                             atw->typ[atw->count ] = 1 << angreifer->typ->weapons.weapon[i].getScalarWeaponType() ;
+   else if (efield->building != NULL) {
+         if ( attacker->getMap()->getPlayer(attacker).diplomacy.isHostile( efield->building->getOwner() ) || efield->building->color == 8*8 )
+            for (int i = 0; i < attacker->typ->weapons.count ; i++)
+               if (attacker->typ->weapons.weapon[i].shootable() )
+                  if (attacker->typ->weapons.weapon[i].offensive() )
+                     if ( attacker->typ->weapons.weapon[i].targetingAccuracy[cmm_building] > 0 ) {
+                        int tm = efield->building->typ->height;
+                        if (tm & attacker->typ->weapons.weapon[i].targ) {
+                           if (fieldvisiblenow(efield, attacker->color/8)) {
+                              int d = beeline(attacker->xpos,attacker->ypos,x,y);
+                              if (d <= attacker->typ->weapons.weapon[i].maxdistance)
+                                 if (d >= attacker->typ->weapons.weapon[i].mindistance) {
+                                    if (attacker->height & attacker->typ->weapons.weapon[i].sourceheight)
+                                       if ( attacker->typ->weapons.weapon[i].efficiency[6 + getheightdelta ( log2( attacker->height), log2(tm))] )
+                                          if (attacker->ammo[i] > 0) {
+                                             atw->strength[atw->count ] = attacker->weapstrength[i];
+                                             atw->typ[atw->count ] = 1 << attacker->typ->weapons.weapon[i].getScalarWeaponType() ;
                                              atw->num[atw->count ] = i;
                                              atw->target = AttackWeap::building;
                                              atw->count++;
@@ -786,9 +899,7 @@ pattackweap  attackpossible( const Vehicle*     angreifer, int x, int y)
                            }
                         }
                      }
-      }
-
-   if ( efield->objects.size() ) {
+   } else if ( efield->objects.size() ) {
       int n = 0;
       for ( tfield::ObjectContainer::iterator j = efield->objects.begin(); j != efield->objects.end(); j++ )
          if ( j->typ->armor > 0 )
@@ -798,26 +909,27 @@ pattackweap  attackpossible( const Vehicle*     angreifer, int x, int y)
          if ((efield->vehicle == NULL) && ( efield->building == NULL)) {
             bool found = false;
             for ( tfield::ObjectContainer::reverse_iterator j = efield->objects.rbegin(); j != efield->objects.rend(); ++j ) {
-               for ( int i = 0; i <= angreifer->typ->weapons.count - 1; i++)
-                  if (angreifer->typ->weapons.weapon[i].shootable() )
-                     if ( angreifer->typ->weapons.weapon[i].getScalarWeaponType() == cwcannonn ||
-                          angreifer->typ->weapons.weapon[i].getScalarWeaponType() == cwlasern ||
-                          angreifer->typ->weapons.weapon[i].getScalarWeaponType() == cwcruisemissile ||
-                          angreifer->typ->weapons.weapon[i].getScalarWeaponType() == cwlargemissilen ||
-                          angreifer->typ->weapons.weapon[i].getScalarWeaponType() == cwsmallmissilen ||
-                          angreifer->typ->weapons.weapon[i].getScalarWeaponType() == cwbombn ) {
-                        if ( angreifer->typ->weapons.weapon[i].targetingAccuracy[cmm_building] )
-                              if (fieldvisiblenow(efield, angreifer->color/8)) {
-                                 int d = beeline(angreifer->xpos,angreifer->ypos,x,y);
-                                 if (d <= angreifer->typ->weapons.weapon[i].maxdistance)
-                                    if (d >= angreifer->typ->weapons.weapon[i].mindistance) {
-                                       if (angreifer->height & angreifer->typ->weapons.weapon[i].sourceheight )
-                                          if ( angreifer->typ->weapons.weapon[i].targ & j->typ->getEffectiveHeight() )
-                                             if ( angreifer->typ->weapons.weapon[i].efficiency[6 + getheightdelta ( log2( angreifer->height), log2(j->typ->getEffectiveHeight()))] )
-                                                if (angreifer->ammo[i] > 0) {
-                                                   atw->strength[atw->count ] = angreifer->weapstrength[i];
+               for ( int i = 0; i <= attacker->typ->weapons.count - 1; i++)
+                  if (attacker->typ->weapons.weapon[i].shootable() )
+                     if ( attacker->typ->weapons.weapon[i].getScalarWeaponType() == cwcannonn ||
+                          attacker->typ->weapons.weapon[i].getScalarWeaponType() == cwlasern ||
+                          attacker->typ->weapons.weapon[i].getScalarWeaponType() == cwcruisemissile ||
+                          attacker->typ->weapons.weapon[i].getScalarWeaponType() == cwlargemissilen ||
+                          attacker->typ->weapons.weapon[i].getScalarWeaponType() == cwsmallmissilen ||
+                          attacker->typ->weapons.weapon[i].getScalarWeaponType() == cwtorpedon ||
+                          attacker->typ->weapons.weapon[i].getScalarWeaponType() == cwbombn ) {
+                        if ( attacker->typ->weapons.weapon[i].targetingAccuracy[cmm_building] )
+                              if (fieldvisiblenow(efield, attacker->color/8)) {
+                                 int d = beeline(attacker->xpos,attacker->ypos,x,y);
+                                 if (d <= attacker->typ->weapons.weapon[i].maxdistance)
+                                    if (d >= attacker->typ->weapons.weapon[i].mindistance) {
+                                       if (attacker->height & attacker->typ->weapons.weapon[i].sourceheight )
+                                          if ( attacker->typ->weapons.weapon[i].targ & j->typ->getEffectiveHeight() )
+                                             if ( attacker->typ->weapons.weapon[i].efficiency[6 + getheightdelta ( log2( attacker->height), log2(j->typ->getEffectiveHeight()))] )
+                                                if (attacker->ammo[i] > 0) {
+                                                   atw->strength[atw->count ] = attacker->weapstrength[i];
                                                    atw->num[atw->count ] = i;
-                                                   atw->typ[atw->count ] = 1 << angreifer->typ->weapons.weapon[i].getScalarWeaponType();
+                                                   atw->typ[atw->count ] = 1 << attacker->typ->weapons.weapon[i].getScalarWeaponType();
                                                    atw->target = AttackWeap::object;
                                                    atw->count++;
                                                    found = true;
@@ -838,42 +950,40 @@ pattackweap  attackpossible( const Vehicle*     angreifer, int x, int y)
 }
 
 
-bool attackpossible2u( const Vehicle* attacker, const Vehicle* target, pattackweap atw, int targetheight )
+bool attackpossible2u( const Vehicle* attacker, const Vehicle* target, AttackWeap* atw, int targetheight )
 {
    if ( targetheight == -1 )
       targetheight = target->height;
 
-   const Vehicle* angreifer = attacker;
-   const Vehicle* verteidiger = target;
    int result = false;
    if ( atw )
       atw->count = 0;
 
-   if ( !angreifer )
+   if ( !attacker )
      return false ;
 
-   if ( !verteidiger )
+   if ( !target )
      return false ;
 
-   if (angreifer->typ->weapons.count == 0)
+   if (attacker->typ->weapons.count == 0)
      return false ;
 
-   if ( actmap->player[angreifer->getOwner()].diplomacy.isHostile( verteidiger->getOwner() )  )
-      for ( int i = 0; i < angreifer->typ->weapons.count ; i++)
+   if ( attacker->getMap()->player[attacker->getOwner()].diplomacy.isHostile( target->getOwner() )  )
+      for ( int i = 0; i < attacker->typ->weapons.count ; i++)
          for ( int h = 0; h < 8; h++ )
             if ( targetheight & (1<<h))
-               if (angreifer->typ->weapons.weapon[i].shootable() )
-                  if (angreifer->typ->weapons.weapon[i].offensive() )
-                     if ( (1<<h) & angreifer->typ->weapons.weapon[i].targ )
-                        if (angreifer->height & angreifer->typ->weapons.weapon[i].sourceheight )
-                           if ( angreifer->typ->weapons.weapon[i].targetingAccuracy[ verteidiger->typ->movemalustyp] > 0 )
-                              if ( angreifer->typ->weapons.weapon[i].efficiency[6 + getheightdelta ( log2( angreifer->height), h)] )
-                                 if (angreifer->ammo[i] > 0) {
+               if (attacker->typ->weapons.weapon[i].shootable() )
+                  if (attacker->typ->weapons.weapon[i].offensive() )
+                     if ( (1<<h) & attacker->typ->weapons.weapon[i].targ )
+                        if (attacker->height & attacker->typ->weapons.weapon[i].sourceheight )
+                           if ( attacker->typ->weapons.weapon[i].targetingAccuracy[ target->typ->movemalustyp] > 0 )
+                              if ( attacker->typ->weapons.weapon[i].efficiency[6 + getheightdelta ( log2( attacker->height), h)] )
+                                 if (attacker->ammo[i] > 0) {
                                     result = true;
                                     if ( atw ) {
-                                       atw->strength[atw->count] = angreifer->weapstrength[i] * angreifer->typ->weapons.weapon[i].targetingAccuracy[ verteidiger->typ->movemalustyp] / 100;
+                                       atw->strength[atw->count] = attacker->weapstrength[i] * attacker->typ->weapons.weapon[i].targetingAccuracy[ target->typ->movemalustyp] / 100;
                                        atw->num[atw->count ] = i;
-                                       atw->typ[atw->count ] = 1 << angreifer->typ->weapons.weapon[i].getScalarWeaponType();
+                                       atw->typ[atw->count ] = 1 << attacker->typ->weapons.weapon[i].getScalarWeaponType();
                                        atw->target = AttackWeap::vehicle;
                                        atw->count++;
                                     }
@@ -884,43 +994,40 @@ bool attackpossible2u( const Vehicle* attacker, const Vehicle* target, pattackwe
 
 
 
-bool attackpossible28( const Vehicle* attacker, const Vehicle* target, pattackweap atw, int targetHeight )
+bool attackpossible28( const Vehicle* attacker, const Vehicle* target, AttackWeap* atw, int targetHeight )
 {
-   const Vehicle* angreifer = attacker;
-   const Vehicle* verteidiger = target;
-
    if ( targetHeight < 0 )
       targetHeight = target->height;
    
-   int result = false;
+   bool result = false;
    if ( atw )
       atw->count = 0;
 
-   if (angreifer == NULL)
+   if (attacker == NULL)
      return false ;
 
-   if (verteidiger == NULL)
+   if (target == NULL)
      return false ;
 
-   if (angreifer->typ->weapons.count == 0)
+   if (attacker->typ->weapons.count == 0)
      return false ;
 
-//   if ( actmap->player[angreifer->getOwner()].diplomacy.isHostile( verteidiger->getOwner() )  )
-      for ( int i = 0; i < angreifer->typ->weapons.count ; i++)
-         if (angreifer->typ->weapons.weapon[i].shootable() )
-            if (angreifer->typ->weapons.weapon[i].offensive() )
-               if (targetHeight & angreifer->typ->weapons.weapon[i].targ )
-                  if (minmalq <= angreifer->typ->weapons.weapon[i].maxdistance)
-                     if (minmalq >= angreifer->typ->weapons.weapon[i].mindistance)
-                        if (angreifer->height & angreifer->typ->weapons.weapon[i].sourceheight )
-                           if ( angreifer->typ->weapons.weapon[i].targetingAccuracy[ verteidiger->typ->movemalustyp ] > 0)
-                              if ( angreifer->typ->weapons.weapon[i].efficiency[6 + getheightdelta ( log2( angreifer->height), log2(targetHeight))] )
-                                 if (angreifer->ammo[i] > 0) {
+//   if ( attacker->getMap()->player[attacker->getOwner()].diplomacy.isHostile( target->getOwner() )  )
+      for ( int i = 0; i < attacker->typ->weapons.count ; i++)
+         if (attacker->typ->weapons.weapon[i].shootable() )
+            if (attacker->typ->weapons.weapon[i].offensive() )
+               if (targetHeight & attacker->typ->weapons.weapon[i].targ )
+                  if (minmalq <= attacker->typ->weapons.weapon[i].maxdistance)
+                     if (minmalq >= attacker->typ->weapons.weapon[i].mindistance)
+                        if (attacker->height & attacker->typ->weapons.weapon[i].sourceheight )
+                           if ( attacker->typ->weapons.weapon[i].targetingAccuracy[ target->typ->movemalustyp ] > 0)
+                              if ( attacker->typ->weapons.weapon[i].efficiency[6 + getheightdelta ( log2( attacker->height), log2(targetHeight))] )
+                                 if (attacker->ammo[i] > 0) {
                                     result =  true;
                                     if ( atw ) {
-                                       atw->strength[atw->count] = angreifer->weapstrength[i] * angreifer->typ->weapons.weapon[i].targetingAccuracy[ verteidiger->typ->movemalustyp] / 100;
+                                       atw->strength[atw->count] = attacker->weapstrength[i] * attacker->typ->weapons.weapon[i].targetingAccuracy[ target->typ->movemalustyp] / 100;
                                        atw->num[atw->count ] = i;
-                                       atw->typ[atw->count ] = 1 << angreifer->typ->weapons.weapon[i].getScalarWeaponType();
+                                       atw->typ[atw->count ] = 1 << attacker->typ->weapons.weapon[i].getScalarWeaponType();
                                        atw->target = AttackWeap::vehicle;
                                        atw->count++;
                                     }
@@ -930,43 +1037,40 @@ bool attackpossible28( const Vehicle* attacker, const Vehicle* target, pattackwe
 }
 
 
-bool attackpossible2n( const Vehicle* attacker, const Vehicle* target, pattackweap atw )
+bool attackpossible2n( const Vehicle* attacker, const Vehicle* target, AttackWeap* atw )
 {
-   const Vehicle* angreifer = attacker;
-   const Vehicle* verteidiger = target;
-
    int result = false;
    if ( atw )
       atw->count = 0;
 
-   if (angreifer == NULL)
+   if (attacker == NULL)
      return false ;
 
-   if (verteidiger == NULL)
+   if (target == NULL)
      return false ;
 
-   if (angreifer->typ->weapons.count == 0)
+   if (attacker->typ->weapons.count == 0)
      return false ;
 
-   int dist = beeline ( angreifer, verteidiger );
-   if ( actmap->player[angreifer->getOwner()].diplomacy.isHostile( verteidiger->getOwner() ) )
-      if ( !angreifer->attacked )
-         if ( !angreifer->typ->wait || !angreifer->hasMoved() || angreifer->reactionfire.getStatus() == Vehicle::ReactionFire::ready)
-            for ( int i = 0; i < angreifer->typ->weapons.count ; i++)
-               if (angreifer->typ->weapons.weapon[i].shootable() )
-                  if (angreifer->typ->weapons.weapon[i].offensive() )
-                     if (verteidiger->height & angreifer->typ->weapons.weapon[i].targ )
-                        if (dist <= angreifer->typ->weapons.weapon[i].maxdistance)
-                           if (dist >= angreifer->typ->weapons.weapon[i].mindistance)
-                              if (angreifer->height & angreifer->typ->weapons.weapon[i].sourceheight )
-                                 if ( angreifer->typ->weapons.weapon[i].efficiency[6 + getheightdelta ( log2( angreifer->height), log2(verteidiger->height))] )
-                                    if ( angreifer->typ->weapons.weapon[i].targetingAccuracy[ verteidiger->typ->movemalustyp ] > 0)
-                                       if (angreifer->ammo[i] > 0) {
+   int dist = beeline ( attacker, target );
+   if ( attacker->getMap()->player[attacker->getOwner()].diplomacy.isHostile( target->getOwner() ) )
+      if ( !attacker->attacked )
+         if ( !attacker->typ->wait || !attacker->hasMoved() || attacker->reactionfire.getStatus() == Vehicle::ReactionFire::ready)
+            for ( int i = 0; i < attacker->typ->weapons.count ; i++)
+               if (attacker->typ->weapons.weapon[i].shootable() )
+                  if (attacker->typ->weapons.weapon[i].offensive() )
+                     if (target->height & attacker->typ->weapons.weapon[i].targ )
+                        if (dist <= attacker->typ->weapons.weapon[i].maxdistance)
+                           if (dist >= attacker->typ->weapons.weapon[i].mindistance)
+                              if (attacker->height & attacker->typ->weapons.weapon[i].sourceheight )
+                                 if ( attacker->typ->weapons.weapon[i].efficiency[6 + getheightdelta ( log2( attacker->height), log2(target->height))] )
+                                    if ( attacker->typ->weapons.weapon[i].targetingAccuracy[ target->typ->movemalustyp ] > 0)
+                                       if (attacker->ammo[i] > 0) {
                                           result = true;
                                           if ( atw ) {
-                                             atw->strength[atw->count] = angreifer->weapstrength[i] * angreifer->typ->weapons.weapon[i].targetingAccuracy[ verteidiger->typ->movemalustyp ] / 100;
+                                             atw->strength[atw->count] = attacker->weapstrength[i] * attacker->typ->weapons.weapon[i].targetingAccuracy[ target->typ->movemalustyp ] / 100;
                                              atw->num[atw->count ] = i;
-                                             atw->typ[atw->count ] = 1 << angreifer->typ->weapons.weapon[i].getScalarWeaponType();
+                                             atw->typ[atw->count ] = 1 << attacker->typ->weapons.weapon[i].getScalarWeaponType();
                                              atw->target = AttackWeap::vehicle;
                                              atw->count++;
                                           }
@@ -1033,7 +1137,7 @@ float WeapDist::getWeaponStrength ( const SingleWeapon* weap, int weather, int d
 
    if ( reldiff == -1) {
       if ( dist < weap->mindistance || dist > weap->maxdistance ) {
-         warning("tweapdist::getweapstrength: invalid range: \n min = " + ASCString::toString(weap->mindistance ) + " ; max = " + ASCString::toString( weap->maxdistance ) + " ; req = " + ASCString::toString( dist));
+         warningMessage("tweapdist::getweapstrength: invalid range: \n min = " + ASCString::toString(weap->mindistance ) + " ; max = " + ASCString::toString( weap->maxdistance ) + " ; req = " + ASCString::toString( dist));
          return 0;
       }
 
@@ -1065,10 +1169,10 @@ float WeapDist::getWeaponStrength ( const SingleWeapon* weap, int weather, int d
 
       if ( attacker_height >= chtieffliegend && weather != 0 && defender_height != -1) {
          int weatherRelevantHeightDelta = min( abs( getheightdelta ( log2 ( attacker_height ), log2 ( defender_height ))), 3);
-         if ( weather == 1 || weather == 3 || weather == 5 )
+         if ( weather == 1 || weather == 3  )
             weatherFactor = 1 - 0.07*weatherRelevantHeightDelta;
          else
-            if ( weather == 2 || weather == 4 )
+            if ( weather == 2 || weather == 4 || weather == 5 )
                weatherFactor = 1 - 0.2 * weatherRelevantHeightDelta;
       }
 
