@@ -27,8 +27,8 @@
 #include <math.h>
 #include "global.h"
 #include "basegfx.h"
-#include "newfont.h"
 #include "misc.h"
+#include "newfont.h"
 
 #ifdef _DOS_
  #include "dos/vesa.h"
@@ -848,167 +848,6 @@ dacpalette256 ppal;
 
 
 
-TrueColorImage* zoomimage ( void* buf, int xsize, int ysize, dacpalette256 pal, int interpolate, int proportional )
-{
-   if ( !newpalgenerated ) 
-      for ( int i = 0; i < 256; i++ )
-         for ( int j = 0; j<3; j++ )
-            ppal[i][j] = pal[i][j] << 2;
-
-
-
-   int orgx;
-   int orgy;
-   getpicsize ( buf, orgx, orgy );
-
-   {
-      float ratiox = (float)xsize / (float)orgx;
-      float ratioy = (float)ysize / (float)orgy;
-      if ( proportional ) {
-         if ( ratiox > ratioy )
-            xsize = (int) (ratioy * orgx);
-         else
-            ysize = (int) (ratiox * orgy);
-      }
-   }
-
-   TrueColorImage* img = new TrueColorImage ( xsize, ysize );
-   
-
-   float dx = (float)orgx / (float)xsize;
-   float dy = (float)orgy / (float)ysize;
-
-   float oy = 0;
-   for ( int y = 0; y < ysize; y++, oy+=dy ) {
-
-      int oyi = f2i ( oy );
-      float oyif = oyi;
-
-      float ox = 0;
-      for ( int x = 0; x < xsize; x++,ox+=dx ) {
-
-         float r = 0;
-         float g = 0;
-         float b = 0;
-         float alpha = 0;
-
-         if ( interpolate ) {
-            float d;
-
-            int oxi = f2i ( ox );
-            float oxif = oxi;
-   
-            float dist = 0;
-            int alphanum = 0;
-
-            // pixel links oben
-            int px1 = getpixelfromimage ( buf, oxi, oyi );
-            int px2 = getpixelfromimage ( buf, oxi+1, oyi );
-            int px3 = getpixelfromimage ( buf, oxi, oyi+1 );
-            int px4 = getpixelfromimage ( buf, oxi+1, oyi+1 );
-            if ( px1 == 255 && px2 == 255 && px3 == 255 && px4 == 255 ) {
-                r = 0;
-                g = 0;
-                b = 0;
-                alpha = alphabase;
-            } else {
-
-               if ( px1 >= 0 ) {
-                  d = ( 1-(ox - oxif)) * ( 1-( oy - oyif ));
-      
-                  if ( px1 == 255 ) {
-                     alpha += ((float)alphabase) * d;
-                     alphanum++;
-                  }
-                  else {
-                     dist += d;
-                     r += ppal[px1][0] * d ;
-                     g += ppal[px1][1] * d ;
-                     b += ppal[px1][2] * d ;
-                  }
-               }
-                  
-               // pixel rechts oben
-               if ( px2 >= 0 ) {
-                  d = (ox - oxif) * ( 1 -  ( oy - oyif ));
-      
-                  if ( px2 == 255 ) {
-                     alpha += ((float)alphabase) * d;
-                     alphanum++;
-                  }
-                  else {
-                     dist += d;
-                     r += ppal[px2][0] * d ;
-                     g += ppal[px2][1] * d ;
-                     b += ppal[px2][2] * d ;
-                  }
-               }
-   
-   
-               // pixel links unten
-               if ( px3 >= 0 ) {
-                  d = ( 1- (ox - oxif)) *  (oy - oyif);
-      
-                  if ( px3 == 255 ) {
-                     alpha += ((float)alphabase) * d;
-                     alphanum++;
-                  }
-                  else {
-                     dist += d;
-                     r += ppal[px3][0] * d ;
-                     g += ppal[px3][1] * d ;
-                     b += ppal[px3][2] * d ;
-                  }
-               }
-   
-               // pixel rechts unten
-               if ( px4 >= 0 ) {
-                  d = (ox - oxif) *  (oy - oyif);
-      
-                  if ( px4 == 255 ) {
-                     alpha += ((float)alphabase) * d;
-                     alphanum++;
-                  }
-                  else {
-                     dist += d;
-                     r += ppal[px4][0] * d ;
-                     g += ppal[px4][1] * d ;
-                     b += ppal[px4][2] * d ;
-                  }
-               }
-   
-               if ( alphanum <= 2 ) 
-                  alpha = 0;
-   
-               if ( dist ) {
-                  r /= dist;
-                  g /= dist;
-                  b /= dist;
-                  alpha /= dist;
-               } else {
-                  r = 0;
-                  g = 0;
-                  b = 0;
-                  alpha = alphabase;
-               }
-            }
-         } else {
-             int px = getpixelfromimage ( buf, f2i2(ox-0.01), f2i2(oy-0.01) );
-             if ( px == 255 ) 
-                alpha += alphabase ;
-             else {
-                r += ppal[px][0] ;
-                g += ppal[px][1] ;
-                b += ppal[px][2] ;
-             }
-
-         }
-         img->setpix ( x, y, f2i(r), f2i(g), f2i(b), f2i(alpha) );
-      }
-   }
-   
-   return img;
-}
 
 
 TrueColorImage* smoothimage ( TrueColorImage* src )
@@ -1053,16 +892,6 @@ TrueColorImage* smoothimage ( TrueColorImage* src )
 #endif
 
 
-TrueColorImage* convertimage2tc ( void* buf, dacpalette256 pal )
-{
-   if ( !buf ) 
-      return NULL;
-
-   int x, y;
-   getpicsize ( buf, x, y );
-   return zoomimage ( buf, x,y , pal, 0, 0 );
-}
-
 int
 imagesize(int x1, int y1, int x2, int y2)
 {
@@ -1178,7 +1007,7 @@ void* uncompress_rlepict ( void* pict )
       getpicsize( pict, w, h );
       w++;
       h++;
-      void* newbuf = asc_malloc ( w * h + 4 );
+      void* newbuf = malloc ( w * h + 4 );
       Uint16* wp = (Uint16*) newbuf;
       char* dest = (char*) newbuf;
       wp[0] = w-1;
@@ -1258,7 +1087,7 @@ tinitgfxengine::tinitgfxengine ( void )
    hgmp = (tgraphmodeparameters *) & hardwaregraphmodeparameters;
    agmp = (tgraphmodeparameters *) & activegraphmodeparameters;
    activepalette256 = (dacpalette256*) &activepalette;
-   xlatbuffer = asc_malloc ( xlatbuffersize );
+   xlatbuffer = malloc ( xlatbuffersize );
 }
 
 
@@ -1266,7 +1095,7 @@ void tvirtualdisplay :: init ( int x, int y, int color, int depth )
 {
    agmp = (tgraphmodeparameters *) & activegraphmodeparameters;
    oldparams = *agmp ;
-   char* cbuf = (char*) asc_malloc ( x * y * depth/8 );
+   char* cbuf = (char*) malloc ( x * y * depth/8 );
    if ( !cbuf )
       throw fatalgraphicserror ( "could not allocate memory !");
 
@@ -1318,7 +1147,7 @@ tvirtualdisplay :: tvirtualdisplay ( int x, int y, int color, int depth )
 tvirtualdisplay :: ~tvirtualdisplay ( )
 {
    delete agmp->surface;
-   asc_free (  buf ) ;
+   free (  buf ) ;
    *agmp = oldparams;
 }
 
