@@ -31,12 +31,10 @@
 #include <stdio.h>
 #include <time.h>
 #include <vector>
+#include <list>
 #include <queue>
 
-#ifdef _SDL_
- #include <SDL.h>
-#endif
-
+#include <SDL.h>
 
 #include "global.h"
 #include "basestreaminterface.h"
@@ -46,16 +44,8 @@
 #include "simplestream.h"
 
 
-#ifdef USE_SYSTEM_BZ2
- #include <bzlib.h>
-#else
- extern "C" {
-  #include "libs/bzlib/bzlib.h"
- }
-#endif
 
-#ifdef _SDL_
- extern SDL_RWops *SDL_RWFromStream( pnstream stream );
+extern SDL_RWops *SDL_RWFromStream( pnstream stream );
 
 class RWOPS_Handler {
       SDL_RWops *rwo;
@@ -66,7 +56,6 @@ class RWOPS_Handler {
       ~RWOPS_Handler() { Close(); };
 };
 
-#endif
 
 
 
@@ -124,7 +113,10 @@ class tinvalidversion : public tfileerror {
 ////////////        dynamic data structures
 ///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
 
-
+/** DEPRECATED
+    Use containers of the C++ Standard Template Library instead!    
+    Container for managing objects of type T
+*/    
 template<class T>
 class dynamic_array {
            protected:
@@ -143,17 +135,6 @@ class dynamic_array {
               void reset ( void );
 };
 
-template<class T>
-class dynamic_initialized_array : public dynamic_array<T> {
-              T initval;
-           protected:
-              virtual void resize ( int newsize );
-
-           public:
-              dynamic_initialized_array ( T ival );
-              dynamic_initialized_array ( T ival, int sze );
-         };
-
 
 ///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
 ////////////        Streams
@@ -337,11 +318,11 @@ class t_compressor_2ndbuf_filter : public t_compressor_stream_interface {
 
 typedef t_compressor_stream_interface *p_compressor_stream_interface;
 
+class PrivateCompressionData;
+
 class libbzip_compression {
-            bz_stream bzs;
-            char* outputbuf;
-            int outputbufsize;
             p_compressor_stream_interface stream;
+            PrivateCompressionData* data;
          public:
              void close_compression ( void );
              void writedata ( const void* buf, int size );
@@ -350,14 +331,11 @@ class libbzip_compression {
 
 };
 
-class libbzip_decompression {
-            bz_stream bzs;
-            char* inputbuf;
-            int inputbufsize;
-            int inputbufused;
-            int inputbufread;
-            p_compressor_stream_interface stream;
+class PrivateDecompressionData;
 
+class libbzip_decompression {
+            p_compressor_stream_interface stream;
+            PrivateDecompressionData* data;
          public:
              int  readdata  ( void* buf, int size, bool excpt = true  );
              libbzip_decompression ( p_compressor_stream_interface strm );
@@ -445,6 +423,7 @@ class tn_c_lzw_filestream : public tnstream, protected tanycompression {
             virtual time_t get_time ( void );
             virtual int getSize( void );
             ASCString getLocation();
+            ASCString getArchive();
      };
 
 typedef tn_c_lzw_filestream tnfilestream ;
@@ -563,6 +542,10 @@ class tfindfile {
         */
         ASCString getnextname ( int* loc = NULL, bool* inContainer = NULL, ASCString* location = NULL );
         bool getnextname ( FileInfo& fi );
+        
+        int getFoundFileNum() {
+           return found;
+        };
 
      };
 
@@ -666,6 +649,19 @@ template<typename BaseType>
    }
 }
 
+template<typename BaseType>
+      void readPointerContainer ( list<BaseType*>& v, tnstream& stream  )
+{
+   stream.readInt(); // version
+   int num = stream.readInt();
+   v.clear();
+   for ( int i = 0; i < num; ++i ) {
+      BaseType* bt = new BaseType;
+      bt->read( stream );
+      v.push_back( bt );
+   }
+}
+
 
 
 template<typename C>
@@ -695,6 +691,22 @@ template<typename C>
    }
 }
 
+template<typename C>
+      void readClassContainerStaticConstructor ( C& c, tnstream& stream  )
+{
+   int version = stream.readInt();
+   if ( version != 1 )
+      throw tinvalidversion( stream.getLocation(), 1, version );
+      
+   int num = stream.readInt();
+   c.clear();
+   for ( int i = 0; i < num; ++i ) {
+      typedef typename C::value_type VT;
+      c.push_back( VT::newFromStream( stream ) );
+   }
+}
+
+
 template<>
       inline void writeClassContainer<> ( const vector<ASCString>& c, tnstream& stream  )
 {
