@@ -30,27 +30,27 @@
 
 class tfield;
 
-//! An actual building on the map, which references a #BuildingType
+/** An actual building on the map, which references a #BuildingType
+    Buildings have an owner,
+
+    Unlike a #Vehicle, a Building can not move, can not shoot and may cover several
+    fields on the map */
 class  Building : public ContainerBase {
     MapCoordinate entryPosition;
 
     char         _completion;
 
     friend class tprocessminingfields;
-  protected:
-
-    //! the percantage that this build has already been repaired this turn. The maximum percentage may be limited by a gameparameter
-    int           repairedThisTurn;
     
    public:
     const BuildingType* typ;
+    
+    const BuildingType* getType() const { return typ; };
+    
 
     //! the ammo that is stored in the building
     int         ammo[waffenanzahl];
 
-    //! the building's name
-    ASCString    name;
-
     //! a bitmapped variable containing the status of the resource-net connection. \see execnetcontrol() \deprecated
     int          netcontrol;
 
@@ -70,7 +70,7 @@ class  Building : public ContainerBase {
     bool canRepair ( const ContainerBase* item ) const;
 
     static Building* newFromStream ( GameMap* gamemap, tnstream& stream, bool chainToField = true );
-    void write ( tnstream& stream, bool includeLoadedUnits = true );
+    void write ( tnstream& stream, bool includeLoadedUnits = true ) const;
     void read ( tnstream& stream );
   private:
     void readData ( tnstream& stream, int version );
@@ -90,7 +90,7 @@ class  Building : public ContainerBase {
     //! the current storage of Resources
     Resources   actstorage;
     
-    int getIdentification();
+    int getIdentification() const;
 
 
     //! returns the picture of the building. It may depend on the current weather of the fields the building is standing on
@@ -105,8 +105,12 @@ class  Building : public ContainerBase {
     
     
     //! changes the building's owner. \param player range: 0 .. 8
-    void convert ( int player );
+    void convert ( int player, bool recursive = true );
 
+    //! this is a low level functions that changes the registration in the map. It's called by convert(int,bool)
+    void registerForNewOwner( int player );
+
+    
     //! Adds the view and jamming of the building to the player's global radar field
     void addview();
 
@@ -167,23 +171,24 @@ class  Building : public ContainerBase {
 
 
     //! returns the bitmapped level of height. Only one bit will be set, of course
-    int getHeight() const { return typ->buildingheight; };
+    int getHeight() const { return typ->height; };
     
     
     ~Building();
 
-    virtual int repairableDamage();
+    virtual int repairableDamage() const;
 
      int getMemoryFootprint() const;
 
   protected:
      ResourceMatrix repairEfficiency;
-     const ResourceMatrix& getRepairEfficiency ( void ) { return repairEfficiency; };
+     const ResourceMatrix& getRepairEfficiency() const { return repairEfficiency; };
      virtual void postRepair ( int oldDamage );
      vector<MapCoordinate> getCoveredFields();
 };
 
 
 extern void doresearch ( GameMap* actmap, int player );
+extern ASCString getBuildingReference ( Building* bld );
 
 #endif
