@@ -69,7 +69,6 @@ BuildingType :: BuildingType ( void )
 {
    for ( int x = 0; x < 4; x++ )
       for ( int y = 0; y < 6; y++ ) {
-         destruction_objects [x][y] = 0;
          field_Exists[x][y] = false;
          for ( int w = 0; w < cwettertypennum; ++w )
             for ( int c = 0; c < maxbuildingpicnum; ++c )
@@ -206,7 +205,7 @@ BuildingType::LocalCoordinate BuildingType::getLocalCoordinate( const MapCoordin
 
 
 
-const int building_version = 11;
+const int building_version = 14;
 
 
 void BuildingType :: read ( tnstream& stream )
@@ -275,7 +274,7 @@ void BuildingType :: read ( tnstream& stream )
       }
 
       technologylevel = stream.readChar( );
-      researchid = stream.readChar( );
+      stream.readChar( ); // was: researchid 
 
       terrainaccess.read ( stream );
 
@@ -303,7 +302,7 @@ void BuildingType :: read ( tnstream& stream )
       }
 
 
-      buildingheight = 1 << log2 ( stream.readInt() );
+      height = 1 << log2 ( stream.readInt() );
       stream.readInt( ); // was: unitheight_forbidden =
       externalloadheight = stream.readInt( );
 
@@ -312,16 +311,18 @@ void BuildingType :: read ( tnstream& stream )
 
 
       if ( version >= 2 ) {
-         for ( int x = 0; x < 4; x++ )
-            for ( int y = 0; y < 6; y++ )
-                destruction_objects[x][y] = stream.readInt( );
+         if ( version <= 11 ) {
+            for ( int x = 0; x < 4; x++ )
+               for ( int y = 0; y < 6; y++ ) {
+                  int id = stream.readInt();
+                  if ( id > 0 )
+                     destructionObjects.insert( make_pair(LocalCoordinate(x,y), id));
+               }
+         }
       } else {
          for ( int w = 0; w < 9; w++ )
-             stream.readInt( );     // dummy
-
-         for ( int x = 0; x < 4; x++ )
-            for ( int y = 0; y < 6; y++ )
-                destruction_objects[x][y] = 0;
+             stream.readInt( ); 
+         destructionObjects.clear();
       }
 
       if ( __loadName )
@@ -353,6 +354,23 @@ void BuildingType :: read ( tnstream& stream )
 
       if ( version >= 11 )
          buildingNotRemovable = stream.readInt();
+      
+      if ( version >= 12 ) {
+         int num = stream.readInt();
+         for ( int i = 0; i < num; ++i ) {
+            int x = stream.readInt();
+            int y = stream.readInt();
+            int id = stream.readInt();
+            destructionObjects.insert( make_pair( LocalCoordinate(x,y),id));
+         }
+      }
+      
+      if ( version >= 13 )
+         for ( int w = 0; w < cwettertypennum; ++w )
+            originalImageFilename[w] = stream.readString();
+      
+      if ( version >= 14 )
+         description = stream.readString();
    } else
       throw tinvalidversion  ( stream.getLocation(), building_version, version );
 }
@@ -399,7 +417,7 @@ void BuildingType :: write ( tnstream& stream ) const
    stream.writeInt ( productionCost.material );
    stream.writeInt ( productionCost.fuel );
    stream.writeChar ( technologylevel );
-   stream.writeChar ( researchid );
+   stream.writeChar ( 0 );
 
    terrainaccess.write ( stream );
 
@@ -425,16 +443,12 @@ void BuildingType :: write ( tnstream& stream ) const
    stream.writeInt ( defaultProduction.material );
    stream.writeInt ( defaultProduction.fuel );
 
-   stream.writeInt ( buildingheight );
+   stream.writeInt ( height );
    stream.writeInt ( 0 );
    stream.writeInt ( externalloadheight );
 
    stream.writeInt ( 0 );
 
-   for ( int x = 0; x < 4; x++ )
-      for ( int y = 0; y < 6; y++ )
-          stream.writeInt ( destruction_objects[x][y] );
-
    if ( !name.empty() )
       stream.writeString ( name );
 
@@ -455,6 +469,18 @@ void BuildingType :: write ( tnstream& stream ) const
 
     stream.writeString ( infotext );
     stream.writeInt( buildingNotRemovable );
+    
+    stream.writeInt( destructionObjects.size());
+    for ( DestructionObjects::const_iterator i = destructionObjects.begin(); i != destructionObjects.end(); ++i) {
+       stream.writeInt( i->first.x);
+       stream.writeInt( i->first.y);
+       stream.writeInt( i->second);
+    }
+    
+    for ( int w = 0; w < cwettertypennum; ++w )
+       stream.writeString( originalImageFilename[w] );
+    
+    stream.writeString( description );
 }
 
 
@@ -574,7 +600,9 @@ void BuildingType :: runTextIO ( PropertyContainer& pc )
                                           s,
                                           SPoint( 500*c + x * fielddistx + (y&1)*fielddisthalfx, y * fielddisty), 
                                           nullParam, nullParam, nullParam, nullParam );
-                  pc.addImage ( weatherTags[w], s, extractFileName_withoutSuffix ( filename )+weatherAbbrev[w]+".png", false  );
+                  ASCString file = extractFileName_withoutSuffix ( filename )+weatherAbbrev[w]+".png";
+                  pc.addImage ( weatherTags[w], s, file, false  );
+                  pc.addString( ASCString(weatherTags[w]) + "_OriginalImageFilename", originalImageFilename[w] );
                }
          } else {
             for ( int w = 0; w < cwettertypennum; w++ )
@@ -582,6 +610,7 @@ void BuildingType :: runTextIO ( PropertyContainer& pc )
                   ASCString fileName = extractFileName_withoutSuffix ( filename )+weatherAbbrev[w]+".png";
                   Surface s;
                   pc.addImage ( weatherTags[w], s, fileName, false );
+                  originalImageFilename[w] = fileName;
 
 //                  if ( s.GetPixelFormat().BitsPerPixel() != 8 )
                      //fatalError("Building image " + filename + " does not have 8 Bit color depth!");
@@ -610,17 +639,26 @@ void BuildingType :: runTextIO ( PropertyContainer& pc )
          pc.closeBracket();
       }
 
-      bool rubble = false;
-      for ( int i = 0; i < 4; i++ )
-         for ( int j = 0; j < 6; j++ )
-            if ( destruction_objects[i][j] > 0 )
-               rubble = true;
-
+      bool rubble = destructionObjects.size() > 0;
       pc.addBool ( "RubbleObjects", rubble );
       if ( rubble ) {
+         if ( pc.isReading() ) 
+            destructionObjects.clear();
+         
          pc.openBracket ( "Rubble");
-         for ( Fields::iterator i = fields.begin(); i != fields.end(); i++ )
-            pc.addInteger ( i->toString(), destruction_objects[i->x][i->y] );
+         for ( Fields::iterator i = fields.begin(); i != fields.end(); i++ ) {
+            vector<int> ids;
+            typedef DestructionObjects::const_iterator J;
+            pair<J,J> b = destructionObjects.equal_range(*i);
+            for ( J j = b.first; j != b.second; ++j)
+               ids.push_back(j->second);
+            pc.addIntegerArray ( i->toString(), ids );
+            
+            if ( pc.isReading() ) {
+               for ( vector<int>::iterator j = ids.begin(); j != ids.end(); ++j )
+                  destructionObjects.insert(make_pair(*i, *j));
+            }
+         }
          pc.closeBracket();
       }
 
@@ -652,7 +690,7 @@ void BuildingType :: runTextIO ( PropertyContainer& pc )
        productionCost.runTextIO ( pc );
       pc.closeBracket ();
 
-      pc.addTagInteger( "Height", buildingheight, choehenstufennum, heightTags );
+      pc.addTagInteger( "Height", height, choehenstufennum, heightTags );
 
       pc.addTagInteger( "ExternalLoading", externalloadheight, choehenstufennum, heightTags );
 
@@ -669,7 +707,7 @@ void BuildingType :: runTextIO ( PropertyContainer& pc )
 void BuildingType::convertOldFunctions( int abilities, const ASCString& location  )
 {
    features.reset();
-   if ( abilities & 1 ) warning ( location + ": The HQ function for buildings is not supported any more");
+   if ( abilities & 1 ) warningMessage ( location + ": The HQ function for buildings is not supported any more");
    if ( abilities & 2 ) features.set( TrainingCenter );
    if ( abilities & (1 << 3) ) features.set( InternalVehicleProduction );
    if ( abilities & (1 << 4) ) features.set( AmmoProduction );
