@@ -29,6 +29,8 @@
 #include "gamemap.h"
 #include "spfst.h"
 #include "dlg_box.h"
+#include "statistics.h"
+#include "spfst-legacy.h"
 
 
 const char* clipboardFileExtension = "*.ascclipboard";
@@ -43,24 +45,42 @@ ClipBoardBase::ClipBoardBase()
 void ClipBoardBase::clear()
 {
    buf.clear();
+   properties.clear();
    objectNum = 0;
 }
 
 
-void ClipBoardBase::addUnit ( Vehicle* unit )
+void ClipBoardBase::setProperties( const ContainerBase* unit )
+{
+   properties["strength"] = ASCString::toString( StatisticsCalculator::strength( unit, true ) );
+   properties["materialindex"] = ASCString::toString( StatisticsCalculator::resource( unit, true ).material / 1000 );
+   properties["productioncost"] = ASCString::toString( StatisticsCalculator::unitCost( unit, true ));
+   properties["unitcount"] = ASCString::toString( StatisticsCalculator::unitCount( unit, true ));
+   properties["name"] = unit->getName();
+   properties["player"] = unit->getMap()->getPlayer( unit ).getName();
+   properties["turn"] = ASCString::toString( unit->getMap()->time.turn() );
+   properties["map"] = unit->getMap()->maptitle;
+}
+
+void ClipBoardBase::addUnit ( const Vehicle* unit )
 {
   tmemorystream stream ( &buf, tnstream::appending );
   stream.writeInt( ClipVehicle );
   unit->write ( stream );
   objectNum++;
+  
+  setProperties( unit );
 }
 
-void ClipBoardBase::addBuilding ( Building* bld )
+void ClipBoardBase::addBuilding ( const Building* bld )
 {
   tmemorystream stream ( &buf, tnstream::appending );
   stream.writeInt( ClipBuilding );
   bld->write ( stream );
   objectNum++;
+  
+  setProperties( bld );
+
 }
 
 
@@ -161,3 +181,11 @@ void ClipBoardBase::read( tnstream& stream )
    buf.readfromstream ( &stream );
 }
 
+void ClipBoardBase::writeProperties( PropertyContainer& pc ) const
+{
+   for ( map<ASCString,ASCString>::const_iterator i = properties.begin(); i != properties.end(); ++i ) {
+      ASCString temp = i->second;
+      pc.addString( i->first, temp );
+   }
+}
+
