@@ -15,6 +15,9 @@
 #include "resourcenet.h"
 #include <iostream>
 
+#include "tos.h"
+
+using namespace toString;
 
 
 template <class T, ContainerBaseType::ContainerFunctions f>
@@ -102,6 +105,10 @@ void AutoHarvestObjects::harvestObject( const MapCoordinate& pos, const ObjectTy
    if ( !currentField )
       return;
 
+   Object* object = currentField->checkForObject( obj );
+   if ( !object )
+      return;
+
    // we are only harvesting on fields who have non-harvestable neighbouring field, 
    // to prevent clearing of areas where the 'island' fields that are supposed to remain
    // are not covered by the oject
@@ -112,7 +119,7 @@ void AutoHarvestObjects::harvestObject( const MapCoordinate& pos, const ObjectTy
       if ( !harvestOnPosition(nextField)) {
          tfield* fld = base->getMap()->getField( nextField);
          if ( fld ) {
-            if ( fld->checkforobject(obj))
+            if ( fld->checkForObject(obj))
                ++regrowFields ;
             else {
                if ( obj->buildable(fld))
@@ -125,7 +132,6 @@ void AutoHarvestObjects::harvestObject( const MapCoordinate& pos, const ObjectTy
       return;
 
 
-   Object* object = currentField->checkforobject( obj );
 
    if( object != NULL ) {
       Resources removeValue = object->typ->removecost;
@@ -153,9 +159,12 @@ void AutoHarvestObjects::harvestObject( const MapCoordinate& pos, const ObjectTy
          if( !justQuery ) {
             base->getResource( removeCost, false,  1, base->getOwner());
             base->putResource( removeBenefit, false,  1, base->getOwner() );
-            currentField->removeobject( obj, true );
+            currentField->removeObject( obj, true );
          }
          ++fieldCounter;
+
+         logMessage("ResourceWork", "AutoHarvester " + tos( base->getPosition() ) + ": harvested object " + tos(obj->id) + " on field " + tos(pos) + " and got " + removeBenefit.toString() );
+
       }
    }
 }
@@ -171,7 +180,7 @@ void AutoHarvestObjects::processField( const MapCoordinate& pos )
 
    if ( fieldCounter >= base->baseType->autoHarvest.maxFieldsPerTurn )
       return;
-   
+
    for ( vector<IntRange>::const_iterator i = base->baseType->autoHarvest.objectsHarvestable.begin(); i != base->baseType->autoHarvest.objectsHarvestable.end();  ++i )
       for( int id=i->from; id <= i->to; ++id ) {
          const ObjectType *obj = base->getMap()->getobjecttype_byid( id );
@@ -305,6 +314,9 @@ bool MatterConverter :: run()
 {
    int perc = percentage;
 
+   logMessage("ResourceWork", "MatterConverter " + tos( bld->getPosition() ) + ": still needs to do " + tos(perc) + "% of work");
+
+
    int usageNum = 0;
    for ( int r = 0; r < 3; r++ )
       if ( bld->plus.resource(r) < 0 )
@@ -323,6 +335,9 @@ bool MatterConverter :: run()
          }
    }
 
+   logMessage("ResourceWork", "MatterConverter " + tos(bld->getPosition()) + ": checked storage, limiting to " + tos(perc) + "% of work");
+
+
    Resources toGet = bld->plus * perc / 100  ;
    for ( int r = 0; r < 3; r++ )
       if ( toGet.resource(r) < 0 )
@@ -341,21 +356,32 @@ bool MatterConverter :: run()
       }
    }
 
+   logMessage("ResourceWork", "MatterConverter " + tos( bld->getPosition() ) + ": checked required res, limiting to " + tos(perc) + "% of work");
 
    bool didSomething = false;
 
    for ( int r = 0; r < 3; r++ )
       if ( bld->plus.resource(r) > 0 ) {
-         bld->putResource( bld->plus.resource(r) * perc  / 100, r , false, 1, bld->getOwner() );
-         if ( bld->plus.resource(r) * perc / 100  > 0)
+         int toput = bld->plus.resource(r) * perc  / 100;
+         int didput = bld->putResource( toput, r , false, 1, bld->getOwner() );
+         if ( toput > 0)
             didSomething = true;
 
+         logMessage("ResourceWork", "MatterConverter " + tos( bld->getPosition()) + ": generated " + tos(toput) + " of " + Resources::name(r) + ", successfully stored " + tos(didput) + " of it");
+
       } else {
-         if ( bld->plus.resource(r) < 0 )
-            bld->getResource( -bld->plus.resource(r) * perc  / 100, r , false, 1, bld->getOwner());
+         if ( bld->plus.resource(r) < 0 ) {
+            int toget = -bld->plus.resource(r) * perc  / 100;
+            int got = bld->getResource( toget, r , false, 1, bld->getOwner());
+            logMessage("ResourceWork", "MatterConverter " + tos( bld->getPosition()) + ": needed " + tos(toget) + " of " + Resources::name(r) + ", successfully got " + tos(got) + " of it");
+         }
       }
 
+
    percentage -= perc;
+
+   logMessage("ResourceWork", "MatterConverter " + tos(bld->getPosition()) + ": did " + tos(perc) + "% of work, " + tos(percentage) + "% left");
+
    return didSomething;
 }
 
@@ -614,7 +640,7 @@ bool MiningStation :: run()
       spaceAvail = bld->putResource( toExtract_thisTurn, true, 1, bld->getOwner() );
       for ( int r = 0; r <3; ++r )
          if ( spaceAvail.resource(r) < 0 ) {
-            warning( ASCString("map corruption detected; building space availability is negative! ") + resourceNames[r] );
+            warningMessage( ASCString("map corruption detected; building space availability is negative! ") + resourceNames[r] );
             spaceAvail.resource( r ) = 0;
          }
    } else
@@ -629,7 +655,7 @@ bool MiningStation :: run()
       powerAvail = bld->getResource( toConsume, true, 1, bld->getOwner() );
       for ( int r = 0; r <3; ++r )
          if ( powerAvail.resource(r) < 0 ) {
-            warning( ASCString("map corruption detected; available power for mining station is negative! ") + resourceNames[r] );
+            warningMessage( ASCString("map corruption detected; available power for mining station is negative! ") + resourceNames[r] );
             powerAvail.resource( r ) = 0;
          }
    } else
@@ -691,7 +717,7 @@ void MiningStation :: testfield ( const MapCoordinate& mc )
                   perc = min  ( perc, float( double(powerAvail.resource(i)) / usageRatio[i] * toExtract_thisTurn.resource(r)));
 
             if ( perc < 0 ) {
-               warning("Warning: mining station inconsistency\n");
+               warningMessage("Warning: mining station inconsistency\n");
                perc = 0;
             }
 
@@ -703,7 +729,7 @@ void MiningStation :: testfield ( const MapCoordinate& mc )
             spaceAvail.resource(r) -= ex;
             for ( int i = 0; i < 3; ++i) {
                if ( spaceAvail.resource(i) < -2 )  // we allow for small deviations due to rounding errors
-                  warning("Warning: mining station inconsistency 2!\n");
+                  warningMessage("Warning: mining station inconsistency 2!\n");
 
                if ( spaceAvail.resource(i) < 0 )
                   spaceAvail.resource(i) = 0;
