@@ -7,7 +7,8 @@
  ***************************************************************************/
 
 /*! \file containerbase.cpp
-    \brief Implementation of methods that buildings and vehicles have in common
+    \brief Implementation of the common base class that that buildings and 
+	       vehicles share
 */
 
 /***************************************************************************
@@ -29,16 +30,19 @@
 #include "graphics/ColorTransform_PlayerColor.h"
 #include "containercontrols.h"
 #include "resourcenet.h"
+#include "accessconstraints.h"
 
 
 ContainerBase ::  ContainerBase ( const ContainerBaseType* bt, GameMap* map, int player ) : gamemap ( map ), cargoParent(NULL), baseType (bt)
 {
+   view = bt->view;
    damage = 0;
    color = player*8;
    maxresearchpoints = baseType->defaultMaxResearchpoints;
    researchpoints = min ( maxresearchpoints, baseType->nominalresearchpoints );
    maxplus = baseType->maxplus;
-   
+   repairedThisTurn = 0;
+  
    for ( int i = 0; i < map->getVehicleTypeNum(); ++i ) {
       const Vehicletype* vt = map->getvehicletype_bypos(i);
       if ( vt ) 
@@ -99,13 +103,13 @@ int ContainerBase :: repairItem   ( ContainerBase* item, int newDamage  )
    return newDamage;
 }
 
-int ContainerBase :: getMaxRepair ( const ContainerBase* item )
+int ContainerBase :: getMaxRepair ( const ContainerBase* item ) const
 {
    Resources res;
    return getMaxRepair ( item, 0, res );
 }
 
-int ContainerBase :: getMaxRepair ( const ContainerBase* item, int newDamage, Resources& cost, bool ignoreCost  )
+int ContainerBase :: getMaxRepair ( const ContainerBase* item, int newDamage, Resources& cost, bool ignoreCost  ) const
 {
    if ( !canRepair( item ) )
       return item->damage;
@@ -127,7 +131,7 @@ int ContainerBase :: getMaxRepair ( const ContainerBase* item, int newDamage, Re
       needed.resource(i) = maxNeeded.resource(i) * (item->damage-newDamage) / 100;
 
    if ( !ignoreCost ) {
-      Resources avail = getResource ( needed, 1 );
+      Resources avail = getResource ( needed );
    
       for ( i = 0; i < resourceTypeNum; i++ )
          if ( needed.resource(i) ) {
@@ -175,84 +179,45 @@ int ContainerBase :: cargoNestingDepth()
       return 0;
 }
 
-
-const ContainerBase* ContainerBase :: findParent ( const ContainerBase* veh ) const
+void ContainerBase :: compactCargo()
 {
-   for ( Cargo::const_iterator i = cargo.begin(); i != cargo.end(); ++i )
-      if ( *i ) {
-         if ( *i == veh )
-            return this;
-         else {
-            const ContainerBase* cb = (*i)->findParent( veh );
-            if ( cb )
-               return cb;
-         }
-      }
-
-   return NULL;
+   for ( Cargo::iterator i = cargo.begin(); i != cargo.end(); ) {
+      if ( *i == NULL )
+         i = cargo.erase(i);
+      else
+         ++i;
+   }
 }
 
-ContainerBase* ContainerBase :: findParent ( const ContainerBase* veh )
-{
-   for ( Cargo::iterator i = cargo.begin(); i != cargo.end(); ++i )
-      if ( *i ) {
-         if ( *i == veh )
-            return this;
-         else {
-            ContainerBase* cb = (*i)->findParent( veh );
-            if ( cb )
-               return cb;
-         }
-      }
 
-   return NULL;
+Vehicle* ContainerBase :: getCargo( int i )
+{
+   if ( i < 0 || i >= cargo.size() )
+      return NULL;
+   else  
+      return cargo.at(i);
 }
 
+
+
 ContainerBase* ContainerBase :: getCarrier() const
 {
    return cargoParent;
-   /*
-   tfield* fld = getMap()->getField( getPosition() );
-   if ( fld->vehicle == this )
-      return NULL;
-   
-   if ( fld->building )
-      return fld->building->findParent( this );
-   
-   if ( fld->vehicle )
-      return fld->vehicle->findParent( this );
-
-   return NULL;
-   */
 }
 
 
-bool ContainerBase::unitLoaded( int nwid )
-{
-   for ( Cargo::const_iterator i = cargo.begin(); i != cargo.end(); ++i )
-      if ( *i ) {
-         if ( (*i)->networkid == nwid )
-            return true;
-         else {
-            if ( (*i)->unitLoaded( nwid ) )
-               return true;
-         }
-      }
-
-   return false;
-
-}
-
-Vehicle* ContainerBase :: findUnit ( int nwid )
+Vehicle* ContainerBase :: findUnit ( int nwid, bool recursive ) const
 {
    for ( Cargo::const_iterator i = cargo.begin(); i != cargo.end(); ++i )
       if ( *i ) {
          if ( (*i)->networkid == nwid )
             return *i;
          else {
-            Vehicle* cb = (*i)->findUnit( nwid );
-            if ( cb )
-               return cb;
+            if ( recursive ) {
+               Vehicle* cb = (*i)->findUnit( nwid );
+               if ( cb )
+                  return cb;
+            }
          }
       }
 
@@ -358,6 +323,7 @@ void ContainerBase::paintField ( const Surface& img, Surface& dest, SPoint pos,
             blitter.blit( img, dest, pos );
          } else {
             MegaBlitter<4,gamemapPixelSize,ColorTransform_PlayerTrueCol, ColorMerger_AlphaMerge, SourcePixelSelector_CacheRotation> blitter;
+            /// blitter.setPlayer( getOwner() );
             blitter.setColor( gamemap->player[getOwner()].getColor() );
             blitter.setAngle( img, directionangle[dir] );
             blitter.blit( img, dest, pos );
@@ -379,16 +345,22 @@ void ContainerBase :: clearCargo()
 }
 
 
-void ContainerBase :: addToCargo( Vehicle* veh )
+void ContainerBase :: addToCargo( Vehicle* veh, int position )
 {
+   if ( veh == this )
+      fatalError ("Trying to add unit to its own cargo");
+   
+   bool slotFound = false;
    for ( Cargo::iterator i = cargo.begin(); i != cargo.end(); ++i )
-      if ( ! (*i) ) {
+      if ( ! (*i) && (position == -1 || position == i - cargo.begin() )) {
          *i = veh;
-         veh->cargoParent = this;
-         return;
+         slotFound = true;
+         break;
       }
 
-   cargo.push_back( veh );
+   if ( !slotFound )
+      cargo.push_back( veh );
+   
    veh->cargoParent = this;
    veh->setnewposition(getPosition());
    cargoChanged();
@@ -414,6 +386,13 @@ bool ContainerBase :: removeUnitFromCargo( int nwid, bool recursive )
          if ( (*i)->networkid == nwid ) {
             (*i)->cargoParent = NULL;
             *i = NULL;
+
+            if ( cargo.size() > baseType->maxLoadableUnits ) {
+               // we only compact the cargo when we have more slots than allowed (may happen when conquering fully occupied buildings).
+               // because else we would confuse the user if the position of units in the cargo dialog changes
+               compactCargo();
+            }
+
             cargoChanged();
             return true;
          }
@@ -425,13 +404,29 @@ bool ContainerBase :: removeUnitFromCargo( int nwid, bool recursive )
    return false;
 }
 
+bool ContainerBase :: canCarryWeight( int additionalWeight, const Vehicle*  vehicle) const
+{
+   // if the unit already carries this unit, there is no additional weight to check
+   if ( vehicle && findUnit( vehicle->networkid ))
+      additionalWeight = 0;
+
+
+   if ( cargoWeight() + additionalWeight > baseType->maxLoadableWeight )
+      return false;
+   else
+      if ( getCarrier() )
+         return getCarrier()->canCarryWeight( additionalWeight, vehicle );
+      else
+         return true;
+}
+
 
 bool ContainerBase :: vehicleFit ( const Vehicle* vehicle ) const
 {
    bool isConquering = isBuilding() && getMap()->getPlayer(this).diplomacy.isHostile( vehicle) && vehicle->color != color;
    if ( baseType->vehicleFit ( vehicle->typ )) // checks size and type
       if ( vehiclesLoaded() < baseType->maxLoadableUnits || isConquering )
-         if ( cargoWeight() + vehicle->weight() <= baseType->maxLoadableWeight || findParent ( vehicle ) || isConquering) // if the unit is already  loaded, the container already bears its weight
+         if ( canCarryWeight( vehicle->weight(), vehicle ) || findUnit ( vehicle->networkid ) || isConquering) // if the unit is already  loaded, the container already bears its weight
             return true;
 
    return false;
@@ -466,7 +461,7 @@ bool  ContainerBase :: vehicleLoadable ( const Vehicle* vehicle, int uheight, co
                                  if ( getOwner() == 8 )
                                     return true;
                                  if ( gamemap->getPlayer(this).diplomacy.isHostile( vehicle->getOwner())  )
-                                    if (damage >= mingebaeudeeroberungsbeschaedigung  || vehicle->typ->hasFunction( ContainerBaseType::ConquerBuildings ) )
+                                    if (damage >= minimumBuildingDamageForConquering  || vehicle->typ->hasFunction( ContainerBaseType::ConquerBuildings ) )
                                        return true;
                               }
                            }
@@ -474,16 +469,21 @@ bool  ContainerBase :: vehicleLoadable ( const Vehicle* vehicle, int uheight, co
    return false;
 }
 
-int  ContainerBase :: vehicleUnloadable ( const Vehicletype* vehicleType ) const
+int  ContainerBase :: vehicleUnloadable ( const Vehicletype* vehicleType, int carrierHeight ) const
 {
    int height = 0;
 
+   if ( carrierHeight == -1 )
+      carrierHeight = getPosition().getBitmappedHeight();
+   else
+      carrierHeight = 1 << carrierHeight;
+   
    if ( baseType->vehicleFit ( vehicleType ))
       for ( ContainerBaseType::EntranceSystems::const_iterator i = baseType->entranceSystems.begin(); i != baseType->entranceSystems.end(); i++ )
          if ( i->mode & ContainerBaseType::TransportationIO::Out )
-            if ( (i->container_height & getPosition().getBitmappedHeight()) || (i->container_height == 0))
+            if ( (i->container_height & carrierHeight) || (i->container_height == 0))
                if ( i->vehicleCategoriesLoadable & (1<<vehicleType->movemalustyp))
-                  if ( vehicleType->hasAnyFunction(i->requiresUnitFeature) || i->requiresUnitFeature.none() )
+                  if ( vehicleType->hasAnyFunction(i->requiresUnitFeature) || i->requiresUnitFeature.none() ) {
                      if ( i->height_abs != 0 && i->height_rel != -100 ) {
                         int h = 0;
                         for ( int hh = 0; hh < 8; ++hh)
@@ -495,6 +495,7 @@ int  ContainerBase :: vehicleUnloadable ( const Vehicletype* vehicleType ) const
                            height |= 1 << (getPosition().getNumericalHeight() + i->height_rel) ;
                         else
                            height |= i->height_abs ;
+                  }
    return height & vehicleType->height;
 }
 
@@ -505,7 +506,7 @@ const ContainerBaseType::TransportationIO* ContainerBase::vehicleUnloadSystem (
          if ( i->mode & ContainerBaseType::TransportationIO::Out )
             if ( (i->container_height & getPosition().getBitmappedHeight()) || (i->container_height == 0))
                if ( i->vehicleCategoriesLoadable & (1<<vehicleType->movemalustyp))
-                  if ( vehicleType->hasAnyFunction(i->requiresUnitFeature) || i->requiresUnitFeature.none() )
+                  if ( vehicleType->hasAnyFunction(i->requiresUnitFeature) || i->requiresUnitFeature.none() ) {
                      if ( i->height_abs != 0 && i->height_rel != -100 ) {
                         if ( height & ( i->height_abs & (1 << (getPosition().getNumericalHeight() + i->height_rel ))))
                            return &(*i);
@@ -516,6 +517,7 @@ const ContainerBaseType::TransportationIO* ContainerBase::vehicleUnloadSystem (
                         } else
                            if ( height & i->height_abs )
                               return &(*i);
+                  }
    return NULL;
 
 }
@@ -531,7 +533,7 @@ int  ContainerBase :: vehicleDocking ( const Vehicle* vehicle, bool out ) const
       for ( ContainerBaseType::EntranceSystems::const_iterator i = baseType->entranceSystems.begin(); i != baseType->entranceSystems.end(); i++ )
          if ( i->mode & ContainerBaseType::TransportationIO::Docking )
             if ( (i->container_height & getPosition().getBitmappedHeight()) || (i->container_height == 0))
-               if ( i->vehicleCategoriesLoadable & (1<<vehicle->typ->movemalustyp))
+               if ( i->vehicleCategoriesLoadable & (1<<vehicle->typ->movemalustyp)) {
                   if ( i->dockingHeight_abs != 0 && i->dockingHeight_rel != -100 )
                      height |= i->dockingHeight_abs & (1 << (getPosition().getNumericalHeight() + i->dockingHeight_rel ));
                   else
@@ -539,15 +541,19 @@ int  ContainerBase :: vehicleDocking ( const Vehicle* vehicle, bool out ) const
                         height |= 1 << (getPosition().getNumericalHeight() + i->dockingHeight_rel) ;
                      else
                         height |= i->dockingHeight_abs ;
+               }
    return height;
 }
 
 const ContainerBase::Production& ContainerBase::getProduction() const
 {
    if ( productionCache.empty() && !internalUnitProduction.empty() ) {
-      for ( Production::const_iterator i = internalUnitProduction.begin(); i != internalUnitProduction.end(); ++i )
-         if ( vehicleUnloadable( *i ) || baseType->hasFunction( ContainerBaseType::ProduceNonLeavableUnits ) )
-            productionCache.push_back ( *i );
+      for ( int height = 0; height < 8; ++height )
+         if ( (1 << height) & baseType->height )
+            for ( Production::const_iterator i = internalUnitProduction.begin(); i != internalUnitProduction.end(); ++i )
+               if ( baseType->hasFunction( ContainerBaseType::ProduceNonLeavableUnits ) || vehicleUnloadable( *i, height ) )
+                  if( find ( productionCache.begin(), productionCache.end(), *i ) == productionCache.end() )
+                     productionCache.push_back ( *i );
    }
 
    return productionCache;
@@ -585,9 +591,23 @@ void ContainerBase :: setProductionLines( const Production& production  )
 }
 
 
+void ContainerBase :: setName ( const ASCString& name )
+{
+   if ( !checkModificationConstraints( this ) )
+      return;
+   
+   this->name = name;
+}
+
+
 ContainerBase :: ~ContainerBase ( )
 {
-   for ( Cargo::iterator i = cargo.begin(); i != cargo.end(); ++i )
+   /* removing a unit from cargo (which is done when deleting) may cause a call to compactCargo, 
+      which in turn invalidates the iterators to cargo 
+      That's why we iterate through a copy of cargo */ 
+   Cargo toDelete = cargo;
+   
+   for ( Cargo::iterator i = toDelete.begin(); i != toDelete.end(); ++i )
       if ( *i )
          delete *i;
 
@@ -623,27 +643,6 @@ void TemporaryContainerStorage :: restore (  )
 
 void ContainerBase::endOwnTurn( void )
 {
-#ifndef karteneditor
-   if ( baseType->hasFunction( ContainerBaseType::TrainingCenter  ) ) {
-      for ( Cargo::iterator i = cargo.begin(); i != cargo.end(); ++i )
-         if ( *i ) {
-            /*
-            bool ammoFull = true;
-            for ( int w = 0; w < (*i)->typ->weapons.count; ++w )
-               if ( (*i)->ammo[w] < (*i)->typ->weapons.weapon[w].count )
-                  ammoFull = false;
-
-            if ( ammoFull ) {
-            */
-               ContainerControls cc ( this );
-               if ( cc.unitTrainingAvailable( *i )) {
-                  cc.trainUnit( *i );
-                  cc.refillAmmo ( *i );
-               }
-            // }
-         }
-   }
-#endif
 }
 
 void ContainerBase::endAnyTurn( void )
@@ -652,6 +651,7 @@ void ContainerBase::endAnyTurn( void )
 
 void ContainerBase::endRound ( void )
 {
+   view = baseType->view;
 }
 
 
