@@ -36,7 +36,11 @@
 #include "gameoptions.h"
 #include "graphics/ColorTransform_PlayerColor.h"
 
+#include "actions/actioncontainer.h"
+#include "widgets/textrenderer.h"
+
 #include "sg.h"
+#include "spfst-legacy.h"
 
 class WeaponInfoLine;
 
@@ -209,6 +213,7 @@ void DashboardPanel::painter ( const PG_Rect &src, const ASCString& name, const
    if( name == "showplayercolor0" || name == "showplayercolor1" ) {
       MegaBlitter<4,4,ColorTransform_PlayerTrueCol,ColorMerger_PlainOverwrite> blitter;
       blitter.setColor( actmap->player[actmap->actplayer].getColor() );
+      // blitter.setPlayer(actmap->actplayer);
       blitter.blit( IconRepository::getIcon("show_playercolor.png"), screen, SPoint(dst.x, dst.y));
       return;
    }
@@ -226,7 +231,7 @@ void DashboardPanel::painter ( const PG_Rect &src, const ASCString& name, const
                                               "terrain_weather_heavysnow.png",
                                               "terrain_weather_ice.png" };
 
-         blitter.blit ( IconRepository::getIcon(weathernames[actmap->getField(mc)->getweather()]), screen, SPoint(dst.x, dst.y) );
+         blitter.blit ( IconRepository::getIcon(weathernames[actmap->getField(mc)->getWeather()]), screen, SPoint(dst.x, dst.y) );
       }
       return;
    }
@@ -262,6 +267,7 @@ void DashboardPanel::painter ( const PG_Rect &src, const ASCString& name, const
             if ( height1 & (1 << i )) {
                MegaBlitter<4,4,ColorTransform_PlayerTrueCol,ColorMerger_PlainOverwrite> blitter;
                blitter.setColor( actmap->player[player].getColor() );
+               // blitter.setPlayer( player );
                blitter.blit( IconRepository::getIcon("height-b" + ASCString::toString(i) + ".png"), screen, SPoint(dst.x, dst.y + (7-i) * 13));
             } else
                if ( height2 & (1 << i ))
@@ -307,7 +313,7 @@ void DashboardPanel::eval()
 
    setLabelText( "windspeed", actmap->weather.windSpeed );
 
-   if ( mc.valid() && fieldvisiblenow( fld, actmap->getPlayerView() )) {
+   if ( fld && mc.valid() && fieldvisiblenow( fld, actmap->getPlayerView() )) {
       setLabelText( "terrain_harbour", fld->bdt.test(cbharbour) ? "YES" : "NO" );
       setLabelText( "terrain_pipe", fld->bdt.test(cbpipeline) || fld->building ? "YES" : "NO" );
 
@@ -446,7 +452,7 @@ void DashboardPanel::showUnitData( Vehicle* veh, Building* bld, tfield* fld,  bo
          setLabelText( "unittypename", "" );
          setLabelText( "unitname", "" );
          bool objectFound = false;
-         if ( fld && fld->objects.size() ) {
+         if ( fld && fld->objects.size() && fieldvisiblenow( fld )) {
             for ( tfield::ObjectContainer::iterator i = fld->objects.begin(); i != fld->objects.end(); ++i )
                if ( i->typ->armor > 0 ) {
                   setBargraphValue( "unitdamage", float(100-i->damage) / 100  );
@@ -816,8 +822,8 @@ MapInfoPanel::MapInfoPanel (PG_Widget *parent, const PG_Rect &r, MapDisplayPG* m
       zoomSlider->SetPosition( 100 - mapDisplay->getZoom() );
    }   
 
-   const int labelnum = 4;
-   const char* label[labelnum] = { "pipes", "container", "resources", "visibilityvalue"};
+   const int labelnum = 5;
+   const char* label[labelnum] = { "pipes", "container", "resources", "visibilityvalue", "reactionfire" };
    for ( int i = 0; i < labelnum; ++i ) {
       PG_CheckButton* cb = dynamic_cast<PG_CheckButton*>( FindChild( label[i], true ) );
       if ( mapDisplay->layerActive( label[i] )
@@ -842,11 +848,12 @@ MapInfoPanel::MapInfoPanel (PG_Widget *parent, const PG_Rect &r, MapDisplayPG* m
 void MapInfoPanel::layerChanged( bool state, const ASCString& label )
 {
    PG_CheckButton* cb = dynamic_cast<PG_CheckButton*>( FindChild( label, true ) );
-   if ( cb && ! changeActive ) 
+   if ( cb && ! changeActive ) {
       if ( state )
          cb->SetPressed();
       else
          cb->SetUnpressed();
+   }
 }
 
 bool MapInfoPanel::showWeaponRange()
@@ -888,3 +895,24 @@ bool MapInfoPanel::checkBox( bool state, const char* name )
 }
 
 
+ActionInfoPanel::ActionInfoPanel (PG_Widget *parent, const PG_Rect &r ) : DashboardPanel( parent, r, "ActionInfo" )
+{
+   ActionContainer::actionListChanged.connect( SigC::slot( *this, &ActionInfoPanel::update ));
+}
+
+void ActionInfoPanel::update( GameMap* map )
+{
+   TextRenderer* text = dynamic_cast<TextRenderer*>( FindChild( "ActionList", true ));
+   if ( text && map) {
+      /*
+      vector<ASCString> list;
+      map->actions.getActionDescriptions( list );
+      
+      ASCString s;
+      for ( vector<ASCString>::const_iterator i = list.begin(); i != list.end(); ++i )
+         s += *i + "\n";
+      */
+      text->SetText( "Function removed, please use Action/Manage dialog" );
+   }
+}
+
