@@ -46,7 +46,7 @@ class AdminGameWindow : public ASC_PG_Dialog {
       PG_LineEdit* currentPlayer;
       PG_LineEdit* filename;
 
-      typedef Loki::Functor<void, TYPELIST_1(int) > PlayerActionFunctor;
+      typedef Loki::Functor<void, LOKI_TYPELIST_1(int) > PlayerActionFunctor;
       typedef PG_ListBoxDataItem<PlayerActionFunctor> ActionItem;
 
       PG_ScrollWidget* scrollwidget;
@@ -61,20 +61,18 @@ class AdminGameWindow : public ASC_PG_Dialog {
          if ( turn )
             turn->SetText( ASCString::toString( gamemap->time.turn() ));
 
-         if ( currentPlayer )
+         if ( currentPlayer ) {
             if ( gamemap->actplayer >= 0 )
                currentPlayer->SetText( gamemap->player[gamemap->actplayer].getName() );
             else
                currentPlayer->SetText( ASCString::toString( gamemap->actplayer ) );
+         }
       }
 
       bool skipPlayer()
       {
          if ( turnSkipper ) {
-            GameMap* temp = actmap;
-            actmap = gamemap;
             (*turnSkipper)(gamemap);
-            actmap = temp;
          }
 
          updateTurn();
@@ -136,8 +134,8 @@ class AdminGameWindow : public ASC_PG_Dialog {
       {
          for ( int j = 0; j< gamemap->getPlayerCount(); ++j ) {
             if ( j != player ) {
-               gamemap->getPlayer(player).diplomacy.setState( j, WAR, false );
-               gamemap->getPlayer(j).diplomacy.setState( player, WAR, false );
+               gamemap->getPlayer(player).diplomacy.setState( j, WAR );
+               gamemap->getPlayer(j).diplomacy.setState( player, WAR );
             }
          }
       }
@@ -169,6 +167,14 @@ class AdminGameWindow : public ASC_PG_Dialog {
             delete *gamemap->player[player].buildingList.begin();
 
       }
+
+      void neutralBuildings( int player )
+      {
+         while ( gamemap->player[player].buildingList.begin() != gamemap->player[player].buildingList.end() )
+            gamemap->player[player].buildingList.front()->convert(8);
+
+      }
+      
       
       void deleteProduction( int player )
       {
@@ -233,14 +239,19 @@ class AdminGameWindow : public ASC_PG_Dialog {
       {
          apply();
          if ( playerSetup->Apply() ) {
+            GameTransferMechanism* gtm = gamemap->network;
             gamemap->network = NULL;
 
+            if ( gamemap->_resourcemode == 1)
+               warningMessage("This map uses BI resource mode. Saving as map will delete all global resources of all players");
+               
             ASCString name = selectFile( mapextension, false);
             if ( !name.empty() ) {
                StatusMessageWindowHolder smw = MessagingHub::Instance().infoMessageWindow( "saving " + name );
-               savemap( name.c_str(), gamemap );
+               savemap( name, gamemap );
             }
 
+            gamemap->network = gtm;
             QuitModal();
             return true;
          } else
@@ -293,13 +304,14 @@ class AdminGameWindow : public ASC_PG_Dialog {
          new ActionItem( actionlistbox, 20, "reset view", PlayerActionFunctor( this, &AdminGameWindow::resetView) );
          new ActionItem( actionlistbox, 20, "reset research", PlayerActionFunctor( this, &AdminGameWindow::resetResearch));
          new ActionItem( actionlistbox, 20, "reset tribute", PlayerActionFunctor( this, &AdminGameWindow::resetTribute));
-         new ActionItem( actionlistbox, 20, "reset diplomacy (->truce)", PlayerActionFunctor( this, &AdminGameWindow::resetDiplomacy));
+         new ActionItem( actionlistbox, 20, "reset diplomacy (->war)", PlayerActionFunctor( this, &AdminGameWindow::resetDiplomacy));
          new ActionItem( actionlistbox, 20, "reset password", PlayerActionFunctor( this, &AdminGameWindow::resetPassword));
          new ActionItem( actionlistbox, 20, "delete units", PlayerActionFunctor( this, &AdminGameWindow::deleteUnits));
          new ActionItem( actionlistbox, 20, "delete buildings", PlayerActionFunctor( this, &AdminGameWindow::deleteBuildings));
          new ActionItem( actionlistbox, 20, "delete production", PlayerActionFunctor( this, &AdminGameWindow::deleteProduction));
          new ActionItem( actionlistbox, 20, "delete resources + ammo", PlayerActionFunctor( this, &AdminGameWindow::deleteResources));
          new ActionItem( actionlistbox, 20, "delete mines", PlayerActionFunctor( this, &AdminGameWindow::deleteMines));
+         new ActionItem( actionlistbox, 20, "make buildings neutral", PlayerActionFunctor( this, &AdminGameWindow::neutralBuildings));
          new ActionItem( actionlistbox, 20, "enter new password", PlayerActionFunctor( this, &AdminGameWindow::newPassword));
          new ActionItem( actionlistbox, 20, "edit email", PlayerActionFunctor( this, &AdminGameWindow::newEmail));
          new ActionItem( actionlistbox, 20, "export technology", PlayerActionFunctor( this, &AdminGameWindow::exportTechs));
@@ -374,7 +386,7 @@ class AdminGameWindow : public ASC_PG_Dialog {
 
 bool adminGame( GameMap* actmap, TurnSkipper* turnSkipper )
 {
-   AdminGameWindow agw( actmap, NULL, turnSkipper );
+   AdminGameWindow agw( actmap, NULL, turnSkipper);
    agw.Show();
    agw.RunModal();
    return agw.GetStatus();
