@@ -34,6 +34,9 @@
 
 #include "pgmultilineedit.h"
 #include "../widgets/textrenderer.h"
+#include "../actions/directresearchcommand.h"
+#include "../sg.h"
+#include "../dialog.h"
 
 class TechWidget: public SelectionWidget  {
       const Technology* tech;
@@ -149,21 +152,8 @@ Surface TechWidget::clippingSurface;
    
    void TechnologySelectionItemFactory::restart()
    {
-      items.clear();
-      
-      for (int i = 0; i < technologyRepository.getNum(); i++) {
-         const Technology* tech = technologyRepository.getObject_byPos( i );
-         if ( tech ) {
-            if ( allTechs ) {
-               if ( tech->eventually_available( player.research, NULL ))
-                  items.push_back( tech );
-            } else {
-               ResearchAvailabilityStatus a = player.research.techAvailable ( tech );
-               if ( a == Available )
-                  items.push_back ( tech );
-            }
-         }
-      }
+      DirectResearchCommand drc( player );
+      items = drc.getAvailableTechnologies( allTechs );
       
       sort( items.begin(), items.end(), techComp );
       it = items.begin();
@@ -200,10 +190,12 @@ class ChooseTech : public ASC_PG_Dialog
    TechnologySelectionItemFactory* factory;
    PG_MultiLineEdit* techList;
    Player& player;
+   PG_CheckButton* allTechsCheckButton;
 
    PG_Label* pointsLabel;
    PG_Label* availLabel;
    const Technology* goal;
+   bool okPressed;
 
    bool changeTechView( bool all )
    {
@@ -213,12 +205,22 @@ class ChooseTech : public ASC_PG_Dialog
       
    }
    
+   void toggleAllTechButton() 
+   {
+      if ( allTechsCheckButton->GetPressed() )
+         allTechsCheckButton->SetUnpressed();
+      else
+         allTechsCheckButton->SetPressed();
+      // changeTechView( allTechsCheckButton->GetPressed() );
+   }         
+
+
    void techSelected( const Technology* tech )
    {
       ASCString s;
       list<const Technology*> techs;
       if ( !tech->eventually_available( player.research , &techs )) {
-         warning("Inconsistency in ChooseTech::techSelected");
+         warningMessage("Inconsistency in ChooseTech::techSelected");
          return;
       }
 
@@ -233,8 +235,7 @@ class ChooseTech : public ASC_PG_Dialog
 
       pointsLabel->SetText( ASCString("Sum: ") + ASCString::toString(points) + " Points" );
 
-      goal = player.research.goal = tech;
-      player.research.activetechnology = *techs.begin();
+      goal = tech;
    };
 
    protected:
@@ -249,12 +250,32 @@ class ChooseTech : public ASC_PG_Dialog
       bool ok()
       {
          if ( goal || !itemSelector->getItemNum() ) {
-            QuitModal();
+            
+            if ( goal ) {
+               auto_ptr<DirectResearchCommand> drc ( new DirectResearchCommand( player ));
+               drc->setTechnology( goal );
+               ActionResult res = drc->execute( createContext( player.getParentMap() ));
+               if ( res.successful() ) {
+                  drc.release();
+                  okPressed = true;
+                  QuitModal();
+               } else
+                  displayActionError( res);
+            } else
+               QuitModal();
             return true;
          } else
             return false;
       }
 
+      bool cancel()
+      {
+         player.research.goal = NULL;
+         player.research.activetechnology = NULL;
+         QuitModal();
+         return true;
+      }
+      
       bool showPrerequisites()
       {
          if ( goal ) {
@@ -266,16 +287,36 @@ class ChooseTech : public ASC_PG_Dialog
          }
          return true;
       }
+      
+      bool eventKeyDown(const SDL_KeyboardEvent* key)
+      {
+         int mod = SDL_GetModState() & ~(KMOD_NUM | KMOD_CAPS | KMOD_MODE);
+
+         if ( !mod  ) 
+            if ( key->keysym.sym == SDLK_RETURN ) 
+               return ok();
+         
+
+         if ( mod & KMOD_CTRL ) {
+            switch ( key->keysym.unicode ) {
+                  case 1: // A
+                     toggleAllTechButton();
+                     return true;
+            };
+         }
+
+         return false;
+      };
 
    
    public:
-      ChooseTech( Player& my_player ) : ASC_PG_Dialog( NULL, PG_Rect( -1, -1, 770, 600), "Choose Technology" ) , factory(NULL), player( my_player ), goal(NULL)
+      ChooseTech( Player& my_player ) : ASC_PG_Dialog( NULL, PG_Rect( -1, -1, 770, 600), "Choose Technology" ) , factory(NULL), player( my_player ), goal(NULL), okPressed(false)
       {
          factory = new TechnologySelectionItemFactory( player );
          factory->techSelected.connect( SigC::slot( *this, &ChooseTech::techSelected ));
          itemSelector = new ItemSelectorWidget( this, PG_Rect( 10, 40, 400, Height() - 80 ), factory );
 
-         (new PG_CheckButton( this, PG_Rect( 10, Height() - 40, 300, 25 ), "Show All Technologies"))->sigClick.connect( SigC::slot( *this, &ChooseTech::changeTechView));
+         (allTechsCheckButton = new PG_CheckButton( this, PG_Rect( 10, Height() - 40, 300, 25 ), "Show ~A~ll Technologies"))->sigClick.connect( SigC::slot( *this, &ChooseTech::changeTechView));
          techList = new PG_MultiLineEdit( this, PG_Rect ( 450, 40, 300, 200 ));
          techList->SetEditable(false);
          pointsLabel = new PG_Label( this, PG_Rect( 450, 250, 300, 25 ));
@@ -283,25 +324,26 @@ class ChooseTech : public ASC_PG_Dialog
 
          (new PG_Button( this, PG_Rect( 450, 320, 300, 20), "List Prerequisites" ))->sigClick.connect( SigC::slot( *this, &ChooseTech::showPrerequisites ));
 
+         AddStandardButton("~C~ancel")->sigClick.connect( SigC::slot( *this, &ChooseTech::cancel ));
          AddStandardButton("~O~K")->sigClick.connect( SigC::slot( *this, &ChooseTech::ok ));
       };
+      
+      bool selectionPerformed()
+      {
+         return okPressed;
+      }
+      
 };
 
 
 
 
-void chooseTechnology( Player& player )
+bool chooseSingleTechnology( Player& player )
 {
-#if 0
-   ItemSelectorWindow isw( NULL, );
-   isw.Show();
-   isw.RunModal();
-#else
    ChooseTech ct( player);
    ct.Show();
    ct.RunModal();
-#endif
+   return ct.selectionPerformed();
 }
 
 
-
