@@ -38,93 +38,135 @@
 #include "../textfileparser.h"
 #include "../textfile_evaluation.h"
 
+#include "editgameoptions.h"
 
 
-class GetVideoModes {
-   public:
-      typedef vector<ASCString> VList;
-   private:
-      SDL_Rect **modes;
-      VList list;
-
-      vector <pair<int,int> > listedmodes;
-     
-   public:
-      GetVideoModes() {
-         int i;
-
-
-         SDL_PixelFormat format;
-         format.palette = NULL;
-         format.BitsPerPixel = 32;
-         format.BytesPerPixel = 4;
-         format.Rloss = format.Gloss = format.Bloss = format.Aloss = 0;
-         format.Rshift = 0;
-         format.Gshift = 8;
-         format.Bshift = 16;
-         format.Ashift = 24;
-         format.Rmask = 0xff;
-         format.Gmask = 0xff00;
-         format.Bmask = 0xff0000;
-         format.Amask = 0xff000000;
-         format.colorkey = 0;
-         format.alpha = 0;
-         
-         
-         /* Get available fullscreen/hardware modes */
-         modes=SDL_ListModes(&format, SDL_FULLSCREEN);
-
-
-         list.push_back( "graphic mode not listed");
-         listedmodes.push_back( make_pair( 0, 0));
-         
-         /* Check is there are any modes available */
-         if(modes == (SDL_Rect **)0){
-            return;
+bool GetVideoModes::comparator( const ModeRes& a, const ModeRes& b )
+{
+   if ( a.first == 0 )
+      return true;
+   
+   if ( b.first == 0 )
+      return false;
+   
+   if ( a.first > b.first ) {
+      return true;
+   } else {
+      if ( a.first < b.first )
+         return false;
+      else {
+         if ( a.second > b.second )
+            return true;
+         else {
+            return false;
          }
-         
-         /* Check if our resolution is restricted */
-         if(modes == (SDL_Rect **)-1){
-            warning("All resolutions available.\n");
-            return;
-         }
-         else{
-            for(i=0;modes[i];++i) {
-
-               if ( find ( listedmodes.begin(), listedmodes.end(), make_pair( int(modes[i]->w), int(modes[i]->h ))) != listedmodes.end() )
-                  continue;
-               
-               if ( modes[i]->w >= 800 && modes[i]->h >= 600 ) {
-                  ASCString s;
-                  s.format( "%d*%d", modes[i]->w, modes[i]->h );
-                  list.push_back ( s );
-                  listedmodes.push_back( make_pair( int(modes[i]->w), int(modes[i]->h )));
-               }
-            }
-            
-         }
-         return;
-      };
-
-      VList& getList() { return list; };
-
-      int getx( int index ) {
-         return listedmodes.at(index).first;
-      };
-      
-      int gety( int index ) {
-         return listedmodes.at(index).second;
-      };
-
-      int findmodenum( int x, int y ) {
-         for ( int j = 0; j < listedmodes.size(); ++j )
-            if ( listedmodes[j].first == x && listedmodes[j].second == y )
-               return j;
-         return 0;
       }
-      
+
+   }
+}
+
+/*
+bool GetVideoModes::comparator( const ModeRes& a, const ModeRes& b )
+{
+   if ( a.first > b.first ) {
+      return 1;
+   } else {
+      if ( a.first < b.first )
+         return -1;
+      else {
+         if ( a.second > b.second )
+            return 1;
+         else {
+            if ( a.second < b.second )
+               return -1;
+            else
+               return 0;
+         }
+      }
+
+   }
+}
+*/
+
+GetVideoModes::GetVideoModes() 
+{
+   int i;
+
+
+   SDL_PixelFormat format;
+   format.palette = NULL;
+   format.BitsPerPixel = 32;
+   format.BytesPerPixel = 4;
+   format.Rloss = format.Gloss = format.Bloss = format.Aloss = 0;
+   format.Rshift = 0;
+   format.Gshift = 8;
+   format.Bshift = 16;
+   format.Ashift = 24;
+   format.Rmask = 0xff;
+   format.Gmask = 0xff00;
+   format.Bmask = 0xff0000;
+   format.Amask = 0xff000000;
+   format.colorkey = 0;
+   format.alpha = 0;
+   
+   
+   /* Get available fullscreen/hardware modes */
+   modes=SDL_ListModes(&format, SDL_FULLSCREEN);
+
+
+   listedmodes.push_back( make_pair( 0, 0));
+   
+   /* Check is there are any modes available */
+   if(modes == (SDL_Rect **)0){
+      return;
+   }
+   
+   /* Check if our resolution is restricted */
+   if(modes == (SDL_Rect **)-1){
+      warningMessage("All resolutions available.\n");
+      return;
+   }
+   else{
+      for(i=0;modes[i];++i) {
+
+         if ( find ( listedmodes.begin(), listedmodes.end(), make_pair( int(modes[i]->w), int(modes[i]->h ))) != listedmodes.end() )
+            continue;
+         
+         if ( modes[i]->w >= 800 && modes[i]->h >= 600 ) {
+            listedmodes.push_back( make_pair( int(modes[i]->w), int(modes[i]->h )));
+         }
+      }
+      sort ( listedmodes.begin(), listedmodes.end(), &GetVideoModes::comparator );
+
+      for ( vector <ModeRes > ::iterator i = listedmodes.begin(); i != listedmodes.end(); ++i ) {
+         ASCString s;
+         if (  i->first )
+            s.format( "%d*%d", i->first, i->second );
+         else 
+            s = "graphic mode not listed"; 
+         
+         list.push_back ( s );
+      }
+
+   }
+   return;
 };
 
+int GetVideoModes::getx( int index ) {
+   return listedmodes.at(index).first;
+};
+
+int GetVideoModes::gety( int index ) {
+   return listedmodes.at(index).second;
+};
+
+int GetVideoModes::findmodenum( int x, int y ) {
+   for ( int j = 0; j < listedmodes.size(); ++j )
+      if ( listedmodes[j].first == x && listedmodes[j].second == y )
+         return j;
+   return 0;
+}
+      
 
 
 
@@ -219,6 +261,7 @@ class EditGameOptions : public ASC_PG_Dialog {
             new PG_PropertyField_Checkbox<bool>( propertyEditor, "Game running Fullscreen", &o->forceWindowedMode, true );
             new PG_PropertyField_Checkbox<bool>( propertyEditor, "Produce Ammo when refuelling", &o->autoproduceammunition );
             new PG_PropertyField_IntDropDown<int>( propertyEditor, "InfoPanels", &o->panelColumns, infoPanelNames );
+            new PG_PropertyField_Integer<int>( propertyEditor, "Aircraft Crash Warning Time", &o->aircraftCrashWarningTime );
          } else {
             new PG_PropertyField_Checkbox<bool>( propertyEditor, "MapEd running Fullscreen", &o->mapeditWindowedMode, true );
             new PG_PropertyField_Checkbox<bool>( propertyEditor, "Modal Selection Window", &o->maped_modalSelectionWindow );
@@ -250,8 +293,8 @@ class EditGameOptions : public ASC_PG_Dialog {
          if ( mainApp ) 
             new PG_PropertyField_String<ASCString>( propertyEditor , "Startup Map", &o->startupMap );
 
-         if ( mainApp ) 
-            (new PG_PropertyField_String<ASCString>( propertyEditor , "Default Password", &defaultPassword ))->SetPassHidden('*');
+         //if ( mainApp ) 
+         (new PG_PropertyField_String<ASCString>( propertyEditor , "Default Password", &defaultPassword ))->SetPassHidden('*');
 
          new PG_PropertyField_Checkbox<bool>( propertyEditor, "DEV: Cache GUI Definition (*.ascgui)", &o->cacheASCGUI );
          new PG_PropertyField_Checkbox<bool>( propertyEditor, "DEV: View own replay", &o->debugReplay );
@@ -259,10 +302,17 @@ class EditGameOptions : public ASC_PG_Dialog {
          if ( mainApp )  {
             new PG_PropertyField_Checkbox<bool>( propertyEditor, "Replays as Movies (not saved)", &o->replayMovieMode );
 
+            new PG_PropertyField_Checkbox<bool>( propertyEditor, "Record Campaign Map solutions", &o->recordCampaignMaps );
+            
             new PG_PropertyField_String<ASCString>( propertyEditor, "PBEM server hostname", &o->pbemServer.hostname );
             new PG_PropertyField_String<ASCString>( propertyEditor, "PBEM server username", &o->pbemServer.username );
             new PG_PropertyField_Integer<int>( propertyEditor, "PBEM server port", &o->pbemServer.port);
          }
+         
+         (new PG_PropertyField_String<ASCString>( propertyEditor , "Language Override", &o->languageOverride ));
+         
+         if ( !mainApp )
+            new PG_PropertyField_Checkbox<bool>( propertyEditor, "DEV: Save event message seperately", &o->saveEventMessagesExternal );
 
 
         
