@@ -29,6 +29,31 @@
 
 extern void editGameOptions ( bool mainApp = true );
 
+class GetVideoModes {
+   public:
+      typedef vector<ASCString> VList;
+   private:
+      SDL_Rect **modes;
+      VList list;
+
+      typedef pair<int,int> ModeRes;
+      vector <ModeRes > listedmodes;
+     
+      static bool comparator( const ModeRes& a, const ModeRes& b );
+
+   public:
+      GetVideoModes();
+
+      VList& getList() { return list; };
+
+      int getx( int index );
+      
+      int gety( int index );
+
+      int findmodenum( int x, int y );
+    
+};
+
 
 #endif
 
