@@ -20,6 +20,8 @@
 
 #include <sstream>
 #include <pgimage.h>
+#include <pglistbox.h>
+#include <pglistboxitem.h>
 
 #include "newgame.h"
 #include "editmapparam.h"
@@ -34,15 +36,35 @@
 #include "../viewcalculation.h"
 #include "../networkinterface.h"
 #include "../spfst.h"
+#include "../spfst-legacy.h"
 #include "../windowing.h"
 #include "../controls.h"
 #include "../sg.h"
 #include "../gamedlg.h"
 #include "../network/simple_file_transfer.h"
 #include "../network/pbem-server.h"
+#include "../textfile_evaluation.h"
+#include "../textfileparser.h"
+#include "../widgets/textrenderer.h"
+#include "../mapdisplayinterface.h"
+#include "../campaignactionrecorder.h"
 
 class GameParameterEditorWidget;
 
+const int pageCount = 11;
+
+class Campaign {
+   public:
+      ASCString name;
+      ASCString description;
+      ASCString file;
+      
+      void runTextIO ( PropertyContainer& pc ) {
+         pc.addString ( "name", name);
+         pc.addString ( "description", description );
+         pc.addString ( "file", file );
+      }
+};
 
 class StartMultiplayerGame: public ConfigurableWindow {
    private:
@@ -50,10 +72,10 @@ class StartMultiplayerGame: public ConfigurableWindow {
       bool success;
       GameMap* newMap;
       
-      enum Pages { ModeSelection = 1, FilenameSelection, PlayerSetup, EmailSetup, AllianceSetup, MapParameterEditor, MultiPlayerOptions, PasswordSearch, PBEMServerSetup }; 
+      enum Pages { ModeSelection = 1, FilenameSelection, PlayerSetup, EmailSetup, AllianceSetup, MapParameterEditor, MultiPlayerOptions, PasswordSearch, PBEMServerSetup, CampaignChooser }; 
       Pages page;
      
-      enum Mode { NewCampagin, ContinueCampaign, Skirmish, Hotseat, PBEM, PBP, PBEM_Server };
+      enum Mode { NewCampaign, ContinueCampaign, Skirmish, Hotseat, PBEM, PBP, PBEM_Server };
       int mode;
       
       static const char* buttonLabels[];
@@ -69,6 +91,7 @@ class StartMultiplayerGame: public ConfigurableWindow {
       
       AllianceSetupWidget* allianceSetup;
       PG_Widget* allianceSetupParent;
+      DirectAllianceSetupStrategy directAllianceSetupStrategy;
       
       PlayerSetupWidget* playerSetup;
       PG_Widget* playerSetupParent;
@@ -86,6 +109,12 @@ class StartMultiplayerGame: public ConfigurableWindow {
       PBEMServer* pbemserver;
       
       void loadPBEMServerDefaults();
+
+      typedef deallocating_vector<Campaign*> Campaigns;
+      Campaigns campaigns;
+      void loadCampaigns();
+      Campaign* getSelectedCampaign();
+      bool campaignSelected();
       
       void fileNameSelected( const ASCString& filename )
       {
@@ -111,12 +140,12 @@ class StartMultiplayerGame: public ConfigurableWindow {
 
       void showSupervisorWidgets()
       {
-         if ( page == MultiPlayerOptions )
+         if ( page == MultiPlayerOptions ) {
             if ( supervisorEnabled )
                show( "SuperVisor" );
             else
                hide( "SuperVisor" );
-
+         }
       }
       
       bool togglesupervisor( bool on )
@@ -230,17 +259,18 @@ StartMultiplayerGame::StartMultiplayerGame(PG_MessageObject* c): ConfigurableWin
     
     
     PG_CheckButton* cb = dynamic_cast<PG_CheckButton*>( FindChild( "Replay", true ));
-    if ( cb )
+    if ( cb ) {
        if ( replay )
          cb->SetPressed( );
        else
          cb->SetUnpressed();
-
+    }
     
     showPage();
     showButtons(false,false,true);
     
     loadPBEMServerDefaults();
+    loadCampaigns();
 }
 
 
@@ -253,6 +283,68 @@ StartMultiplayerGame::~StartMultiplayerGame()
       delete pbemserver;
 }      
 
+bool StartMultiplayerGame::campaignSelected()
+{
+   
+   TextRenderer* text = dynamic_cast<TextRenderer*>( FindChild( "CampaignDescription", true ));
+   if ( text ) {
+      Campaign* c = getSelectedCampaign();
+      if ( c )
+         text->SetText( c->description );
+   }
+   return true;
+}
+
+void StartMultiplayerGame::loadCampaigns()
+{
+   PG_ListBox* list = dynamic_cast<PG_ListBox*>( FindChild( "CampaignList", true ));
+   if ( !list ) 
+      return;
+   
+   list->sigSelectItem.connect( SigC::slot( *this, &StartMultiplayerGame::campaignSelected ));
+   
+   
+   tfindfile ff ( "*.asccampaign" );
+   tfindfile::FileInfo fi;
+   while ( ff.getnextname( fi) ) {
+      try {
+         tnfilestream f ( fi.name, tnstream::reading );
+         TextFormatParser parser(&f);
+         
+         PropertyReadingContainer prc ( "campaign", parser.run());
+         Campaign* c = new Campaign();
+         c->runTextIO(prc);
+   
+         campaigns.push_back ( c );
+         
+         if ( list )
+            new PG_ListBoxItem( list, 20, c->name );
+         
+      } catch ( ParsingError pe ) {
+         errorMessage( pe.getMessage( ));
+      }
+   }
+}
+
+
+Campaign* StartMultiplayerGame::getSelectedCampaign()
+{
+   PG_ListBox* list = dynamic_cast<PG_ListBox*>( FindChild( "CampaignList", true ));
+      
+   if ( list )
+      for ( int i = 0; i < list->GetWidgetCount(); ++i ) {
+         PG_ListBoxBaseItem* bi = dynamic_cast<PG_ListBoxBaseItem*>(list->FindWidget(i));
+         if ( bi && bi->IsSelected() )
+            for ( Campaigns::iterator i = campaigns.begin(); i != campaigns.end(); ++i )
+               if ( (*i)->name == bi->GetText() )
+                  return *i;
+      }
+
+   
+   return NULL;
+}
+
+
 bool StartMultiplayerGame::Apply()
 {
    switch ( page )  {
@@ -262,7 +354,7 @@ bool StartMultiplayerGame::Apply()
                   delete newMap;
                   newMap = mapLoadingExceptionChecker( filename, MapLoadingFunction( tmaploaders::loadmap ));
                   if ( newMap ) {
-                     if ( mode != NewCampagin && mode != ContinueCampaign )
+                     if ( mode != NewCampaign && mode != ContinueCampaign )
                         newMap->campaign.avail = false;
                      else
                         replay = false;
@@ -323,7 +415,7 @@ bool StartMultiplayerGame::Apply()
                   Password pwd2;
                   pwd2.setEncoded( s2->GetText() );
                   if ( pwd != pwd2 ) {
-                     warning ( "Passwords don't match!");
+                     warningMessage ( "Passwords don't match!");
                      return false;
                   }
                }
@@ -366,6 +458,7 @@ bool StartMultiplayerGame::Apply()
                      return false;
                   }
                   newMap = map;
+                  replay = false;
                   // start();
                   return true;
                }
@@ -373,6 +466,37 @@ bool StartMultiplayerGame::Apply()
          }
          break;
 
+      case CampaignChooser: {
+         Campaign* c =getSelectedCampaign();
+         if ( c ) {
+            if ( c->file.empty() )
+               return false;
+            
+            tfindfile ff ( c->file );
+            if ( ff.getFoundFileNum() < 1 )
+               return false;
+            
+            if ( ff.getFoundFileNum() == 1 ) {
+               tfindfile::FileInfo fi;
+               if ( !ff.getnextname( fi) )
+                  return false;
+               
+               filename = fi.name;
+               replay = false;
+               return true;
+            } else {
+               filename =  selectFile( c->file, true );
+               if ( filename.empty() ) {
+                  return false;
+               } else {
+                  replay = false;
+                  return true;
+               }
+            }
+         } else 
+            return false;
+      }
+         
       default: 
            break;
    }
@@ -420,7 +544,10 @@ bool StartMultiplayerGame::nextPage(PG_Button* button)
             if ( mode == ContinueCampaign )
                page = PasswordSearch;
             else
-               page = FilenameSelection;
+               if ( mode == NewCampaign )
+                  page = CampaignChooser;
+               else
+                  page = FilenameSelection;
          }
          break;
          
@@ -429,12 +556,12 @@ bool StartMultiplayerGame::nextPage(PG_Button* button)
                page = PlayerSetup;
             break;
       case PlayerSetup: 
-            if ( Apply() )
+            if ( Apply() ) {
                if ( mode == PBEM || mode == PBP || mode == PBEM_Server )
                   page = EmailSetup;
                else
                   page = AllianceSetup;
-                  
+            }      
             break;       
       case EmailSetup: 
             if ( Apply() ) {
@@ -476,7 +603,7 @@ bool StartMultiplayerGame::nextPage(PG_Button* button)
       
       if ( page == AllianceSetup && allianceSetupParent ) {
          delete allianceSetup;
-         allianceSetup = new AllianceSetupWidget( newMap, true, allianceSetupParent, PG_Rect( 0, 0, allianceSetupParent->Width(), allianceSetupParent->Height() ));
+         allianceSetup = new AllianceSetupWidget( newMap, &directAllianceSetupStrategy, true, allianceSetupParent, PG_Rect( 0, 0, allianceSetupParent->Width(), allianceSetupParent->Height() ));
       }   
 
       if ( page == MapParameterEditor && mapParameterEditorParent ) {
@@ -521,7 +648,7 @@ void StartMultiplayerGame::loadPBEMServerDefaults()
 
 void StartMultiplayerGame::showPage()
 {
-   for ( int i = 0; i < 10; ++i ) {
+   for ( int i = 0; i < pageCount; ++i ) {
       ASCString name = "Page" + ASCString::toString(i);
       if ( page == i )
          show( name );
@@ -531,8 +658,12 @@ void StartMultiplayerGame::showPage()
    showSupervisorWidgets();
    
    switch ( page ) {
+      case CampaignChooser:
+         showButtons(true,false,false);
+         break;
+   
       case FilenameSelection: 
-         if ( mode == NewCampagin || mode == ContinueCampaign )
+         if ( mode == NewCampaign || mode == ContinueCampaign )
             showButtons(true,false,false);
          else
             showButtons(false,true,true);
@@ -585,7 +716,7 @@ void StartMultiplayerGame::userHandler( const ASCString& label, PropertyReadingC
 bool StartMultiplayerGame::checkPlayerStat()
 {
    ASCString msg;
-   if ( mode == NewCampagin || mode == ContinueCampaign ) {
+   if ( mode == NewCampaign || mode == ContinueCampaign ) {
       bool humanFound = false;
       for ( int i = 0; i < newMap->getPlayerCount(); ++i )
          if ( newMap->player[i].exist() )
@@ -593,7 +724,7 @@ bool StartMultiplayerGame::checkPlayerStat()
                humanFound = true;
 
       if ( !humanFound ) {
-         warning("no human players on map!");
+         warningMessage("no human players on map!");
          return false;
       }
       return true;
@@ -603,13 +734,13 @@ bool StartMultiplayerGame::checkPlayerStat()
       bool humanFound = false;
       for ( int i = 0; i < newMap->getPlayerCount(); ++i )
          if ( newMap->player[i].exist() )
-            if ( newMap->player[i].stat == Player::human || newMap->player[i].stat == Player::supervisor )
+            if ( newMap->player[i].stat == Player::human || newMap->player[i].stat == Player::supervisor ) {
                if ( humanFound ) {
                   newMap->player[i].stat = Player::computer;
                   msg += newMap->player[i].getName() + " has been switch to AI\n";
                } else
                   humanFound = true;   
-                  
+            }      
       if ( !humanFound )            
          for ( int i = 0; i < newMap->getPlayerCount(); ++i )
             if ( newMap->player[i].exist() )
@@ -770,15 +901,17 @@ bool StartMultiplayerGame::start()
    computeview( actmap );
    hookGuiToMap(actmap);
   
-   next_turn();
+   next_turn( actmap, NextTurnStrategy_AskUser(), &getDefaultMapDisplay() );
    displaymap();
    updateFieldInfo();
-   moveparams.movestatus = 0;
    success = true;
 
    if ( mode == PBP || mode == PBEM ) 
       sendGameParameterAsMail ( actmap );
 
+   if ( (mode == NewCampaign || mode == ContinueCampaign) && CGameOptions::Instance()->recordCampaignMaps ) 
+      actmap->actionRecorder = new  CampaignActionLogger( actmap );
+  
    
    return true;
 }
@@ -796,7 +929,7 @@ bool startMultiplayerGame()
     smg.Hide();
    }
    catch ( ShutDownMap sdm ) {
-      warning("No players found on map");
+      warningMessage("No players found on map");
       throw sdm;
    }
     /*
