@@ -9,12 +9,14 @@
  ***************************************************************************/
 
 #include <pgimage.h>
+#include <pgeventsupplier.h>
 #include "../paradialog.h"
 #include "../typen.h"
 #include "../vehicletype.h"
 #include "../vehicle.h"
 #include "../iconrepository.h"
 #include "../spfst.h"
+#include "../spfst-legacy.h"
 #include "../dialog.h"
 
 #include "../textfiletags.h"
@@ -87,8 +89,10 @@ class UnitInfoDialog : public Panel {
          void registerSpecialInput( const ASCString& name )
          {
             SpecialInputWidget* siw = dynamic_cast<SpecialInputWidget*>( FindChild( name, true ) );
-            if ( siw )
+            if ( siw ) {
                siw->sigMouseButtonDown.connect( SigC::slot( *this, &UnitInfoDialog::onClick ));
+               siw->sigMouseButtonUp.connect( SigC::slot( *this, &UnitInfoDialog::onRelease ));
+            }
          };
 
          bool onClick ( PG_MessageObject* obj, const SDL_MouseButtonEvent* event ) {
@@ -100,6 +104,16 @@ class UnitInfoDialog : public Panel {
             return false;
          };
 
+         bool onRelease ( PG_MessageObject* obj, const SDL_MouseButtonEvent* event ) {
+            PG_Widget* w = dynamic_cast<PG_Widget*>(obj);
+            if ( w ) {
+               release( w->GetName() );
+               return true;
+            } 
+            return false;
+         };
+         
+         
          bool onEntranceClick ( PG_MessageObject* obj, const SDL_MouseButtonEvent* event, int entranceNum ) {
             if ( vt ) {
                PG_Widget* swi = FindChild( "single_weapon_info", true );
@@ -203,11 +217,12 @@ class UnitInfoDialog : public Panel {
             Surface screen = Surface::Wrap( PG_Application::GetScreen() );
 
             if ( name == "unitpad_unitsymbol" ) 
-               if ( vt )
+               if ( vt ) {
                   if ( veh ) 
                      vt->paint( screen, SPoint( dst.x, dst.y ), veh->getMap()->getPlayer(veh).getPlayerColor() );
                   else
                      vt->paint( screen, SPoint( dst.x, dst.y ));
+               }
 
             if ( name == "unitpad_weapon_diagram" ) {
                if ( vt && weaponGraph ) {
@@ -279,12 +294,17 @@ class UnitInfoDialog : public Panel {
                    show( paneName[i] );
          };
 
+         void release( const ASCString& name ) {
+            if ( name == "padclick_exit" ) {
+               QuitModal();
+            }
+            
+         }
+         
          void click( const ASCString& name ) {
             for ( int i = 0; i < paneNum; ++i)
                if ( name == ASCString("padclick_") + paneName[i] ) 
                   activate(paneName[i]);
-            if ( name == "padclick_exit" )
-               QuitModal();
          };
 
      public:
