@@ -5,10 +5,13 @@
      which was never finished.
 */
 
-//     $Id: dlgraph.cpp,v 1.15 2007-04-13 16:15:53 mbickel Exp $
+//     $Id: dlgraph.cpp,v 1.16 2008-12-20 14:25:47 mbickel Exp $
 //
 //     $Log: dlgraph.cpp,v $
-//     Revision 1.15  2007-04-13 16:15:53  mbickel
+//     Revision 1.16  2008-12-20 14:25:47  mbickel
+//      Merged Undo-Branch into Trunk
+//
+//     Revision 1.15  2007/04/13 16:15:53  mbickel
 //      Merged ASC2 branch
 //
 //     Revision 1.13.2.1  2006/03/01 21:00:50  mbickel
@@ -145,10 +148,10 @@ void         tdialogbox::knopfdruck(int      xx1,
      mt = mouseparams.taste;
      mousevisible(false);
 
-     void*    pq = asc_malloc ( imagesize ( xx1 , yy1 , xx2 , yy2 ));
+     void*    pq = malloc ( imagesize ( xx1 , yy1 , xx2 , yy2 ));
      getimage( xx1 , yy1 , xx2 , yy2 , pq );
 
-     p = asc_malloc ( imagesize ( xx1 + 1,yy1 + 1,xx2 - 2,yy2 - 2 ));
+     p = malloc ( imagesize ( xx1 + 1,yy1 + 1,xx2 - 2,yy2 - 2 ));
      getimage(xx1 + 1,yy1 + 1,xx2 - 2,yy2 - 2, p);
      putimage(xx1 + 3,yy1 + 3, p );
      rahmen(true, xx1, yy1, xx2, yy2); 
@@ -167,10 +170,10 @@ void         tdialogbox::knopfdruck(int      xx1,
      mousevisible(false); 
 
      putimage(xx1 + 1,yy1 + 1, p);
-     asc_free ( p );
+     free ( p );
 
      putimage( xx1 , yy1 , pq );
-     asc_free ( pq );
+     free ( pq );
 
      mousevisible(true); 
 } 
@@ -204,7 +207,7 @@ void         tdialogbox::newknopfdruck2(int      xx1,
        collategraphicoperations cgs ( xx1 -1, yy1 -1, xx2 + 1, yy2 + 1 );
        mt = mouseparams.taste;
        mousevisible(false);
-       p = asc_malloc ( imagesize ( xx1 + 1,yy1 + 1,xx2 - 2,yy2 - 2 ) );
+       p = malloc ( imagesize ( xx1 + 1,yy1 + 1,xx2 - 2,yy2 - 2 ) );
        getimage(xx1 + 1,yy1 + 1,xx2 - 2,yy2 - 2, p);
        putimage(xx1 + 3,yy1 + 3, p);
 
@@ -238,7 +241,7 @@ void         tdialogbox::newknopfdruck2(int      xx1,
 
        if (knopfsuccessful == false)
           putimage (xx1 + 1,yy1 + 1,p);
-       asc_free ( p );
+       free ( p );
 
        if (knopfsuccessful == false) {
           line(xx1 - 1,yy1 - 1,xx2 + 1,yy1 - 1, 8);
@@ -268,11 +271,11 @@ void         tdialogbox::newknopfdruck3(int      xx1,
 
      // vom reingedr?ckten in den Normalzustand
 
-     p = asc_malloc ( imagesize ( xx1 + 4,yy1 + 4,xx2 - 2,yy2 - 2 ));
+     p = malloc ( imagesize ( xx1 + 4,yy1 + 4,xx2 - 2,yy2 - 2 ));
      getimage(xx1 + 4,yy1 + 4,xx2 - 2,yy2 - 2, p );
 
      putimage(xx1 + 2,yy1 + 2,p);
-     asc_free ( p );              
+     free ( p );              
 
      rahmen(false,xx1,yy1,xx2,yy2); 
 } 
@@ -290,10 +293,10 @@ void         tdialogbox::newknopfdruck4(int      xx1,
    void*      p;
     // reindr?cken
 
-     p = asc_malloc ( imagesize ( xx1 + 2,yy1 + 2,xx2 - 4,yy2 - 4 )) ;
+     p = malloc ( imagesize ( xx1 + 2,yy1 + 2,xx2 - 4,yy2 - 4 )) ;
      getimage(xx1 + 2,yy1 + 2,xx2 - 4,yy2 - 4, p);
      putimage(xx1 + 4,yy1 + 4, p);
-     asc_free ( p );
+     free ( p );
 
      rahmen(true,xx1,yy1,xx2,yy2); 
 } 
@@ -314,11 +317,11 @@ void         tdialogbox::newknopfdruck(int      xx1,
      mt = mouseparams.taste;
      mousevisible(false); 
 
-     pq = asc_malloc ( imagesize ( xx1 , yy1 , xx2 , yy2 ));
+     pq = malloc ( imagesize ( xx1 , yy1 , xx2 , yy2 ));
      getimage( xx1 , yy1 , xx2 , yy2 , pq );
 
 
-     p = asc_malloc ( imagesize ( xx1 + 1,yy1 + 1,xx2 - 2,yy2 - 2 ));
+     p = malloc ( imagesize ( xx1 + 1,yy1 + 1,xx2 - 2,yy2 - 2 ));
      getimage(xx1 + 1,yy1 + 1,xx2 - 2,yy2 - 2, p);
      putimage(xx1 + 3,yy1 + 3, p);
 
@@ -351,7 +354,7 @@ void         tdialogbox::newknopfdruck(int      xx1,
      collategraphicoperations cgo( xx1-1, yy1-1, xx2+1, yy2+1 );
      mousevisible(false); 
      putimage(xx1 + 1,yy1 + 1,p);
-     asc_free ( p ) ;
+     free ( p ) ;
 
      line(xx1 - 1,yy1 - 1,xx2 + 1,yy1 - 1, 8);
      line(xx1 - 1,yy1 - 1,xx1 - 1,yy2 + 1, 8);
@@ -366,7 +369,7 @@ void         tdialogbox::newknopfdruck(int      xx1,
      line(xx1,yy2,xx2,yy2, 8);
 
      putimage( xx1 , yy1 , pq );
-     asc_free ( pq );
+     free ( pq );
 
      mousevisible(true);
    }
