@@ -27,7 +27,7 @@
 
 extern int infomessage( char* formatstring, ... );
 
-const int execactionscount = 109;
+const int execactionscount = 112;
 
 extern const char*  execactionnames[execactionscount];
 
@@ -138,12 +138,18 @@ enum tuseractions {
      act_dumpAllBuildings,
      act_locateItemByID,
      act_mirrorMap,
-		 act_copyArea,
-		 act_pasteArea
+     act_copyArea,
+     act_pasteArea,
+     asc_nullPointerCrash,
+     asc_testFunction,
+     act_unitGuideDialog,
+     act_runLuaScript,
+     act_runTranslationScript
+
  };
 
 class SelectionHolder : public SigC::Object {
-     const MapComponent* currentItem;
+     const Placeable* currentItem;
      int actplayer;
      int currentWeather;
      
@@ -158,11 +164,11 @@ class SelectionHolder : public SigC::Object {
      int getWeather() { return currentWeather; };
      
      int brushSize;
-     const MapComponent* getSelection();
-     void setSelection( const MapComponent& component ) ;
+     const Placeable* getSelection();
+     void setSelection( const Placeable& component ) ;
      void pickup ( tfield* fld );
      
-     SigC::Signal1<void,const MapComponent*> selectionChanged;
+     SigC::Signal1<void,const Placeable*> selectionChanged;
      void clear(); 
 };
  
