@@ -37,6 +37,7 @@
 
 #include "dialogs/vehicletypeselector.h"
 #include "unitset.h"
+#include "spfst-legacy.h"
 
 SigC::Signal0<void> filtersChangedSignal;
 
@@ -130,26 +131,25 @@ void MapComponent::displayClip( PG_Widget* parent, SDL_Surface * surface, const
 
 
 
-// template<class Vehicletype> Surface BasicItem<Vehicletype>::clippingSurface;
-int VehicleItem::place( const MapCoordinate& mc ) const
+int VehicleItem::place( GameMap* gamemap, const MapCoordinate& mc, const Vehicletype* v, int owner )
 {
-   tfield* fld = actmap->getField(mc);
+   tfield* fld = gamemap->getField(mc);
    if ( !fld )
       return -1;
 
-   const Vehicletype* veh = item;
+   const Vehicletype* veh = v;
    
    if ( !veh )
       return -2;
      
-   if ( selection.getPlayer()  == 8 )
+   if ( owner  >= 8 )
       return -3;
 
    bool accessible = veh->terrainaccess.accessible ( fld->bdt );
    if ( veh->height >= chtieffliegend )
          accessible = true;
 
-   if ( fld->building  || ( !accessible && !actmap->getgameparameter( cgp_movefrominvalidfields)) ) 
+   if ( fld->building  || ( !accessible && !gamemap->getgameparameter( cgp_movefrominvalidfields)) ) 
       return -3;
    
    
@@ -158,18 +158,18 @@ int VehicleItem::place( const MapCoordinate& mc ) const
          delete fld->vehicle;
          fld->vehicle = NULL;
       } else {
-         fld->vehicle->convert( selection.getPlayer()  );
+         fld->vehicle->convert( owner  );
          return 1;
       }
    }
-   fld->vehicle = new Vehicle ( veh, actmap, selection.getPlayer()  );
+   fld->vehicle = new Vehicle ( v, gamemap, owner  );
    fld->vehicle->setnewposition ( mc );
    fld->vehicle->fillMagically();
    
    for ( int i = 0; i < 9; ++i ) {
       if ( fld->vehicle->typ->height & (1 << i )) {
          fld->vehicle->height = 1 << i;
-         if ( terrainaccessible( fld, fld->vehicle ) == 2 ) //  || actmap->getgameparameter( cgp_movefrominvalidfields)
+         if ( terrainaccessible( fld, fld->vehicle ) == 2 ) //  || gamemap->getgameparameter( cgp_movefrominvalidfields)
             break;
       }
       if ( i == 8 ) { // no height found
@@ -183,11 +183,18 @@ int VehicleItem::place( const MapCoordinate& mc ) const
    
 }
 
+int VehicleItem::place( const MapCoordinate& mc ) const
+{
+   return place( actmap, mc, item, selection.getPlayer() );
+}
 
 Surface BuildingItem::clippingSurface;
 Surface BuildingItem::fullSizeImage;
 int BuildingItem::place( const MapCoordinate& mc ) const
 {
+   if ( selection.getPlayer()  > 8 )
+      return -3;
+   
    int f = 0;
    for ( int x = 0; x < 4; x++ )
       for ( int y = 0; y < 6; y++ )
@@ -204,7 +211,7 @@ int BuildingItem::place( const MapCoordinate& mc ) const
          return -1 ;
    }      
 
-   putbuilding( actmap->getCursor(), selection.getPlayer()  * 8, bld, bld->construction_steps );
+   putbuilding( actmap, actmap->getCursor(), selection.getPlayer()  * 8, bld, bld->construction_steps );
    return 0;
 }
 
@@ -238,8 +245,8 @@ int ObjectItem::place( const MapCoordinate& mc ) const
 
 bool ObjectItem::remove ( const MapCoordinate& mc ) const
 {
-   if ( item && actmap->getField(mc)->checkforobject(item) ) {
-      actmap->getField(mc)->removeobject ( item );
+   if ( item && actmap->getField(mc)->checkForObject(item) ) {
+      actmap->getField(mc)->removeObject ( item );
       return true;
    }
    return false;
@@ -252,7 +259,7 @@ int TerrainItem::place( const MapCoordinate& mc ) const
 {
    tfield* fld = actmap->getField(mc);
    fld->typ = item->weather[0]; 
-   fld->setweather( selection.getWeather() );
+   fld->setWeather( selection.getWeather() );
    fld->setparams();
    for ( int d = 0; d < 6; ++d ) {
       MapCoordinate pos = getNeighbouringFieldCoordinate( mc, d );
@@ -267,7 +274,7 @@ int TerrainItem::place( const MapCoordinate& mc ) const
 
 int MineItem::place( const MapCoordinate& mc ) const
 {
-   actmap->getField(mc)->putmine( selection.getPlayer(), item->id, MineBasePunch[item->id-1] );
+   actmap->getField(mc)->putmine( selection.getPlayer(), MineTypes(item->id), MineBasePunch[item->id-1] );
    return 0;
 }
 
