@@ -28,7 +28,6 @@
 
 class GameMap;
 
-extern void  choosetechnology(void);
 extern void  settributepayments ( void );
 
 
@@ -37,8 +36,9 @@ extern void choosetechlevel ( void );
 
 
 class tfield;
+class Context;
 
-extern void giveunitaway ( tfield* fld );
+extern void giveunitaway ( tfield* fld, const Context& context );
 
 extern void showGameParameters ( void );
 
