@@ -2,7 +2,7 @@
     \brief Interface to game event dialogs
 */
 
-//     $Id: gameevent_dialogs.h,v 1.6 2007-09-08 14:13:51 mbickel Exp $
+//     $Id: gameevent_dialogs.h,v 1.9 2009-05-01 12:24:45 mbickel Exp $
 
 /*
     This file is part of Advanced Strategic Command; http://www.asc-hq.de
@@ -28,38 +28,22 @@
 #ifndef gameevent_dialogsH
 #define gameevent_dialogsH
 
+#include <list>
+
 #include "mappolygons.h"
 #include "gameevents.h"
 #include "dlg_box.h"
 #include "research.h"
+#include "researchexecution.h"
 
 #include "dialogs/fieldmarker.h"
 
-    //! checks, which vehicle types are newly available
-    class NewVehicleTypeDetection  {
-                    bool*  buf;
-                public:
-                    NewVehicleTypeDetection( );
 
-                    //! checks, which vehicle are now available that where not available when initbuffer was called. The new ones are displayed by calling #tshownewtanks
-                    void evalbuffer( void );
+class ShowNewTechnology : public TechnologyPresenter {
+   public:
+      virtual void showTechnology( const Technology* tech, const Gadgets& newGadgetsAvailable );
+};
 
-                    ~NewVehicleTypeDetection();
-               };
-
-     //! displays a list of vehicles that are newly available
-     class   tshownewtanks : public tdialogbox {
-                         public:
-                           void          init ( bool*      buf2 );
-                           virtual void  run  ( void );
-                           virtual void  buttonpressed ( int id );
-                         protected:
-                           bool*      buf;
-                           int           status;
-                        };
-
-
-extern void showtechnology( const Technology* t);
 extern void selectFields( FieldAddressing::Fields& fields );
 extern bool chooseWeather( int& weather );
 extern bool chooseTerrain( int& terrainID );
@@ -74,8 +58,12 @@ class ReinforcementSelector : public SelectFromMap {
    private:
       tmemorystreambuf& buf;
       int& objectNum;
+      void cut( const MapCoordinate& pos );
+      CoordinateList cutPositions;
+
    protected:
       bool mark();
+      virtual bool isOk();
    public:
       ReinforcementSelector( CoordinateList& list, GameMap* map, tmemorystreambuf& buffer, int& objNum ) : SelectFromMap( list, map ), buf( buffer), objectNum(objNum) {};
 };
