@@ -32,9 +32,7 @@
 #include "global.h"
 #include "buildingtype.h"
 #include "vehicletype.h"
-#include "newfont.h"
 #include "typen.h"
-// #include "basegfx.h"
 
 #include "gameevent_dialogs.h"
 #include "spfst.h"
@@ -47,36 +45,16 @@
 #include "messagedlg.h"
 #include "mapdisplayinterface.h"
 
+#include "actions/diplomacycommand.h"
+
 #ifdef sgmain
-// # include "gamedlg.h"
 # include "viewcalculation.h"
 # include "resourcenet.h"
-# include "unitctrl.h"
+# include "sg.h"
+# include "turncontrol.h"
 #endif
 
 
-void    viewtextmessage ( int id, int player )
-{
-   ASCString txt = readtextmessage( id );
-   if ( !txt.empty() ) {
-
-      int to;
-      if ( player < 8 )
-         to = (1 << player);
-      else
-         to = 0xff;
-
-      new Message ( txt, actmap, to );
-      #ifdef sgmain
-      if ( player == actmap->actplayer )
-         viewunreadmessages ( actmap->player[ actmap->actplayer ] );
-      #endif
-   } else
-      displaymessage( "Message %d not found", 1, id );
-}
-
-
-
 
 /////////////////////////////////////////////////////////////////////////////
 // Trigger
@@ -208,7 +186,7 @@ void BuildingConquered::arm()
 void BuildingConquered::triggered()
 {
    if ( isFulfilled() )
-      eventReady();
+      eventReady( gamemap );
 }
 
 
@@ -285,7 +263,7 @@ EventTrigger::State BuildingSeen::getState( int player )
             tfield* fld = bld->getField ( BuildingType::LocalCoordinate( x, y) );
             if ( fld ) {
                int vis = (fld-> visible >> (player*2) ) & 3;
-               if ( bld->typ->buildingheight >= chschwimmend && bld->typ->buildingheight <= chhochfliegend ) {
+               if ( bld->typ->height >= chschwimmend && bld->typ->height <= chhochfliegend ) {
                   if ( vis >= visible_now )
                      cnt++;
                } else {
@@ -321,7 +299,7 @@ void BuildingSeen::arm()
 void BuildingSeen::triggered()
 {
    if ( isFulfilled() )
-      eventReady();
+      eventReady( gamemap );
 }
 
 
@@ -395,7 +373,7 @@ void UnitTrigger::setup()
 void UnitTrigger::triggered()
 {
    state( -1 );
-   eventReady();
+   eventReady( gamemap );
 }
 
 
@@ -576,7 +554,7 @@ void EventTriggered::arm()
 void EventTriggered::triggered()
 {
    state( -1 );
-   eventReady();
+   eventReady( gamemap );
 }
 
 
@@ -610,14 +588,14 @@ void AllEnemyUnitsDestroyed::arm()
 void AllEnemyUnitsDestroyed::triggered()
 {
    if ( isFulfilled() )
-      eventReady();
+      eventReady( gamemap );
 }
 
 
 EventTrigger::State AllEnemyBuildingsDestroyed::getState( int player )
 {
    for ( int i = 0; i < 8; i++ )
-      if ( actmap->getPlayer(player).diplomacy.isHostile(i))
+      if ( gamemap->getPlayer(player).diplomacy.isHostile(i))
          if ( !gamemap->player[i].buildingList.empty() )
             return unfulfilled;
 
@@ -645,7 +623,7 @@ void AllEnemyBuildingsDestroyed::arm()
 void AllEnemyBuildingsDestroyed::triggered()
 {
    if ( isFulfilled() )
-      eventReady();
+      eventReady( gamemap );
 }
 
 
@@ -663,8 +641,8 @@ void SpecificUnitEntersPolygon::fieldOperator( const MapCoordinate& mc )
 {
    tfield* fld = gamemap->getField ( mc );
    if ( !arming ) {
-      if ( fld && fld->vehicle )
-         if ( fld->vehicle->networkid == unitID || unitID == -1 )
+      if ( fld && fld->getVehicle() )
+         if ( fld->getVehicle()->networkid == unitID || unitID == -1 )
             found = true;
 
       Vehicle* veh = gamemap->getUnit( unitID );
@@ -726,10 +704,10 @@ void SpecificUnitEntersPolygon::arm ()
    #endif
 }
 
-void SpecificUnitEntersPolygon::triggered()
+void SpecificUnitEntersPolygon::triggered( const Context& context )
 {
    if ( isFulfilled() )
-      eventReady();
+      eventReady( gamemap );
 }
 
 EventTrigger::State AnyUnitEntersPolygon::getState( int player )
@@ -746,8 +724,8 @@ void AnyUnitEntersPolygon::fieldOperator( const MapCoordinate& mc )
 {
    tfield* fld = gamemap->getField ( mc );
    if ( !arming ) {
-      if ( fld && fld->vehicle )
-         if ( (1 << fld->vehicle->getOwner()) & player )
+      if ( fld && fld->getVehicle() )
+         if ( (1 << fld->getVehicle()->getOwner()) & player )
             found = true;
 
       if ( fld && fld->building )
@@ -811,10 +789,10 @@ void AnyUnitEntersPolygon::arm()
    #endif
 }
 
-void AnyUnitEntersPolygon::triggered()
+void AnyUnitEntersPolygon::triggered(const Context& context )
 {
    if ( isFulfilled() )
-      eventReady();
+      eventReady( gamemap );
 }
 
 
@@ -875,7 +853,7 @@ void ResourceTribute::arm()
 void ResourceTribute::triggered()
 {
    if ( isFulfilled() )
-      eventReady();
+      eventReady( gamemap );
 }
 
 /////////////////////////////////////////////////////////////////////////////
@@ -973,13 +951,33 @@ void ChangeGameParameter::setup()
 }
 
 
+void  DisplayMessage :: viewtextmessage ( GameMap* actmap, int id, int player )
+{
+   ASCString txt = readtextmessage( id );
+   if ( !txt.empty() ) {
+
+      int to;
+      if ( player < 8 )
+         to = (1 << player);
+      else
+         to = 0xff;
+
+      new Message ( txt, actmap, to );
+#ifdef sgmain
+      if ( player == actmap->actplayer )
+         viewunreadmessages ( actmap->player[ actmap->actplayer ] );
+#endif
+   } else
+      displaymessage( "Message %d not found", 1, id );
+}
+
 
 void DisplayMessage::execute( MapDisplayInterface* md )
 {
    if ( gamemap->state == GameMap::Replay )
       return;
 
-   viewtextmessage ( messageNum , gamemap->actplayer );
+   viewtextmessage ( gamemap, messageNum , gamemap->actplayer );
 }
 
 void DisplayMessage::readData( tnstream& stream )
@@ -1097,6 +1095,9 @@ void FieldAddressing::writeMapModificationData ( tnstream& stream )
 void MapModificationEvent::execute( MapDisplayInterface* md )
 {
    operate();
+#ifdef sgmain
+   computeview( gamemap );
+#endif
    if ( md ) {
       md->displayMap();
       md->updateDashboard();
@@ -1123,7 +1124,7 @@ void WeatherChange :: fieldOperator( const MapCoordinate& mc )
 {
    tfield* field = gamemap->getField ( mc );
    if ( field ) 
-      field->setweather( weather );
+      field->setWeather( weather );
 }
 
 void WeatherChange :: setup ()
@@ -1156,7 +1157,7 @@ void MapChange :: fieldOperator( const MapCoordinate& mc )
 
    tfield* field = gamemap->getField ( mc );
    if ( field ) {
-      int w = field->getweather();
+      int w = field->getWeather();
       if (typ->weather[w] == NULL)
          w = 0;
 
@@ -1243,12 +1244,17 @@ void RemoveAllObjects :: setup ()
 
 void MapChangeCompleted :: execute( MapDisplayInterface* md )
 {
-   checkobjectsforremoval();
-   checkunitsforremoval ();
+   checkobjectsforremoval( gamemap );
+   checkunitsforremoval ( gamemap );
 
    if ( md ) {
-      md->displayMap();
-      md->updateDashboard();
+      mapChanged( gamemap );
+      
+#ifdef sgmain
+      if ( gamemap->getCurrentPlayer().isHuman() )
+         viewunreadmessages ( gamemap->player[ gamemap->actplayer ] );
+#endif
+      
    }
 
 }
@@ -1276,10 +1282,9 @@ void ChangeBuildingDamage::execute( MapDisplayInterface* md )
          delete fld->building;
          fld->building = NULL;
 
-         if ( md ) {
-           md->displayMap();
-           md->updateDashboard();
-         }
+         if ( md ) 
+            mapChanged( gamemap );
+      
       } else {
          fld->building->damage  = damage;
          if ( md )
@@ -1317,7 +1322,8 @@ void NextMap::execute( MapDisplayInterface* md )
       name.erase( name.find('.') );
 
    
-   savegame( "map-" + name + "-completed" + (savegameextension + 1) );
+   savegame( "map-" + name + "-completed" + (savegameextension + 1), gamemap );
+   gamemap->sigMapWon( gamemap->getCurrentPlayer() );
    throw  LoadNextMap(mapID);
 }
 
@@ -1333,11 +1339,26 @@ void LoseMap::execute( MapDisplayInterface* md )
    if ( gamemap->state == GameMap::Replay )
       return;
 
+   if ( gamemap->getCurrentPlayer().stat != Player::human )
+      return;
+
    if ( !gamemap->continueplaying ) {
       displaymessage ( "You have been defeated !", 3 );
-      delete gamemap;
-      gamemap = NULL;
-      throw NoMapLoaded();
+      gamemap->getCurrentPlayer().stat = Player::off;
+      
+#ifdef sgmain
+      int humanCount = 0;
+      for ( int p = 0; p < gamemap->getPlayerCount(); ++p )
+         if ( gamemap->getPlayer(p).isHuman() && gamemap->getPlayer(p).exist() )
+            humanCount++;
+      
+      if ( humanCount > 1 ) {
+         next_turn( gamemap, NextTurnStrategy_AskUser(), md, -1 );
+      } else {
+         delete gamemap;
+         throw NoMapLoaded();
+      }
+#endif
    }
 }
 
@@ -1366,7 +1387,7 @@ void DisplayEllipse::writeData ( tnstream& stream )
 
 void DisplayEllipse::execute( MapDisplayInterface* md )
 {
-   warning("Ellipses are not supported any more!");
+   warningMessage("Ellipses are not supported any more!");
    /*
    if ( !gamemap->ellipse )
       gamemap->ellipse = new EllipseOnScreen;
@@ -1452,10 +1473,8 @@ void ChangeBuildingOwner :: execute( MapDisplayInterface* md )
       #ifdef sgmain
       evaluateviewcalculation ( gamemap );
       #endif
-      if ( md ) {
-         md->displayMap();
-         md->updateDashboard();
-      }
+      if ( md ) 
+         mapChanged( gamemap );
    }
 }
 
@@ -1514,6 +1533,17 @@ void DisplayImmediateMessage::setup()
 }
 
 
+ASCString DisplayImmediateMessage::getLocalizationString() const
+{
+   return message;
+}
+
+void DisplayImmediateMessage::setLocalizationString( const ASCString& s ) 
+{
+   message = s;
+}
+
+
 
 
 void AddProductionCapability :: readData ( tnstream& stream )
@@ -1581,14 +1611,21 @@ void ChangeDiplomaticStatus :: setup ()
 
 void ChangeDiplomaticStatus :: execute( MapDisplayInterface* md )
 {
-   if ( proposal == Peace )
-      gamemap->getPlayer(proposingPlayer).diplomacy.propose( targetPlayer, PEACE );
-   else
-      if ( proposal == War )
-         gamemap->getPlayer(proposingPlayer).diplomacy.propose( targetPlayer, WAR );
-      else {
-         gamemap->getPlayer(proposingPlayer).diplomacy.setState( targetPlayer, WAR );
+#ifdef sgmain
+   auto_ptr<DiplomacyCommand> dc ( new DiplomacyCommand( gamemap->getPlayer(proposingPlayer) ));  
+   if ( proposal == Peace ) {
+      dc->newstate( PEACE, gamemap->getPlayer(targetPlayer) );
+   } else
+      if ( proposal == War ) {
+         dc->newstate( WAR, gamemap->getPlayer(targetPlayer) );
+      } else {
+         dc->sneakAttack( gamemap->getPlayer(targetPlayer) );
       }
+      
+   ActionResult res = dc->execute( createContext(gamemap) );
+   if ( res.successful() )
+      dc.release();
+#endif   
 }
 
 
@@ -1633,12 +1670,17 @@ class ChangePlayerState : public EventAction {
          list.push_back ( "supervisor");
          list.push_back ( "suspended");
 
+#if 1
+         state = Player::PlayerStatus( chooseString ( "State", list, state ));
+#else
+         /* this is intended as security measure to prevent a player in a duell
+            from switching himself to supervisor and seeing the whole map */
          do {
             state = Player::PlayerStatus( chooseString ( "State", list, state ));
             if ( state == Player::supervisor )
                warning("setting a player to supervisor is not allowed");
          } while ( state == Player::supervisor );
-
+#endif
       }
 
       ASCString getName() const { return "Change player state"; };
@@ -1806,7 +1848,7 @@ void Reinforcements :: execute( MapDisplayInterface* md )
       Type type = Type(stream.readInt());
       if ( type == ReinfVehicle ) {
          try {
-            Vehicle* veh = Vehicle::newFromStream( gamemap, stream, gamemap->getNewNetworkID() );
+            Vehicle* veh = Vehicle::newFromStream( gamemap, stream, gamemap->idManager.getNewNetworkID() );
             FindUnitPlacementPos fupp( gamemap, veh );
          } 
          catch ( InvalidID err ) {
@@ -1860,10 +1902,8 @@ void Reinforcements :: execute( MapDisplayInterface* md )
 
    #endif
 
-   if ( md ) {
-      md->displayMap();
-      md->updateDashboard();
-   }
+   if ( md ) 
+      mapChanged( gamemap );
 }
 
 template<class T>
@@ -1888,13 +1928,13 @@ ASCString ActionNameProvider() {
 template <typename TriggerType > 
 bool registerTrigger( EventTrigger_ID id )
 {
-   return triggerFactory::Instance().registerClass( id, ObjectCreator<EventTrigger, TriggerType>,  TriggerNameProvider<TriggerType>() );
+   return eventTriggerFactory::Instance().registerClass( id, ObjectCreator<EventTrigger, TriggerType>,  TriggerNameProvider<TriggerType>() );
 }
 
 template <typename ActionType > 
 bool registerAction( EventAction_ID id )
 {
-   return actionFactory::Instance().registerClass( id, ObjectCreator<EventAction, ActionType>,  ActionNameProvider<ActionType>() );
+   return eventActionFactory::Instance().registerClass( id, ObjectCreator<EventAction, ActionType>,  ActionNameProvider<ActionType>() );
 }
 
 
