@@ -38,30 +38,30 @@
 
 
 
-void         checktimedevents ( MapDisplayInterface* md )
+void         checktimedevents ( GameMap* gamemap, MapDisplayInterface* md )
 {
-   if ( actmap->eventTimes.empty() )
+   if ( gamemap->eventTimes.empty() )
       return;
                               
-   GameTime t = actmap->eventTimes.back();
-   if ( actmap->time.abstime >= t.abstime ) {
-      checkevents( md );
-      actmap->eventTimes.pop_back();
+   GameTime t = gamemap->eventTimes.back();
+   if ( gamemap->time.abstime >= t.abstime ) {
+      checkevents( gamemap, md );
+      gamemap->eventTimes.pop_back();
    }
    
 }
 
 
 
-void eventReady()
+void eventReady( GameMap* gamemap )
 {
    for ( int i = 0; i < 8; i++ )
-      actmap->player[i].queuedEvents++;
+      gamemap->player[i].queuedEvents++;
 }
 
 
 
-bool checkevents( MapDisplayInterface* md )
+bool checkevents( GameMap* gamemap, MapDisplayInterface* md )
 {
    // this is not for synchronizing between threads, but in the same threat inside the call stack 
    static bool isRunning = false;
@@ -70,12 +70,12 @@ bool checkevents( MapDisplayInterface* md )
 
    VariableLocker l( isRunning );
    
-   actmap->player[actmap->actplayer].queuedEvents++;
-   while ( actmap->player[actmap->actplayer].queuedEvents ) {
+   gamemap->player[gamemap->actplayer].queuedEvents++;
+   while ( gamemap->player[gamemap->actplayer].queuedEvents ) {
 
-      actmap->player[actmap->actplayer].queuedEvents = 0;
+      gamemap->player[gamemap->actplayer].queuedEvents = 0;
 
-      for ( GameMap::Events::iterator ev = actmap->events.begin(); ev != actmap->events.end(); ++ev )
+      for ( GameMap::Events::iterator ev = gamemap->events.begin(); ev != gamemap->events.end(); ++ev )
          (*ev)->check( md );
 
    }
@@ -252,6 +252,9 @@ void Event::execute( MapDisplayInterface* md )
          status = Timed;
       }
       if ( status == Timed && gamemap.time.abstime >= triggerTime.abstime ) {
+         
+         gamemap.actions.breakUndo();
+         
          if ( action )
             action->execute( md );
          
@@ -276,7 +279,7 @@ void Event::execute( MapDisplayInterface* md )
 
 void Event::check( MapDisplayInterface* md )
 {
-   if ( playerBitmap & (1 << gamemap.actplayer ))
+    if ( playerBitmap & (1 << gamemap.actplayer ))
       if ( status == Untriggered ) {
          if ( triggerConnection == AND ) {
             status = Triggered;
@@ -321,13 +324,13 @@ void Event::check( MapDisplayInterface* md )
 
 void Event::spawnAction( EventActionID eai )
 {
-   action = actionFactory::Instance().createObject( eai );
+   action = eventActionFactory::Instance().createObject( eai );
    action->setMap ( &gamemap );
 }
 
 EventTrigger* Event::spawnTrigger( EventTriggerID eti )
 {
-   EventTrigger* et = triggerFactory::Instance().createObject( eti );
+   EventTrigger* et = eventTriggerFactory::Instance().createObject( eti );
    et->setMap ( &gamemap );
    et->setEvent ( this );
    return et;
