@@ -27,12 +27,13 @@
 #define gameeventsystemH
 
 #include <sigc++/sigc++.h>
+#include "loki/Singleton.h"
 
 #include "typen.h"
-#include "libs/loki/Singleton.h"
-#include "factory.h"
+#include "util/factory.h"
 
 #include "mapdisplayinterface.h"
+#include "util/factorywithnames.h"
 
 class MapDisplayInterface;
 
@@ -44,11 +45,11 @@ enum  EventConnections { cconnection_destroy = 1,
                          cconnection_areaentered_specificunit = 32 };
 
 
-extern bool  checkevents( MapDisplayInterface* md );
+extern bool  checkevents( GameMap* gamemap, MapDisplayInterface* md );
 
-extern void  checktimedevents( MapDisplayInterface* md );
+extern void  checktimedevents( GameMap* gamemap, MapDisplayInterface* md );
 
-extern void eventReady();
+extern void eventReady( GameMap* gamemap );
 
 typedef int EventTriggerID;
 typedef int EventActionID;
@@ -87,7 +88,6 @@ class EventTrigger {
 
       virtual ~EventTrigger(){};
 
-      // friend Event* readOldEvent( pnstream stream );
 };
 
 class EventAction {
@@ -101,6 +101,9 @@ class EventAction {
       virtual void writeData ( tnstream& stream ) = 0;
       virtual ASCString getName() const = 0;
 
+      virtual ASCString getLocalizationString() const { return ""; };
+      virtual void setLocalizationString( const ASCString& s ) {};
+      
       virtual void execute( MapDisplayInterface* md ) = 0;
       virtual void setup() = 0;
       void setMap( GameMap* gamemap_ ) { gamemap = gamemap_; };
@@ -162,48 +165,8 @@ class VariableLocker {
 };
 
 
-template < class AbstractProduct, typename IdentifierType, typename ObjectCreatorCallBack = AbstractProduct*(*)(), typename NameType = ASCString >
-class FactoryWithNames : protected Factory<AbstractProduct, IdentifierType>
-{
-   private:
-      typedef std::map<NameType, IdentifierType> NameMap;
-      NameMap names;
-   public:
-      vector<NameType> getNames(){
-         vector<NameType> nameList;
-         
-         for ( typename NameMap::iterator i = names.begin(); i != names.end(); ++i )
-            nameList.push_back( i->first );
-         return nameList;
-      }
-
-      IdentifierType getID( const ASCString& name )
-      {
-         return names[name];
-      }
-
-      bool registerClass( IdentifierType id, typename FactoryWithNames<AbstractProduct, IdentifierType, typename FactoryWithNames::ObjectCreatorCallBack, NameType>::ObjectCreatorCallBack createFn, Loki::Functor<NameType, TYPELIST_1(const IdentifierType&)> nameProvider )
-      {
-         return registerClass( id, createFn, nameProvider(id) );
-      }
-
-      bool registerClass( IdentifierType id, typename FactoryWithNames<AbstractProduct, IdentifierType, typename FactoryWithNames::ObjectCreatorCallBack, NameType>::ObjectCreatorCallBack createFn, NameType name )
-      {
-         if ( Factory<AbstractProduct, IdentifierType>::registerClass ( id, createFn )) {
-            names[name] = id;
-            return true;
-         } else
-            return false;
-      }
-
-      AbstractProduct* createObject( IdentifierType id ) {
-         return Factory<AbstractProduct, IdentifierType>::createObject( id );
-      }
-
-};
-
-typedef Loki::SingletonHolder< FactoryWithNames< EventTrigger, EventTriggerID > > triggerFactory;
-typedef Loki::SingletonHolder< FactoryWithNames< EventAction , EventActionID  > > actionFactory;
+typedef Loki::SingletonHolder< FactoryWithNames< EventTrigger, EventTriggerID > > eventTriggerFactory;
+typedef Loki::SingletonHolder< FactoryWithNames< EventAction , EventActionID  > > eventActionFactory;
 
 
 
