@@ -36,6 +36,9 @@
 #include "overviewmapimage.h"
 #include "gameeventsystem.h"
 #include "spfst.h"
+#include "campaignactionrecorder.h"
+
+#include "packagemanager.h"
 
 
 RandomGenerator::RandomGenerator(int seedValue){
@@ -96,15 +99,15 @@ bool OverviewMapHolder::updateField( const MapCoordinate& pos )
    SPoint imgpos = OverviewMapImage::map2surface( pos );
 
    tfield* fld = map.getField( pos );
-   VisibilityStates visi = fieldVisibility( fld, map.getPlayerView(), &map );
+   VisibilityStates visi = fieldVisibility( fld, map.getPlayerView() );
    if ( visi == visible_not ) {
       OverviewMapImage::fill ( overviewMapImage, imgpos, 0xff545454 );
    } else {
-      if ( fld->building && fieldvisiblenow( fld, map.getPlayerView(), &map) )
+      if ( fld->building && fieldvisiblenow( fld, map.getPlayerView()) )
          OverviewMapImage::fill ( overviewMapImage, imgpos, map.player[fld->building->getOwner()].getColor() );
       else {
 
-         int w = fld->getweather();
+         int w = fld->getWeather();
          fld->typ->getQuickView()->blit( overviewMapImage, imgpos );
          for ( tfield::ObjectContainer::iterator i = fld->objects.begin(); i != fld->objects.end(); ++i )
             if ( visi > visible_ago || i->typ->visibleago )
@@ -218,7 +221,7 @@ void OverviewMapHolder::clearmap( GameMap* actmap )
 
 
 GameMap :: GameMap ( void )
-      : overviewMapHolder( *this ), network(NULL)
+   : actions(this), actionRecorder(NULL), overviewMapHolder( *this ), network(NULL), packageData(NULL)
 {
    randomSeed = rand();
    dialogsHooked = false;
@@ -248,8 +251,6 @@ GameMap :: GameMap ( void )
       player[i].research.chainToMap ( this, i );
    }
           
-   unitnetworkid = 0;
-
    levelfinished = 0;
 
    messageid = 0;
@@ -271,6 +272,9 @@ GameMap :: GameMap ( void )
    weatherSystem  = new WeatherSystem(this, 1, 0.03);
 #endif
    setgameparameter( cgp_objectsDestroyedByTerrain, 1 );
+   
+   
+   nativeMessageLanguage = "en_US";
 }
 
 GameMap::Campaign::Campaign()
@@ -286,7 +290,7 @@ void GameMap :: guiHooked()
    dialogsHooked = true;
 }
 
-const int tmapversion = 21;
+const int tmapversion = 32;
 
 void GameMap :: read ( tnstream& stream )
 {
@@ -325,6 +329,7 @@ void GameMap :: read ( tnstream& stream )
    else
       ___loadtitle = true;
 
+   
    bool loadCampaign = stream.readInt();
    actplayer = stream.readChar();
    time.abstime = stream.readInt();   
@@ -338,6 +343,19 @@ void GameMap :: read ( tnstream& stream )
       if ( stream.readInt() != 0x12345678 )
          throw ASCmsgException("marker not matched when loading GameMap");
    
+   if ( version >= 32 ) {
+      int p = stream.readInt();
+      if ( p ) {
+         packageData = new PackageData();
+         packageData->read( stream );
+         
+         PackageManager::checkGame( this );
+         
+      } else
+         packageData = NULL;
+   } else 
+      packageData = NULL;
+   
       
    for ( int j = 0; j < 4; j++ )
       stream.readChar(); // was: different wind in different altitudes
@@ -385,14 +403,17 @@ void GameMap :: read ( tnstream& stream )
          if ( i < 8 ) // no alliances for neutral 'player'
             for ( int j = 0; j< 8; ++j ) {
                if ( alliances[i][j] == 0 ) 
-                  player[i].diplomacy.setState( j, PEACE, false ); 
+                  player[i].diplomacy.setState( j, PEACE ); 
                else
-                  player[i].diplomacy.setState( j, WAR, false ); 
+                  player[i].diplomacy.setState( j, WAR ); 
             }
       }
       
       if ( version >= 12 )
          player[i].email = stream.readString();
+      
+      if ( version >= 22 )
+         player[i].read( stream );
    }
    
    if ( version >= 11 ) 
@@ -410,7 +431,8 @@ void GameMap :: read ( tnstream& stream )
    }
 
 
-   unitnetworkid = stream.readInt();
+   idManager.readData(stream );
+   
    levelfinished = stream.readChar();
    
    bool alliance_names_not_used_any_more[8];
@@ -560,7 +582,7 @@ void GameMap :: read ( tnstream& stream )
          for ( int j =0; j < 8; j++ ) {
             int sv = stream.readChar();
             if ( sv )
-               player[i].diplomacy.setState( j, PEACE_SV, false );
+               player[i].diplomacy.setState( j, PEACE_SV );
          }      
                
       stream.readInt();
@@ -657,6 +679,23 @@ void GameMap :: read ( tnstream& stream )
       if ( nw ) 
          network = GameTransferMechanism::read( stream );         
     }
+    
+    if ( version >= 25 )
+       actions.read( stream );
+    
+    if ( version >= 30 ) {
+       nativeMessageLanguage = stream.readString();
+       int recorder  = stream.readInt();
+       if ( recorder == 1 ) {
+          actionRecorder = new CampaignActionLogger( this );
+          actionRecorder->readData( stream );
+       } else
+          actionRecorder = NULL;
+    }
+    
+    if( version >= 31 )
+       properties.read( stream );
+    
 }
 
 
@@ -688,6 +727,11 @@ void GameMap :: write ( tnstream& stream )
    
    stream.writeInt( 0x12345678 );
    
+   stream.writeInt( packageData != NULL );
+   if ( packageData ) 
+      packageData->write( stream );
+   
+   
    for  ( i= 0; i < 4; i++ )
       stream.writeChar( 0 );
 
@@ -714,11 +758,13 @@ void GameMap :: write ( tnstream& stream )
       player[i].cursorPos.write( stream );
       player[i].diplomacy.write( stream );
       stream.writeString ( player[i].email );
+      player[i].write(stream);
    }
 
    stream.writeInt( 0x12345678 );
    
-   stream.writeInt( unitnetworkid );
+   idManager.writeData(stream);
+   
    stream.writeChar( levelfinished );
 
    stream.writeInt( 1 );
@@ -845,25 +891,42 @@ void GameMap :: write ( tnstream& stream )
       network->write( stream );
     } else
       stream.writeInt( 0 );
+      
+    actions.write( stream );
+    
+    stream.writeString(nativeMessageLanguage);
+    
+    if ( actionRecorder != NULL ) {
+       stream.writeInt( 1 );
+       actionRecorder->writeData( stream );
+    } else
+       stream.writeInt( 0 );
+    
+    properties.write( stream );
+    
+   
 }
 
 
+MapCoordinate GameMap::findFirstContainer() const
+{
+   for ( int y = 0; y < ysize; ++y )
+      for ( int x = 0; x < xsize; ++x )
+         if ( getField(x,y)->getContainer() )
+            if ( getField(x,y)->getContainer()->getOwner() == actplayer ) 
+               return MapCoordinate(x,y);
+   
+   return MapCoordinate(0,0);
+}
 
 
 MapCoordinate& GameMap::getCursor()
 {
    #ifdef sgmain
    if ( actplayer >= 0 ) {
-      if ( !player[actplayer].cursorPos.valid() ) {
-         bool found = false;
-         for ( int y = 0; y < ysize && !found; ++y )
-            for ( int x = 0; x < xsize  && !found; ++x )
-               if ( getField(x,y)->getContainer() )
-                  if ( getField(x,y)->getContainer()->getOwner() == actplayer ) {
-                     player[actplayer].cursorPos = getField(x,y)->getContainer()->getPosition();
-                     found = true;
-                  }
-      }
+      if ( !player[actplayer].cursorPos.valid() || player[actplayer].cursorPos.x >= xsize || player[actplayer].cursorPos.y >= ysize) 
+         player[actplayer].cursorPos = findFirstContainer();
+      
       return player[actplayer].cursorPos;
    } else
       return player[0].cursorPos;
@@ -872,6 +935,21 @@ MapCoordinate& GameMap::getCursor()
    #endif
 }
 
+MapCoordinate GameMap::getCursor() const
+{
+#ifdef sgmain
+   if ( actplayer >= 0 ) {
+      if ( !player[actplayer].cursorPos.valid() || player[actplayer].cursorPos.x >= xsize || player[actplayer].cursorPos.y >= ysize) 
+         return findFirstContainer();
+      else
+         return player[actplayer].cursorPos;
+   } else
+      return player[0].cursorPos;
+#else
+      return player[8].cursorPos;
+#endif
+}
+
 
 void GameMap :: cleartemps( int b, int value )
 {
@@ -913,7 +991,7 @@ void GameMap :: allocateFields ( int x, int y, TerrainType::Weather* terrain )
 
 void GameMap :: calculateAllObjects ( void )
 {
-   calculateallobjects();
+   calculateallobjects( this );
 }
 
 tfield*  GameMap :: getField(int x, int y)
@@ -924,6 +1002,15 @@ tfield*  GameMap :: getField(int x, int y)
       return (   &field[y * xsize + x] );
 }
 
+const tfield*  GameMap :: getField(int x, int y) const
+{
+   if ((x < 0) || (y < 0) || (x >= xsize) || (y >= ysize))
+      return NULL;
+   else
+      return (   &field[y * xsize + x] );
+}
+
+
 tfield*  GameMap :: getField(const MapCoordinate& pos )
 {
    return getField ( pos.x, pos.y );
@@ -1046,12 +1133,37 @@ int GameMap :: eventpassed( int id, int mapid )
 }
 
 
-int GameMap :: getNewNetworkID()
+void GameMap:: IDManager :: registerUnitNetworkID( Vehicle* veh )
+{
+   vehicleLookupCache[veh->networkid] = veh;
+   if ( unitnetworkid < veh->networkid )
+      unitnetworkid = veh->networkid;
+}
+
+void GameMap:: IDManager :: unregisterUnitNetworkID( Vehicle* veh )
+{
+   VehicleLookupCache::iterator j = vehicleLookupCache.find(veh->networkid);
+   if ( j != vehicleLookupCache.end() )
+      vehicleLookupCache.erase(j);
+}
+
+
+int GameMap :: IDManager :: getNewNetworkID()
 {
    ++unitnetworkid;
    return unitnetworkid;
 }
 
+void GameMap :: IDManager :: readData ( tnstream& stream )
+{
+   unitnetworkid = stream.readInt();
+}
+
+void GameMap :: IDManager :: writeData ( tnstream& stream ) const
+{
+   stream.writeInt( unitnetworkid );
+}
+
 Vehicle* GameMap :: getUnit ( Vehicle* eht, int nwid )
 {
    if ( !eht )
@@ -1074,24 +1186,43 @@ Vehicle* GameMap :: getUnit ( Vehicle* eht, int nwid )
    }
 }
 
-Vehicle* GameMap :: getUnit ( int nwid )
+
+
+Vehicle* GameMap :: getUnit ( int nwid, bool consistencyCheck )
 {
-   VehicleLookupCache::iterator i = vehicleLookupCache.find( nwid );
-   if ( i != vehicleLookupCache.end() )
+   IDManager::VehicleLookupCache::iterator i = idManager.vehicleLookupCache.find( nwid );
+   if ( i != idManager.vehicleLookupCache.end() )
       return i->second;
 
 
-   for ( int p = 0; p < 9; p++ )
-      for ( Player::VehicleList::iterator i = player[p].vehicleList.begin(); i != player[p].vehicleList.end(); i++ )
-         if ( (*i)->networkid == nwid ) {
-            displaymessage("warning: id not registered in VehicleLookupCache!",1);
-            return *i;
-
-         }
+   if ( consistencyCheck ) 
+      for ( int p = 0; p < 9; p++ )
+         for ( Player::VehicleList::iterator i = player[p].vehicleList.begin(); i != player[p].vehicleList.end(); i++ )
+            if ( (*i)->networkid == nwid ) {
+               displaymessage("warning: id not registered in VehicleLookupCache!",1);
+               return *i;
+            }
 
    return NULL;
 }
 
+const Vehicle* GameMap :: getUnit ( int nwid, bool consistencyCheck ) const
+{
+   IDManager::VehicleLookupCache::const_iterator i = idManager.vehicleLookupCache.find( nwid );
+   if ( i != idManager.vehicleLookupCache.end() )
+      return i->second;
+
+
+   if ( consistencyCheck ) 
+      for ( int p = 0; p < 9; p++ )
+         for ( Player::VehicleList::const_iterator i = player[p].vehicleList.begin(); i != player[p].vehicleList.end(); i++ )
+            if ( (*i)->networkid == nwid ) {
+               displaymessage("warning: id not registered in VehicleLookupCache!",1);
+               return *i;
+            }
+
+   return NULL;
+}
 
 Vehicle* GameMap :: getUnit ( int x, int y, int nwid )
 {
@@ -1116,7 +1247,22 @@ ContainerBase* GameMap::getContainer ( int nwid )
    else {
       int x = (-nwid) & 0xffff;
       int y = (-nwid) >> 16;
-      tfield* fld = getfield(x,y);
+      tfield* fld = getField(x,y);
+      if ( !fld )
+         return NULL;
+
+      return fld->building;
+   }
+}
+
+const ContainerBase* GameMap::getContainer ( int nwid ) const 
+{
+   if ( nwid > 0 )
+      return getUnit(nwid);
+   else {
+      int x = (-nwid) & 0xffff;
+      int y = (-nwid) >> 16;
+      const tfield* fld = getField(x,y);
       if ( !fld )
          return NULL;
 
@@ -1135,18 +1281,18 @@ void GameMap::beginTurn()
             replayinfo->guidata[actplayer] = NULL;
          }
             
-   if ( player[actplayer].exist() && player[actplayer].stat != Player::off ) {
+   for ( Player::VehicleList::const_iterator i = player[actplayer].vehicleList.begin(); i != player[actplayer].vehicleList.end(); i++ )
+      (*i)->beginTurn();
+         
+   if ( player[actplayer].exist() && player[actplayer].stat != Player::off ) 
       sigPlayerTurnBegins( player[actplayer] );
       
-//      if ( player[actplayer].stat == Player::human || player[actplayer].stat == Player::supervisor )
-//         sigPlayerUserInteractionEnds( player[actplayer] );
-
-   }
 }
 
 
 void GameMap::endTurn()
 {
+
    player[actplayer].ASCversion = getNumericVersion();
    Player::PlayTime pt;
    pt.turn = time.turn();
@@ -1154,7 +1300,10 @@ void GameMap::endTurn()
    player[actplayer].playTime.push_back ( pt );
    
    sigPlayerTurnEnds( player[actplayer] );
-
+   sigPlayerTurnEndsStatic( this, player[actplayer] );
+         
+   actions.breakUndo(); // commits all actions to the replay log
+   
    for ( int i = 0; i < 9; ++i )
      for ( Player::BuildingList::iterator v = player[i].buildingList.begin(); v != player[i].buildingList.end(); ++v ) {
          if ( i == actplayer )
@@ -1169,7 +1318,7 @@ void GameMap::endTurn()
 
       // Bei Aenderungen hier auch die Windanzeige dashboard.PAINTWIND aktualisieren !!!
 
-      if (( actvehicle->height >= chtieffliegend )   &&  ( actvehicle->height <= chhochfliegend ) && ( getfield(actvehicle->xpos,actvehicle->ypos)->vehicle == actvehicle)) {
+      if (( actvehicle->height >= chtieffliegend )   &&  ( actvehicle->height <= chhochfliegend ) && ( getField(actvehicle->xpos,actvehicle->ypos)->vehicle == actvehicle)) {
          if ( getmaxwindspeedforunit ( actvehicle ) < weather.windSpeed*maxwindspeed ){
             new Message ( getUnitReference( *v ) + " crashed because of the strong wind", this, 1<<(*v)->getOwner());
             toRemove.push_back ( *v );
@@ -1179,9 +1328,7 @@ void GameMap::endTurn()
             if (j < 0) {
                new Message ( getUnitReference( *v ) + " crashed due to lack of fuel", this, 1<<(*v)->getOwner());
                toRemove.push_back ( *v );
-               // logtoreplayinfo( rpl_removeunit, actvehicle->getPosition().x, actvehicle->getPosition().y, actvehicle->networkid );
             } else {
-               // logtoreplayinfo( rpl_refuel2, actvehicle->getPosition().x, actvehicle->getPosition().y, actvehicle->networkid, 1002, j, actvehicle->tank.fuel );
                actvehicle->getResource( actvehicle->getTank().fuel - j, Resources::Fuel, false);
             }
          }
@@ -1195,7 +1342,7 @@ void GameMap::endTurn()
    for ( Player::VehicleList::iterator v = toRemove.begin(); v != toRemove.end(); v++ )
       delete *v;
 
-   checkunitsforremoval();
+   checkunitsforremoval( this );
 
   for ( int i = 0; i < 9; ++i ) 
      for ( Player::VehicleList::iterator v = player[i].vehicleList.begin(); v != player[i].vehicleList.end(); ++v ) 
@@ -1234,32 +1381,45 @@ void GameMap::endRound()
           BuildingWork buildingWork;
 
           for ( Player::BuildingList::iterator j = player[i].buildingList.begin(); j != player[i].buildingList.end(); j++ ) {
+             if ( (*j)->getEntry().x == 19 && (*j)->getEntry().y == 72 )
+                logMessage("dummy", "dummy");
              ContainerBase::Work* w = (*j)->spawnWorkClasses( false );
-             if ( w )
+             if ( w ) {
                 buildingWork.push_back ( w );
+                logMessage("ResourceWork", "Building " + (*j)->getEntry().toString() + " has work to do");
+             } else
+                logMessage("ResourceWork", "Building " + (*j)->getEntry().toString() + " has no work to do");
           }
 
           for ( Player::VehicleList::iterator j = player[i].vehicleList.begin(); j != player[i].vehicleList.end(); j++ ) {
              ContainerBase::Work* w = (*j)->spawnWorkClasses( false );
-             if ( w )
+             if ( w ) {
                 buildingWork.push_back ( w );
+                // logMessage("ResourceWork", "Vehicle " + ASCString::toString((*j)->networkid ) + " has work to do");
+             } 
+                // logMessage("ResourceWork", "Vehicle " + ASCString::toString((*j)->networkid ) + " has no work to do");
           }
           
           
           bool didSomething;
           do {
              didSomething = false;
-             for ( BuildingWork::iterator j = buildingWork.begin(); j != buildingWork.end(); j++ )
+             for ( BuildingWork::iterator j = buildingWork.begin(); j != buildingWork.end(); j++ ) {
                 if ( ! (*j)->finished() ) 
-    //weatherSystem->update(time);
                    if ( (*j)->run() )
                       didSomething = true;
-
+             }
           } while ( didSomething );
           doresearch( this, i );
        }
     }
 
+    int playerMask = 0;
+    for ( int i = 0; i < getPlayerCount(); ++i )
+       if ( !getPlayer(i).exist() )
+          playerMask |= 1 << i;
+    tfield::resetView(this,playerMask);
+
     if ( getgameparameter( cgp_objectGrowthMultiplier) > 0 )
        objectGrowth();
 }
@@ -1282,41 +1442,45 @@ void GameMap::objectGrowth()
       for ( int x = 0; x < xsize; ++x ) {
           tfield* fld = getField( x, y );
           for ( tfield::ObjectContainer::iterator i = fld->objects.begin(); i != fld->objects.end(); ++i)
-             if ( i->typ->growthRate > 0 && i->remainingGrowthTime != 0 )
+             if ( i->typ->growthRate > 0 )
                 for ( int d = 0; d < 6; ++d ) {
                    tfield* fld2 = getField ( getNeighbouringFieldCoordinate( MapCoordinate(x,y), d ));
                    if ( fld2 ) 
                       if ( i->typ->growOnUnits || ((!fld2->vehicle || fld2->vehicle->height >= chtieffliegend) && !fld2->building ))
-                        if ( fld2->objects.empty() || getgameparameter( cgp_objectGrowOnOtherObjects ) > 0 ) {
-                           double d = i->typ->growthRate * getgameparameter( cgp_objectGrowthMultiplier) / 100;
-                           if ( d > 0 ) {
-                              if ( d > 0.9 )
-                                 d = 0.9;
+                        if ( fld2->objects.empty() || getgameparameter( cgp_objectGrowOnOtherObjects ) > 0 ) 
+                           if ( i->remainingGrowthTime != 0 ) {
+                              double d = i->typ->growthRate * getgameparameter( cgp_objectGrowthMultiplier) / 100;
+                              if ( d > 0 ) {
+                                 if ( d > 0.9 )
+                                    d = 0.9;
 
-                              int p = static_cast<int>(std::ceil ( double(1) / d));
-                              if ( p > 1 )
-                                 if ( random ( p ) == 1 )
-                                    if ( i->typ->fieldModification[fld2->getweather()].terrainaccess.accessible( fld2->bdt) > 0 ) {
-                                       newObjects.push_back( make_pair( fld2, i->typ->id ));
-                                       i->remainingGrowthTime -= 1;
-                                       remainingGrowthTime[fld2] = i->remainingGrowthTime;
-                                    }
-                           }
+                                 int p = static_cast<int>(std::ceil ( double(1) / d));
+                                 if ( p > 1 )
+                                    if ( random ( p ) == 1 )
+                                       if ( i->typ->fieldModification[fld2->getWeather()].terrainaccess.accessible( fld2->bdt) > 0 ) {
+                                          newObjects.push_back( make_pair( fld2, i->typ->id ));
+                                          i->remainingGrowthTime -= 1;
+                                          remainingGrowthTime[fld2] = i->remainingGrowthTime;
+                                       }
+                              }
                         }
                 }
       }
 
    for ( NewObjects::iterator i = newObjects.begin(); i != newObjects.end(); ++i )
-      if ( !i->first->checkforobject( getobjecttype_byid( i->second ))) {
+      if ( !i->first->checkForObject( getobjecttype_byid( i->second ))) {
          if ( i->first->addobject ( getobjecttype_byid( i->second ))) {
-            Object* o = i->first->checkforobject( getobjecttype_byid( i->second ));
+            Object* o = i->first->checkForObject( getobjecttype_byid( i->second ));
             assert(o);
             o->remainingGrowthTime = remainingGrowthTime[i->first];
          }
       }
+
+   checkunitsforremoval( this );
 }
 
 SigC::Signal1<void,GameMap&> GameMap::sigMapDeletion;
+SigC::Signal2<void,GameMap*,Player&> GameMap::sigPlayerTurnEndsStatic;
 
 GameMap :: ~GameMap ()
 {
@@ -1359,6 +1523,10 @@ GameMap :: ~GameMap ()
       network = NULL;
    }   
       
+   if ( actionRecorder ) {
+      delete actionRecorder;
+      actionRecorder = NULL;
+   }
 
 #ifdef WEATHERGENERATOR
    delete weatherSystem;
@@ -1369,9 +1537,6 @@ GameMap :: ~GameMap ()
       field = NULL;
    }
 
-   if ( actmap == this )
-      actmap = NULL;
-
 }
 
 /*
@@ -1546,7 +1711,7 @@ int  GameMap::resize( int top, int bottom, int left, int right )  // positive: l
      for ( int x = 0; x < newx; x++ )
         newfield[ x + y * newx ] = defaultfield;
 
-  calculateallobjects();
+  calculateallobjects( this );
 
   for ( int p = 0; p < newx*newy; p++ )
      newfield[p].setparams();
@@ -1594,16 +1759,33 @@ ObjectType* GameMap :: getobjecttype_byid ( int id )
    return objectTypeRepository.getObject_byID ( id );
 }
 
+const ObjectType* GameMap :: getobjecttype_byid ( int id ) const
+{
+   return objectTypeRepository.getObject_byID ( id );
+}
+
+
 Vehicletype* GameMap :: getvehicletype_byid ( int id )
 {
    return vehicleTypeRepository.getObject_byID ( id );
 }
 
+const Vehicletype* GameMap :: getvehicletype_byid ( int id ) const
+{
+   return vehicleTypeRepository.getObject_byID ( id );
+}
+
+
 BuildingType* GameMap :: getbuildingtype_byid ( int id )
 {
    return buildingTypeRepository.getObject_byID ( id );
 }
 
+const BuildingType* GameMap :: getbuildingtype_byid ( int id ) const
+{
+   return buildingTypeRepository.getObject_byID ( id );
+}
+
 const Technology* GameMap :: gettechnology_byid ( int id )
 {
    return technologyRepository.getObject_byID ( id );
@@ -1886,6 +2068,7 @@ AiParameter :: AiParameter ( Vehicle* _unit ) : AiValue ( log2( _unit->height ))
    reset( _unit );
 }
 
+
 void AiParameter :: resetTask ( )
 {
    dest.setnum ( -1, -1, -1 );
@@ -1924,6 +2107,12 @@ bool AiParameter::hasJob ( AiParameter::Job j )
    return find ( jobs.begin(), jobs.end(), j ) != jobs.end();
 }
 
+void AiParameter :: setNewHeight()
+{
+   AiValue::reset ( log2( unit->height ) );
+}
+
+
 void AiParameter :: reset ( Vehicle* _unit )
 {
    unit = _unit;
@@ -2068,13 +2257,14 @@ GameParameterSettings gameParameterSettings[gameparameternum ] = {
       {  "ObjectsDestroyedByTerrain",          1,                    0,   1,                  true,   false,   "objects can be destroyed by terrain"},   //       cgp_objectsDestroyedByTerrain,      
       {  "TrainingIncrement",                  2,                    1,   maxunitexperience,  true,   false,   "training centers: training increment"},   //       cgp_trainingIncrement,              
       {  "ExperienceEffectDivisorAttack",      1,                    1,   10,                 false,  false,   "experience effect divisor for attack"},  //       gp_experienceDivisorAttack
-      {  "DisableDirectView",                  0,                    0,   1,                  false,  false,   "disable direct View"},  //       cgp_disableDirectView
+      {  "DisableDirectView",                  1,                    0,   1,                  false,  false,   "disable direct View"},  //       cgp_disableDirectView
       {  "DisableUnitTrade",                   0,                    0,   1,                  false,  false,   "disable transfering units/buildings to other players"},  //       cgp_disableUnitTransfer
       {  "ExperienceEffectDivisorDefense",     1,                    1,   10,                 false,  false,   "experience effect divisor for defense"},  //       cgp_experienceDivisorDefense
       {  "DebugGameEvents",                    0,                    0,   2,                  true,   false,   "debug game events"},  //       cgp_debugEvents
       {  "ObjectGrowthRate",                   0,                    0,   maxint,             true,   false,   "Object growth rate (percentage)" },  //       cgp_objectGrowthMultiplier
       {  "ObjectsGrowOnOtherObjects",          1,                    0,   1,                  false,  false,   "Objects can grow on fields with other objects"  },  //       cgp_objectGrowOnOtherObjects
       {  "ResearchOutputMultiplier",           1,                    1,   maxint,             false,  false,   "Multiplies the research output of all labs"  },  //       cgp_researchOutputMultiplier
-      {  "ProduceOnlyResearchedStuff",         0,                    0,   1,                  true,   false,   "Produce only researched stuff" }
+      {  "ProduceOnlyResearchedStuffInternally", 0,                  0,   1,                  true,   false,   "Produce only researched stuff internally" },
+      {  "ProduceOnlyResearchedStuffExternally", 1,                  0,   1,                  true,   false,   "Produce only researched stuff externally" }
 };
 
