@@ -42,6 +42,8 @@
  #include "networkinterface.h"
  #include "player.h"
  
+ #include "actions/actioncontainer.h"
+ 
  class RandomGenerator{
    public:
 
@@ -58,43 +60,53 @@
 class WeatherSystem;
 #endif
 
-//! The number of game paramters that can be specified for each map.
-const int gameparameternum = 34;
 
-enum GameParameter { cgp_fahrspur,
+class ActionRecorder {
+   public:
+      virtual void readData ( tnstream& stream ) = 0;
+      virtual void writeData ( tnstream& stream ) = 0;
+      virtual ~ActionRecorder() {};
+};
+
+
+//! The number of game paramters that can be specified for each map.
+const int gameparameternum = 35;
+
+enum GameParameter { cgp_fahrspur, // 0
        cgp_eis,
        cgp_movefrominvalidfields,
        cgp_building_material_factor,
        cgp_building_fuel_factor,
-       cgp_forbid_building_construction,
+       cgp_forbid_building_construction, // 5
        cgp_forbid_unitunit_construction,
        cgp_bi3_training,
        cgp_maxminesonfield,
        cgp_antipersonnelmine_lifetime,
-       cgp_antitankmine_lifetime,
+       cgp_antitankmine_lifetime,      // 10
        cgp_mooredmine_lifetime,
        cgp_floatingmine_lifetime,
        cgp_buildingarmor,
        cgp_maxbuildingrepair,
-       cgp_buildingrepairfactor,
+       cgp_buildingrepairfactor, // 15
        cgp_globalfuel,
        cgp_maxtrainingexperience,
        cgp_initialMapVisibility,
        cgp_attackPower,
-       cgp_jammingAmplifier,
+       cgp_jammingAmplifier,  // 20
        cgp_jammingSlope,
        cgp_superVisorCanSaveMap,
        cgp_objectsDestroyedByTerrain,
        cgp_trainingIncrement,
-       cgp_experienceDivisorAttack,
+       cgp_experienceDivisorAttack,  // 25
        cgp_disableDirectView,
        cgp_disableUnitTransfer,
        cgp_experienceDivisorDefense,
        cgp_debugEvents,
-       cgp_objectGrowthMultiplier,
+       cgp_objectGrowthMultiplier, // 30
        cgp_objectGrowOnOtherObjects,
        cgp_researchOutputMultiplier,
-       cgp_produceOnlyResearchedStuff };
+       cgp_produceOnlyResearchedStuffInternally,
+       cgp_produceOnlyResearchedStuffExternally };
 
 
 struct GameParameterSettings {
@@ -159,7 +171,7 @@ class OverviewMapHolder : public SigC::Object {
 
 
 class Event;
-
+class PackageData;
 
 //! The map. THE central structure of ASC, which holds everything not globally available together
 class GameMap {
@@ -169,8 +181,8 @@ class GameMap {
       //! the size of the map
       int          xsize, ysize;
 
-      //! the coordinate of the map that is displayed on upper left corner of the screen
-      // int          xpos, ypos;
+      int width() const { return xsize; };
+      int height() const { return ysize; };
 
       //! the array of fields
       tfield*       field;
@@ -236,12 +248,20 @@ class GameMap {
          return player[actplayer];
       }
 
+      const Player& getCurrentPlayer() const
+      {
+         return player[actplayer];
+      }
+      
       int getNeutralPlayerNum() const { return 8; };
       
+   private:
+      MapCoordinate findFirstContainer() const;
+   public:
+      
       MapCoordinate& getCursor();
+      MapCoordinate getCursor() const;
       
-      typedef map<int, Vehicle*> VehicleLookupCache;
-      VehicleLookupCache vehicleLookupCache; 
 
       int eventID;
 
@@ -253,10 +273,28 @@ class GameMap {
       int eventpassed ( int saveas, int action, int mapid );
       int eventpassed ( int id, int mapid );
 
-private:
-      int          unitnetworkid;
-public:
-      int getNewNetworkID();
+      class IDManager {
+            int          unitnetworkid;
+            typedef map<int, Vehicle*> VehicleLookupCache;
+            VehicleLookupCache vehicleLookupCache; 
+            
+            friend class SpawnUnit;
+            friend class GameMap;
+                        
+            
+         public:
+            IDManager() : unitnetworkid(0) {};
+            
+            //! returns a new and unique ID 
+            int getNewNetworkID();
+            void registerUnitNetworkID( Vehicle* veh );
+            void unregisterUnitNetworkID( Vehicle* veh );
+            
+            void readData ( tnstream& stream );
+            void writeData ( tnstream& stream ) const;
+      } idManager;
+
+
       char         levelfinished;
 
       enum State { 
@@ -322,9 +360,14 @@ public:
            //! Close the replay logging at the end of a players or the ai's turn.
           void closeLogging();
           ~ReplayInfo ( );
+          
         };
 
+      //! Records all action that have been done for undo/redo purposes
+      ActionContainer actions;
+      ActionRecorder* actionRecorder;
 
+      
       ReplayInfo*  replayinfo;
 
       //! a helper variable to store some information during the loading process. No usage outside.
@@ -357,6 +400,12 @@ public:
       int           gameparameter_num;
       int*          game_parameter;
 
+      /** the language string that describes the messages which are saved in the game events
+          For example: en_US
+          All translations will be maintained in seperate files
+      */
+      ASCString     nativeMessageLanguage;
+      
       int           mineralResourcesDisplayed;
 
       class ArchivalInformation {
@@ -380,13 +429,16 @@ public:
       GameMap ( void );
 
       Vehicle* getUnit ( int x, int y, int nwid );
-      Vehicle* getUnit ( int nwid );
+      Vehicle* getUnit ( int nwid, bool consistencyCheck = true );
+      const Vehicle* getUnit ( int nwid, bool consistencyCheck = true ) const;
       ContainerBase* getContainer ( int nwid );
+      const ContainerBase* getContainer ( int nwid ) const;
       int  getgameparameter ( GameParameter num ) const;
       void setgameparameter ( GameParameter num, int value );
       void cleartemps( int b = -1, int value = 0 );
       bool isResourceGlobal ( int resource );
       tfield* getField ( int x, int y );
+      const tfield* getField ( int x, int y ) const;
       tfield* getField ( const MapCoordinate& pos );
       
       
@@ -414,8 +466,10 @@ public:
       SigC::Signal1<void,Player&> sigPlayerUserInteractionEnds;
       SigC::Signal1<void,Player&> sigPlayerTurnEnds;
       SigC::Signal1<void,Player&> sigPlayerTurnHasEnded;
+      SigC::Signal1<void,Player&> sigMapWon;
 
       static SigC::Signal1<void,GameMap&> sigMapDeletion;
+      static SigC::Signal2<void,GameMap*,Player&> sigPlayerTurnEndsStatic;
       
       //! called when a new round starts (after switching from player 7 to player 0 )
       SigC::Signal0<void> newRound;
@@ -452,8 +506,11 @@ public:
       
       pterraintype getterraintype_byid ( int id );
       ObjectType* getobjecttype_byid ( int id );
+      const ObjectType* getobjecttype_byid ( int id ) const;
       Vehicletype* getvehicletype_byid ( int id );
+      const Vehicletype* getvehicletype_byid ( int id ) const;
       BuildingType* getbuildingtype_byid ( int id );
+      const BuildingType* getbuildingtype_byid ( int id ) const;
       const Technology* gettechnology_byid ( int id );
 
       pterraintype getterraintype_bypos ( int pos );
@@ -481,11 +538,16 @@ public:
       void guiHooked();
       bool getGuiHooked() { return dialogsHooked; };
 
+      //! general key/value storage, primarily for Lua scripts
+      Properties properties;
+      Properties& getProperties() { return properties; };
       
       GameTransferMechanism* network;
 
       int getMemoryFootprint() const;
 
+      mutable PackageData* packageData;
+      
    private:
       Vehicle* getUnit ( Vehicle* eht, int nwid );
 
