@@ -27,7 +27,7 @@
 #include "../basegfx.h"
 #include "../basestrm.h"
 #include "../misc.h"
-#include "../messaginghub.h"
+#include "../util/messaginghub.h"
 #include <iostream>
 
 DI_Color::DI_Color() {
@@ -217,6 +217,14 @@ Surface::Surface(const SDLmm::Surface& other) : SDLmm::Surface ( other ), pixelD
 
 void Surface::write ( tnstream& stream ) const
 {
+   if ( !valid() ) {
+      stream.writeWord( 16974 );
+      stream.writeWord ( 1 );
+      stream.writeChar ( 0 );
+      stream.writeWord ( 0 );
+      stream.writeWord ( 0 );
+      return;
+   } 
    stream.writeWord( 16974 );
    stream.writeWord ( 1 );
    stream.writeChar ( 0 );
@@ -263,6 +271,11 @@ void Surface::read ( tnstream& stream )
   hd.rle = stream.readChar();
   hd.x = stream.readWord();
   hd.y = stream.readWord();
+  
+  if ( hd.x == 0 && hd.y == 0 ) {
+      SetSurface( NULL );
+      return;  
+  }
 
    if (hd.id == 16973) {
       char *pnter = new char [ hd.size + sizeof(hd) ];
@@ -277,7 +290,14 @@ void Surface::read ( tnstream& stream )
       // TODO fix memory leak: uncomp will not be deleted
       delete[] pnter;
 
-      SetSurface( SDL_CreateRGBSurfaceFrom(uncomp+4, hd.x+1, hd.y+1, 8, hd.x+1, 0, 0, 0, 0 ));
+      SDL_Surface* surface = SDL_CreateRGBSurface( SDL_SWSURFACE, hd.x+1, hd.y+1, 8,  0, 0, 0, 0 );
+      for ( int y = 0; y <= hd.y; ++y ) {
+         char* dest = ((char*)surface->pixels) + y * surface->pitch;
+         memcpy( dest, uncomp+4 + y * (hd.x+1), hd.x+1);
+      } 
+      free ( uncomp );
+      
+      SetSurface( surface );
       SetColorKey( SDL_SRCCOLORKEY, 255 );
       assignDefaultPalette();
    }
@@ -681,7 +701,7 @@ Surface rotateSurface( Surface& s, int degrees )
 
 void* Surface::toBGI() const
 {
-   void* p = asc_malloc( imagesize(1,1,w(),h()) );
+   void* p = malloc( imagesize(1,1,w(),h()) );
    char* c = (char*) p;
    Uint16* ww = (Uint16*) p;
    ww[0] = w()-1;
