@@ -34,6 +34,7 @@
 #include "sigc++/retype_return.h"
 #include "asc-mainscreen.h"
 #include "gameoptions.h"
+#include "spfst-legacy.h"
 
 const int guiIconSizeX = 49;
 const int guiIconSizeY = 35;
@@ -47,6 +48,7 @@ const int smallGuiIconSpace = 2;
 const float smallGuiIconSizeFactor = 0.6;
 
 
+
 GuiButton::GuiButton( PG_Widget *parent, const PG_Rect &r ) : PG_Button( parent, r, "", -1, "GuiButton"), func( NULL ), id(-1)
 {
   sigClick.connect ( SigC::slot( *this, &GuiButton::exec ));
@@ -59,7 +61,7 @@ GuiButton::GuiButton( PG_Widget *parent, const PG_Rect &r ) : PG_Button( parent,
 bool GuiButton::exec()
 {
   if ( func ) {
-     func->execute( pos, subject, id );
+     callFunc( pos, subject, id );
      return true;
   }
   return false;
@@ -96,11 +98,21 @@ void GuiButton::eventMouseLeave()
    MessagingHub::Instance().statusInformation("");
 }
 
+void GuiButton::callFunc( const MapCoordinate& pos, ContainerBase* subject, int num )
+{
+   try {
+      func->execute( pos, subject, id );
+   } catch ( ActionResult res ) {
+      errorMessage( res.getMessage() );
+   }
+}
+
+
 bool GuiButton::checkForKey( const SDL_KeyboardEvent* key, int modifier )
 {
    if ( func->available( pos, subject, id ))
       if ( func->checkForKey( key, modifier, id)) {
-         func->execute( pos, subject, id );
+         callFunc( pos, subject, id );
          return true;
       }
    return false;
@@ -321,6 +333,7 @@ class SmallButtonHolder : public SpecialInputWidget {
 
 SmallButtonHolder* NewGuiHost :: smallButtonHolder = NULL;
 
+Command* NewGuiHost :: pendingCommand = NULL;
 
 void NewGuiHost::evalCursor()
 {
@@ -342,6 +355,7 @@ void NewGuiHost::evalCursor()
 
 void NewGuiHost::mapDeleted( GameMap& map )
 {
+   pendingCommand = NULL;
    /*
    while ( theGuiHost->iconHandlerStack.size() >= 1 )
       popIconHandler();
@@ -566,7 +580,7 @@ bool NewGuiHost::setNewButtonPressed( int i )
 
 bool NewGuiHost::eventKeyDown(const SDL_KeyboardEvent* key)
 {
-   int mod = SDL_GetModState() & ~(KMOD_NUM | KMOD_CAPS | KMOD_MODE);
+   int mod = SDL_GetModState() & ~(KMOD_NUM | KMOD_CAPS | KMOD_MODE | SDLK_LSHIFT | SDLK_RSHIFT);
    if ( mod )
       return false;
 
@@ -686,3 +700,12 @@ NewGuiHost::~NewGuiHost()
 
 }
 
+void resetActiveGuiAction( GameMap* map )
+{
+   if ( NewGuiHost::pendingCommand ) {
+      delete NewGuiHost::pendingCommand;
+      NewGuiHost::pendingCommand = NULL;
+      map->cleartemps();
+      repaintMap();
+   }
+}
