@@ -20,9 +20,9 @@
     pipelka@teleweb.at
  
     Last Update:      $Author: mbickel $
-    Update Date:      $Date: 2007-04-13 16:15:56 $
+    Update Date:      $Date: 2008-06-27 19:22:11 $
     Source File:      $Source: /home/cvspsrv/cvsroot/games/asc/source/libs/paragui/include/pgapplication.h,v $
-    CVS/RCS Revision: $Revision: 1.2 $
+    CVS/RCS Revision: $Revision: 1.4 $
     Status:           $State: Exp $
 */
 
@@ -35,6 +35,7 @@
 #define PG_APPLICATION_H
 
 #include "pgmessageobject.h"
+#include "pgscreenupdater.h"
 #include "pgfilearchive.h"
 #include "pgfont.h"
 #include "pgdraw.h"
@@ -186,16 +187,17 @@ class SignalAppIdle : public PG_Signal1<PG_MessageObject*, datatype> {}
 	@return	true - the lock was established successfully
 	*/
 	inline static bool LockScreen() {
-		return (SDL_mutexP(mutexScreen) == 0);
+	   return SDL_mutexP(mutexScreen) == 0;
 	}
-
+	
+	
 	/**
 	Unlock the screen surface
 
 	@return	true - the unlock operation was successful
 	*/
 	inline static bool UnlockScreen() {
-		return (SDL_mutexV(mutexScreen) == 0);
+	   return (SDL_mutexV(mutexScreen) == 0);	   
 	}
 
 	/**
@@ -550,6 +552,17 @@ class SignalAppIdle : public PG_Signal1<PG_MessageObject*, datatype> {}
 	*/
 	static void SetEventSupplier( PG_EventSupplier* eventSupplier );
 
+
+	/**
+	Registers a new class for handling the screen updates. This source will
+	be used for all screen updates throughout Paragui.
+
+	@param screenUpdater the new updater. 
+	Paragui will not delete this object. If NULL is passed, Paragui will obtain 
+	its events directly from SDL 
+	*/
+	static void SetScreenUpdater( PG_ScreenUpdater* screenUpdater ) ;
+
 	/**
 	Returns the EventSupplier that's currently active. \see SetEventSupplier
 	       
@@ -577,8 +590,15 @@ class SignalAppIdle : public PG_Signal1<PG_MessageObject*, datatype> {}
    
    static PG_Char GetHighlightingTag();
 
-   static void UpdateRect(SDL_Surface *screen, Sint32 x, Sint32 y, Sint32 w, Sint32 h);
-   static void UpdateRects(SDL_Surface *screen, int numrects, SDL_Rect *rects);
+   static void UpdateRect(SDL_Surface *screen, Sint32 x, Sint32 y, Sint32 w, Sint32 h) 
+   {
+	my_ScreenUpdater->UpdateRect( screen, x,y,w,h);   	
+   }
+
+   static void UpdateRects(SDL_Surface *screen, int numrects, SDL_Rect *rects)
+   {
+	my_ScreenUpdater->UpdateRects( screen, numrects, rects);   	
+   }
    
 	SignalXMLTag<> sigXMLTag;
 	SignalAppIdle<> sigAppIdle;
@@ -602,6 +622,35 @@ class SignalAppIdle : public PG_Signal1<PG_MessageObject*, datatype> {}
       };
    };
 
+   class ScreenLocker {
+      bool active;
+   public:
+      ScreenLocker() {
+         active = false;
+      };
+      ScreenLocker( bool doLock) {
+         active = false;
+         if ( doLock )
+            lock();
+      };
+      void lock() {
+         if ( !active ) {
+            LockScreen();
+            active = true;
+         }
+      }
+      void unlock() {
+         if ( active ) {
+            UnlockScreen();
+            active = false;
+         }
+      }
+      ~ScreenLocker() {
+         unlock();
+      };
+   };
+   
+
 
 protected:
 
@@ -659,6 +708,8 @@ private:
 	static PG_EventSupplier* my_eventSupplier;
 	static PG_EventSupplier* my_defaultEventSupplier;
 
+	static PG_ScreenUpdater* my_ScreenUpdater;
+
 	static SDL_Surface* my_mouse_pointer;
 	static SDL_Surface* my_mouse_backingstore;
 	static PG_Rect my_mouse_position;
