@@ -20,9 +20,9 @@
     pipelka@teleweb.at
  
     Last Update:      $Author: mbickel $
-    Update Date:      $Date: 2007-04-13 16:15:57 $
+    Update Date:      $Date: 2009-08-16 16:50:27 $
     Source File:      $Source: /home/cvspsrv/cvsroot/games/asc/source/libs/paragui/include/pgwidget.h,v $
-    CVS/RCS Revision: $Revision: 1.2 $
+    CVS/RCS Revision: $Revision: 1.4 $
     Status:           $State: Exp $
 */
 
@@ -886,6 +886,9 @@ protected:
 	*/
 	SDL_Surface* my_srfObject;
 
+	/** creates #my_srfObject */
+	virtual void createMySurface( int w, int h );
+
 	/**
 	text attached to the widget
 	*/
