@@ -20,9 +20,9 @@
     pipelka@teleweb.at
  
     Last Update:      $Author: mbickel $
-    Update Date:      $Date: 2007-12-07 17:38:50 $
+    Update Date:      $Date: 2008-06-27 19:22:12 $
     Source File:      $Source: /home/cvspsrv/cvsroot/games/asc/source/libs/paragui/src/core/pgapplication.cpp,v $
-    CVS/RCS Revision: $Revision: 1.3 $
+    CVS/RCS Revision: $Revision: 1.5 $
     Status:           $State: Exp $
 */
 
@@ -34,6 +34,7 @@
 #include "pgtheme.h"
 #include "pgeventsupplier.h"
 #include "pgsdleventsupplier.h"
+#include "pgsdlscreenupdater.h"
 
 #include <iostream>
 #include <cstring>
@@ -76,6 +77,8 @@ PG_EventSupplier* PG_Application::my_eventSupplier = NULL;
 PG_EventSupplier* PG_Application::my_defaultEventSupplier = NULL;
 bool PG_Application::defaultUpdateOverlappingSiblings = true;
 PG_Char PG_Application::highlightingTag = 0;
+PG_ScreenUpdater* PG_Application::my_ScreenUpdater = NULL;
+
 
 /**
 	new shutdown procedure (called at application termination
@@ -86,6 +89,8 @@ void PARAGUI_ShutDownCode() {
 	SDL_Quit();
 }
 
+PG_SDLScreenUpdater defaultScreenUpdater;
+
 
 PG_Application::PG_Application()
 		: my_quitEventLoop(false), emergencyQuit(false), enableAppIdleCalls(false) {
@@ -438,27 +443,15 @@ PG_Application::CursorMode PG_Application::ShowCursor(CursorMode mode) {
 }
 
 
-#include "../../../../sdl/graphicsqueue.h"
 
-void PG_Application::UpdateRect(SDL_Surface *screen, Sint32 x, Sint32 y, Sint32 w, Sint32 h)
+void PG_Application::SetScreenUpdater( PG_ScreenUpdater* screenUpdater )
 {
-#ifdef WIN32
-   queueOperation( new UpdateRectOp( screen, x, y, w, h ));
-#else
-   SDL_UpdateRect( screen,x,y,w,h);
-   postScreenUpdate(screen);
-#endif
+   if ( screenUpdater != NULL )
+      my_ScreenUpdater = screenUpdater;
+   else
+      my_ScreenUpdater = &defaultScreenUpdater;
 }
 
-void PG_Application::UpdateRects(SDL_Surface *screen, int numrects, SDL_Rect *rects)
-{
-#ifdef WIN32
-   queueOperation( new UpdateRectsOp( screen, numrects, rects ));
-#else
-   SDL_UpdateRects( screen, numrects, rects );
-   postScreenUpdate(screen);
-#endif
-}
 
 
 /**  */
@@ -1296,6 +1289,26 @@ bool PG_Application::GetUpdateOverlappingSiblings() {
 	return defaultUpdateOverlappingSiblings;
 }
 
+volatile int lockCount = 0;
+/*
+bool PG_Application::LockScreen() {
+   
+   int res = SDL_mutexP(mutexScreen);
+   ++lockCount;
+   printf("Lock screen %d\n", lockCount);
+   fflush(stdout);
+   return (res == 0);
+}
+
+bool PG_Application::UnlockScreen() {
+   printf("UnLock screen %d\n", lockCount);
+   --lockCount;
+   fflush(stdout);
+   
+   return (SDL_mutexV(mutexScreen) == 0);
+}
+*/
+
 
 void PG_Application::SetHighlightingTag( PG_Char c )
 {
