@@ -27,7 +27,6 @@
 #include <cstring>
 
 #include "loadbi3.h"
-#include "newfont.h"
 #include "typen.h"
 #include "buildingtype.h"
 #include "vehicletype.h"
@@ -35,6 +34,7 @@
 #include "gameoptions.h"
 #include "graphicset.h"
 #include "spfst.h"
+#include "spfst-legacy.h"
 #include "loaders.h"
 #include "dialog.h"
 #include "itemrepository.h"
@@ -114,7 +114,8 @@ void checkbi3dir ( void )
      if ( notfound ) {
             char bi3path[10000];
             printf("Enter Battle Isle directory:\n" );
-            scanf ( "%s", bi3path );
+            if ( !scanf ( "%s", bi3path ) )
+               bi3path[0] = 0;
 
             CGameOptions::Instance()->BI3directory = bi3path;
             appendbackslash ( CGameOptions::Instance()->BI3directory );
@@ -355,6 +356,21 @@ void Bi3MapTranslationTable :: runTextIO ( PropertyContainer& pc )
 
 PointerVector<Bi3MapTranslationTable*> bi3ImportTables;
 
+Bi3MapTranslationTable* findTable( const ASCString& filename ) {
+   for ( PointerVector<Bi3MapTranslationTable*>::iterator i = bi3ImportTables.begin(); i != bi3ImportTables.end(); ++i )
+      if ( (*i)->filename == filename )
+         return *i;
+   return NULL;
+}
+
+vector<ASCString> getBI3ImportTables()
+{
+   vector<ASCString> list;
+   for ( PointerVector<Bi3MapTranslationTable*>::iterator i = bi3ImportTables.begin(); i != bi3ImportTables.end(); ++i )
+      list.push_back ( (*i)->filename );
+   return list;
+}
+
 
 void BI3TranslationTableLoader::read ( tnstream& stream )
 {
@@ -930,7 +946,7 @@ void        tloadBImap ::   ReadACTNPart(void)
                      ObjectType* obj = objectTypeRepository.getObject_byID ( translationTable->object2IDtranslate[i].second );
                      if ( obj ) {
                         tfield* fld = getfield ( newx, newy );
-                        if ( pass == 1 || obj->getFieldModification(fld->getweather()).terrainaccess.accessible ( fld->bdt )) {
+                        if ( pass == 1 || obj->getFieldModification(fld->getWeather()).terrainaccess.accessible ( fld->bdt )) {
                            fld -> addobject ( obj, 0, true );
                            found |= 1;
                            if ( pass == 0 )
@@ -948,7 +964,7 @@ void        tloadBImap ::   ReadACTNPart(void)
                               for ( unsigned int j = 0; j < obj->weatherPicture[ww].images.size(); j++ )
                                  if ( obj->weatherPicture[ww].bi3pic[j] == xlt[m]  && !(found & 2)  && !( GraphicSetManager::Instance().getMode(xlt[m]) & 256) ) {
                                     tfield* fld = getfield ( newx, newy );
-                                    if ( pass == 1 || obj->getFieldModification(fld->getweather()).terrainaccess.accessible ( fld->bdt )) {
+                                    if ( pass == 1 || obj->getFieldModification(fld->getWeather()).terrainaccess.accessible ( fld->bdt )) {
                                        fld -> addobject ( obj, 0, true );
                                        found |= 1;
                                        if ( pass == 0 )
@@ -1178,7 +1194,7 @@ void       tloadBImap :: ReadSHOPPart( void )
               actpos--;
 
               if ( found ) {
-                 putbuilding ( MapCoordinate(xoffset + newx, yoffset + newy), 0, bldlist[actpos].bld, bldlist[actpos].bld->construction_steps - 1, 1 );
+                 putbuilding ( actmap, MapCoordinate(xoffset + newx, yoffset + newy), 0, bldlist[actpos].bld, bldlist[actpos].bld->construction_steps - 1, 1 );
                  if ( fld->building ) {
                     
                     for ( int m = 0; m < 4; m++ )
@@ -1605,7 +1621,7 @@ void tloadBImap :: LoadFromFile( const char* path, const char* AFileName, Terrai
     } /* endcatch */
 
     if ( !error )
-       calculateallobjects();
+       calculateallobjects( actmap );
     if ( missing[0] ) {
       if ( errorOutput )
          (*errorOutput)=missing;
@@ -1628,48 +1644,17 @@ void setDefaultBI3ImportTranslationTable( const ASCString& filename )
 }
 
 
-Bi3MapTranslationTable* chooseImportTable ( )
-{
-  if ( bi3ImportTables.size() == 0)
-     fatalError ( "No BI3 map translation tables present!");
 
-  if ( bi3ImportTables.size() == 1)
-     return bi3ImportTables[0];
-
-  if ( defaultImportTranslationTable.empty() ) {
-     vector<ASCString> strings;
-     for ( int i = 0; i < bi3ImportTables.size(); i++ )
-        strings.push_back ( bi3ImportTables[i]->filename );
-
-     int t = chooseString ( "Select Import Table", strings );
-     if ( t != 255 )
-        return bi3ImportTables[ t ];
-     else
-        return bi3ImportTables[0];
-  } else {
-     for ( int i = 0; i < bi3ImportTables.size(); i++ )
-         if ( bi3ImportTables[i]->filename.compare_ci ( defaultImportTranslationTable ) == 0 )
-            return bi3ImportTables[i];
-
-     fatalError ( "default BI3 import Table " +defaultImportTranslationTable + " not found !");
-
-     // never executed...
-     return bi3ImportTables[0];
-  }
-}
-
-
-
-void importbattleislemap ( const char* path, const char* filename, TerrainType::Weather* trrn, string* errorOutput, bool fakemap  )
+void importbattleislemap ( const ASCString& path, const ASCString& mapfilename, TerrainType::Weather* trrn, const ASCString& importTable, ASCString* errorOutput, bool fakemap  )
 {
     GameMap* oldmap = actmap;
     actmap = NULL;
     GraphicSetManager::Instance().setActive(1);
     try {
-       ImportBiMap lbim ( chooseImportTable ( ) );
+       ImportBiMap lbim ( findTable(importTable) );
        if ( fakemap )
           lbim.____fakeMap____();
-       lbim.LoadFromFile ( path, filename, trrn, errorOutput );
+       lbim.LoadFromFile ( path.c_str(), mapfilename.c_str(), trrn, errorOutput );
        actmap->_resourcemode = 1;
        actmap->cleartemps ( 7 );
     }
@@ -1682,11 +1667,11 @@ void importbattleislemap ( const char* path, const char* filename, TerrainType::
 }
 
 
-void insertbattleislemap ( int x, int y, const char* path, const char* filename  )
+void insertbattleislemap ( int x, int y, const ASCString& path, const ASCString& mapfilename, const ASCString& importTable  )
 {
     GraphicSetManager::Instance().setActive ( 1 );
-    InsertBiMap lbim ( x, y, chooseImportTable ( ) );
-    lbim.LoadFromFile ( path, filename, NULL, NULL );
+    InsertBiMap lbim ( x, y, findTable(importTable) );
+    lbim.LoadFromFile ( path.c_str(), mapfilename.c_str(), NULL, NULL );
     actmap->_resourcemode = 1;
     actmap->cleartemps ( 7 );
 }
