@@ -32,7 +32,6 @@
 
 #include "typen.h"
 #include "misc.h"
-#include "newfont.h"
 #include "buildingtype.h"
 #include "vehicletype.h"
 #include "spfst.h"
@@ -49,6 +48,12 @@
 #include "prehistoricevents.h"
 #include "gamemap.h"
 #include "gameeventsystem.h"
+#include "graphicset.h"
+#include "gameoptions.h"
+#include "lua/luarunner.h"
+#include "lua/luastate.h"
+#include "packagemanager.h"
+#include "i18n.h"
 
 #ifdef sgmain
 # include "ai/ai.h"
@@ -73,16 +78,14 @@ const int fileterminator = 0xa01a;
  const char* mapextension = "*.map";
  const char* tournamentextension = ".ascpbm";
 
-
+bool suppressMapTriggerExecution = true;
 
 void         seteventtriggers( GameMap* actmap )
 {
-   #ifdef sgmain
-   for ( GameMap::Events::iterator i = actmap->events.begin(); i != actmap->events.end(); ++i )
-       (*i)->arm();
+   if ( !suppressMapTriggerExecution )
+      for ( GameMap::Events::iterator i = actmap->events.begin(); i != actmap->events.end(); ++i )
+         (*i)->arm();
 
-   #endif
-   
    for ( int i = 0; i < 8; i++ )
       actmap->player[i].queuedEvents = 1;
 } 
@@ -100,9 +103,7 @@ void         seteventtriggers( GameMap* actmap )
   #define csm_material 1     /*  b3  */
   #define csm_fuel 2         /*  b3  */
   #define csm_visible 4      /*  b3  */
-//  #define csm_mine 8         /*  b3  */
   #define csm_weather 16     /*  b3  */
-//  #define csm_fahrspur 32    /*  b3  */
   #define csm_object 64      /*  b3  */
   #define csm_b4 128         /* b3 */ 
 
@@ -182,7 +183,7 @@ void   tspfldloaders::writedissections ( void )
 /*        Messages  schreiben / lesen                         */
 /**************************************************************/
 
-const int messageVersion = 0xabcdf0;
+const int messageVersion = 0xabcdf1;
 const int messageMinVersion = 0xabcdef;
 
 void      tspfldloaders:: writemessages ( void )
@@ -202,9 +203,11 @@ void      tspfldloaders:: writemessages ( void )
       stream->writeInt ( (*mi)->id );
       stream->writeInt ( (*mi)->gametime.turn() );
       stream->writeInt ( (*mi)->gametime.move() );
+      stream->writeInt( (*mi)->reminder );
 
 
       ASCString& t = (*mi)->text;
+      
       mi++;
       stream->writeInt ( mi != spfld->messages.end() ? 1 : 0 );
 
@@ -226,7 +229,6 @@ void      tspfldloaders:: writemessages ( void )
 
    stream->writeString ( spfld->gameJournal );
    stream->writeString ( spfld->newJournal );
-
 }
 
 
@@ -259,7 +261,7 @@ void      tspfldloaders:: readmessagelist( MessagePntrContainer& lst )
             mi = mi2;
 
       if ( mi == spfld->messages.end())
-         warning( "message list corrupted !\nplease report this bug!\nthe game will continue, but some messages will probably be missing\nand other instabilities may occur.");
+         warningMessage( "message list corrupted !\nplease report this bug!\nthe game will continue, but some messages will probably be missing\nand other instabilities may occur.");
       lst.push_back ( *mi );
       i = stream->readInt();
    }
@@ -287,11 +289,17 @@ void      tspfldloaders:: readmessages ( void )
       int t = stream->readInt();
       int m = stream->readInt();
       msg->gametime.set ( t, m );
+      
+      if ( magic >= 0xabcdf1 )
+         msg->reminder = stream->readInt();
+      else
+         msg->reminder = false;
 
       spfld->__loadmessages = stream->readInt();
 
       if ( msgtext )
          msg->text = stream->readString( true );
+      
    }
 
    for ( int i = 0; i < 8; i++ )
@@ -344,8 +352,8 @@ void checkForUniqueUnitIDs( GameMap* gamemap )
    for ( int p = 0; p < gamemap->getPlayerCount(); ++p )
       for ( Player::VehicleList::iterator i = gamemap->getPlayer(p).vehicleList.begin(); i != gamemap->getPlayer(p).vehicleList.end(); ++i )
          if ( units[(*i)->networkid]++ > 0 ) {
-            warning("unit with duplicate network ids: " + ASCString::toString( (*i)->networkid ) + "\nThis will lead to replay errors during the next turn." );
-            (*i)->networkid = gamemap->getNewNetworkID();;
+            warningMessage("unit with duplicate network ids: " + ASCString::toString( (*i)->networkid ) + "\nThis will lead to replay errors during the next turn." );
+            (*i)->networkid = gamemap->idManager.getNewNetworkID();;
          }
 }
 
@@ -441,7 +449,7 @@ void        tspfldloaders::readLegacyNetwork ( void )
 /*     fielder schreiben / lesen                              */
 /**************************************************************/
 
-const int objectstreamversion = 3;
+const int objectstreamversion = 4;
 
 void   tspfldloaders::writefields ( void )
 {
@@ -562,6 +570,7 @@ void   tspfldloaders::writefields ( void )
             stream->writeInt ( m->strength );
             stream->writeInt ( m->lifetimer );
             stream->writeInt ( m->player );
+            stream->writeInt ( m->identifier );
          }
 
          stream->writeInt ( fld->objects.size() );
@@ -647,6 +656,7 @@ void tspfldloaders::readfields ( void )
          else 
             weather = 0;
 
+         assertOrThrow( weather >= 0 && weather < cwettertypennum );
 
          int k;
 
@@ -733,7 +743,13 @@ void tspfldloaders::readfields ( void )
                assertOrThrow( strength >= 0 );
                assertOrThrow( type > 0 && type <= 4 );
 
-               Mine m ( type, strength, player, spfld );
+               int id;
+               if ( objectversion >= 4 )
+                  id = stream->readInt();
+               else
+                  id = spfld->idManager.getNewNetworkID();
+               
+               Mine m ( type, strength, player, spfld, id );
                if ( objectversion == 1 ) {
                   int endtime = minetime;
                   int lifetime = spfld->getgameparameter( GameParameter(cgp_antipersonnelmine_lifetime + m.type - 1));
@@ -994,10 +1010,39 @@ GameMap* tmaploaders::_loadmap( const ASCString& name )
 } 
 
 
+GameMap* eventLocalizationMap = NULL;
+
+
+
+void loadLocalizedMessageFile( GameMap* map, const ASCString& filename ) {
+   eventLocalizationMap = map;
+         
+   LuaState state;
+   LuaRunner runner( state );
+   runner.runFile( filename );
+   if ( !runner.getErrors().empty() )
+      errorMessage( runner.getErrors() );
+         
+   eventLocalizationMap = NULL;
+}
+
+void loadLocalizedMessages( GameMap* map, const ASCString& name )
+{
+   Locale locale;
+   ASCString filename = locale.getLocalizedFile( name );
+   if( !filename.empty() && exist( filename ) )
+       loadLocalizedMessageFile( map, filename );
+  
+}
+
 GameMap* tmaploaders::loadmap ( const ASCString& name )
 {
      tmaploaders gl;
-     return gl._loadmap ( name );
+     GameMap* map = gl._loadmap ( name );
+#ifdef sgmain
+   loadLocalizedMessages( map, name );     
+#endif
+     return map;
 }     
    
 
@@ -1017,6 +1062,8 @@ GameMap* tmaploaders::loadmap ( const ASCString& name )
 
 void   tsavegameloaders::savegame( pnstream strm, GameMap* gamemap, bool writeReplays )
 {
+   PackageManager::storeData( gamemap );
+   
    stream = strm;
    spfld = gamemap;
 
@@ -1046,10 +1093,10 @@ void   tsavegameloaders::savegame( pnstream strm, GameMap* gamemap, bool writeRe
 }
 
 
-void         tsavegameloaders::savegame( const ASCString& name)
+void         tsavegameloaders::savegame( GameMap* gamemap, const ASCString& name)
 { 
    tnfilestream filestream ( name, tnstream::writing );
-   savegame ( &filestream, actmap, true );
+   savegame ( &filestream, gamemap, true );
 }
 
 
@@ -1128,6 +1175,7 @@ GameMap*          tsavegameloaders::loadgame( pnstream strm )
 
    mapLoaded( spfld );
 
+   
    GameMap* s = spfld;
    spfld = NULL;  // to avoid that is is deleted by the destructor of tsavegameloaders
    return s;
@@ -1144,6 +1192,9 @@ GameMap*          tsavegameloaders::loadgame( pnstream strm )
 
 int          tnetworkloaders::savenwgame( pnstream strm, const GameMap* gamemap )
 { 
+   
+   PackageManager::storeData( gamemap );
+   
    spfld = const_cast<GameMap*>(gamemap); // yes, this is bad, but spfld can't be made constant because it is also used for loading
    
    stream = strm;
@@ -1271,6 +1322,7 @@ GameMap*  tnetworkloaders::loadnwgame( pnstream strm )
             }
 
    
+            
    GameMap* spfldcopy = spfld;
    spfld = NULL;
 
@@ -1286,15 +1338,52 @@ GameMap*  tnetworkloaders::loadnwgame( pnstream strm )
 
 
 
+ASCString getLuaQuote( bool open, int n ) 
+{
+   ASCString s =  open? "[[" : "]]";
+   for ( int i =0; i < n; ++i )
+      s.insert(1, "=" );
+   return s;
+}
+
+ASCString luaQuote( const ASCString& text )
+{
+   int count = 0;
+   while( text.find_first_of( getLuaQuote( true,  count )) != ASCString::npos ||
+          text.find_first_of( getLuaQuote( false, count )) != ASCString::npos )
+          ++count;
+   
+   return getLuaQuote( true, count) + text + getLuaQuote( false, count );
+}
+
+void writeMessageFile( GameMap* gamemap, tnstream& stream )
+{
+   stream.writeString( "map = asc.getLoadingMap() \n", false);
+   for ( GameMap::Events::const_iterator i = gamemap->events.begin(); i != gamemap->events.end(); ++i ) {
+      ASCString s = (*i)->action->getLocalizationString();
+      if ( !s.empty() )  {
+         stream.writeString ( "--- ===== Event " + ASCString::toString((*i)->id) + "  ======= \n", false );
+         stream.writeString ( "message = " + luaQuote( s ) + "\n", false );
+         stream.writeString ( "asc.setLocalizedEventMessage( map, " + ASCString::toString((*i)->id) + ", message )\n\n", false);
+      }
+   }
+   
+   for ( int p = 0; p <= gamemap->getPlayerCount(); ++p ) {
+      Player& player = gamemap->getPlayer(p);
+      for ( Player::BuildingList::const_iterator b = player.buildingList.begin(); b != player.buildingList.end(); ++b ) {
+         if ( !(*b)->name.empty() ) {
+            stream.writeString ( "--- ===== Building " + ASCString::toString((*b)->getIdentification()) + "  ======= \n", false );
+            ASCString f;
+            f.format( "asc.setLocalizedContainerName( map, asc.MapCoordinate( %d, %d ), %s ) \n", (*b)->getPosition().x, (*b)->getPosition().y, +  luaQuote( (*b)->name ).c_str() );
+            stream.writeString (  f, false  );
+         }
+      }
+   }
+}
 
 
 
-
-
-
-
-
-void  savemap( const char * name, GameMap* gamemap )
+void  savemap( const ASCString& name, GameMap* gamemap )
 {
 
    #ifdef logging
@@ -1304,28 +1393,34 @@ void  savemap( const char * name, GameMap* gamemap )
    try {
      tmaploaders gl;
      gl.savemap ( name, gamemap );
+     
+     
+     if ( CGameOptions::Instance()->saveEventMessagesExternal && gamemap->nativeMessageLanguage.length() ) {
+        tn_file_buf_stream messages ( name + "." + gamemap->nativeMessageLanguage, tnstream::writing );
+        writeMessageFile( gamemap, messages );
+     }
+     
    } /* endtry */
 
    catch ( tfileerror err ) {
       displaymessage( "file error writing map to filename %s ", 1, err.getFileName().c_str() );
    } /* endcatch */
-   catch ( ASCexception err) {
+   catch ( ASCexception ) {
       displaymessage( "error writing map ", 1 );
    } /* endcatch */
 
 }
 
-
-void  savegame( const ASCString& name )
+void  savegame( const ASCString& name, GameMap* gamemap )
 {
    try {
       tsavegameloaders gl;
-      gl.savegame ( name );
+      gl.savegame ( gamemap, name );
    }
    catch ( tfileerror err) {
       displaymessage( "error writing map to filename %s ", 1, err.getFileName().c_str() );
    } /* endcatch */
-   catch ( ASCexception err) {
+   catch ( ASCexception ) {
       displaymessage( "error writing map ", 1 );
    } /* endcatch */
 }
@@ -1352,7 +1447,7 @@ void  savereplay( GameMap* gamemap, int num )
 
       memstream.writeInt ( actreplayversion );
    }
-   catch ( ASCexception err ) {
+   catch ( ASCexception ) {
       displaymessage( "error saving replay information", 1 );
    } /* endcatch */
 }
@@ -1416,6 +1511,10 @@ GameMap* mapLoadingExceptionChecker( const ASCString& filename, MapLoadingFuncti
       displaymessage( err.getMessage().c_str(), 1 );
       return NULL;
    } /* endcatch */
+   catch ( tcompressionerror err ) {
+      displaymessage( "The file %s is corrupted.\nPlease obtain a new copy of that file", 1, filename.c_str() );
+      return NULL;
+   }
    catch ( tfileerror err) {
       displaymessage( "error reading map filename %s ", 1, err.getFileName().c_str() );
       return NULL;
@@ -1590,6 +1689,3 @@ MapConinuationInfo findNextCampaignMap( int id  )
 }
 
 
-
-
-
