@@ -46,6 +46,7 @@
 #include "iconrepository.h"
 #include "mainscreenwidget.h"
 #include "sdl/sound.h"
+#include "spfst-legacy.h"
 
 #ifndef karteneditor
  #include "dialogs/attackpanel.h"
@@ -149,8 +150,13 @@ void ResourceGraphLayer::paintSingleField( const MapRenderer::FieldRenderInfo& f
             visible = true;
 
       if ( visible ) {
-         paintBar( fieldInfo, pos, 0, fieldInfo.fld->resourceview->materialvisible[max(fieldInfo.playerView,0)], Resources::materialColor );
-         paintBar( fieldInfo, pos, 1, fieldInfo.fld->resourceview->fuelvisible[max(fieldInfo.playerView,0)], Resources::fuelColor );
+         if ( fieldInfo.playerView>=0 && fieldInfo.gamemap->getPlayer(fieldInfo.playerView).stat == Player::supervisor ) {
+            paintBar( fieldInfo, pos, 0, fieldInfo.fld->material, Resources::materialColor );
+            paintBar( fieldInfo, pos, 1, fieldInfo.fld->fuel,     Resources::fuelColor );
+         } else {
+            paintBar( fieldInfo, pos, 0, fieldInfo.fld->resourceview->materialvisible[max(fieldInfo.playerView,0)], Resources::materialColor );
+            paintBar( fieldInfo, pos, 1, fieldInfo.fld->resourceview->fuelvisible[max(fieldInfo.playerView,0)], Resources::fuelColor );
+         }
       }
    }
 #else
@@ -211,7 +217,7 @@ void PipeLayer::paintSingleField( const MapRenderer::FieldRenderInfo& fieldInfo,
          pipeline->display( fieldInfo.surface, pos, 63, 0 );
       } else {
          if ( fieldInfo.fld->bdt.test( cbpipeline )) {
-            Object* o = fieldInfo.fld->checkforobject( buried_pipeline );
+            Object* o = fieldInfo.fld->checkForObject( buried_pipeline );
             if ( o )
                pipeline->display( fieldInfo.surface, pos, o->dir, 0 );
             else {
@@ -233,6 +239,27 @@ void PipeLayer::paintSingleField( const MapRenderer::FieldRenderInfo& fieldInfo,
 }
 
 
+
+class ReactionFireLayer : public MapLayer {
+      Surface& image;
+   public: 
+      ReactionFireLayer() : image ( IconRepository::getIcon("rf-icon.png")) {} ;
+      bool onLayer( int layer ) { return layer == 17; };
+      void paintSingleField( const MapRenderer::FieldRenderInfo& fieldInfo,  int layer, const SPoint& pos );
+};
+
+void ReactionFireLayer::paintSingleField( const MapRenderer::FieldRenderInfo& fieldInfo,  int layer, const SPoint& pos )
+{
+   if ( fieldInfo.visibility > visible_ago) {
+      if ( fieldInfo.fld->vehicle && fieldInfo.fld->vehicle->reactionfire.getStatus() != Vehicle::ReactionFire::off && fieldInfo.fld->vehicle->getOwner() == fieldInfo.playerView ) {
+         MegaBlitter<colorDepth,colorDepth,ColorTransform_None,ColorMerger_AlphaMerge> blitter;
+         blitter.blit( image, fieldInfo.surface, pos);
+      }   
+   }
+}
+
+
+
 class WeaponRange : public SearchFields
 {
    public:
@@ -287,7 +314,7 @@ void MapRenderer::readData()
    if ( !icons.mapBackground.valid() ) {
       icons.mapBackground = IconRepository::getIcon("mapbkgr.raw");
       icons.notVisible    = IconRepository::getIcon("hexinvis.raw");
-      icons.markField     = IconRepository::getIcon("markedfield.pcx");
+      icons.markField     = IconRepository::getIcon("markedfield.png");
       icons.markField.detectColorKey();
       icons.markFieldDark = IconRepository::getIcon("markedfielddark.png");
    }
@@ -340,8 +367,8 @@ void MapRenderer::paintSingleField( const MapRenderer::FieldRenderInfo& fieldInf
    if ( fieldInfo.visibility > visible_ago ) {
 
       /* display buildings */
-      if ( fld->building  &&  (fld->building->typ->buildingheight & binaryUnitHeight) && fld->building->visible )
-         if ((fieldInfo.visibility == visible_all) || (fld->building->typ->buildingheight >= chschwimmend) || ( fld->building->getOwner() == fieldInfo.playerView ))
+      if ( fld->building  &&  (fld->building->typ->height & binaryUnitHeight) && fld->building->visible )
+         if ((fieldInfo.visibility == visible_all) || (fld->building->typ->height >= chschwimmend) || ( fld->building->getOwner() == fieldInfo.playerView ))
             fld->building->paintSingleField( fieldInfo.surface, pos, fld->building->getLocalCoordinate( fieldInfo.pos ));
 
 
@@ -358,7 +385,7 @@ void MapRenderer::paintSingleField( const MapRenderer::FieldRenderInfo& fieldInf
          int h = o->typ->imageHeight;
          if ( fieldInfo.visibility > visible_ago || (o->typ->visibleago && fieldInfo.visibility >= visible_ago ))
             if (  h >= ((layer-1)/2)*30 && h < (layer-1)/2*30+30 )
-               o->display ( fieldInfo.surface, pos, fld->getweather() );
+               o->display ( fieldInfo.surface, pos, fld->getWeather() );
       }
 
 
@@ -368,8 +395,10 @@ void MapRenderer::paintSingleField( const MapRenderer::FieldRenderInfo& fieldInf
       /* display mines */
       
       if ( fieldInfo.visibility == visible_all )
-         if ( !fld->mines.empty() && layer == 7 ) 
-            fld->mines.begin()->paint( fieldInfo.surface, pos );
+         if ( !fld->mines.empty() && layer == 7 ) {
+            for ( tfield::MineContainer::const_iterator i = fld->mines.begin(); i != fld->mines.end(); ++i )
+               i->paint( fieldInfo.surface, pos );
+         }
      
 
 
@@ -386,8 +415,8 @@ void MapRenderer::paintSingleField( const MapRenderer::FieldRenderInfo& fieldInf
 
    } else {
       if (fieldInfo.visibility == visible_ago ) {
-         if ( fld->building  &&  (fld->building->typ->buildingheight & binaryUnitHeight) && fld->building->visible )
-            if ((fieldInfo.visibility == visible_all) || (fld->building->typ->buildingheight >= chschwimmend) || ( fld->building->getOwner() == fieldInfo.playerView ))
+         if ( fld->building  &&  (fld->building->typ->height & binaryUnitHeight) && fld->building->visible )
+            if ((fieldInfo.visibility == visible_all) || (fld->building->typ->height >= chschwimmend) || ( fld->building->getOwner() == fieldInfo.playerView ))
                fld->building->paintSingleField( fieldInfo.surface, pos, fld->building->getLocalCoordinate( fieldInfo.pos ));
 
       }
@@ -540,6 +569,7 @@ MapDisplayPG::MapDisplayPG ( MainScreenWidget *parent, const PG_Rect r )
    addMapLayer( new ResourceGraphLayer(), "resources" );
    addMapLayer( new ContainerInfoLayer(), "container" );
    addMapLayer( new PipeLayer()         , "pipes" );
+   addMapLayer( new ReactionFireLayer() , "reactionfire" );
    
    parent->lockOptionsChanged.connect( SigC::slot( *this, &MapDisplayPG::lockOptionsChanged ));
    GameMap::sigMapDeletion.connect( SigC::slot( *this, &MapDisplayPG::sigMapDeleted ));
@@ -579,7 +609,7 @@ MapDisplayPG::Icons MapDisplayPG::icons;
 void MapDisplayPG::readData()
 {
    if ( !icons.cursor.valid() ) {
-      icons.cursor = IconRepository::getIcon( "curshex.raw" );
+      icons.cursor = IconRepository::getIcon( "curshex.png" );
       icons.fieldShape = IconRepository::getIcon("hexinvis.raw");
    }
 }
@@ -734,7 +764,7 @@ void MapDisplayPG::displayCursor( const PG_Rect& dst )
    int y = cursor.pos().y  - offset.y;
    if( x >= field.viewPort.x1 && x < field.viewPort.x2 && y >= field.viewPort.y1 && y < field.viewPort.y2 && x >= 0 && y >= 0 && x < actmap->xsize && y < actmap->ysize ) {
       // surface->Blit( icons.cursor, getFieldPos(x,y));
-      MegaBlitter<1,colorDepth,ColorTransform_None,ColorMerger_AlphaOverwrite,SourcePixelSelector_DirectZoom,TargetPixelSelector_Rect> blitter;
+      MegaBlitter<colorDepth,colorDepth,ColorTransform_None,ColorMerger_AlphaOverwrite,SourcePixelSelector_DirectZoom,TargetPixelSelector_Rect> blitter;
       blitter.setZoom( float(zoom) / 100.0 );
 
       PG_Rect clip= dst.IntersectRect( PG_Application::GetScreen()->clip_rect );
@@ -1254,7 +1284,7 @@ bool ccompare( const MapCoordinate& a, const MapCoordinate& b )
 }   
 
 
-void MapDisplayPG::displayUnitMovement( GameMap* actmap, Vehicle* veh, const MapCoordinate3D& from, const MapCoordinate3D& to )
+void MapDisplayPG::displayUnitMovement( GameMap* actmap, Vehicle* veh, const MapCoordinate3D& from, const MapCoordinate3D& to, int duration )
 {
    static int col = 0xff;
    
@@ -1274,11 +1304,6 @@ void MapDisplayPG::displayUnitMovement( GameMap* actmap, Vehicle* veh, const Map
       return;
 
    int startTime = ticker;
-#ifdef debugmapdisplay
-   int duration = CGameOptions::Instance()->movespeed * 20;
-#else
-   int duration = CGameOptions::Instance()->movespeed;
-#endif
    int endTime = startTime + duration;
    
    // initialisation that is only executed the first time the code runs here
