@@ -36,11 +36,13 @@
 #include "edglobal.h"
 #include "mapdisplay.h"
 #include "spfst.h"
+#include "spfst-legacy.h"
 #include "overviewmappanel.h"
 #include "edselfnt.h"
 #include "edmisc.h"
 #include "gameoptions.h"
 #include "widgets/dropdownselector.h"
+#include "dialogs/fileselector.h"
 
 Maped_MainScreenWidget*  mainScreenWidget = NULL ;
 
@@ -106,6 +108,9 @@ void Menu::setup()
    addbutton ( "Load Clipboard",     act_readClipBoard );
    addbutton ( "Save Clipboard",     act_saveClipboard );
    currentMenu->addSeparator();
+   addbutton ( "Run Script",     act_runLuaScript );
+   addbutton ( "Run Translation Script", act_runTranslationScript );
+   currentMenu->addSeparator();
    addbutton ( "~W~rite map to PNG-File\tctrl+G", act_maptopcx);
     addbutton ( "~I~mport BI map\tctrl-i", act_import_bi_map );
     addbutton ( "Insert ~B~I map", act_insert_bi_map );
@@ -167,7 +172,7 @@ void Menu::setup()
    addbutton ( "PBP Player Statistics", act_pbpstatistics );
 
    addfield ("~O~ptions");
-    addbutton ( "~M~ap values\tctrl+M",          act_changemapvals );
+    addbutton ( "~M~ap properties\tctrl+M",          act_changemapvals );
     addbutton ( "~C~hange players\tO",           act_changeplayers);
     addbutton ( "~E~dit events\tE",              act_events );
 //    addbutton ( "unit production ~L~imitation", act_specifyunitproduction );
@@ -188,10 +193,14 @@ void Menu::setup()
     addbutton ( "Locate Item by ~I~D",    act_locateItemByID );
    currentMenu->addSeparator();
     addbutton ( "Exchange ~G~raphics",          act_exchangeGraphics );
-
-
+/*
+   addfield("Debug");
+    addbutton ( "Crash map editor", asc_nullPointerCrash );
+    addbutton ( "TestFunctionHolder", asc_testFunction );
+*/
    addfield ("~H~elp");
     addbutton ( "~U~nit Information\tctrl+U",    act_unitinfo );
+    addbutton ( "unit guide dialog",             act_unitGuideDialog );
     addbutton ( "unit~S~et Information",        act_unitSetInformation );
     addbutton ( "~T~errain Information\t7",        act_terraininfo );
    currentMenu->addSeparator();
@@ -434,6 +443,10 @@ Maped_MainScreenWidget::Maped_MainScreenWidget( PG_Application& application )
    button5->sigClick.connect( SigC::slot( *this, &Maped_MainScreenWidget::selectMine ));
    ypos += 25;
     
+   PG_Button* button6 = new PG_Button( this, PG_Rect( xpos, ypos, w, 20), "Lua Brush" );
+   button6->sigClick.connect( SigC::slot( *this, &Maped_MainScreenWidget::selectLuaBrush ));
+   ypos += 25;
+   
    
    new PG_Label( this, PG_Rect( xpos, ypos, w/2 - 5, 20), "Brush:");
    brushSelector = new DropDownSelector( this, PG_Rect( xpos+w/2+5, ypos, w/2-5, 20));
@@ -530,11 +543,16 @@ void Maped_MainScreenWidget::updateStatusBar()
    coordinateDisplay->SetText( ASCString::toString( pos.x) + " / " + ASCString::toString( pos.y ));
 }
 
-void Maped_MainScreenWidget::selectionChanged( const MapComponent* item )
+void Maped_MainScreenWidget::selectionChanged( const Placeable* item )
 {
    if ( item ) {
-      selectionName->SetText( item->getItemType()->getName() );
-      selectionName2->SetText( "ID: " + ASCString::toString( item->getItemType()->getID() ) );
+      selectionName->SetText( item->getName() );
+      
+      const MapComponent* mc = dynamic_cast<const MapComponent*>(item);
+      if (  mc ) 
+         selectionName2->SetText( "ID: " + ASCString::toString( mc->getItemType()->getID() ) );
+      else
+         selectionName2->SetText( "" );
    } else {
       selectionName->SetText( "" );
       selectionName2->SetText( "" );
@@ -662,6 +680,7 @@ bool Maped_MainScreenWidget::eventKeyDown(const SDL_KeyboardEvent* key)
 
    if ( !mod  ) {
       switch ( key->keysym.sym ) {
+         case SDLK_KP_ENTER:
          case SDLK_RETURN:
          case SDLK_SPACE: execaction_ev( act_primaryAction );
                         return true;
@@ -762,70 +781,90 @@ bool Maped_MainScreenWidget::eventKeyDown(const SDL_KeyboardEvent* key)
          case SDLK_F4 : execaction_ev(act_selobjectAll);
                         return true;
 
-         case SDLK_a:  execaction_ev(act_setupalliances);
-                        return true;
-   
-         case SDLK_b:  execaction_ev(act_toggleresourcemode);
-                        return true;
-   
-         case SDLK_c:  execaction_ev(act_copyToClipboard);
-                        return true;
-
-         case SDLK_f: execaction_ev(act_createresources);
-                        return true;
-   
-         case SDLK_g: execaction_ev(act_maptopcx);
-                        return true;
-   
-         case SDLK_h: execaction_ev(act_setunitfilter);
-                        return true;
-   
-         case SDLK_i: execaction_ev (act_import_bi_map );
-                        return true;
-   
-         case SDLK_l: execaction_ev(act_loadmap);
-                        return true;
-   
-         case SDLK_m: execaction_ev(act_changemapvals);
-                        return true;
-   
-         case SDLK_n: execaction_ev(act_newmap);
-                        return true;
-   
-         case SDLK_o: execaction_ev(act_polymode);
-                        return true;
-   
-         case SDLK_p: execaction_ev(act_changeproduction);
-                        return true;
-   
-         case SDLK_q: execaction_ev(act_end);
-                        return true;
-                        
-         case SDLK_r: execaction_ev(act_repaintdisplay);
-                        return true;
-   
-         case SDLK_s : execaction_ev(act_savemap);
-                        return true;
-
-         case SDLK_u: execaction_ev(act_unitinfo);
-                        return true;
-   
-         case SDLK_v: execaction_ev(act_pasteFromClipboard);
-                        return true;
-   
-         case SDLK_w: execaction_ev(act_setactweatherglobal);
-                        return true;
-
-         case SDLK_x: execaction_ev(act_cutToClipboard);
-                        return true;
-
          case SDLK_SPACE: execaction_ev( act_primaryAction );
                         return true;
                         
-                                                
          default:;
    
        } 
+      switch ( key->keysym.unicode ) {
+         case 1:  // A
+            execaction_ev(act_setupalliances);
+            return true;
+   
+         case 2:  // B
+            execaction_ev(act_toggleresourcemode);
+            return true;
+   
+         case 3:  // C
+            execaction_ev(act_copyToClipboard);
+            return true;
+
+         case 6: // F
+            execaction_ev(act_createresources);
+            return true;
+   
+         case 7: //G
+            execaction_ev(act_maptopcx);
+            return true;
+   
+         case 8:  // H
+            execaction_ev(act_setunitfilter);
+            return true;
+   
+         case 9:  // I
+            execaction_ev (act_import_bi_map );
+            return true;
+   
+         case 12:  // L
+            execaction_ev(act_loadmap);
+            return true;
+   
+         case 13:  // M
+            execaction_ev(act_changemapvals);
+            return true;
+   
+         case 14:  // N
+            execaction_ev(act_newmap);
+            return true;
+   
+         case 15:  // O
+            execaction_ev(act_polymode);
+            return true;
+   
+         case 16:  // P
+            execaction_ev(act_changeproduction);
+            return true;
+   
+         case 17:  // Q
+            execaction_ev(act_end);
+            return true;
+                        
+         case 18: // R
+            execaction_ev(act_repaintdisplay);
+            return true;
+   
+         case 19 :  // S
+            execaction_ev(act_savemap);
+            return true;
+
+         case 21: // U
+            execaction_ev(act_unitinfo);
+            return true;
+   
+         case 22:  // V
+            execaction_ev(act_pasteFromClipboard);
+            return true;
+   
+         case 23: // W
+            execaction_ev(act_setactweatherglobal);
+            return true;
+
+         case 24: // X
+            execaction_ev(act_cutToClipboard);
+            return true;
+
+      }
    }
 
    if ( mod & KMOD_SHIFT ) {
@@ -958,6 +997,19 @@ bool Maped_MainScreenWidget :: selectMine()
    return true;
 }
 
+bool Maped_MainScreenWidget :: selectLuaBrush()
+{
+   ASCString file = selectFile( "*.brush.lua", true );
+   if ( file.empty () )
+      return true;
+   
+   LuaBrush brush ( file );
+   selection.setSelection( brush );
+
+   
+   return true;
+}
+
 
 
 void displaymessage2( const char* formatstring, ... )
