@@ -30,7 +30,7 @@
 #include "graphics/drawing.h"
 #include "fieldimageloader.h"
 #include "memsize_interface.h"
-
+#include "iconrepository.h"
 
 
 #ifndef converter
@@ -80,7 +80,7 @@ bool  ObjectType :: buildable ( tfield* fld ) const
    if ( fld->building && !growOnUnits )
       return false;
 
-   if ( getFieldModification( fld->getweather() ).terrainaccess.accessible ( fld->bdt ) <= 0 )
+   if ( getFieldModification( fld->getWeather() ).terrainaccess.accessible ( fld->bdt ) <= 0 )
        return false;
 
    #endif
@@ -142,8 +142,24 @@ const Surface& ObjectType :: getPicture ( int i, int w ) const
 
    if ( weatherPicture[w].bi3pic[i] > 0 )
       return GraphicSetManager::Instance().getPic(weatherPicture[w].bi3pic[i]);
-   else
-      return weatherPicture[w].images[i];
+   else {
+      const Surface& s = weatherPicture[w].images[i];
+      if ( s.valid() )
+         return s;
+      else {
+         if ( i < 65 && weatherPicture[w].images.size() >= 66 ) {
+            const Surface& s2 = weatherPicture[w].images[65];
+            if ( s2.valid())
+               return s2;
+         } 
+         if ( i < 64 && weatherPicture[w].images.size() >= 65 ) {
+            const Surface& s2 = weatherPicture[w].images[64];
+            if ( s2.valid())
+               return s2;
+         } 
+         return IconRepository::getIcon("red-x-field.png");
+      }
+   }
 }
 
 
@@ -201,10 +217,19 @@ void ObjectType::realDisplay ( Surface& surface, const SPoint& pos, int dir, int
       flip = weatherPicture[weather].flip[dir];
 
    if ( displayMethod==1 ) { // SHADOW: buried pipeline, tracks, ...
-      megaBlitter<ColorTransform_None, ColorMerger_AlphaLighter, SourcePixelSelector_DirectFlip,TargetPixelSelector_All>(getPicture( dir, weather), surface, pos, nullParam, 0.7, flip, nullParam); 
+      const Surface& s = getPicture( dir, weather);
+      if ( dir != 0 && rotateImage ) {
+         megaBlitter<ColorTransform_None, ColorMerger_AlphaLighter, SourcePixelSelector_CacheRotation,TargetPixelSelector_All>(s, surface, pos, nullParam,0.7,make_pair(&s,directionangle[dir%6]),nullParam); 
+      } else {
+         megaBlitter<ColorTransform_None, ColorMerger_AlphaLighter, SourcePixelSelector_DirectFlip,TargetPixelSelector_All>(s, surface, pos, nullParam, 0.7, flip, nullParam); 
+      }
    } else
       if ( displayMethod == 2 ) {  // translation
-         if ( getPicture( dir, weather).GetPixelFormat().BitsPerPixel() == 8 ) {
+         const Surface& s = getPicture( dir, weather);
+         if ( !s.valid() )
+            return;
+         
+         if ( s.GetPixelFormat().BitsPerPixel() == 8 ) {
             MegaBlitter< 1,4,
                         ColorTransform_None, 
                         ColorMerger_Alpha_XLAT_TableShifter, 
@@ -214,12 +239,15 @@ void ObjectType::realDisplay ( Surface& surface, const SPoint& pos, int dir, int
                      >
                blitter;
             blitter.setFlipping( flip & 1, flip & 2 );
-            blitter.blit( getPicture( dir, weather), surface, pos );
+            blitter.blit( s, surface, pos );
          } // else
            // warning("objects with palette translation as display method may not have truecolor images");
       } else
          if ( displayMethod == 4 ) {
             const Surface& s = getPicture( dir, weather);
+            if ( !s.valid() )
+               return;
+   
             if ( dir != 0 && rotateImage ) {
                megaBlitter<ColorTransform_None, ColorMerger_AlphaMixer, SourcePixelSelector_CacheRotation ,TargetPixelSelector_All>(s, surface, pos, nullParam,nullParam,make_pair(&s,directionangle[dir%6]),nullParam); 
             } else {
@@ -233,9 +261,11 @@ void ObjectType::realDisplay ( Surface& surface, const SPoint& pos, int dir, int
             #endif
             if ( disp ) {
                const Surface& s = getPicture( dir, weather);
+               if ( !s.valid() )
+                  return;
                if ( flip ) {
                   // if ( s.flags() & SDL_SRCALPHA )
-                     megaBlitter<ColorTransform_None, ColorMerger_AlphaMerge, SourcePixelSelector_DirectFlip,TargetPixelSelector_All>(getPicture( dir, weather), surface, pos, nullParam,nullParam, flip, nullParam); 
+                     megaBlitter<ColorTransform_None, ColorMerger_AlphaMerge, SourcePixelSelector_DirectFlip,TargetPixelSelector_All>(s, surface, pos, nullParam,nullParam, flip, nullParam); 
                   // else   
                   //   megaBlitter<ColorTransform_None, ColorMerger_AlphaOverwrite, SourcePixelSelector_DirectFlip,TargetPixelSelector_All>(getPicture( dir, weather), surface, pos, nullParam,nullParam, flip, nullParam); 
                } else {  
@@ -246,7 +276,7 @@ void ObjectType::realDisplay ( Surface& surface, const SPoint& pos, int dir, int
                      //   megaBlitter<ColorTransform_None, ColorMerger_AlphaOverwrite, SourcePixelSelector_CacheRotation,TargetPixelSelector_All>(s, surface, pos, nullParam,nullParam,make_pair(&s,directionangle[dir]),nullParam); 
                   } else {
                      // if ( s.flags() & SDL_SRCALPHA )
-                        megaBlitter<ColorTransform_None, ColorMerger_AlphaMerge, SourcePixelSelector_Plain,TargetPixelSelector_All>(getPicture( dir, weather), surface, pos, nullParam,nullParam,nullParam,nullParam); 
+                        megaBlitter<ColorTransform_None, ColorMerger_AlphaMerge, SourcePixelSelector_Plain,TargetPixelSelector_All>(s, surface, pos, nullParam,nullParam,nullParam,nullParam); 
                      // else
                      //   megaBlitter<ColorTransform_None, ColorMerger_AlphaOverwrite, SourcePixelSelector_Plain,TargetPixelSelector_All>(getPicture( dir, weather), surface, pos, nullParam,nullParam,nullParam,nullParam); 
                   }
@@ -419,7 +449,7 @@ class Smoothing {
                tfield* fld = getfield ( x1, y1 );
                if ( fld ) {
 
-                  Object* obj = fld->checkforobject ( o );
+                  Object* obj = fld->checkForObject ( o );
                   if ( obj )
                      if ( obj->typ->weather.test(0) )
                         if ( IsInSetOfWord ( obj->typ->weatherPicture[0].bi3pic[ obj->dir ], Arr ))
@@ -496,7 +526,7 @@ class Smoothing {
            for ( int Y = 0 ; Y < actmap->ysize; Y++ )
              for ( int X = 0; X < actmap->xsize; X++ ) {
                  if ( TerObj ) {
-                    Object* obj = getfield ( X, Y )-> checkforobject ( TerObj );
+                    Object* obj = getfield ( X, Y )-> checkForObject ( TerObj );
                     if ( obj  && obj->typ->weather.test(0) ) {
                        int Old = obj->dir; // bipicnum
                                            //    Old:= TRawArrEck(Mission.ACTN[Y, X])[TerObj];  // bisherige Form / oder Bildnummer ?
@@ -760,7 +790,7 @@ void calculateforest( GameMap* actmap, ObjectType* woodObj )
                      tfield* fld2 = actmap->getField(a,b);
 
                      if ( fld2 ) {
-                        Object* oi = fld2->checkforobject ( o->typ );
+                        Object* oi = fld2->checkForObject ( o->typ );
                         if ( oi )
                            if ( oi->dir <= 20  ||  run == 0 )
                               c |=  1 << i ;
