@@ -60,9 +60,11 @@
 
 #include "graphics/drawing.h"
 
-#include "messaginghub.h"
+#include "util/messaginghub.h"
 
 
+#include "sdl/graphicsqueue.h"
+
 class EventSupplier: public PG_SDLEventSupplier {
    public:
 
@@ -121,6 +123,30 @@ void signalQuit( int i )
 }
 
 
+class ASC_PG_ScreenUpdater : public PG_ScreenUpdater {
+   public:
+      void UpdateRect(SDL_Surface *screen, Sint32 x, Sint32 y, Sint32 w, Sint32 h)
+      {
+      #ifdef WIN32
+         queueOperation( new UpdateRectOp( screen, x, y, w, h ));
+      #else
+         SDL_UpdateRect( screen,x,y,w,h);
+         postScreenUpdate(screen);
+      #endif
+      };
+
+      void UpdateRects(SDL_Surface *screen, int numrects, SDL_Rect *rects)
+      {
+      #ifdef WIN32
+         queueOperation( new UpdateRectsOp( screen, numrects, rects ));
+      #else
+         SDL_UpdateRects( screen, numrects, rects );
+         postScreenUpdate(screen);
+      #endif
+      }
+} ascScreenUpdater;
+
+
 ASC_PG_App :: ASC_PG_App ( const ASCString& themeName )  : fullScreen(false), bitsperpixel(0)
 {
    this->themeName = themeName;
@@ -153,6 +179,7 @@ ASC_PG_App :: ASC_PG_App ( const ASCString& themeName )  : fullScreen(false), bi
 
    pgApp = this;
    SetEventSupplier ( &eventSupplier );
+   SetScreenUpdater ( &ascScreenUpdater );
    
    signal ( SIGINT, &signalQuit );
    
@@ -309,6 +336,8 @@ StartupScreen::StartupScreen( const ASCString& filename, SigC::Signal0<void>& ti
       float enw = float(PG_Application::GetScreenWidth() )/float(fullscreenImage.w());
       float enh = float(PG_Application::GetScreenHeight())/float(fullscreenImage.h());
 
+	  displayLogMessage(6, "Startupscreen : %f / %f", enw, enh ); 
+
       // we allow a asymetric stretch of 5% 
       if ( enw / enh < 0.95 || enw / enh > 1.05 ) 
          enh = enw = min( enw, enh );
@@ -394,10 +423,8 @@ void ASC_PG_App::processEvent( )
 int ASC_PG_App::Run ( )
 {
    enableLegacyEventHandling ( false );
-
    PG_Application::Run();
-   
-   enableLegacyEventHandling ( true );
+  
    return 0;
 }
 
@@ -743,3 +770,25 @@ ASCString editString2( const ASCString& title, const ASCString& defaultValue )
    se.RunModal();
    return se.GetEditedText();
 }
+
+int choiceDialog(const ASCString& text, const ASCString& button1, const ASCString& button2, const ASCString& shortLabel )
+{
+
+   map<ASCString,int>& answers = CGameOptions::Instance()->dialogAnswers;
+
+   map<ASCString,int>::iterator i = answers.find( shortLabel );
+   if ( i != answers.end() )
+      return i->second;
+
+   bool saveResult = false;
+   int result = new_choice_dlg(text, shortLabel, button1, button2, saveResult);
+
+   if ( saveResult ) {
+      answers[shortLabel] = result;
+      CGameOptions::Instance()->setChanged();
+   }
+
+   return  result;
+
+}
+
