@@ -33,10 +33,6 @@
 PlayerID::PlayerID( const Player& p ) : num( p.getPosition() ) {};
 
 
-SigC::Signal4<void,GameMap*,int,int,DiplomaticStates> DiplomaticStateVector::anyStateChanged;
-SigC::Signal3<void,GameMap*,int,int> DiplomaticStateVector::shareViewChanged;
-
-
 const char* diplomaticStateNames[diplomaticStateNum+1] = 
 {
    "War",
@@ -70,159 +66,63 @@ void DiplomaticStateVector::turnBegins()
             // this should have been handled when the other player set his proposal 
          } else {
             // he did not answer
+            // we are still doing nothing and deleting the request with the .clear below
          }
       }
    }
    queuedStateChanges.clear();
+   
+   // changes to war will be effective as soon as a new player is active,
+   // so that the other player will get reaction fire if he approaches the war-declaring player
+   for ( int p = 0; p < player.getParentMap()->getPlayerCount(); ++p ) {
+      Player& pl = player.getParentMap()->getPlayer(p);
+      if ( pl.exist() && p != player.getPosition() ) {
+         DiplomaticStates state;
+         if ( getProposal(p, &state ) )
+            if ( state < getState(p)) {
+               setState( p, state );
+               pl.diplomacy.setState( player.getPosition(), state );
+            }
+      }
+   }
 }
 
 
 
-DiplomaticStates DiplomaticStateVector::getState( int towardsPlayer ) const
+DiplomaticStates DiplomaticStateVector::getState( PlayerID towardsPlayer ) const
 {
-   if ( towardsPlayer < 0 )
+   if ( towardsPlayer.getID() < 0 )
       return WAR;
 
-   if ( player.getPosition() == towardsPlayer )
+   if ( player.getPosition() == towardsPlayer.getID() )
       return ALLIANCE;
       
-   if ( towardsPlayer < states.size() )
-      return states[towardsPlayer];
+   if ( towardsPlayer.getID() < states.size() )
+      return states[towardsPlayer.getID()];
    else
       return WAR;
 }
 
-void DiplomaticStateVector::setState( int towardsPlayer, DiplomaticStates s, bool fireSignal )
+void DiplomaticStateVector::resize( int size )
 {
-   assert( towardsPlayer >= 0 );
-
-   if ( towardsPlayer >= states.size() ) 
-      states.resize(towardsPlayer+1);
-      
-   states[towardsPlayer] = s;
-   
-   if ( fireSignal )
-      anyStateChanged( player.getParentMap(), player.getPosition(), towardsPlayer,s);
-}
-
-void DiplomaticStateVector::sneakAttack( int towardsPlayer )
-{
-   assert( towardsPlayer >= 0 );
-
-   setState( towardsPlayer, WAR );         
-   player.getParentMap()->player[ towardsPlayer ].diplomacy.setState( player.getPosition(), WAR );
-   
-   int to = 0;
-   for ( int j = 0; j < 8; j++ )
-      if ( j != player.getPosition() )
-         to |= 1 << j;
-
-   ASCString txt;
-   txt.format ( getmessage( 10001 ), player.getName().c_str(), player.getParentMap()->player[towardsPlayer].getName().c_str() );
-   new Message ( txt, player.getParentMap(), to );
+   int oldsize  = states.size();
+   states.resize(size);
+   for ( int i = oldsize; i < size; ++i )
+      states[i] = WAR;
 }
 
 
-void DiplomaticStateVector::changeToState( int towardsPlayer, DiplomaticStates s, bool mail )
+void DiplomaticStateVector::setState( PlayerID towardsPlayer, DiplomaticStates s )
 {
-   assert( towardsPlayer >= 0 );
+   assert( towardsPlayer.getID() >= 0 );
 
-   int msgid;
-   if ( s > getState( towardsPlayer ))
-      msgid = 10003;  //  propose peace
-   else
-      msgid = 10002;  // declare war
-
-
-   bool oldShareView = sharesView(towardsPlayer);
-
-   setState( towardsPlayer, s );
-   
-   DiplomaticStateVector& targ = player.getParentMap()->player[ towardsPlayer ].diplomacy;
-   targ.setState( player.getPosition(), s );
-
-   if( sharesView(towardsPlayer) != oldShareView )
-      shareViewChanged( player.getParentMap(), player.getPosition(), towardsPlayer );
-
-   if ( mail ) {
-      ASCString txt;
-      txt.format( getmessage( msgid ), player.getName().c_str(), diplomaticStateNames[s]  );
-      new Message ( txt, player.getParentMap(), 1 << towardsPlayer );
-   }
-/*
-   for ( int p = 0; p < player.getParentMap()->getPlayerCount(); ++p )
-      if ( p != player.getPosition() ) {
-         if ( getState( p ) == ALLIANCE ) {
-            // we have an allied player which must now react
-            DiplomaticStateVector& ally = player.getParentMap()->getPlayer( p ).diplomacy;
-            if ( ally.getState( towardsPlayer ) != s )
-               ally.changeToState( towardsPlayer, s );
+   if ( towardsPlayer.getID() >= states.size() ) 
+      resize(towardsPlayer.getID()+1);
       
-         }
-         
-         if ( targ.getState( p ) == ALLIANCE ) {
-            // the opponent has an ally which must now react
-            if ( getState( p ) != s )
-               changeToState( p, s );
-      
-         }
-      }
-      */
+   states[towardsPlayer.getID()] = s;
 }
 
 
-void DiplomaticStateVector::propose( int towardsPlayer, DiplomaticStates s )
-{
-   assert( towardsPlayer >= 0 );
-
-   DiplomaticStateVector& targ = player.getParentMap()->player[ towardsPlayer ].diplomacy;
-
-   QueuedStateChanges::iterator i = targ.queuedStateChanges.find( player.getPosition() );
-         
-   DiplomaticStates currentState = getState( towardsPlayer ) ;
-
-   if ( i == targ.queuedStateChanges.end() || (s < currentState && i->second > currentState)) {
-      // we only send a message if this is an initial proposal OR
-      // if the other player proposed a more peaceful state, but we are setting a more hostile state
-      ASCString txt;
-      int msgid;
-      if ( s > getState( towardsPlayer )) 
-         msgid = 10003;  //  propose peace
-      else
-         msgid = 10002;  //  declare war
-            
-      txt.format( getmessage( msgid ), player.getName().c_str(), diplomaticStateNames[s]  ); 
-      new Message ( txt, player.getParentMap(), 1 << towardsPlayer );
-      
-      queuedStateChanges[towardsPlayer] = s;
-   }  else {
-      // we are answering a proposal by the other player
-      
-      if ( s > getState( towardsPlayer )) {
-         // our proposal is about going to a more peaceful state 
-
-         if ( s > i->second ) {
-            // we are proposing even more peace, but we'll only set the state he proposed and keep our proposal
-            changeToState( towardsPlayer, i->second );
-         } else {
-            // he proposes less or equal peace, but we'll set the state he proposed and delete his proposal, because it's fulfilled
-            changeToState( towardsPlayer, s, true );  // s < i->second would only send mail if the proposal differs
-            targ.queuedStateChanges.erase( i );
-         }
-      } else {
-         if ( s < i->second ) {
-            // we go to an even more hostile state 
-            changeToState( towardsPlayer, i->second );
-            targ.queuedStateChanges.erase( i );
-         } else {
-            // we are going to a state that is more hostile than the current one, but less hostile then the other players declaration
-            changeToState( towardsPlayer, s, false );
-         }
-
-      }
-   }
-   
-}
 
 bool DiplomaticStateVector::getProposal( int fromPlayer, DiplomaticStates* state )
 {
@@ -314,6 +214,10 @@ void Player :: turnBegins( Player& p )
    if ( &p == this ) {
       diplomacy.turnBegins();
    }   
+   
+   if ( stat == supervisor ) {
+      parentMap->supervisorpasswordcrc = passwordcrc;  
+   }
 }
 
 void Player :: userInteractionBegins( Player& p )
@@ -410,6 +314,30 @@ bool Player::exist() const
   return !(buildingList.empty() && vehicleList.empty());
 }
 
+
+const int playerVersion = 2; 
+
+void Player::read ( tnstream& stream )
+{
+   int version  = stream.readInt();
+   if ( version > playerVersion )
+      throw tinvalidversion ( "Player", playerVersion, version );
+   
+   // to be filled with future attributes
+}
+
+void Player::write ( tnstream& stream ) const
+{
+   stream.writeInt( playerVersion );
+   // to be filled with future attributes
+}
+
+
+
+
+
+
+
 template <typename T> void swapData( T& t1, T& t2 )
 {
    T temp = t1;
@@ -453,10 +381,10 @@ void Player::swap ( Player& secondPlayer )
 
    BL bl2 = secondPlayer.buildingList;
    for ( BLI i = bl2.begin(); i != bl2.end(); ++i)
-      (*i)->convert( player );
+      (*i)->convert( player, false );
 
    for ( BLI i = bl.begin(); i != bl.end(); ++i)
-      (*i)->convert( secondPlayer.player );
+      (*i)->convert( secondPlayer.player, false );
 
    for (int i =0; i < parentMap->xsize * parentMap->ysize ;i++ ) {
       tfield* fld = &parentMap->field[i];
@@ -536,8 +464,15 @@ void DiplomaticStateVector::swap( int secondPlayer )
    swapData( states, secondDSV.states );
    swapData( queuedStateChanges, secondDSV.queuedStateChanges );
 
-   for ( int i= 0; i < player.getParentMap()->getPlayerCount(); ++i )
-      swapData( player.getParentMap()->getPlayer(i).diplomacy.states[secondPlayer],  player.getParentMap()->getPlayer(i).diplomacy.states[player.getPosition()] );
+   
+
+   for ( int i= 0; i < player.getParentMap()->getPlayerCount(); ++i ) {
+      DiplomaticStates sec = player.getParentMap()->getPlayer(i).diplomacy.getState(secondPlayer);
+      DiplomaticStates fir = player.getParentMap()->getPlayer(i).diplomacy.getState(player.getPosition());
+
+      player.getParentMap()->getPlayer(i).diplomacy.setState(secondPlayer, fir );
+      player.getParentMap()->getPlayer(i).diplomacy.setState(player.getPosition(), sec );
+   }
 
 }
 
