@@ -56,36 +56,34 @@ extern const char* diplomaticStateNames[diplomaticStateNum+1];
 class DiplomaticStateVector : public SigC::Object {
 
       friend class AllianceSetupWidget;
+      friend class ChangeDiplomaticState;
+      friend class DiplomacyCommand;
 
       Player& player; 
       
       typedef vector<DiplomaticStates> States;
       States states;
       
+       
       typedef map<int,DiplomaticStates> QueuedStateChanges;
       QueuedStateChanges queuedStateChanges;
 
-      void changeToState( int towardsPlayer, DiplomaticStates s, bool mail = true );
+      void resize( int size );
       
    public:
       DiplomaticStateVector( Player& _player );
       
-      DiplomaticStates getState( int towardsPlayer ) const;
-      void setState( int towardsPlayer, DiplomaticStates s, bool fireSignal = true );
-      void propose ( int towardsPlayer, DiplomaticStates s );
-      void sneakAttack( int towardsPlayer );
+      DiplomaticStates getState( PlayerID towardsPlayer ) const;
+      void setState( PlayerID towardsPlayer, DiplomaticStates s );
       
-      bool isHostile( PlayerID towardsPlayer ) { return getState( towardsPlayer.getID() ) == WAR; };
-      bool sharesView( PlayerID receivingPlayer ) { return getState( receivingPlayer.getID() ) >= PEACE_SV; };
-      bool isAllied( PlayerID towardsPlayer ) { return getState( towardsPlayer.getID() ) >= ALLIANCE; };
+      bool isHostile( PlayerID towardsPlayer ) const { return getState( towardsPlayer.getID() ) == WAR; };
+      bool sharesView( PlayerID receivingPlayer ) const { return getState( receivingPlayer.getID() ) >= PEACE_SV; };
+      bool isAllied( PlayerID towardsPlayer ) const { return getState( towardsPlayer.getID() ) >= ALLIANCE; };
 
       void turnBegins();
       
       void swap( int secondPlayer );
             
-      static SigC::Signal4<void,GameMap*,int,int,DiplomaticStates> anyStateChanged;
-      static SigC::Signal3<void,GameMap*,int,int> shareViewChanged;
-
       //! \returns true if there are any proposals
       bool getProposal( int fromPlayer, DiplomaticStates* state );
       
@@ -105,9 +103,10 @@ class Player : public SigC::Object {
       void userInteractionBegins( Player& p );
       void turnEnds( Player& p );
       
+      
    public:
       Player();
-      
+
       int getPosition() const { return player; };
       
       const GameMap* getParentMap() const { return parentMap; };
@@ -139,6 +138,8 @@ class Player : public SigC::Object {
 
       //! the status of the scientific research
       Research    research;
+      
+      Research& getResearch() { return research; };
 
       //! if the player is run by an AI, this is the pointer to it
       BaseAI*      ai;
@@ -156,6 +157,7 @@ class Player : public SigC::Object {
       ASCString getName( ) const;
       
       void setName( const ASCString& name ) { this->name = name; };
+      void setName( const char* name ) { setName( ASCString(name)); };
 
       //! the Password required for playing this player
       Password passwordcrc;
@@ -212,6 +214,8 @@ class Player : public SigC::Object {
       PlayerColor getPlayerColor() const;
       void setColor( const DI_Color& color ); 
 
+      void read ( tnstream& stream );
+      void write ( tnstream& stream ) const;
 
       void resetView();
       void resetResearch();
