@@ -12,6 +12,8 @@
 #include "spfst.h"
 #include "viewcalculation.h"
 #include "events.h"
+#include "gameoptions.h"
+#include "spfst-legacy.h"
 
 int ReplayMapDisplay :: checkMapPosition ( int x, int y )
 {
@@ -60,7 +62,7 @@ int ReplayMapDisplay :: checkMapPosition ( int x, int y )
 }
 
 
-int ReplayMapDisplay :: displayMovingUnit ( const MapCoordinate3D& start, const MapCoordinate3D& dest, Vehicle* vehicle, int fieldnum, int totalmove, SoundStartCallback startSound )
+int ReplayMapDisplay :: displayMovingUnit ( const MapCoordinate3D& start, const MapCoordinate3D& dest, Vehicle* vehicle, int fieldnum, int totalmove, SoundStartCallback startSound, int duration )
 {
    if ( actmap->getPlayerView() < 0 )
       return 0;
@@ -72,7 +74,7 @@ int ReplayMapDisplay :: displayMovingUnit ( const MapCoordinate3D& start, const
    tfield* fld2 = actmap->getField ( dest );
    
    if ( actmap->player[actmap->getPlayerView()].diplomacy.sharesView( vehicle->getOwner() )) 
-      view2 = fieldVisibility ( fld2, actmap->getPlayerView(), actmap );
+      view2 = fieldVisibility ( fld2, actmap->getPlayerView() );
    else {
       // This is a workaround to estimate if the target field will be visible or not once the units is there 
       view2 = calcvisibilityfield ( actmap, fld2, actmap->getPlayerView(), -1, actmap->getgameparameter ( cgp_initialMapVisibility ), vehicle->typ->jamming );
@@ -91,7 +93,7 @@ int ReplayMapDisplay :: displayMovingUnit ( const MapCoordinate3D& start, const
       if ( checkMapPosition  ( start.x, start.y ))
          displayMap();
 
-      int fc = mapDisplay->displayMovingUnit ( start, dest, vehicle, fieldnum, totalmove, startSound );
+      int fc = mapDisplay->displayMovingUnit ( start, dest, vehicle, fieldnum, totalmove, startSound, duration );
       if ( fc == 1 ) {
          mapDisplay->resetMovement();
          mapDisplay->displayMap();
@@ -155,3 +157,10 @@ void ReplayMapDisplay :: playPositionalSound( const MapCoordinate& pos, Sound* s
 }
 
 
+int ReplayMapDisplay :: getUnitMovementDuration() const 
+{ 
+   int speed = CGameOptions::Instance()->movespeed;
+   int factor = max( 10, CGameOptions::Instance()->replayMoveSpeedFactor );
+   return speed * 100 / factor;
+}
+
