@@ -27,7 +27,8 @@
  #include <cstring>
  #include <map>
 
- #include "libs/loki/Functor.h"
+ #include "loki/Functor.h"
+ #include "loki/Typelist.h"
 
  #include "basestreaminterface.h"
  #include "basictypes.h"
@@ -57,7 +58,7 @@ class TechDependency: public LoadableItemType {
      TechDependency(){ requireAllListedTechnologies = true; };
 
      bool available( const Research& research ) const;
-     typedef Loki::Functor<bool, TYPELIST_1(int) > CheckTechAvailabilityFunctor;
+     typedef Loki::Functor<bool, LOKI_TYPELIST_1(int) > CheckTechAvailabilityFunctor;
      ResearchAvailabilityStatus available( CheckTechAvailabilityFunctor checkTechAvailability ) const;
 
      /** this will recursively scan through the tech tree to check if this item will be available sometime.
@@ -99,7 +100,9 @@ class TechAdapter: public LoadableItemType {
 };
 
 class TechAdapterDependency {
+   public:
      typedef vector<ASCString> RequiredTechAdapter;
+   private:  
      RequiredTechAdapter requiredTechAdapter;
      bool         requireAllListedTechAdapter;
    public:
@@ -112,6 +115,8 @@ class TechAdapterDependency {
      void runTextIO ( PropertyContainer& pc, const ASCString& defaultTechAdapter = "");
 
      void writeInvertTreeOutput ( const ASCString& tech, tnstream& stream, const vector<IntRange>* onlyWithBaseTechs = NULL ) const;
+     
+     const RequiredTechAdapter& listAdapters() const { return requiredTechAdapter; };
 };
 
 
@@ -155,7 +160,7 @@ class TechAdapterDependency {
 
 
  class Research {
-
+     friend class DirectResearchCommand;
      GameMap* map;
      int player;
 
@@ -168,7 +173,7 @@ class TechAdapterDependency {
    public:
    #endif   
 
-     typedef std::map<ASCString,bool> TriggeredTechAdapter;
+     typedef std::set<ASCString> TriggeredTechAdapter;
      TriggeredTechAdapter triggeredTechAdapter;
 
 
@@ -188,41 +193,48 @@ class TechAdapterDependency {
      //! the technology that is defined as long-term goal
      const Technology* goal;
 
-     bool vehicletypeavailable ( const Vehicletype* fztyp ) const;
-
      void read ( tnstream& stream );
      void write ( tnstream& stream );
 
      void read_struct ( tnstream& stream, bool merge = false );
      void read_techs ( tnstream& stream, bool merge = false );
 
-     void evalTechAdapter();
+     /** checks for TechAdapters whose preconditions are now satisfied.
+         These TechAdapters will be activated.
+         \returns a list of TechAdapters that were activated 
+      */
+     vector<ASCString> evalTechAdapter();
      bool techAdapterAvail( const ASCString& ta ) const;
 
      void settechlevel ( int techlevel );
 
      void chainToMap ( GameMap* _map, int _player ) { map = _map; player = _player; };
 
-     //! Move the technology that is currently being reseached to the list of discovered technologies
-     void addtechnology();
-
-     void addanytechnology ( const Technology* tech );
+     /** adds the technology to the list of available technologies
+         \returns a list of TechAdapters that were activated due to this new technology 
+     */
+     vector<ASCString> addanytechnology ( const Technology* tech );
 
      ResearchAvailabilityStatus techAvailable ( const Technology* tech ) const;
 
      bool isBlocked( const Technology* tech ) const;
 
      /** is used by the chooseTechnology dialog: the first time no techs are available this variable is still true,
-         so the dialog shows "now techs avail". THen it sets techAvail to false, preventing the same message at the
+         so the dialog shows "no techs avail". THen it sets techAvail to false, preventing the same message at the
          beginning of each turn */
      bool techsAvail;
 
+
      int getResearchPerTurn() const;
      int currentTechAvailableIn() const;
 
      void setMultiplier( int m ) { multiplier = m; };
      int getMultiplier() { return multiplier; };
      
+     const vector<ASCString>& getPredefinedTechAdapter() { return predefinedTechAdapter; };
+     void setPredefinedTechAdapter( const set<ASCString>& adapter );
+     void addPredefinedTechAdapter( const ASCString& techAdapter );
+     
      Research ( );
      void clear();
      ~Research ();
