@@ -48,6 +48,7 @@
 #include "gameoptions.h"
 
 #ifdef _WIN32_
+ #include <direct.h>
  #include <windows.h>
  #include <winreg.h>
  #include <shlobj.h>
@@ -95,7 +96,7 @@ void loadpalette ( void )
       stream.readdata( &truecolor2pal_table,  sizeof ( truecolor2pal_table ));
       stream.readdata( &bi2asc_color_translation_table,  sizeof ( bi2asc_color_translation_table ));
 #endif
-      xlatpictgraytable = (ppixelxlattable) asc_malloc( sizeof(*xlatpictgraytable) );
+      xlatpictgraytable = (ppixelxlattable) malloc( sizeof(*xlatpictgraytable) );
       // xlatpictgraytable = new tpixelxlattable;
       generategrayxlattable(xlatpictgraytable,160,16,&pal);
 
@@ -178,8 +179,9 @@ ASCString getDirectory( ASCString filename )
    if( boost::regex_match( filename, what, dir)) {
       directory.assign( what[1].first, what[1].second );
       return directory;
-   } else
+   } else {
       return ".";
+   }
 }
 
 
@@ -190,7 +192,17 @@ void ConfigurationFileLocatorCore::setCommandLineParam( const ASCString& path )
 
 void ConfigurationFileLocatorCore::setExecutableLocation( const ASCString& path )
 {
-   exePath = getDirectory( path );
+    exePath = getDirectory( path );
+
+#ifdef WIN32
+   char buffer[_MAX_PATH];
+   if( _getcwd( buffer, _MAX_PATH ) ) 
+	  if ( exePath == "."  || exePath == ".\\" ) 
+         exePath = buffer;
+	  
+   
+#endif
+
 }
 
 
@@ -389,8 +401,10 @@ int readgameoptions ( const ASCString& filename )
         CGameOptions::Instance()->setChanged();
         if ( writegameoptions( filename ))
            displayLogMessage ( 6, "A config file has been sucessfully written to " + filename + " ");
-        else
+        else {
+           warningMessage("Unable to write file " + filename );
            displayLogMessage ( 6, "Failed to write config file to " + filename + " ");
+        }
      }
    }
 
@@ -498,22 +512,6 @@ void initFileIO ( const ASCString& configFileName, int skipChecks )
 
    if ( ! (skipChecks & 1 ))
       checkFileLoadability ( "palette.pal" );
-
-   if ( ! (skipChecks & 2 ))
-      checkFileLoadability ( "data.version" );
-
-   if ( ! (skipChecks & 8 ))
-      checkFileLoadability ( "trrobj.version" );
-
-   if ( ! (skipChecks & 0x10 ))
-      checkFileLoadability ( "trrobj2.version" );
-
-   if ( ! (skipChecks & 0x20 ))
-      checkFileLoadability ( "buildings.version" );
-
-   if ( ! (skipChecks & 0x40 ))
-      checkFileLoadability ( "markedfielddark.png" );
-   
 }
 
 void versionError( const ASCString& filename, const ASCString& location )
@@ -523,32 +521,3 @@ void versionError( const ASCString& filename, const ASCString& location )
    msg += "The old file is " + location;
    fatalError( msg );
 }
-
-void checkFileVersion( const ASCString& filename, const ASCString& containername, int version )
-{
-   ASCString location;
-   bool dataOk = true;
-   if ( exist ( filename )) {
-      tnfilestream s ( filename, tnstream::reading );
-      ASCString str = s.readString();
-      int v = atoi ( str.c_str() );
-      if ( v < version )
-         dataOk = false;
-      location = s.getLocation();
-   } else
-      dataOk = false;
-
-   if ( !dataOk )
-      versionError ( containername, location );
-
-}
-
-void checkDataVersion( )
-{
-   ASCString location;
-   checkFileVersion( "main.version", "main.ascdat", 8 );
-
-   if ( exist( "pbp.ascdat" ))
-      checkFileVersion( "pbp.version", "pbp.ascdat", 25 );
-}
-
