@@ -20,6 +20,7 @@
 
 #include <string.h>
 #include <sys/stat.h>
+#include <errno.h>
 #include "simplestream.h"
 #include "errors.h"
 
@@ -202,8 +203,9 @@ tn_file_buf_stream::tn_file_buf_stream( const ASCString& _fileName, IOMode mode)
 
      devicename = s;
 
-   } else
-     throw tfileerror( s.c_str() );
+   } else {
+      throw tfileerror( s + " : " + strerror(errno) );
+   }
 
 }
 
@@ -224,8 +226,9 @@ void tn_file_buf_stream::seek( int newpos )
          actmempos = newpos - ( actfilepos - datasize );
       else {
          fseek( fp, newpos, SEEK_SET );
+
          if ( ferror ( fp ) )
-            throw  tfileerror ( getDeviceName() );
+            throw  tfileerror ( getDeviceName() + " : " + strerror(errno) );
       
          actmempos = 0; 
          actfilepos = newpos; 
@@ -266,8 +269,8 @@ tn_file_buf_stream::~tn_file_buf_stream()
       writebuffer();
 
    int res = fclose( fp );
-   if ( res != 0 )
-      throw  tfileerror ( getDeviceName() );
+   if ( res != 0 ) 
+      throw  tfileerror ( getDeviceName() + " : " + strerror(errno));
       
    _mode = uninitialized;
 
