@@ -40,6 +40,8 @@
 #include "buildings.h"
 #include "mapfield.h"
 
+#include "spfst-legacy.h"
+
 SigC::Signal0<void> repaintMap;
 SigC::Signal0<void> repaintDisplay;
 SigC::Signal0<void> updateFieldInfo;
@@ -49,6 +51,7 @@ SigC::Signal1<void,Vehicletype*> showVehicleTypeInfo;
 SigC::Signal0<void> viewChanged;
 SigC::Signal1<void,GameMap*> mapChanged;
 SigC::Signal0<bool> idleEvent;
+SigC::Signal1<void, const Context&> fieldCrossed;
 
 
 
@@ -131,7 +134,7 @@ int         fieldAccessible( const tfield*        field,
       uheight = vehicle->height;
 
    if ( !ignoreVisibility ) {
-      int c = fieldVisibility ( field, vehicle->color/8 );
+      int c = fieldVisibility ( field, vehicle->getOwner() );
 
       if (field == NULL)
         return 0;
@@ -169,16 +172,17 @@ int         fieldAccessible( const tfield*        field,
            if ( terrainaccessible ( field, vehicle, uheight ) ) {
               if (vehicleplattfahrbar(vehicle,field))
                  return 2;
-               else 
-                 if ( getheightdelta(log2(field->vehicle->height), log2(vehicle->height)) || (attackpossible28(field->vehicle,vehicle) == false) ||  actmap->player[actmap->actplayer].diplomacy.getState( field->vehicle->getOwner()) >= PEACE )
+               else {
+                  if ( getheightdelta(log2(field->vehicle->height), log2(vehicle->height)) || (attackpossible28(field->vehicle,vehicle) == false) || ( vehicle->getMap()->getPlayer(vehicle).diplomacy.getState( field->vehicle->getOwner()) >= PEACE ))
                     return 1;
+               }
            }
       }
       else {   // building
         if ((field->bdt & getTerrainBitType(cbbuildingentry) ).any() && field->building->vehicleLoadable ( vehicle, uheight, attacked ))
            return 2;
         else
-           if (uheight >= chtieffliegend || (field->building->typ->buildingheight <= chgetaucht && uheight >=  chschwimmend ))
+           if (uheight >= chtieffliegend || (field->building->typ->height <= chgetaucht && uheight >=  chschwimmend ))
               return 1;
            else
               return 0;
@@ -203,7 +207,8 @@ tfield*        getfield(int          x,
 
 
 
-void         putbuilding( const MapCoordinate& entryPosition,
+void         putbuilding( GameMap* actmap,
+                          const MapCoordinate& entryPosition,
                          int          color,
                          const BuildingType* buildingtyp,
                          int          completion,
@@ -236,122 +241,42 @@ void         putbuilding( const MapCoordinate& entryPosition,
 }
 
 
-void         putbuilding2( const MapCoordinate& entryPosition,
-                           int         color,
-                           BuildingType* buildingtyp)
-{ 
-   if ( color & 7 )
-      fatalError("putbuilding muss eine farbe aus 0,8,16,24,.. uebergeben werden !",2);
-
-   for ( int a = 0; a < 4; a++)
-      for ( int b = 0; b < 6; b++ )
-         if ( buildingtyp->fieldExists ( BuildingType::LocalCoordinate( a, b ) ) ) {
-            tfield* field = actmap->getField( buildingtyp->getFieldCoordinate( entryPosition, BuildingType::LocalCoordinate(a,b) ));
-            if (field == NULL)
-               return ;
-            else {
-               if ( field->vehicle )
-                  return;
-            }
-         }
-
-   if ( !actmap->getField(entryPosition)->building ) {
-      Building* gbde = new Building ( actmap, entryPosition, buildingtyp, color/8 );
-
-      Resources maxplus;
-      Resources actplus;
-      Resources biplus;
-      /*
-      int maxresearch = 0;
-      bool found = false;
-      for ( GameMap::Player::BuildingList::iterator i = actmap->player[color/8].buildingList.begin(); i != actmap->player[ color/8].buildingList.end(); i++ ) {
-         Building* bld = *i;
-         if ( bld->typ == gbde->typ  && bld != gbde ) {
-            found = true;
-
-            for ( int r = 0; r < 3; r++ ) {
-               if ( bld->maxplus.resource(r) > maxplus.resource(r) )
-                 maxplus  = bld->maxplus;
-
-               if ( bld->bi_resourceplus.resource(r) > biplus.resource(r) )
-                  biplus = bld->bi_resourceplus;
-
-               if ( bld->plus.resource(r) > actplus.resource(r) )
-                  actplus = bld->plus;
-            }
-
-            if ( bld->maxresearchpoints > maxresearch )
-               maxresearch = bld->maxresearchpoints;
-         }
-      }
-
-      gbde->damage = 0;
-      if ( found ) {
-         if ( actmap->_resourcemode == 1 ) {
-            gbde->plus.energy = biplus.energy;
-            gbde->plus.material = biplus.material;
-            gbde->plus.fuel = biplus.fuel;
-         } else {
-            gbde->plus.energy = maxplus.energy;
-            gbde->plus.material = maxplus.material;
-            gbde->plus.fuel = maxplus.fuel;
-         }
-         gbde->maxplus = maxplus;
-         gbde->bi_resourceplus = biplus;
-      } else {
-         gbde->plus = gbde->defaultProduction;
-         gbde->maxplus = gbde->defaultProduction;
-         gbde->bi_resourceplus = gbde->defaultProduction;
-      }
-      */
-      
-      gbde->plus = gbde->typ->defaultProduction;
-      gbde->maxplus = gbde->typ->defaultProduction;
-      gbde->bi_resourceplus = gbde->typ->defaultProduction;
-
-      gbde->actstorage.fuel = 0;
-      gbde->actstorage.material = 0;
-      gbde->actstorage.energy = 0;
-      gbde->netcontrol = 0;
-      gbde->connection = 0;
-      gbde->visible = true;
-      gbde->setCompletion ( 0 );
-   }
-   else {
-      Building* gbde = actmap->getField(entryPosition)->building;
-      if (gbde->getCompletion() < gbde->typ->construction_steps-1)
-         gbde->setCompletion( gbde->getCompletion()+1 );
-
-   }
-}
 
 
 
-
-
-
-void checkobjectsforremoval ( void )
+void checkobjectsforremoval ( GameMap* gamemap )
 {
-   for ( int y = 0; y < actmap->ysize; y++ )
-      for ( int x = 0; x < actmap->xsize; x++ ) {
+   for ( int y = 0; y < gamemap->ysize; y++ )
+      for ( int x = 0; x < gamemap->xsize; x++ ) {
          tfield* fld = getfield ( x, y );
          for ( tfield::ObjectContainer::iterator i = fld->objects.begin(); i != fld->objects.end();  )
-            if ( i->typ->getFieldModification(fld->getweather()).terrainaccess.accessible ( fld->bdt ) < 0 ) {
-               fld->removeobject ( i->typ, true );
+            if ( i->typ->getFieldModification(fld->getWeather()).terrainaccess.accessible ( fld->bdt ) < 0 ) {
+               fld->removeObject ( i->typ, true );
                i = fld->objects.begin();
             } else
                i++;
       }
 }
 
-void  checkunitsforremoval ( void )
+void  checkunitsforremoval ( GameMap* gamemap )
 {
+   ASCString messages[playerNum];
+   for ( int y = 0; y < gamemap->ysize; y++ )
+      for ( int x = 0; x < gamemap->xsize; x++ ) {
+         tfield* fld = gamemap->getField ( x, y );
+         if ( (fld->building && fld->building->typ->terrainaccess.accessible( fld->bdt ) < 0) && (fld->building->typ->height <= chfahrend) ) {
+            messages[fld->building->getOwner()] += getBuildingReference( fld->building ) + " was destroyed \n\n";
+            delete fld->building;
+         }
+      }
+
+
    for ( int c=0; c<=8 ;c++ ) {
-      ASCString msg;
-      for ( Player::VehicleList::iterator i = actmap->player[c].vehicleList.begin(); i != actmap->player[c].vehicleList.end();  ) {
+      ASCString msg = messages[c];
+      for ( Player::VehicleList::iterator i = gamemap->player[c].vehicleList.begin(); i != gamemap->player[c].vehicleList.end();  ) {
 
           Vehicle* eht = *i;
-          tfield* field = getfield(eht->xpos,eht->ypos);
+          tfield* field = gamemap->getField(eht->xpos,eht->ypos);
           bool erase = false;
 
           ASCString reason;
@@ -362,7 +287,7 @@ void  checkunitsforremoval ( void )
                    reason = "was swallowed by the ground";
                 }
              if ( eht )
-                if ( getmaxwindspeedforunit( eht ) < actmap->weather.windSpeed*maxwindspeed ) {
+                if ( getmaxwindspeedforunit( eht ) < gamemap->weather.windSpeed*maxwindspeed ) {
                    reason = "was blown away by the wind";
                    erase = true;
                 }
@@ -372,20 +297,20 @@ void  checkunitsforremoval ( void )
              msg += "\n\n";
 
              Vehicle* pv = *i;
-             actmap->player[c].vehicleList.erase ( i );
+             gamemap->player[c].vehicleList.erase ( i );
              delete pv;
 
 
 
              /* if the unit was a transport and had other units loaded, these units have been deleted as well.
                 We don't know which elements of the container are still valid, so we start from the beginning again. */
-             i = actmap->player[c].vehicleList.begin();
+             i = gamemap->player[c].vehicleList.begin();
           } else
              i++;
       }
 
       if ( !msg.empty() )
-         new Message ( msg, actmap, 1<<c);
+         new Message ( msg, gamemap, 1<<c);
    }
 }
 
@@ -406,7 +331,7 @@ int  getwindheightforunit ( const Vehicle* eht, int uheight )
 
 int  getmaxwindspeedforunit ( const Vehicle* eht )
 {
-   tfield* field = getfield(eht->xpos,eht->ypos);
+   tfield* field = eht->getMap()->getField(eht->xpos,eht->ypos);
    if ( field->vehicle == eht) {
       if (eht->height >= chtieffliegend && eht->height <= chhochfliegend ) //    || ((eht->height == chfahrend) && ( field->typ->art & cbwater ))) ) 
          return eht->typ->movement[log2(eht->height)] * 256 ;
@@ -485,22 +410,23 @@ int getheightdelta ( int height1, int height2 )
    return hd;
 }
 
-bool fieldvisiblenow( const tfield* pe, int player, Vehicle* veh, GameMap* actmap )
+bool fieldvisiblenow( const tfield* pe, Vehicle* veh, int player )
 {
+   GameMap* gamemap = pe->getMap();
    if ( player == -1 )
       return true;
 
    if ( player < -1 )
       return false;
 
-   if ( !actmap )
+   if ( !gamemap )
       return false;
   
    if ( pe ) {
       int c = (pe->visible >> ( player * 2)) & 3;
 
-      if ( c < actmap->getInitialMapVisibility( player ) )
-         c = actmap->getInitialMapVisibility( player );
+      if ( c < gamemap->getInitialMapVisibility( player ) )
+         c = gamemap->getInitialMapVisibility( player );
 
       if (c > visible_ago) {
          if ( !veh )
@@ -512,7 +438,7 @@ bool fieldvisiblenow( const tfield* pe, int player, Vehicle* veh, GameMap* actma
          }
          else
             if (pe->building != NULL) {
-            if ((c == visible_all) || (pe->building->typ->buildingheight >= chschwimmend) || (pe->building->color == player*8))
+            if ((c == visible_all) || (pe->building->typ->height >= chschwimmend) || (pe->building->color == player*8))
                return true;
             }
             else
@@ -523,27 +449,17 @@ bool fieldvisiblenow( const tfield* pe, int player, Vehicle* veh, GameMap* actma
 }
 
 
-bool fieldvisiblenow( const tfield* pe, Vehicle* veh, int player  )
+
+
+VisibilityStates fieldVisibility( const tfield* pe )
 {
-   return fieldvisiblenow( pe, player, veh, veh->getMap());
+   return fieldVisibility( pe, pe->getMap()->actplayer );
 }
 
-bool fieldvisiblenow( const tfield* pe, int player, GameMap* actmap )
-{
-   return fieldvisiblenow( pe, player, NULL, actmap );
-
-}
-
-
-
-
 VisibilityStates fieldVisibility( const tfield* pe, int player )
 {
-   return fieldVisibility( pe, player, actmap );
-}
-
-VisibilityStates fieldVisibility( const tfield* pe, int player, GameMap* gamemap )
-{
+   GameMap* gamemap = pe->getMap();
+   
    if ( player < 0 )
       return visible_all;
 
@@ -582,7 +498,7 @@ void         calculateobject( int       x,
    }
 
    tfield* fld = actmap->getField(x,y) ;
-   Object* oi2 = fld-> checkforobject (  obj  );
+   Object* oi2 = fld-> checkForObject (  obj  );
 
    int c = 0;
    for ( int dir = 0; dir < sidenum; dir++) {
@@ -593,7 +509,7 @@ void         calculateobject( int       x,
 
       if ( fld2 ) {
          if ( obj->netBehaviour & ObjectType::NetToSelf )
-            if ( fld2->checkforobject ( obj )) {
+            if ( fld2->checkForObject ( obj )) {
                c |=  1 << dir ;
                if ( mof )
                   calculateobject ( a, b, false, obj, actmap );
@@ -602,7 +518,7 @@ void         calculateobject( int       x,
 
          for ( int oj = 0; oj < int(obj->linkableObjects.size()); oj++ ) {
             for ( int id = obj->linkableObjects[oj].from; id <= obj->linkableObjects[oj].to; ++id ) {
-               Object* oi = fld2->checkforobject ( actmap->getobjecttype_byid ( id ) );
+               Object* oi = fld2->checkForObject ( actmap->getobjecttype_byid ( id ) );
                if ( oi ) {
                   c |=  1 << dir ;
                   if ( mof )
@@ -652,7 +568,7 @@ void         calculateobject( int       x,
    }
 
    if ( oi2 ) {
-     oi2->setdir ( c );
+     oi2->setDir ( c );
      fld->setparams();
    }
 
