@@ -30,47 +30,21 @@
  #include <sigc++/sigc++.h>
   
   
-  #include "newfont.h"
   #include "typen.h"
   #include "gamemap.h"
+  
 
-   struct Schriften {
-               pfont        smallarial;
-               pfont        smallsystem;
-               pfont        large;
-               pfont        arial8; 
-               pfont        guifont;
-               pfont        guicolfont;
-               pfont        monogui;
-            } ; 
-   extern Schriften schriften;
-
-
-  extern GameMap* actmap; 
-
-//! returns the field at the given coordinates
-extern tfield* getfield(int x, int y);
-
-
-// ! returns the diplomatic status between actmap->actplayer and the player with color b (note that the color is playernum*8 ) 
-//extern int getdiplomaticstatus( int b );
-
-// ! returns the diplomatic status between the players with color c and b (note that the color is playernum*8 ) 
-// extern int getdiplomaticstatus2( int c, int b);
-
-extern void  putbuilding( const MapCoordinate& entryPosition,
+//! puts a building onto the map. To be used by the map editor! For ingame usage, see ConstructBuildingCommand or SpawnBuilding
+extern void  putbuilding( GameMap* gamemap,
+                          const MapCoordinate& entryPosition,
                          int          color,
                          const BuildingType* buildingtyp,
                          int          completion,
-                         int          ignoreunits = 0 ); // f?r Kartened
-
-extern void  putbuilding2( const MapCoordinate& entryPosition,
-                          int color,
-                          BuildingType* buildingtyp);  // f?r Spiel
+                         int          ignoreunits = 0 ); 
 
 
 //! recalculates the connection (like road interconnections) of all objects on the map
-extern void  calculateallobjects( GameMap* m = actmap );
+extern void  calculateallobjects( GameMap* m );
 
 /** recalculates the connection (like road interconnections) of an object
       \param x The x coordinate of the field
@@ -83,12 +57,12 @@ extern void  calculateobject(int  x,
                              int  y,
                              bool mof,
                              const ObjectType* obj,
-                             GameMap* gamemap = actmap );
+                             GameMap* gamemap  );
 
 extern void  calculateobject( const MapCoordinate& pos, 
                              bool mof,
                              const ObjectType* obj,
-                             GameMap* gamemap = actmap );
+                             GameMap* gamemap  );
 
 
 
@@ -108,10 +82,10 @@ extern int fieldAccessible( const tfield*        field,
 
 /** removes all units that cannot exist any more, either due to invalid terrin
     (like tanks on melting ice) or too much wind (eg. hoovercrafts in a storm) */
-extern void  checkunitsforremoval ( void );
+extern void  checkunitsforremoval( GameMap* gamemap );
 
 //! removes all objects that cannot exist where they currently are (terrainaccess)
-extern void checkobjectsforremoval ( void );
+extern void checkobjectsforremoval( GameMap* gamemap );
 
 //! returns the maximum wind speed that the unit can endure
 extern int          getmaxwindspeedforunit ( const Vehicle* eht );
@@ -176,18 +150,20 @@ extern int getheightdelta ( const ContainerBase* c1, const ContainerBase* c2 );
     \param player the player who is 'looking'
   */
 //! {@ 
-  extern bool fieldvisiblenow( const tfield* pe, int player = actmap->actplayer, GameMap* map = actmap );
-  extern bool fieldvisiblenow( const tfield* pe, Vehicle* veh, int player = actmap->actplayer  );
+  extern bool fieldvisiblenow( const tfield* pe, Vehicle* veh, int player   );
+  inline bool fieldvisiblenow( const tfield* pe, int player ) { return fieldvisiblenow( pe, NULL, player); };
+  inline bool fieldvisiblenow( const tfield* pe, Vehicle* veh  ) { return fieldvisiblenow( pe, veh, pe->getMap()->actplayer); };
+  inline bool fieldvisiblenow( const tfield* pe ) { return fieldvisiblenow( pe, pe->getMap()->actplayer);};
 //! }@
 
   /*!
     evaluates the visibility of a field
     \param pe the field to be evaluated
-    \param player the player who is 'looking'
+    \param player the player who is 'looking'; without player the active player will be used
    */
 //! {@ 
-  extern VisibilityStates fieldVisibility  ( const tfield* pe, int player = actmap->actplayer );
-  extern VisibilityStates fieldVisibility  ( const tfield* pe, int player, GameMap* gamemap );
+  extern VisibilityStates fieldVisibility  ( const tfield* pe );
+  extern VisibilityStates fieldVisibility  ( const tfield* pe, int player );
 //! }@
 
 
@@ -202,6 +178,7 @@ extern SigC::Signal1<void,GameMap*> mapChanged;
 extern SigC::Signal1<void,ContainerBase*> showContainerInfo;
 extern SigC::Signal1<void,Vehicletype*> showVehicleTypeInfo;
 extern SigC::Signal0<bool> idleEvent;
+extern SigC::Signal1<void, const Context&> fieldCrossed;
 
 extern void displaymap();
 
