@@ -99,8 +99,7 @@ class PropertyContainer {
          void addNamedInteger ( const ASCString& name, int& property, int tagNum, const char** tags, int defaultValue );
          void addBreakpoint();
         #ifdef ParserLoadImages
-         // void addImage ( const ASCString& name, void* &property, const ASCString& fileName );
-         void addImage ( const ASCString& name, Surface& property, const ASCString& fileName, bool applyFieldMask );
+         void addImage ( const ASCString& name, Surface& property, ASCString& fileName, bool applyFieldMask );
          // void addImageArray ( const ASCString& name, vector<void*> &property, const ASCString& fileName );
          void addImageArray ( const ASCString& name, vector<Surface> &property, const ASCString& fileName );
         #endif
@@ -116,10 +115,11 @@ class PropertyContainer {
          void error ( const ASCString& errmsg );
          bool find ( const ASCString& name );
          virtual ASCString getFileName (  ) = 0;
+         virtual ASCString getArchive (  ) = 0;
          virtual ~PropertyContainer ( ) { };
+         virtual ASCString getLocation() = 0;
       protected:
          PropertyContainer ( const ASCString& baseName, TextPropertyGroup* tpg, bool reading_ ) : reading( reading_ ), levelDepth ( 0 ), textPropertyGroup( tpg ) { };
-         virtual ASCString getLocation() = 0;
       private:
          virtual void writeProperty ( Property& p, const ASCString& value ) = 0;
          void setup ( Property* p, const ASCString& name );
@@ -127,10 +127,10 @@ class PropertyContainer {
 };
 
 class PropertyReadingContainer : public PropertyContainer {
-   protected:
-         virtual ASCString getLocation() { return textPropertyGroup->location; };
    public:
-         virtual ASCString getFileName (  ) { return textPropertyGroup->fileName; };
+      virtual ASCString getLocation() { return textPropertyGroup->location; };
+      virtual ASCString getFileName() { return textPropertyGroup->fileName; };
+         virtual ASCString getArchive() { return textPropertyGroup->archive; };
          PropertyReadingContainer ( const ASCString& baseName, TextPropertyGroup* tpg );
          ~PropertyReadingContainer (  );
          void writeProperty ( Property& p, const ASCString& value );
@@ -138,10 +138,10 @@ class PropertyReadingContainer : public PropertyContainer {
 
 class PropertyWritingContainer : public PropertyContainer {
          tnstream& stream;
-   protected:
-         virtual ASCString getLocation() { return stream.getLocation(); };
    public:
-         virtual ASCString getFileName (  ) { return stream.getDeviceName(); };
+         virtual ASCString getLocation() { return stream.getLocation(); };
+         virtual ASCString getFileName() { return stream.getDeviceName(); };
+         virtual ASCString getArchive() { return stream.getArchive(); };
          PropertyWritingContainer ( const ASCString& baseName, tnstream& stream );
          ~PropertyWritingContainer();
 
