@@ -20,11 +20,10 @@
 
 // #include <config.h>
 
+#include <fstream>
 #include <cstring>
 #include <cerrno>
 #include <cstdio>
-#include <iostream>
-#include <fstream>
 
 #include <SDL_types.h>
 #include <SDL_endian.h>
@@ -60,6 +59,8 @@
 #define WriteCode(a,b,c) fprintf( outfile, "%c%c%c", a, b, c )
 
 
+void displayLogMessage( int verbosity, const ASCString& message )
+{}
 
 void fatalError ( const ASCString& s )
 {
@@ -351,7 +352,7 @@ void testcompress ( const char* name, int size )
    }
    if ( comp ) {
       if ( verbose )
-         printf ( "is already compressed ! " );
+         printf ( " is already compressed ! " );
       copyfile ( name, extractFileName ( buf2, name ), filesize ( name ) );
    } else {
 
