@@ -41,8 +41,10 @@
 #include "viewcalculation.h"
 #include "network/simple_file_transfer.h"
 #include "dialogs/fileselector.h"
+#include "researchexecution.h"
+#include "gameeventsystem.h"
 
-bool authenticateUser ( GameMap* actmap, int forcepasswordchecking = 0, bool allowCancel = true, bool lockView = true, bool throwOnFailure = false  )
+bool authenticateUser ( GameMap* actmap, bool allowCancel = true, bool lockView = true, bool throwOnFailure = false  )
 {
    for ( int p = 0; p < 8; p++ )
       actmap->player[p].existanceAtBeginOfTurn = actmap->player[p].exist() && actmap->player[p].stat != Player::off;
@@ -53,7 +55,7 @@ bool authenticateUser ( GameMap* actmap, int forcepasswordchecking = 0, bool all
          if ( actmap->player[i].isHuman() )
             humannum++;
 
-   if ( humannum > 1  ||  forcepasswordchecking > 0 ) {
+   if ( humannum > 1  ) {
       MapDisplayPG::LockDisplay ld ( !lockView );
 
       bool firstRound = actmap->time.turn() == 1;
@@ -62,24 +64,18 @@ bool authenticateUser ( GameMap* actmap, int forcepasswordchecking = 0, bool all
 
       if ( (!actmap->player[actmap->actplayer].passwordcrc.empty() && actmap->player[actmap->actplayer].passwordcrc != CGameOptions::Instance()->getDefaultPassword() )
          || firstRound  ) {
-            if ( forcepasswordchecking < 0 ) {
-               delete actmap;
-               actmap = NULL;
-               throw NoMapLoaded();
-            } else {
-               bool stat;
-               actmap->setPlayerView ( actmap->actplayer );  // the idle handler of enterpassword starts generating the overview map, so we need to have the correct view prior to enterpassword
-               do {
-                  stat = enterpassword ( actmap->player[actmap->actplayer].passwordcrc, specifyPassword, allowCancel );
-                  if ( !stat ) {
-                     if ( throwOnFailure ) {
-                        delete actmap;
-                        throw NoMapLoaded();
-                     } else
-                        return false;
-                  }
-               } while ( actmap->player[actmap->actplayer].passwordcrc.empty() && stat && viewtextquery ( 910, "warning", "~e~nter password", "~c~ontinue without password" ) == 0 ); /* enddo */
-            }
+            bool stat;
+            actmap->setPlayerView ( actmap->actplayer );  // the idle handler of enterpassword starts generating the overview map, so we need to have the correct view prior to enterpassword
+            do {
+               stat = enterpassword ( actmap->player[actmap->actplayer].passwordcrc, specifyPassword, allowCancel, true, actmap->player[actmap->actplayer].getName() );
+               if ( !stat ) {
+                  if ( throwOnFailure ) {
+                     delete actmap;
+                     throw NoMapLoaded();
+                  } else
+                     return false;
+               }
+            } while ( actmap->player[actmap->actplayer].passwordcrc.empty() && stat && viewtextquery ( 910, "warning", "~e~nter password", "~c~ontinue without password" ) == 0 ); /* enddo */
       } else {
          infoMessage("next player is " + actmap->player[actmap->actplayer].getName() );
          actmap->setPlayerView ( actmap->actplayer );
@@ -88,8 +84,6 @@ bool authenticateUser ( GameMap* actmap, int forcepasswordchecking = 0, bool all
    } else
       actmap->overviewMapHolder.clear( true );
 
-   moveparams.movestatus = 0;
-
    actmap->setPlayerView ( actmap->actplayer );
 
    return true;
@@ -99,7 +93,7 @@ bool authenticateUser ( GameMap* actmap, int forcepasswordchecking = 0, bool all
 
 
 
-void runai( GameMap* actmap, int playerView )
+void runai( GameMap* actmap, int playerView, MapDisplayInterface* display )
 {
    MapDisplayPG::CursorHiding cusorHiding;
    actmap->setPlayerView ( playerView );
@@ -109,7 +103,7 @@ void runai( GameMap* actmap, int playerView )
    if ( !actmap->player[ actmap->actplayer ].ai )
       actmap->player[ actmap->actplayer ].ai = new AI ( actmap, actmap->actplayer );
 
-   actmap->player[ actmap->actplayer ].ai->run();
+   actmap->player[ actmap->actplayer ].ai->run( display );
    updateFieldInfo();
 }
 
@@ -154,7 +148,7 @@ void clearReplays( GameMap* actmap )
 }
 
 
-void iterateToNextPlayer( GameMap* actmap, bool saveNetwork, int lastPlayer, int lastTurn  )
+void iterateToNextPlayer( GameMap* actmap, bool saveNetwork, int lastPlayer, int lastTurn, MapDisplayInterface* display  )
 {
    int loop = 0;
    bool closeLoop = false;
@@ -185,7 +179,7 @@ void iterateToNextPlayer( GameMap* actmap, bool saveNetwork, int lastPlayer, int
       
       if ( actmap->player[nextPlayer].stat == Player::computer ) {
          actmap->beginTurn();
-         runai( actmap, lastPlayer );
+         runai( actmap, lastPlayer, display );
          actmap->endTurn();
       }
       
@@ -207,83 +201,105 @@ void iterateToNextPlayer( GameMap* actmap, bool saveNetwork, int lastPlayer, int
 
 }
 
-void next_turn ( int playerView )
+bool NextTurnStrategy_AskUser::continueWhenLastPlayer() const
 {
-   int lastPlayer = actmap->actplayer;
-   int lastTurn = actmap->time.turn();
+   viewtext2(904);
+   if ( choice_dlg("Do you want to continue playing ?","~y~es","~n~o") == 2) 
+      return false;
+   else
+      return true;
+}
+
+bool NextTurnStrategy_AskUser::authenticate( GameMap* actmap ) const
+{
+   return authenticateUser( actmap );
+}
+
+
+void next_turn ( GameMap* gamemap, const NextTurnStrategy& nextTurnStrategy, MapDisplayInterface* display, int playerView  )
+{
+   int lastPlayer = gamemap->actplayer;
+   int lastTurn = gamemap->time.turn();
 
    if  ( lastPlayer >= 0 )
-      actmap->endTurn();
+      gamemap->endTurn();
    
    int pv;
    if ( playerView == -2 ) {
-      if ( actmap->time.turn() <= 0 || actmap->actplayer < 0 )
+      if ( gamemap->time.turn() <= 0 || gamemap->actplayer < 0 )
          pv = -1;
       else
-         if ( actmap->player[actmap->actplayer].stat != Player::human )
+         if ( gamemap->player[gamemap->actplayer].stat != Player::human )
             pv = -1;
          else
-            pv = actmap->actplayer;
+            pv = gamemap->actplayer;
    } else
       pv = playerView;
 
       
-   if ( findNextPlayer( actmap ) == lastPlayer ) {
-      if ( !actmap->continueplaying ) {
-         viewtext2(904);
-         if (choice_dlg("Do you want to continue playing ?","~y~es","~n~o") == 2) {
-            delete actmap;
-            actmap = NULL;
+   if ( findNextPlayer( gamemap ) == lastPlayer ) {
+      if ( !gamemap->continueplaying ) {
+         
+         
+         if ( !nextTurnStrategy.continueWhenLastPlayer() ) {
+            delete gamemap;
+            gamemap = NULL;
             throw NoMapLoaded();
          } else {
-            actmap->continueplaying = true;
-            if ( actmap->replayinfo ) {
-               delete actmap->replayinfo;
-               actmap->replayinfo = NULL;
+            gamemap->continueplaying = true;
+            if ( gamemap->replayinfo ) {
+               delete gamemap->replayinfo;
+               gamemap->replayinfo = NULL;
             }
          }
       }   
    }
       
  
-   iterateToNextPlayer( actmap, true, lastPlayer, lastTurn );
+   iterateToNextPlayer( gamemap, true, lastPlayer, lastTurn, display );
    
-   actmap->setPlayerView ( -1 );
+   gamemap->setPlayerView ( -1 );
    
-   if ( !authenticateUser( actmap, 0, false, true, true )) {
-      delete actmap;
+   if ( ! nextTurnStrategy.authenticate(gamemap) ) {
+      delete gamemap;
       throw NoMapLoaded();
    }
    
-   actmap->beginTurn();
-   actmap->setPlayerView ( actmap->actplayer );
-   actmap->overviewMapHolder.clear();
-   actmap->sigPlayerUserInteractionBegins( actmap->player[actmap->actplayer] );
+   gamemap->beginTurn();
+   gamemap->setPlayerView ( gamemap->actplayer );
+   gamemap->overviewMapHolder.clear();
+   
+   gamemap->sigPlayerUserInteractionBegins( gamemap->player[gamemap->actplayer] );
+   
+   checktimedevents( gamemap, display );
+   checkevents( gamemap, display );
 }
 
 
 void skipTurn( GameMap* gamemap )
 {
    if ( gamemap->actplayer >= 0 ) {
+      SuppressTechPresentation stp;
+            
       gamemap->beginTurn();
       gamemap->endTurn();
    }
-   iterateToNextPlayer( gamemap, false, -1, -1 );
+   iterateToNextPlayer( gamemap, false, -1, -1, NULL );
 }
 
 void checkUsedASCVersions ( Player& currentPlayer )
 {
    for ( int i = 0; i < 8; i++ )
-      if  ( actmap->player[i].exist() )
-         if ( actmap->actplayer != i )
-            if ( actmap->player[i].ASCversion > 0 )
-               if ( (actmap->player[i].ASCversion & 0xffffff00) > getNumericVersion() ) {
-                  new Message ( ASCString("Player ") + actmap->player[i].getName()
+      if  ( currentPlayer.getParentMap()->player[i].exist() )
+         if ( currentPlayer.getParentMap()->actplayer != i )
+            if ( currentPlayer.getParentMap()->player[i].ASCversion > 0 )
+               if ( (currentPlayer.getParentMap()->player[i].ASCversion & 0xffffff00) > getNumericVersion() ) {
+                  new Message ( ASCString("Player ") + currentPlayer.getParentMap()->player[i].getName()
                            + " is using a newer version of ASC. \n"
                            "Please check www.asc-hq.org for updates.\n\n"
                            "Please do NOT report any problems with this version of ASC until "
                            "you have confirmed that they are also present in the latest "
-                           "version of ASC.", actmap, 1<<actmap->actplayer );
+                           "version of ASC.", currentPlayer.getParentMap(), 1<<currentPlayer.getParentMap()->actplayer );
                   return;
                }
 
@@ -292,7 +308,7 @@ void checkUsedASCVersions ( Player& currentPlayer )
 
 
 
-bool continuenetworkgame ( bool mostRecent )
+GameMap* continueNetworkGame ( bool mostRecent )
 {
    ASCString filename;
    if ( !mostRecent ) {
@@ -320,88 +336,89 @@ bool continuenetworkgame ( bool mostRecent )
    }
 
    if ( filename.empty() )
-      return false;
+      return NULL;
 
    StatusMessageWindowHolder smw = MessagingHub::Instance().infoMessageWindow( "loading " + filename );
 
-   return continuenetworkgame( filename );
+   return continueNetworkGame( filename );
 }
 
-bool continuenetworkgame ( const ASCString& filename )
+GameMap* continueNetworkGame ( const ASCString& filename )
 {
    FileTransfer ft;
    auto_ptr<GameMap> newMap ( mapLoadingExceptionChecker( filename, MapLoadingFunction( &ft, &FileTransfer::loadPBEMFile )));
    if ( !newMap.get() )
-      return false;
+      return NULL;
 
-   if ( !authenticateUser( newMap.get() , 0, true, false ))
-      return false;
+   if ( !authenticateUser( newMap.get(), true, false ))
+      return NULL;
    
-   delete actmap;
-   actmap = newMap.release();
-
-   computeview( actmap );
-   actmap->beginTurn();
-   actmap->setPlayerView ( actmap->actplayer );
-   return true;
+   computeview( newMap.get() );
+   newMap->beginTurn();
+   newMap->setPlayerView ( newMap->actplayer );
+   return newMap.release();
 }
 
 
 
-void  checkforvictory ( )
+void  checkforvictory ( GameMap* gamemap, bool hasTurnControl )
 {
-   if ( !actmap->continueplaying ) {
+   if ( !gamemap->continueplaying ) {
       int plnum = 0;
       for ( int i = 0; i < 8; i++ )
-         if ( !actmap->player[i].exist() && actmap->player[i].existanceAtBeginOfTurn ) {
+         if ( !gamemap->player[i].exist() && gamemap->player[i].existanceAtBeginOfTurn ) {
             int to = 0;
             for ( int j = 0; j < 8; j++ )
                if ( j != i )
                   to |= 1 << j;
 
 
-            if ( !actmap->campaign.avail ) {
+            if ( !gamemap->campaign.avail ) {
                char txt[1000];
                const char* sp = getmessage( 10010 ); // Message "player has been terminated"
    
-               sprintf ( txt, sp, actmap->player[i].getName().c_str() );
-               new Message ( txt, actmap, to  );
+               sprintf ( txt, sp, gamemap->player[i].getName().c_str() );
+               new Message ( txt, gamemap, to  );
             } 
 
-            actmap->player[i].existanceAtBeginOfTurn = false;
+            gamemap->player[i].existanceAtBeginOfTurn = false;
 
-            if ( i == actmap->actplayer ) {
-               displaymessage ( getmessage ( 10011 ),1 );
+            if ( i == gamemap->actplayer ) {
+               if ( gamemap->getPlayerView() == i && gamemap->getPlayer(i).stat == Player::human )
+                  displaymessage ( getmessage ( 10011 ),1 );
 
                int humannum=0;
                for ( int j = 0; j < 8; j++ )
-                  if (actmap->player[j].exist() && actmap->player[j].stat == Player::human )
+                  if (gamemap->player[j].exist() && gamemap->player[j].stat == Player::human )
                      humannum++;
-               if ( humannum )
-                  next_turn();
-               else {
-                  delete actmap;
-                  actmap = NULL;
-                  throw NoMapLoaded();
+
+               if ( hasTurnControl ) {
+                  if ( humannum )
+                     next_turn(gamemap, NextTurnStrategy_AskUser(), &getDefaultMapDisplay() );
+                  else {
+                     delete gamemap;
+                     gamemap = NULL;
+                     throw NoMapLoaded();
+                  }
                }
             }
          } else
             plnum++;
 
       if ( plnum <= 1 ) {
-         if ( actmap->player[actmap->actplayer].ai &&  actmap->player[actmap->actplayer].ai->isRunning() ) {
+         if ( gamemap->player[gamemap->actplayer].ai &&  gamemap->player[gamemap->actplayer].ai->isRunning() ) {
             displaymessage("You lost!",1);
          } else {
             displaymessage("Congratulations!\nYou won!",1);
             if (choice_dlg("Do you want to continue playing ?","~y~es","~n~o") == 2) {
-               delete actmap;
-               actmap = NULL;
+               delete gamemap;
+               gamemap = NULL;
                throw NoMapLoaded();
             } else {
-               actmap->continueplaying = 1;
-               if ( actmap->replayinfo ) {
-                  delete actmap->replayinfo;
-                  actmap->replayinfo = 0;
+               gamemap->continueplaying = 1;
+               if ( gamemap->replayinfo ) {
+                  delete gamemap->replayinfo;
+                  gamemap->replayinfo = 0;
                }
             }
          }
