@@ -25,6 +25,7 @@
 
 class BuildingType;
 
+class Context;
 
 class UnitHooveringLogic {
    //! the percentage of fuel that is required for a flying plane just hoovering in the air (unit: percentage)
@@ -75,11 +76,14 @@ class UnitHooveringLogic {
 
     Vehicle ( const Vehicletype* t, GameMap* actmap, int player );
 
-    int repairableDamage() { return damage; };
+    int repairableDamage() const { return damage; };
 
     bool isBuilding() const { return false; };
 
     const Vehicletype* typ;
+    
+    const Vehicletype* getType() const { return typ; };
+    
     int          ammo[16];
     int          weapstrength[16];
     int          experience;
@@ -89,6 +93,9 @@ class UnitHooveringLogic {
 
     //! the the unit and all cargo to #attacked = true
     void setAttacked();
+    
+    //! the the unit and (optionally) all cargo to #attacked = true
+    void setAttacked( bool recursive, const Context& context );
 
     //! the current level of height ( BITMAPPED ! )
     int          height;
@@ -114,11 +121,12 @@ class UnitHooveringLogic {
        (and not only the network protocols where is was first used, hence the name)
     */
     int          networkid;
-    ASCString    name;
 
     //! The class that manages the reaction fire which causes a unit to attack approaching enemies even if it is not the unit's turn
     class  ReactionFire {
          friend class Vehicle;
+         friend class UnitPropertyEditor;
+         friend class ChangeUnitProperty;
 
          Vehicle* unit;
        public:
@@ -129,8 +137,12 @@ class UnitHooveringLogic {
          // int enemiesAttackable;
          Status getStatus() const { return status;};
 
-         //! enables the reaction fire or returns an error code ( result < 0 = -errorNr ) 
+         /** enables the reaction fire or returns an error code ( result < 0 = -errorNr ) 
+         \deprecated
+         */
          int enable();
+         
+         //! \deprecated 
          void disable();
 
          vector<int> weaponShots;
@@ -182,14 +194,10 @@ class UnitHooveringLogic {
     //@}
     
     
-
-    //! changes a units height and adjusts the movement so that the percentage of used movepoints remains constant
-    void setNewHeight( int bitmappedheight );
-
     /** add the objects like tracks or broken ice
         \returns true if any objects were created
     */
-    bool spawnMoveObjects( const MapCoordinate& start, const MapCoordinate& dest );
+    bool spawnMoveObjects( const MapCoordinate& start, const MapCoordinate& dest, const Context& context );
 
     //! @name Resource related functions
     //@{
@@ -232,7 +240,7 @@ class UnitHooveringLogic {
     //@}
 
   protected:
-    const ResourceMatrix& getRepairEfficiency ( void ) { return repairEfficiency; };
+    const ResourceMatrix& getRepairEfficiency() const { return repairEfficiency; };
 
     virtual void postRepair ( int oldDamage );
 
@@ -247,48 +255,54 @@ class UnitHooveringLogic {
     //! Returns the size of a unit. A size is equal to the weight of the unit without any cargo or carried resources.
     int size ( void );
 
+    //! hook that called when the turn for a player starts
+    void beginTurn();
+    
     //! hook that is called when a player ends his turn
     void endOwnTurn( void );
 
     //! hook that is called when any player (including owner) ends turn
     void endAnyTurn( void );
 
-
     //! hook that is called the next round begins ( active player switching from player8 to player1 )
     void endRound ( void );
 
-    int getIdentification() { return networkid; };
+    int getIdentification() const { return networkid; };
 
 
     //! @name Construction related functions
     //@{
-    //! constructs a vehicle at the given position.
-    Vehicle* constructvehicle ( Vehicletype* tnk, int x, int y );
-
     //! checks whether the unit can construct a vehicle of the given type at the given position.
-    bool vehicleconstructable ( Vehicletype* tnk, int x, int y );
+    bool vehicleconstructable ( const Vehicletype* tnk, int x, int y );
+    
+    Resources getExternalVehicleConstructionCost( const Vehicletype* tnk ) const;
 
     /** checks whether the unit can construct a building of the given type.
     This method does not check if there is enough space around the unit to place
     the building */
-    bool buildingconstructable ( const BuildingType* bld, bool checkResources = true );
+    //bool buildingconstructable ( const BuildingType* bld, bool checkResources = true );
     //@}
     
     //! sets the unit (and its cargo) the a new position (the unit will not be chained to a field)
     void setnewposition ( int x, int y );
     void setnewposition ( const MapCoordinate& mc );
+    void setnewposition ( const MapCoordinate& mc, const Context& context );
 
     /** converts the unit so it is owned by 'player'. Note that the player is passed
-        as parameter (values [0..8]), and not his color (which would be the values
-        [0,8,16,..,64] )
+        as parameter (values [0..8]) )
     */
-    void convert ( int player );
+    void convert ( int player, bool recursive = true );
+    
+    
+    //! this is a low level functions that changes the registration in the map. It's called by convert(int,bool)
+    void registerForNewOwner( int player );
 
     //! sets the status of the unit's energy generator
     void setGeneratorStatus( bool status );
     bool getGeneratorStatus () const { return generatoractive; };
 
     //! callback that is called after the unit has attacked
+    void postAttack( bool reactionFire, const Context& context );
     void postAttack( bool reactionFire );
 
     /** adds the units view to the map. The view must then be evaluated by functions like #evaluateviewcalculation ( GameMap*, int)
@@ -298,19 +312,15 @@ class UnitHooveringLogic {
     /** removes the units view to the map. The view must then be evaluated by functions like #evaluateviewcalculation ( GameMap*, int)
         \sa viewcalculation.cpp */
     void removeview();
+    
+    /** resets the internal view state, so that addview() can be executed again */
+    void resetview();
 
     /** returns true if the units view is currently added to the maps global visibility.
         \sa viewcalculation.cpp */
     bool isViewing ( ) const { return viewOnMap; };
-    const SingleWeapon *getWeapon( unsigned weaponNum );
-
-
-    /** searches for mineral resources.
-        \returns > 0 on success ; < 0 on failure (error number is returned => message.txt )
-    */
-    int searchForMineralResources( ) const;
-
-//    bool searchForMineralResourcesAvailable();
+    
+    const SingleWeapon *getWeapon( unsigned weaponNum ) const;
 
     //! returns the units position
     MapCoordinate3D getPosition ( ) const;
@@ -319,17 +329,13 @@ class UnitHooveringLogic {
     MapCoordinate3D getPosition3D ( ) const;
 
 
-    // Searches this and all nested transport for the unit. Returns the direct parent carrier.
-    // Vehicle* findCarrierUnit ( const Vehicle* veh );
-
-    
     /** can the unit repair anything? This does not necessarily mean that the unit can repair
         other units, because a service weapon is additionally needed for that. It may be the
         case that the unit can only repair itself.
     */
     bool canRepair( const ContainerBase* item ) const;
 
-    /** fills a unit with all resources it can carry and sets it class to "to be determined on map load".
+    /** fills a unit with all resources it can carry 
         This function should only be called in the mapeditor !
     */
     void fillMagically( bool ammo = true, bool resources = true );
@@ -345,7 +351,7 @@ class UnitHooveringLogic {
     */
     void read ( tnstream& stream );
 
-    void write ( tnstream& stream, bool includeLoadedUnits = true );
+    void write ( tnstream& stream, bool includeLoadedUnits = true ) const ;
 
     //! displays the unit at position spos on s
     void paint ( Surface& s, SPoint pos, int shadowDist = -1 ) const;
@@ -375,7 +381,7 @@ class UnitHooveringLogic {
     const Vehicletype::HeightChangeMethod* getHeightChange( int dir, int height = 0 ) const;
 
     //! Is the unit able to shoot ?
-    bool weapexist ( void );
+    bool weapexist() const;
     void prepareForCleanRemove() { cleanRemove = true; };
     ~Vehicle ( );
 
