@@ -60,8 +60,8 @@ extern const char*  cvehiclefunctions[];
  #define cfvehiclefunctionsanzeige 0xFFFFFFFF
 */
 
-#define cwaffentypennum 13
- extern const char*  cwaffentypen[cwaffentypennum] ;
+#define weaponTypeNum 13
+ extern const char*  cwaffentypen[weaponTypeNum] ;
  #define cwcruisemissile 0
  #define cwcruisemissileb ( 1 << cwcruisemissile )
  #define cwminen 1
@@ -89,10 +89,10 @@ extern const char*  cvehiclefunctions[];
  #define cwserviceb ( 1 << cwservicen )
  #define cwobjectplacementn 12
  #define cwobjectplacementb ( 1 << cwobjectplacementn )
- extern const int cwaffenproduktionskosten[cwaffentypennum][3];  /*  Angabe: Waffentyp; energy - Material - Sprit ; jeweils fuer 5er Pack */
+ extern const int ammoProductionCost[weaponTypeNum][3];  /*  Angabe: Waffentyp; energy - Material - Sprit ; jeweils fuer 5er Pack */
 
 
- extern const bool weaponAmmo[cwaffentypennum];
+ extern const bool weaponAmmo[weaponTypeNum];
 
 
 
@@ -177,20 +177,17 @@ extern const char*  cvehiclefunctions[];
  class Vehicletype : public ContainerBaseType {
         //! the image of the unit.
         Surface  image;
+        
+        //! the filename from which the image was loaded
+        ASCString imageFilename;
+        
     public:
         static const int legacyVehicleFunctionNum = 29;
         
-        //! short description of the units role, for example "strategic bomber"
-        ASCString    description;
-
         ASCString    getName() const;
 
         int armor;
 
-        
-        //! the levels of height which this unit can enter
-        int          height;
-
         //! if a transport moves the movement for the units inside a transport is decreased by 1/n of the tranport's distance
         double     cargoMovementDivisor;
 
@@ -251,6 +248,14 @@ extern const char*  cvehiclefunctions[];
         //! the damage this unit can repair itself automatically each turn.
         int          autorepairrate;
 
+        //! if this unit constructs another unit externally (for example a turret), it costs this much of its movement (in percent) 
+        int  unitConstructionMoveCostPercentage;
+        
+        //! the minimal distance (measured in number of fields) in which units can be externally constructed
+        int unitConstructionMinDistance;
+        //! the maximum distance (measured in number of fields) in which units can be externally constructed
+        int unitConstructionMaxDistance;
+        
         //! if the unit is destroyed, it can leave an wreckage object behind ( < 0 to disable )
         vector<int> wreckageObject;
 
