@@ -19,6 +19,7 @@
 #ifndef viewcalculationH
  #define viewcalculationH
 
+ #include "actions/context.h"
  #include "typen.h"
  #include "mapalgorithms.h"
  #include "gamemap.h"
@@ -61,7 +62,7 @@
       \param disableShareView sharing the view between different players is disabled. 
       \returns the number of fields that have a changed visibility for the given players. If nothing changes, the map must not be displayed again after the view calculation
   */
-  extern int computeview( GameMap* gamemap, int player_fieldcount_mask = 0, bool disableShareView = false );
+  extern int computeview( GameMap* gamemap, int player_fieldcount_mask = 0, bool disableShareView = false, const Context* context = NULL );
 
   /** evaluates the view on a given field and saves it for that field. Calls #calcvisibilityfield for the calculation
       Radar and jamming values must have already been applied to the field! 
@@ -80,18 +81,20 @@
       \param disableShareView sharing the view between different players is disabled. 
       \returns the number of fields which have a changed visibility status
   */
-  extern int  evaluateviewcalculation ( GameMap* gamemap, int player_fieldcount_mask = 0, bool disableShareView = false );
+  extern int  evaluateviewcalculation ( GameMap* gamemap, int player_fieldcount_mask = 0, bool disableShareView = false, const Context* context = NULL );
 
   /** evaluates the view on a part of the map.
       Radar and jamming values must have already been applied to the field! 
       \param gamemap the map that the view is calculated of
       \param pos  the central position around which the view is calculated
-      \param distance the radius of the circle around pos in which the view is evaluated. The view is calculated in AT LEAST this circle, in reality it is a rectangle containing this circle.
+      \param distance the radius of the circle around pos in which the view is evaluated. 
+                      The view is calculated in AT LEAST this circle, in reality it is a rectangle containing this circle.
+                      This is not the number of fields, but the distance in points (with 10 points = 1 field) 
       \param player_fieldcount_mask determines, which players should be counted when the view has changed
       \param disableShareView sharing the view between different players is disabled. 
       \returns the number of fields which have a changed visibility status
   */
-  extern int  evaluateviewcalculation ( GameMap* gamemap, const MapCoordinate& pos, int distance, int player_fieldcount_mask = 0, bool disableShareView = false);
+  extern int  evaluateviewcalculation ( GameMap* gamemap, const MapCoordinate& pos, int distance, int player_fieldcount_mask = 0, bool disableShareView = false, const Context* context = NULL );
 
   /** calculates the view on a given field.
       Radar and jamming values must have already been applied to the field! 
@@ -118,8 +121,9 @@
 
      void removeFieldView( const MapCoordinate& pos );
      void addFieldView( const MapCoordinate& pos );
+     const Context* context;
    public:
-      RecalculateAreaView( GameMap* gamemap, const MapCoordinate& pos, int range );
+      RecalculateAreaView( GameMap* gamemap, const MapCoordinate& pos, int range, const Context* context );
       void removeView();
       void addView();
       ~RecalculateAreaView();
