@@ -436,11 +436,13 @@ PG_Widget* TextRenderer :: render( const ASCString& token )
 
 void TextRenderer::clear()
 {
+   BulkGraphicUpdates bgu( this );
    for ( Widgets::iterator i = widgets.begin(); i != widgets.end(); ++i )
       delete *i;
      
    widgets.clear();
    attributes.clear();
+   bgu.release();
 }
 
 
@@ -513,6 +515,7 @@ bool ViewFormattedText :: eventKeyDown(const SDL_KeyboardEvent* key)
    switch ( key->keysym.sym ) {
       case SDLK_ESCAPE:
       case SDLK_RETURN:
+      case SDLK_KP_ENTER:
       case SDLK_SPACE:
          QuitModal();
          return true;
