@@ -64,10 +64,10 @@
 #include "graphics/drawing.h"
 #include "widgets/textrenderer.h"
 
-#include "messaginghub.h"
+#include "util/messaginghub.h"
 
 
-const int widgetTypeNum = 16;
+const int widgetTypeNum = 17;
 const char* widgetTypes[widgetTypeNum]
 =
    { "image",
@@ -85,7 +85,8 @@ const char* widgetTypes[widgetTypeNum]
      "checkbox",
      "lineedit",
      "slider",
-     "plain"
+     "plain",
+     "listbox"
    };
 
 enum  WidgetTypes  { Image,
@@ -103,7 +104,8 @@ enum  WidgetTypes  { Image,
                      CheckBox,
                      LineEdit,
                      Slider,
-                     Plain };
+                     Plain,
+                     ListBox };
 
 const int imageModeNum = 5;
 const char* imageModes[imageModeNum]
@@ -444,7 +446,7 @@ void ASCGUI_Window::parsePanelASCTXT ( PropertyReadingContainer& pc, PG_Widget*
                widgetParams.assign ( img );
                parsePanelASCTXT( pc, img, widgetParams );
             } catch ( tfileerror ) {
-               warning( "unable to load " + filename );
+               warningMessage( "unable to load " + filename );
             }
          } else {
             PG_Image* img = new PG_Image( parent, PG_Point(r.x, r.y ), NULL, false, PG_Draw::BkMode(imgMode) );
@@ -452,19 +454,17 @@ void ASCGUI_Window::parsePanelASCTXT ( PropertyReadingContainer& pc, PG_Widget*
             widgetParams.assign ( img );
             parsePanelASCTXT( pc, img, widgetParams );
          }
-
-      }
+      }  else
       if ( type == Area ) {
          bool mode;
          pc.addBool( "in", mode, true );
 
-         Emboss* tw = new Emboss ( parent, r );
+         Emboss* tw = new Emboss ( parent, r, mode );
          // PG_ThemeWidget* tw = new PG_ThemeWidget ( parent, r, style );
          widgetParams.assign ( tw );
          parsePanelASCTXT( pc, tw, widgetParams );
          newWidget = tw;
-      }
-
+      } else
       if ( type == StaticLabel ) {
          ASCString text;
          pc.addString( "text", text );
@@ -474,7 +474,7 @@ void ASCGUI_Window::parsePanelASCTXT ( PropertyReadingContainer& pc, PG_Widget*
             widgetParams.assign ( lb );
          parsePanelASCTXT( pc, lb, widgetParams );
          newWidget = lb;
-      }
+      } else
       if ( type == TextOutput ) {
          PG_Label* lb = new PG_Label ( parent, r, PG_NULLSTR, style );
 
@@ -482,7 +482,7 @@ void ASCGUI_Window::parsePanelASCTXT ( PropertyReadingContainer& pc, PG_Widget*
             widgetParams.assign ( lb );
          parsePanelASCTXT( pc, lb, widgetParams );
          newWidget = lb;
-      }
+      } else
       if ( type == MultiLineText ) {
          /*
          PG_MultiLineEdit* lb = new PG_MultiLineEdit ( parent, r, style );
@@ -500,8 +500,7 @@ void ASCGUI_Window::parsePanelASCTXT ( PropertyReadingContainer& pc, PG_Widget*
          
          parsePanelASCTXT( pc, tr, widgetParams );
          newWidget = tr;
-      }
-
+      } else
       if ( type == BarGraph ) {
          int dir;
          pc.addNamedInteger( "direction", dir, barDirectionNum, barDirections, 0 );
@@ -524,29 +523,25 @@ void ASCGUI_Window::parsePanelASCTXT ( PropertyReadingContainer& pc, PG_Widget*
          widgetParams.assign ( bg );
          parsePanelASCTXT( pc, bg, widgetParams );
          newWidget = bg;
-      }
-
-
+      } else
       if ( type == SpecialDisplay ) {
          SpecialDisplayWidget* sw = new SpecialDisplayWidget ( parent, r );
          widgetParams.assign ( sw );
 
          parsePanelASCTXT( pc, sw, widgetParams );
          newWidget = sw;
-      }
-
+      } else
       if ( type == SpecialInput ) {
          SpecialInputWidget* sw = new SpecialInputWidget ( parent, r );
 
          parsePanelASCTXT( pc, sw, widgetParams );
          newWidget = sw;
-      }
-
+      } else
       if ( type == Dummy ) {
          SpecialInputWidget* sw = new SpecialInputWidget ( parent, r );
          parsePanelASCTXT( pc, sw, widgetParams );
          newWidget = sw;
-      }
+      } else
       if ( type == ScrollArea ) {
          PG_ScrollWidget* sw = new PG_ScrollWidget( parent, r, style );
          ASCString scrollbar;
@@ -563,8 +558,7 @@ void ASCGUI_Window::parsePanelASCTXT ( PropertyReadingContainer& pc, PG_Widget*
             
          parsePanelASCTXT( pc, sw, widgetParams );
          newWidget = sw;
-      }
-      
+      } else
       if ( type == Button ) {
          PG_Button* sw = new PG_Button( parent, r, style );
 
@@ -580,8 +574,7 @@ void ASCGUI_Window::parsePanelASCTXT ( PropertyReadingContainer& pc, PG_Widget*
             
          parsePanelASCTXT( pc, sw, widgetParams );
          newWidget = sw;
-      }
-
+      } else
       if ( type == RadioButton ) {
          PG_RadioButton* sw = new PG_RadioButton( parent, r, style );
          if ( radioButtonGroup )
@@ -600,8 +593,7 @@ void ASCGUI_Window::parsePanelASCTXT ( PropertyReadingContainer& pc, PG_Widget*
             
          parsePanelASCTXT( pc, sw, widgetParams );
          newWidget = sw;
-         
-      }
+      } else
       if ( type == CheckBox ) {
          PG_CheckButton* sw = new PG_CheckButton( parent, r, style );
 
@@ -616,8 +608,7 @@ void ASCGUI_Window::parsePanelASCTXT ( PropertyReadingContainer& pc, PG_Widget*
             
          parsePanelASCTXT( pc, sw, widgetParams );
          newWidget = sw;
-         
-      }
+      } else
       if ( type == LineEdit ) {
          PG_LineEdit* sw = new PG_LineEdit( parent, r, style );
          
@@ -632,7 +623,7 @@ void ASCGUI_Window::parsePanelASCTXT ( PropertyReadingContainer& pc, PG_Widget*
             
          parsePanelASCTXT( pc, sw, widgetParams );
          newWidget = sw;
-      }
+      } else
       if ( type == Slider ) {
          
          int direction = 0;
@@ -646,7 +637,7 @@ void ASCGUI_Window::parsePanelASCTXT ( PropertyReadingContainer& pc, PG_Widget*
             
          parsePanelASCTXT( pc, sb, widgetParams );
          newWidget = sb;
-      }
+      } else
       if ( type == Plain ) {
          
          PG_ThemeWidget* w = new PG_ThemeWidget( parent, r );
@@ -656,6 +647,15 @@ void ASCGUI_Window::parsePanelASCTXT ( PropertyReadingContainer& pc, PG_Widget*
             
          parsePanelASCTXT( pc, w, widgetParams );
          newWidget = w;
+      } else
+      if ( type == ListBox) {
+         PG_ListBox* l = new PG_ListBox( parent, r );
+         
+         if ( !hasStyle )
+            widgetParams.assign ( l );
+            
+         parsePanelASCTXT( pc, l, widgetParams );
+         newWidget = l;
       }
             
       if ( newWidget && newWidget->GetName().empty() ) 
@@ -833,7 +833,10 @@ bool ASCGUI_Window::setup()
 
       PropertyReadingContainer pc ( "panel", textPropertyGroup );
 
-      int w, h;
+      
+     
+      int w = 0;
+      int h = 0;
       pc.addInteger( "width", w, 0 );
       pc.addInteger( "height", h, 0 );
 
@@ -870,6 +873,35 @@ bool ASCGUI_Window::setup()
          MoveWidget( x1, y1, false );
       }
 
+      int w2 = Width();
+      int h2 = Height();
+      
+      if ( w == 0 ) {
+         int x2 ;
+         pc.addInteger( "x2", x2, 0 );
+         
+         if ( x2 < 0 ) 
+            w2 = PG_Application::GetScreenWidth() - my_xpos + x2;
+         else
+            if ( x2 > 0 )
+               w2 = my_xpos + x2;
+      }
+      
+      if ( h == 0 ) {
+         int y2;
+         pc.addInteger( "y2", y2, 0 );
+         if ( y2 < 0 )
+            h2 = PG_Application::GetScreenHeight() - my_ypos + y2;
+         else
+            if ( y2 > 0 )
+               h2 = my_ypos + y2;
+      }
+      
+      if ( h2 != Height() || w2 != Width() )
+         SizeWidget( w2, h2, false );
+         
+      
+      
       int titlebarHeight;
       pc.addInteger("Titlebarheight", titlebarHeight, -1 );
       if ( titlebarHeight != -1 )
@@ -887,9 +919,9 @@ bool ASCGUI_Window::setup()
 
       return true;
    } catch ( ParsingError err ) {
-      warning( "parsing error: " + err.getMessage());
+      warningMessage( "parsing error: " + err.getMessage());
    } catch ( tfileerror err ) {
-      warning( "could not acces file: " + err.getFileName() );
+      warningMessage( "could not acces file: " + err.getFileName() );
    }
    return false;
 
